/*
    IO.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef IO_H
#define IO_H

#include <iostream>
#include <pthread.h>

#include "util/mytype.h"
#include "Widi/Option.h"
#include "Widi/State.h"
#include "Widi/WidiController.h"

using namespace std;

class SongManager;


class IO
{
protected:
  Option* option;
  State* state;
  State cacheState;
  WidiController* const widiController;
  
  //sf,mi$B$O(BSMF$B$N(Bkey signature$B$N(Bsf,mi$B$G$"$k!#(B
  static const char* getKeySignature (const int sf, const int mi);

  //BPM(Beat Per Minute)$B$rF@$k(B
  uint32 getBPM (uint32 tempo)
    {//tempo 4$BJ,2;Id$ND9$5(B(usec)
      return static_cast <uint32> (60.0*1000000/tempo);
    };
  virtual void update () {};//default$B$G$O2?$b$7$J$$!#(B
  //pause$B$7$F$$$k4VDd;_$9$k!#(B
  inline virtual void checkPause ()
    {
      if (cacheState.songStatus.state == SongStatus::PAUSING)
	{//pause$B$J$N$G%]!<%:2r=|$5$l$k$^$GBT$D!#(B
	  DEBUG_MESSAGE ("IO::checkPause pausing");
	  state->lock ();
	  while (state->songStatus.state == SongStatus::PAUSING)
	    state->waitChange ();
	  state->unlock ();
	  DEBUG_MESSAGE ("IO::checkPause all done");
	}
    };

  //playLoop$B$rAv$i$;$k!#(B
  static void* startPlayLoopThread (void* me);
  
public:
  IO (State* state__, WidiController* widiController__, Option* option__):
    option (option__), state(state__), cacheState (),
    widiController (widiController__)
    {
      assert (state != NULL);
      assert (option != NULL);
      assert (widiController != NULL);
    };
  virtual ~IO (){/* do nothing*/;};

  //$B1iAUA0$K3F<oJQ?t$r=i4|2=$9$k!#(B
  //sub class$B$G$OEvA3?F%/%i%9$N(Breset$B$b8F$s$G$*$/$Y$-!#(B
  virtual void reset (){cacheState.reset ();};//$BFbIt$N>uBV$N=i4|2=(B
  virtual void initialize (){/*do nothing*/;};//$B=i4|2=$r9T$&!#(B
  virtual void release (){/*do nothing*/;};//$B=t!9$N2rJ|$r9T$&!#(B
  //$B6J$N1iAUA0$K8F$P$l$k!#(B
  virtual void prePlay ()
    {state->lock ();state->getState (&cacheState);state->unlock ();reset ();};
  virtual void postPlay (){reset ();};//$B6J$N1iAU8e$K8F$P$l$k!#(B
  
  //$B$3$l$G(Bthread$B$H$7$FAv$k!#(B
  virtual int createPlayLoopThread (pthread_t* thread, pthread_attr_t* attr);
  virtual void playLoop ();//$B$3$l$O(BMainIO$B$GL5$$;~$K(Bloop$B$9$kJ*!#(B
  //MainIO$B$H$7$FF0:n$9$k$H$-$O(BmainLoop$B$r;H$&!#(B

  //////////////////////////////////////////////////////////exceptional class
  class IError
    {
    public:
      IError ();
      IError (const char* errorMessage_):errorMessage (errorMessage) {};
      IError (string errorMessage_):errorMessage (errorMessage_) {};
      string errorMessage;
    };

  //IO$B$N8F$S=P$5$lJ}(B
  //
  //$B5/F0;~(B
  //constructor
  //
  //io->initialize ();
  //while ($B1iAU$9$k6J$,;D$C$F$$$k(B)
  //{
  //  io->prePlay ();
  //  $B1iAU3+;O(B
  //  io->createPlayLoopThread ();
  //      $B1iAU>uBV$K1~$8$F!"?'!9I=<($7$?$j$9$k!#(B
  //      $B1iAU=*N;$N9g?^$G(Bio$B<+$i(BplayLoop$B$+$iH4$1=P$9!#(B
  //      $B8F$S=P$7B&$+$i(Bpthread_join ();
  //  io->postPlay ();
  //}
  //io->relrease ();
  //
  //$B=*N;;~(B
  //destructor
};

#endif // IO_H
