/*
    MainIO.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>
#include <strstream>

#include "util/schedYield.h"
#include "util/PThreadAttribute.h"
#if 0
#include "util/PThreadStartRoutine.h"
#endif
#include "MainIO.h"
#include "Widi/ServerCommand.h"

using namespace std;

const int MainIO::MAX_MESSAGE_NUMBER;
const int MainIO::MAX_MESSAGE_LENGTH;

MainIO::MainIO (State* state__, WidiController* widiController__,
		Option* option__):
  IO (state__, widiController__, option__),
  lock (),
  notEmpty (), notFull (), writepos (0), readpos (0),
  notErrorEmpty (), notErrorFull (), writeErrorPos (0), readErrorPos (0)
{
  messageBuffer = new char*[MAX_MESSAGE_NUMBER];
  errorMessageBuffer = new char*[MAX_MESSAGE_NUMBER];
  for (int i = 0; i < MAX_MESSAGE_NUMBER; i++)
    {
      messageBuffer[i] = new char[MAX_MESSAGE_LENGTH];
      memset (messageBuffer[i], 0, MAX_MESSAGE_LENGTH);
      errorMessageBuffer[i] = new char[MAX_MESSAGE_LENGTH];
      memset (errorMessageBuffer[i], 0, MAX_MESSAGE_LENGTH);
    }
}

MainIO::~MainIO ()
{
  for (int i = 0; i < MAX_MESSAGE_NUMBER; i++)
    {
      delete[] messageBuffer[i];
      delete[] errorMessageBuffer[i];
    }
  delete[] errorMessageBuffer;
  delete[] messageBuffer;
}

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////threadϢ
////////////////////////////////////////////////////////////////////////////
void* MainIO::startMainLoopThread (void* me)
{
  assert (me != NULL);
  static_cast<MainIO*>(me)->mainLoop ();
  return NULL;
}

void* MainIO::startCoutLoopThread (void* me)
{
  assert (me != NULL);
  static_cast<MainIO*>(me)->coutLoop ();
  return NULL;
}

void* MainIO::startCerrLoopThread (void* me)
{
  assert (me != NULL);
  static_cast<MainIO*>(me)->cerrLoop ();
  return NULL;
}


int MainIO::createMainLoopThread (pthread_t* thread, pthread_attr_t* attr)
{
  assert (thread != NULL);
    return pthread_create (thread, attr,
			   &startMainLoopThread,
			   static_cast<void*> (this));
#if 0
  return pthread_create (thread, attr,
			 &PThreadStartRoutine<MainIO, &MainIO::mainLoop>,
			 static_cast<void*> (this));
#endif
}

/////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////ɽ
/////////////////////////////////////////////////////////////////////
void MainIO::initialize ()
{
  PThreadAttribute attribute;
  attribute.setDetachState (PTHREAD_CREATE_DETACHED);
  attribute.setScope (PTHREAD_SCOPE_SYSTEM);

  const int ret1 = pthread_create (&coutThread,
				   &attribute.getPThreadAttribute (),
				   &startCoutLoopThread,
				   static_cast<void*>(this));
#if 0
  const int ret1 = pthread_create (&coutThread,
				   &attribute.getPThreadAttribute (),
				   &PThreadStartRoutine<MainIO,
				                        &MainIO::coutLoop>,
				   static_cast<void*>(this));
#endif
  if (ret1 != 0)
    {
      cerr << "MainIO::initialize: can not create thread:"
	   << strerror (ret1) << endl;
      //XXXäȤޤʥ顼Ȧ
      exit (1);

    }
  const int ret2 = pthread_create (&cerrThread,
				   &attribute.getPThreadAttribute (),
				   &startCerrLoopThread,
				   static_cast<void*>(this));
#if 0
  const int ret2 = pthread_create (&cerrThread,
				   &attribute.getPThreadAttribute (),
				   &PThreadStartRoutine<MainIO,
				                        &MainIO::cerrLoop>,
				   static_cast<void*>(this));
#endif
  if (ret2 != 0)
    {
      cerr << "MainIO::inittialize: can not create thread:"
	   << strerror (ret2) << endl;
      //XXXäȤޤʥ顼Ȧ
      exit (1);

    }

}

void MainIO::release ()
{
  //do nothing
}

void MainIO::update ()
{
  //do nothing
}

void MainIO::mainLoop ()
{
  DEBUG_MESSAGE ("MainIO::mainLoop");
  playLoop ();
  DEBUG_MESSAGE ("MainIO::mainLoop playLoop done");
  widiController->serverCommand->done ();
  DEBUG_MESSAGE ("MainIO::mainLoop all done");
}


/////////////////////////////////////////////////////////////////////
////////////////////////////////////////message, error messageɽ
/////////////////////////////////////////////////////////////////////
void MainIO::coutLoop ()
{
  DEBUG_MESSAGE ("MainIO::coutLoop called");
  while (1)
    {
      lock.lock ();
      //DEBUG_MESSAGE ("DEBUG:MainIO::coutLoop:now waiting.");
      while (writepos == readpos)
	notEmpty.wait (lock);

#ifdef DEBUG
      //DEBUG_MESSAGE ("DEBUG:MainIO::coutLoop locked.");
      //cerr << "DEBUG:readpos=" << readpos << ":writepos=" << writepos);
      //DEBUG_MESSAGE (messageBuffer[readpos]);
#endif
      //cout << messageBuffer[readpos]);
      coutMessage (messageBuffer[readpos]);
      (++(readpos)) %= MAX_MESSAGE_NUMBER;

      //DEBUG_MESSAGE ("now out putting cout done.");
      lock.unlock ();
      notFull.signal ();
      schedYield ();
    }
}

void MainIO::coutMessage (const char* message)
{
  //DEBUG_MESSAGE ("MainIO::coutMesage");
  cout << message << endl;
  //DEBUG_MESSAGE ("MainIO::coutMesage done");
}

void MainIO::cerrLoop ()
{
  DEBUG_MESSAGE ("MainIO::cerrLoop called");
  while (1)
    {
      lock.lock ();
      while (writeErrorPos == readErrorPos)
	notErrorEmpty.wait (lock);

      cerrMessage (messageBuffer[readErrorPos]);
      (++(readErrorPos)) %= MAX_MESSAGE_NUMBER;
      
      lock.unlock ();
      notErrorFull.signal ();
      schedYield ();
    }
}

void MainIO::cerrMessage (const char* message)
{
  cerr << message << endl;
}

////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////producer(message롣)
////////////////////////////////////////////////////////////////////////////

/////////cout
void MainIO::storeMessage (const char* message)
{
  lock.lock ();
  while ((writepos+1)%MAX_MESSAGE_NUMBER == readpos)
    notFull.wait (lock);

#ifdef DEBUG
  //DEBUG_MESSAGE ("now storing message length=" << strlen (message));
  //cerr << "readpos=" << readpos << "writepos=" << writepos << endl;
#endif
  strncpy (messageBuffer[writepos], message, MAX_MESSAGE_LENGTH-1);
  (++(writepos)) %= MAX_MESSAGE_NUMBER;
  
  lock.unlock ();
  notEmpty.signal ();
}
 

void MainIO::normalMessage (const char* message)
{
  storeMessage (message);
}

void MainIO::normalMessage (const char* message, int size)
{
  char* tmpMessage = new char[size+1];
  tmpMessage[size] = '\0';
  memcpy (tmpMessage, message, size);
  storeMessage (tmpMessage);
  delete[] tmpMessage;
}


//////cerr
void MainIO::storeErrorMessage (const char* message)
{
  lock.lock ();
  while ((writeErrorPos+1)%MAX_MESSAGE_NUMBER == readErrorPos)
    notErrorFull.wait (lock);

#ifdef DEBUG
  //cerr << "now storing error length=" << strlen (message) << endl;
#endif
  strncpy (errorMessageBuffer[writeErrorPos], message, MAX_MESSAGE_LENGTH-1);
  (++(writeErrorPos)) %= MAX_MESSAGE_NUMBER;
  
  lock.unlock ();
  notErrorEmpty.signal ();
}

void MainIO::errorMessage (const char* errorMessage)
{
  storeErrorMessage (errorMessage);
}

void MainIO::errorMessage (const char* errorMessage, int size)
{
  char* tmpMessage = new char[size+1];
  tmpMessage[size] = '\0';
  memcpy (tmpMessage, errorMessage, size);
  storeErrorMessage (tmpMessage);
  delete[] tmpMessage;
}
