/*
    MainIO.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef MainIO_H
#define MainIO_H

#include "util/CondVar.h"
#include "IO.h"

class MainIO:public IO
{
protected:
  ///////////////////////////////for thread
  Mutex lock;

  //$B$3$3$N%a%C%;!<%8$N<h$j07$$$O0l;~E*!#(B
  //$BK\Ev$O(BSTL$B$N(BLinked List$B$r;H$C$FITDjD9$KBP1~=PMh$k$h$&$K$9$k$3$H!#(B
  //$B0?$OF0E*$K%a%b%j!<$r3NJ]$9$k!#(B
  static const int MAX_MESSAGE_NUMBER = 1024;
  static const int MAX_MESSAGE_LENGTH = 512;
  //message$B$r(Bbuffering$B$7$F$*$/(B
  char** messageBuffer;
  //message$B$,;D$C$F$$$k$+$I$&$+$K$h$jF14|$r<h$k0Y$N(Bcondition variable$B!#(B
  CondVar notEmpty;
  CondVar notFull;
  //$B:#$I$N0LCV$rFI$_=q$-$7$F$$$k$+3P$($F$*$/!#(B
  int writepos;
  int readpos;
  
  //error message$B$r07$&$H$$$&;v$GF1>e(B
  //error message$B$r(Bbuffering$B$7$F$*$/(B
  char** errorMessageBuffer;
  //error message$B$,;D$C$F$$$k$+$I$&$+$K$h$jF14|$r<h$k0Y$N(Bcondition variable
  CondVar notErrorEmpty;
  CondVar notErrorFull;
  //$B0J2<$I$N0LCV$rFI$_=q$-$7$F$$$k$+3P$($F$*$/!#(B
  int writeErrorPos;
  int readErrorPos;
  
  ///////////////////////////////////////////for pthread_create
  static void* startMainLoopThread (void* me);
  static void* startCoutLoopThread (void* me);
  static void* startCerrLoopThread (void* me);
  
  /////////////////////////////////////////interface$BI=<(MQ!#(B
  virtual void update ();//$B2hLL$N99?7!#(B(mainLoop,playLoop$B$+$i8F$S=P$5$l$k!#(B)
protected:
  //message$B$r=PNO$9$k(Bthread
  pthread_t coutThread;
  //error message$B$r=PNO$9$k(Bthread
  pthread_t cerrThread;
  
  //message,error message$B$r(Bbuffer$B$K3JG<$9$k!#(B
  virtual void storeMessage (const char* message);
  virtual void storeErrorMessage (const char* message);
 ////////////////////////////sub class$B$G$ODL>o0J2<$N(B5$B$D$r(Bover ride$B$9$l$PNI$$!#(B
  virtual void coutLoop ();//message$B$r=PNO$9$k%k!<%W(B
  virtual void cerrLoop ();//error message$B$r=PNO$9$k%k!<%W(B
  //$B<B:]$K%a%C%;!<%8$rI=<($9$k(Bmethod coutLoop$B$+$i8F$S=P$5$l$k!#(B
  virtual void coutMessage (const char* message);
  //error message$B$rI=<($9$k%a%=%C%I(B cerrLoop$B$+$i8F$S=P$5$l$k!#(B
  virtual void cerrMessage (const char* message);
  
public:
  MainIO (State* state__, WidiController* widiController__, Option* option__);
  virtual ~MainIO ();
  virtual void initialize ();//$B3F<o=i4|2=!#(B
  virtual void release ();//$B3F<o8e;OKv!#(B

  //IO$B$H$7$FF0:n$9$k$H$-$N%k!<%W(B
  virtual void mainLoop ();
  //$B$3$l$G(Bthread$B$H$7$FAv$k!#(B
  virtual int createMainLoopThread (pthread_t* thread,pthread_attr_t* attr);
  ///$B0J2<$N%a%C%;!<%8I=<($O(Bnon blocking call$B$G$"$k!#(B
  //$B0J2<$K$h$jB>$N(Bobject$B$,(BMainIO object$B$K(Bmessage,error message$B$NI=<($r(B
  //$B$7$F$b$i$&!#(B
  //stdout
  virtual void normalMessage (const char* message);
  virtual void normalMessage (const char* message, int size);
  //stderr
  virtual void errorMessage (const char* errorMessage);
  virtual void errorMessage (const char* errorMessage, int size);
};
#endif // MainIO_H
