/*
    LCDInfo.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cassert>

#include "LCDInfo.h"

const int LCDInfo::LCD_LINES;
const int LCDInfo::LCD_COLS;

LCDInfo::LCDInfo ()
{
  reset ();
}

LCDInfo::~LCDInfo ()
{
  //do nothing
}

void LCDInfo::reset ()
{
  for (int i = 0; i < LCD_LINES; i++)
    {
      for (int j = 0; j < LCD_COLS; j++)
	isOn[i][j] = false;
    }
}

inline void LCDInfo::setLevel (const int part,
			       uchar level, uchar maxLevel)
{
  assert (0 <= part);
  assert (part < LCD_COLS);
  //assert (0<=level);
  //assert (level<=LCD_LINES);
  //assert (0<=maxLevel);
  //assert (maxLevel<=LCD_LINES);

  //check
  if (level > LCD_LINES)
    level = LCD_LINES;
  if (maxLevel > LCD_LINES)
    maxLevel = LCD_LINES;
  if (maxLevel < level)
    maxLevel = level;
  
  for (int i = 1; i <= level; i++)
    isOn[LCD_LINES-i][part] = true;
  if (maxLevel != 0)
    isOn[LCD_LINES-maxLevel][part] = true;
}


inline void LCDInfo::setLCD (const int page,
			     const uchar dotData[SequencerStatus::PAGE_NUM][SequencerStatus::DOT_SIZE],
			     bool lcd[LCD_LINES][LCD_COLS],
			     const int dataOffset, const int columnOffset)
{
  for (int i = 0; i < 16; i++)
    {
      for (int j = 0; j < 5; j++)
        {
          if (((dotData[page][i+dataOffset])&(0x10 >> j)) != 0)
	    lcd[i][j+columnOffset+MIDI::MAX_CHANNEL] = true;
	  //'+MIDI::MAX_CHANNEL'$B$O(B17-32$B$N0LCV$KI=<($5$;$k0Y!#(B
	  //lcd[i][j+columnOffset]=true;
        }
    }
}


inline void LCDInfo::setPage (int page, const State& nowState)
{
  //level$B$N>C5n(B
  for (int x = MIDI::MAX_CHANNEL; x < LCD_COLS; x++)
    {
      for (int y = 0; y < LCD_LINES; y++)
	isOn[y][x] = false;
    }
  
  ///lcd$B$N9=@.(B
  //dotData 00-15
  setLCD (page, nowState.sequencerStatus.dotData, isOn, 0, 0);
  //dotData 16-31
  setLCD (page, nowState.sequencerStatus.dotData, isOn, 16, 5);
  //dotData 16-31
  setLCD (page, nowState.sequencerStatus.dotData, isOn, 32, 10);
  //dotData 48-63
  for (int i = 0; i < 16; i++)
    {
      if ((nowState.sequencerStatus.dotData[page][i+48] & 0x10))
	isOn[i][15+MIDI::MAX_CHANNEL]=true;
      //isOn[i][15]=true;
    }
}


inline void LCDInfo::setDotData (const State& nowState)
{
#define DISPLAY_TIME ((double) nowState.sequencerStatus.displayTime*nowState.songStatus.rate*720/100/15)
  //displayTime$B$,(B1$BJU$j(B7.2/15second$B$J$N$G!#(B
  if (nowState.sequencerStatus.pageMachineTicks
      <= nowState.sequencerStatus.dotMachineTicks
      &&
      nowState.sequencerStatus.changedPage == 0
      &&
      nowState.songStatus.machineTicks-
      nowState.sequencerStatus.dotMachineTicks < DISPLAY_TIME
      )
    setPage (0, nowState);//page$B$,@_Dj$5$l$F$$$J$1$l$P(B0$B%Z!<%8$rI=<((B
  else if (nowState.songStatus.machineTicks-
	   nowState.sequencerStatus.pageMachineTicks < DISPLAY_TIME)
    setPage (nowState.sequencerStatus.displayPage, nowState);

  //$B;~4V$,2a$.$F$$$l$P2?$b$7$J$$!#(B
#undef DISPLAY_TIME 
}


void LCDInfo::update (const State& nowState,
		      const uchar level[LCD_COLS],
		      const uchar maxLevel[LCD_COLS])
{
  reset ();
  for (int i = 0; i < LCD_COLS; i++)
    setLevel (i, level[i], maxLevel[i]);
  setDotData (nowState);
}


void LCDInfo::update (const State& nowState,
		      const LevelInfo levelInfo[MIDI::MAX_PORT][MIDI::MAX_CHANNEL])
{
  reset ();
  //note:LCD_COLS = MIDI::MAX_CHANNEL*2
  for (int port = 0; port < 2; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
#if 0
	  setLevel (port*MIDI::MAX_CHANNEL+channel, 
		    (uint) ((((double)levelInfo[port][channel].nowLevel)*(LCD_LINES+1))/MAX_LEVEL),
		    (uint) ((((double)levelInfo[port][channel].maxLevel)*(LCD_LINES+1))/MAX_LEVEL));
#else
	  setLevel (port*MIDI::MAX_CHANNEL+channel, 
		    (uint) ((((double)levelInfo[port][channel].nowLevel)*(LCD_LINES+0.5))/MAX_LEVEL),
		    (uint) ((((double)levelInfo[port][channel].maxLevel)*(LCD_LINES+0.5))/MAX_LEVEL));

#endif
	}
    }
  setDotData (nowState);
}

