/*
    LevelInfo.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "LevelInfo.h"

//used WtkLCD.cc
void LevelInfo::getLevel (const State& nowState,
			  const int port, const int channel)
{
  const uchar maxVelocity = nowState.part[port][channel].getMaxVelocity (nowState.songStatus);
  const double tmpMaxLevel = 
    ((double)nowState.sequencerStatus.masterVolume*
      nowState.part[port][channel].channelVolume*
      maxVelocity*
      nowState.part[port][channel].expression);
     
  const uchar noteOnVelocity = nowState.part[port][channel].getNoteOnVelocity (nowState.songStatus);
  const double noteOnLevel = 
    (((double)nowState.sequencerStatus.masterVolume*
      nowState.part[port][channel].channelVolume*
      noteOnVelocity*
      nowState.part[port][channel].expression))
     *90/100;
  //90/100$B$O(Bsustin$B$NJ,!#(B90$B$K?<$$0UL#$O$J$$!#(B

  //$B$I$l$@$1$ND9$5$K$J$k$+7W;;!#(B
  double velocityLevel = tmpMaxLevel;
  if (noteOnLevel > tmpMaxLevel)
    velocityLevel = noteOnLevel;
      
  //$B:GBg$NItJ,$H2<$,$C$F$$$/ItJ,$N0LCV$N7W;;!#(B
#define DECREASING_TIME_MAX_DELAY (nowState.songStatus.rate*29/100) //0.29sec
#define DECREASING_TIME_MAX (nowState.songStatus.rate*7/100) //0.07sec
#define DECREASING_RATE_MAX (MAX_LEVEL*5/100)
#define DECREASING_TIME_NOW (nowState.songStatus.rate*1/100) // 0.01sec
#define DECREASING_RATE_NOW (MAX_LEVEL*2/100)
  while (nowState.songStatus.machineTicks - maxMachineTicks
	 >= DECREASING_TIME_MAX)
    {
      maxMachineTicks += DECREASING_TIME_MAX;
      if (maxLevel <= DECREASING_RATE_MAX)
	{
	  maxLevel = 0;
	  break;
	}
      maxLevel -= DECREASING_RATE_MAX;
    }
  if (velocityLevel >= maxLevel)
    {
      maxLevel = velocityLevel;
      maxMachineTicks = 
	nowState.songStatus.machineTicks+DECREASING_TIME_MAX_DELAY;
    }
  
#define DECREASING_DIFF_TIME (nowState.songStatus.rate) // 1sec  
  //seeking$BEy$,$"$k$+$iI,MW(B
  if (maxMachineTicks > nowState.songStatus.machineTicks+DECREASING_TIME_MAX+DECREASING_TIME_MAX_DELAY+DECREASING_DIFF_TIME)
    {
      DEBUG_MESSAGE ("LevelInfo::getLevel maxMachineTicks is too large.");
      maxLevel = velocityLevel;
      maxMachineTicks = 
	nowState.songStatus.machineTicks+DECREASING_TIME_MAX_DELAY;
    }
#undef DECREASING_DIFF_TIME
  

  while (nowState.songStatus.machineTicks-nowMachineTicks
	 >= DECREASING_TIME_NOW)
    {
      nowMachineTicks += DECREASING_TIME_NOW;
      if (nowLevel <= DECREASING_RATE_NOW)
	{
	  nowLevel = 0;
	  break;
	}
      nowLevel -= DECREASING_RATE_NOW;
    }
  if (velocityLevel >= nowLevel)
    {
      nowLevel = velocityLevel;
      nowMachineTicks = nowState.songStatus.machineTicks;
    }

#define DECREASING_DIFF_TIME (nowState.songStatus.rate) // 1sec  
  //seeking$BEy$,$"$k$+$iI,MW!#(B
  if (nowMachineTicks > nowState.songStatus.machineTicks+DECREASING_DIFF_TIME)
    {
      DEBUG_MESSAGE ("LevelInfo::getLevel nowMachineTicks is too large.");
      nowLevel = velocityLevel;
      nowMachineTicks = nowState.songStatus.machineTicks;
    }
#undef DECREASING_DIFF_TIME
  
#undef DECREASING_TIME_MAX_DELAY
#undef DECREASING_TIME_MAX
#undef DECREASING_RATE_MAX
#undef DECREASING_TIME_NOW
#undef DECREASING_RATE_NOW
}


//used IOXlib.cc,WtkContainerPart2,WtkPart2
void LevelInfo::getLevel2 (const State& nowState,
			   const int port, const int channel)
{
  const uchar maxVelocity = nowState.part[port][channel].getMaxVelocity2 (nowState.songStatus);
  const double tmpMaxLevel = 
    ((double)nowState.sequencerStatus.masterVolume*
      nowState.part[port][channel].channelVolume*
      maxVelocity*
      nowState.part[port][channel].expression);
     
  const uchar noteOnVelocity = nowState.part[port][channel].getNoteOnVelocity2 (nowState.songStatus);
  const double noteOnLevel = 
    (((double)nowState.sequencerStatus.masterVolume*
      nowState.part[port][channel].channelVolume*
      noteOnVelocity*
      nowState.part[port][channel].expression));
  
  //$B$I$l$@$1$ND9$5$K$J$k$+7W;;!#(B
  double velocityLevel = tmpMaxLevel;
  if (noteOnLevel>tmpMaxLevel)
    velocityLevel = noteOnLevel;

  //cerr << "LevelInfo:part=" << part << " maxVelocity=" << static_cast<int> (maxVelocity) << " noteOnVelocity=" << static_cast<int> (noteOnVelocity) << " velocity Level=" << velocityLevel << endl;
  
  //$B:GBg$NItJ,$H2<$,$C$F$$$/ItJ,$N0LCV$N7W;;!#(B
#define DECREASING_TIME_MAX_DELAY (nowState.songStatus.rate*29/100) //0.29sec
#define DECREASING_TIME_MAX (nowState.songStatus.rate*13/100) //0.13sec
#define DECREASING_RATE_MAX (MAX_LEVEL*20/100)
#define DECREASING_TIME_NOW (nowState.songStatus.rate*1/100) // 0.01sec
#define DECREASING_RATE_NOW (MAX_LEVEL*6/1000)
  while (nowState.songStatus.machineTicks-maxMachineTicks
	 >= DECREASING_TIME_MAX)
    {
      maxMachineTicks += DECREASING_TIME_MAX;
      if (maxLevel <= DECREASING_RATE_MAX)
	{
	  maxLevel = 0;
	  break;
	}
      maxLevel -= DECREASING_RATE_MAX;
    }
  if (velocityLevel >= maxLevel)
    {
      maxLevel = velocityLevel;
      maxMachineTicks = 
	nowState.songStatus.machineTicks+DECREASING_TIME_MAX_DELAY;
    }

#define DECREASING_DIFF_TIME (nowState.songStatus.rate) // 1sec  
  //seeking$B$,$"$k$+$iI,MW!#(B
  if (maxMachineTicks > nowState.songStatus.machineTicks+DECREASING_TIME_MAX+DECREASING_TIME_MAX_DELAY + DECREASING_DIFF_TIME)
    {
      DEBUG_MESSAGE ("LevelInfo::getLevel2 maxMachineTicks is too large.");
      maxLevel = velocityLevel;
      maxMachineTicks = 
	nowState.songStatus.machineTicks+DECREASING_TIME_MAX_DELAY;
    }
#undef DECREASING_DIFF_TIME

  while (nowState.songStatus.machineTicks-nowMachineTicks
	 >= DECREASING_TIME_NOW)
    {
      nowMachineTicks += DECREASING_TIME_NOW;
      if (nowLevel <= DECREASING_RATE_NOW)
	{
	  nowLevel = 0;
	  break;
	}
      nowLevel -= DECREASING_RATE_NOW;
    }
  if (velocityLevel >= nowLevel)
    {
      nowLevel = velocityLevel;
      nowMachineTicks = nowState.songStatus.machineTicks;
    }
  
#define DECREASING_DIFF_TIME (nowState.songStatus.rate) // 1sec  
  //seeking$B$,$"$k$+$iI,MW!#(B
  if (nowMachineTicks > nowState.songStatus.machineTicks+DECREASING_DIFF_TIME)
    {
      DEBUG_MESSAGE ("LevelInfo::getLevel2 nowMachineTicks is too large.");
      nowLevel = velocityLevel;
      nowMachineTicks = nowState.songStatus.machineTicks;
    }
#undef DECREASING_DIFF_TIME
  
#undef DECREASING_TIME_MAX_DELAY
#undef DECREASING_TIME_MAX
#undef DECREASING_RATE_MAX
#undef DECREASING_TIME_NOW
#undef DECREASING_RATE_NOW
}

