/*
    IONTrace.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cctype>
#include <cassert>

#include "util/schedYield.h"
#include "IONTrace.h"

using namespace std;

IONTrace::IONTrace (State* state__, WidiController* widiController__,
		    Option* option__):
  IONCurses (state__, widiController__, option__),octaves (-1)
{
}

IONTrace::~IONTrace ()
{
  //do nothing
}

void IONTrace::initialize () throw (Error)
{
  screen=getScreen ();
  lockCurses ();
  (void) set_term (screen);
  infoWindowLines=4;
  infoWindowCols=COLS;
  infoWindow=newwin (infoWindowLines,infoWindowCols,0,0);

  if (LINES-infoWindowLines-MIDI::MAX_PARTS-1>3)
    messageWindowLines=LINES-infoWindowLines-MIDI::MAX_PARTS-1;
  else
    messageWindowLines=3;
  messageWindowCols=COLS;
  messageWindow=newwin (messageWindowLines,messageWindowCols,
			infoWindowLines,0);

  mainWindowLines=LINES-infoWindowLines-messageWindowLines;
  mainWindowCols=COLS;
  mainWindow=newwin (mainWindowLines,mainWindowCols,
		     infoWindowLines+messageWindowLines,0);

  assert (infoWindow!=NULL);
  assert (messageWindow!=NULL);
  assert (mainWindow!=NULL);
  unlockCurses ();

  //////////////////////////////////////////////////̤ν
#define LEFT_SPACE_LENGTH 27
  mvwaddstr(mainWindow,0,0,"Ch ");
  octaves=(COLS-LEFT_SPACE_LENGTH)/OCTAVE;
  if (octaves>MAX_OCTAVES-1)
    octaves=MAX_OCTAVES;
  
  for(int i=0;i<octaves;i++)
    {
      for(int j=0;j<OCTAVE;j++)
	{
	  if(islower(noteName[j]))
	    waddch(mainWindow, noteName[j]);
	  else
	    waddch(mainWindow, ' ');
	}
    }
  mvwaddstr (mainWindow,0,octaves*OCTAVE+3," Inst         Vol Exp PanSB");

  memset (notes,'.',sizeof (notes));
  notes[octaves*OCTAVE]='\0';
  for (int i = 0; i < MIDI::MAX_PARTS; i++)
    mvwprintw (mainWindow,i+1,0,"%02d %s",i+1,notes);

  MainIO::initialize ();
}

/////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////ɽ
/////////////////////////////////////////////////////////////////////
const char IONTrace::noteName[OCTAVE]=
{
  'c', 'C', 'd', 'D', 'e', 'f', 'F', 'g', 'G', 'a', 'A', 'b'
};

void IONTrace::updateMainWindow ()
{
#define DELAY (cacheState.songStatus.rate*20/100)//0.2 second
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  //note
	  for (int i=0;i<Part::MAX_NOTES;i++)
	    {
	      if (cacheState.part[port][channel].note[i] ||
		  cacheState.songStatus.machineTicks-
		  cacheState.part[port][channel].noteOnMachineTicks[i]
		  <DELAY)
		{
		  notes[i%(octaves*OCTAVE)]=noteName[i%OCTAVE];
		}
	      else
		{
		  notes[i]='.';
		}
	    }
#undef DELAY
      
	  mvwaddnstr (mainWindow, port*MIDI::MAX_CHANNEL+channel+1, 3,
		      notes,octaves*OCTAVE);

	  char* pan;
	  if (cacheState.part[port][channel].pan<(0x40 << 7))
	    pan=" L ";
	  else if (cacheState.part[port][channel].pan>(0x40 << 7)+0x7F)
	    pan=" R ";
	  else
	    pan=" C ";

	  char* sustine;
	  if (cacheState.part[port][channel].damperPedal>0)
	    sustine="S";
	  else
	    sustine=" ";

	  char* pitchBend;
	  if (cacheState.part[port][channel].pitchBend>1)
	    pitchBend="^";
	  else if (cacheState.part[port][channel].pitchBend<0)
	    pitchBend="v";
	  else
	    pitchBend=" ";
      
	  //instrument prg..ɽ
	  mvwprintw (mainWindow,
		     (port*MIDI::MAX_CHANNEL+channel)+1, octaves*OCTAVE+4,
		     "%-12s %3d %3d %s%s%s",
		     cacheState.part[port][channel].instrument,
		     (cacheState.part[port][channel].channelVolume >>7),
		     (cacheState.part[port][channel].expression >> 7),
		     pan,
		     sustine,
		     pitchBend
		     );
	}
    }
}






////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////for dynamic loading
///////////////////////////////////////////////////////////////newIO<T>
///////////////////////////////////////////////////////////////newMainIO<T>
///////////////////////////////////////////////////////////////infoIO<T>
////////////////////////////////////////////////////////////////////////////

#include "IO/InfoIO.h"
static IO* newIONTrace (State* state, WidiController* widiController,
			Option* option)
{
  return new IONTrace (state, widiController, option);
}

static MainIO* newMainIONTrace (State* state, WidiController* widiController,
				Option* option)
{
  return new IONTrace (state, widiController, option);
}

InfoIO infoIONTrace=
{
  newIONTrace,
  newMainIONTrace,
  false
};
