/*
    IONWrd.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cassert>

#include "IONWrd.h"

using namespace std;

const short IONWrd::BLACK;
const short IONWrd::RED;
const short IONWrd::GREEN;
const short IONWrd::YELLOW;
const short IONWrd::BLUE;
const short IONWrd::MAGENTA;
const short IONWrd::CYAN;
const short IONWrd::WHITE;

IONWrd::IONWrd (State* state__, WidiController* widiController__,
		Option* option__):
  NCursesBase (),Wrd (state__, widiController__, option__),
  mainWindow (NULL),mainWindowLines (0),mainWindowCols (0),
  hasColors (false),currentAttribute (0)
{
  /*do nothing*/;
}

IONWrd::~IONWrd ()
{
  //do nothing
}


/////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////ɽ
/////////////////////////////////////////////////////////////////////
void IONWrd::initialize ()
{
  screen=getScreen ();
  lockCurses ();
  (void) set_term (screen);
  mainWindowLines=LINES;
  mainWindowCols=COLS;
  mainWindow=newwin (mainWindowLines,mainWindowCols,0,0);
  assert (mainWindow!=NULL);
  start_color ();
  if (has_colors ())
    {
      hasColors=true;
      initColors ();
    }
  unlockCurses ();
  
  Wrd::initialize ();
}

void IONWrd::release ()
{
  //window˴
  delwin (mainWindow);
  //screen(terminal)˴
  lockCurses ();
  (void) set_term (screen);
  endwin ();
  delscreen (screen);
  unlockCurses ();
  
  Wrd::release ();
}

void IONWrd::initColors ()
{
  //init_pair (BLACK,  COLOR_BLACK,  COLOR_BLACK);
  init_pair (RED,    COLOR_RED,    COLOR_BLACK);
  init_pair (GREEN,  COLOR_GREEN,  COLOR_BLACK);
  init_pair (YELLOW, COLOR_YELLOW, COLOR_BLACK);
  init_pair (BLUE,   COLOR_BLUE,   COLOR_BLACK);
  init_pair (MAGENTA,COLOR_MAGENTA,COLOR_BLACK);
  init_pair (CYAN,   COLOR_CYAN,   COLOR_BLACK);
  init_pair (WHITE,  COLOR_WHITE,  COLOR_BLACK);
#if 1
  attribute[0]=COLOR_PAIR (WHITE)|A_NORMAL;
  attribute[1]=COLOR_PAIR (WHITE)|A_BOLD;
  attribute[4]=COLOR_PAIR (WHITE)|A_UNDERLINE;
  attribute[5]=COLOR_PAIR (WHITE)|A_BLINK;
  attribute[6]=COLOR_PAIR (BLACK)|A_NORMAL;
  attribute[7]=COLOR_PAIR (WHITE)|A_REVERSE;
  attribute[8]=COLOR_PAIR (BLACK)|A_NORMAL;

  attribute[16]=COLOR_PAIR (BLACK)  |A_NORMAL;
  attribute[17]=COLOR_PAIR (RED)    |A_NORMAL;
  attribute[18]=COLOR_PAIR (BLUE)   |A_NORMAL;
  attribute[19]=COLOR_PAIR (MAGENTA)|A_NORMAL;
  attribute[20]=COLOR_PAIR (GREEN)  |A_NORMAL;
  attribute[21]=COLOR_PAIR (YELLOW) |A_NORMAL;
  attribute[22]=COLOR_PAIR (CYAN)   |A_NORMAL;
  attribute[23]=COLOR_PAIR (WHITE)  |A_NORMAL;  
  
  attribute[30]=COLOR_PAIR (BLACK)  |A_NORMAL;  
  attribute[31]=COLOR_PAIR (RED)    |A_NORMAL;    
  attribute[32]=COLOR_PAIR (GREEN)  |A_NORMAL;
  attribute[33]=COLOR_PAIR (YELLOW) |A_NORMAL;   
  attribute[34]=COLOR_PAIR (BLUE)   |A_NORMAL;
  attribute[35]=COLOR_PAIR (MAGENTA)|A_NORMAL; 
  attribute[36]=COLOR_PAIR (CYAN)   |A_NORMAL;   
  attribute[37]=COLOR_PAIR (WHITE)  |A_NORMAL;

  attribute[40]=COLOR_PAIR (BLACK)  |A_REVERSE;  
  attribute[41]=COLOR_PAIR (RED)    |A_REVERSE;    
  attribute[42]=COLOR_PAIR (GREEN)  |A_REVERSE;
  attribute[43]=COLOR_PAIR (YELLOW) |A_REVERSE;   
  attribute[44]=COLOR_PAIR (BLUE)   |A_REVERSE;
  attribute[45]=COLOR_PAIR (MAGENTA)|A_REVERSE; 
  attribute[46]=COLOR_PAIR (CYAN)   |A_REVERSE;   
  attribute[47]=COLOR_PAIR (WHITE)  |A_REVERSE;
#else
  attribute[0]=COLOR_PAIR (WHITE);
  attribute[1]=COLOR_PAIR (WHITE);
  attribute[4]=COLOR_PAIR (WHITE);
  attribute[5]=COLOR_PAIR (WHITE);
  attribute[6]=COLOR_PAIR (BLACK);
  attribute[7]=COLOR_PAIR (WHITE);
  attribute[8]=COLOR_PAIR (BLACK);

  attribute[16]=COLOR_PAIR (BLACK)  ;
  attribute[17]=COLOR_PAIR (RED)    ;
  attribute[18]=COLOR_PAIR (BLUE)   ;
  attribute[19]=COLOR_PAIR (MAGENTA);
  attribute[20]=COLOR_PAIR (GREEN)  ;
  attribute[21]=COLOR_PAIR (YELLOW) ;
  attribute[22]=COLOR_PAIR (CYAN)   ;
  attribute[23]=COLOR_PAIR (WHITE)  ;  
  
  attribute[30]=COLOR_PAIR (BLACK)  ;  
  attribute[31]=COLOR_PAIR (RED)    ;    
  attribute[32]=COLOR_PAIR (GREEN)  ;
  attribute[33]=COLOR_PAIR (YELLOW) ;   
  attribute[34]=COLOR_PAIR (BLUE)   ;
  attribute[35]=COLOR_PAIR (MAGENTA); 
  attribute[36]=COLOR_PAIR (CYAN)   ;   
  attribute[37]=COLOR_PAIR (WHITE)  ;

  attribute[40]=COLOR_PAIR (BLACK)  ;  
  attribute[41]=COLOR_PAIR (RED)    ;    
  attribute[42]=COLOR_PAIR (GREEN)  ;
  attribute[43]=COLOR_PAIR (YELLOW) ;   
  attribute[44]=COLOR_PAIR (BLUE)   ;
  attribute[45]=COLOR_PAIR (MAGENTA); 
  attribute[46]=COLOR_PAIR (CYAN)   ;   
  attribute[47]=COLOR_PAIR (WHITE)  ;
#endif
  
  currentAttribute=attribute[0];
}


void IONWrd::prePlay ()
{
  Wrd::prePlay ();

  redrawwin (mainWindow);
  wclear (mainWindow);
  doUpdate ();
}


void IONWrd::doUpdate ()
{
  //windowι
  updateMainWindow ();
  //ºݤΥߥʥɽι
  lockCurses ();
  set_term (screen);
  wnoutrefresh (mainWindow);
  doupdate ();
  unlockCurses ();
}


void IONWrd::updateMainWindow ()
{
  char text[TEXT_SIZE_X+1];
  text[TEXT_SIZE_X]='\0';
  if (hasColors)
    {
      for (uint y_=0;y_<TEXT_SIZE_Y;y_++)
	{
	  memset (text,' ',TEXT_SIZE_X);
	  int length=0;
	  for (uint x_=0;x_<TEXT_SIZE_X;x_++)
	    {
	      const TextScreen::OneChar& oneChar=textScreen.getText (x_+1,y_+1);
	      if (oneChar.getChar ()!='\0')
		text[x_]=oneChar.getChar ();
	      length++;

	      if (currentAttribute!=attribute[oneChar.getColor ()] ||
		  x_==TEXT_SIZE_X-1)
		{
		  currentAttribute=attribute[oneChar.getColor ()];
		  wattrset (mainWindow,currentAttribute);
		  mvwaddnstr (mainWindow,y_,x_,text+x_-length-1,length);
		  length=0;
		}
	    }
	}
    }
  else
    {
      for (uint y_=0;y_<TEXT_SIZE_Y;y_++)
	{
	  memset (text,' ',TEXT_SIZE_X);
	  for (uint x_=0;x_<TEXT_SIZE_X;x_++)
	    {
	      const char tmp=textScreen.getText (x_+1,y_+1).getChar ();
	      if (tmp!='\0')
		text[x_]=tmp;
	    }
	  mvwaddstr (mainWindow,y_,0,text);
	}
    }
}

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////for dynamic loading
///////////////////////////////////////////////////////////////newIO<T>
///////////////////////////////////////////////////////////////newMainIO<T>
///////////////////////////////////////////////////////////////infoIO<T>
////////////////////////////////////////////////////////////////////////////

#include "IO/InfoIO.h"
static IO* newIONWrd (State* state, WidiController* widiController,
		      Option* option)
{
  return new IONWrd (state, widiController, option);
}

InfoIO infoIONWrd=
{
  newIONWrd,
  NULL,
  false
};
