#include "PNG.h"
#include "util/mytype.h"

void PNG::pngErrorHandler (png_structp ,png_const_charp)
{
  throw PNG::PNGError ();
}


PNG::PNG (const string& filename):
  width (0), height (0), bitDepth (-1), colorType (0), interlaceType (0),
  filterType (0), channels (0), rowbytes (0), signature(NULL),
  
  numText (-1), textPtr (NULL),
  
  palette (NULL), numPalette (-1),gamma (0),
  intent (0),sigBit (NULL),trans (NULL), numTrans (-1),transValues (NULL),
  hist (NULL),modTime (NULL),background (NULL),
  offsetX (0), offsetY (0), oFFsUnitType (0),
  resX (0), resY (0), pHYsUnitType (0),

  rowPointers (NULL)
{
  pngPtr = png_create_read_struct (PNG_LIBPNG_VER_STRING,
				   static_cast<png_voidp> (NULL),
				   &PNG::pngErrorHandler,
				   static_cast<png_error_ptr> (NULL));
  if (!pngPtr)
    throw PNG::CanNotAllocate();

  infoPtr = png_create_info_struct (pngPtr);
  if (!infoPtr)
    {
      png_destroy_read_struct (&pngPtr,
			       static_cast<png_infopp> (NULL),
			       static_cast<png_infopp> (NULL));
      throw PNG::CanNotAllocate ();
    }

  endInfo = png_create_info_struct (pngPtr);
  if (!endInfo)
    {
      png_destroy_read_struct (&pngPtr, &infoPtr,
			       static_cast<png_infopp> (NULL));
      throw PNG::CanNotAllocate ();
    }

  load (filename);
}

PNG::~PNG ()
{
  png_destroy_read_struct (&pngPtr, &infoPtr, &endInfo);
  if (rowPointers != NULL)
    {
      for (uint i = 0; i < height; i++)
	delete[] rowPointers[i];
      delete[] rowPointers;
    }
}


void PNG::getInfo (png_structp pngPtr__, png_infop infoPtr__)
{
  png_get_IHDR (pngPtr__, infoPtr__, &width, &height,
		&bitDepth, &colorType,
		&interlaceType, &compressionType, &filterType);
  channels = png_get_channels (pngPtr__, infoPtr__);
  rowbytes = png_get_rowbytes (pngPtr__, infoPtr__);
  signature = png_get_signature (pngPtr__, infoPtr__);
  
  png_get_text (pngPtr__, infoPtr__, &textPtr, &numText);

#ifdef DEBUG
  cerr << "PNG::getInfo width=" << width << " height=" << height
       << " bitDepth=" << bitDepth << " colorType=" << colorType
       << " interlaceType=" << interlaceType
       << " compressionType=" << compressionType
       << " filterType=" << filterType
       << " channels=" << static_cast<int> (channels)
       << " rowbytes=" << rowbytes
       << endl;
#endif
  
  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_PLTE))
    {
      png_get_PLTE (pngPtr__, infoPtr__, &palette, &numPalette);
#ifdef DEBUG
      cerr << "PNG::getInfo numPalette=" << numPalette << endl;
#endif
    }
  
  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_gAMA))
    {
      png_get_gAMA (pngPtr__, infoPtr__, &gamma);
#ifdef DEBUG
      cerr << "PNG::getInfo gamma=" << gamma << endl;
#endif
    }
  
  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_sRGB))
    {
      png_get_sRGB (pngPtr__, infoPtr__, &intent);
#ifdef DEBUG
      cerr << "PNG::getInfo intent=" << intent << endl;
#endif
    }
  
  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_sBIT))
    {
      png_get_sBIT (pngPtr__, infoPtr__, &sigBit);
#ifdef DEBUG
      cerr << "PNG::getInfo png has sigBit" << endl;
#endif
    }
  
  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_tRNS))
    {
      png_get_tRNS (pngPtr__, infoPtr__, &trans, &numTrans, &transValues);
#ifdef DEBUG
      cerr << "PNG::getInfo numTrans=" << numTrans << endl;
#endif
    }

  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_hIST))
    {
      png_get_hIST (pngPtr__, infoPtr__, &hist);
#ifdef DEBUG
      cerr << "PNG::getInfo png has hist" << endl;
#endif
    }

  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_tIME))
    {
      png_get_tIME (pngPtr__, infoPtr__, &modTime);
#ifdef DEBUG
      cerr << "PNG::getInfo png has modTime" << endl;
#endif
    }

  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_bKGD))
    {
      png_get_bKGD (pngPtr__, infoPtr__, &background);
#ifdef DEBUG
      cerr << "PNG::getInfo png has background" << endl;
#endif
    }

  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_oFFs))
    {
      png_get_oFFs (pngPtr__, infoPtr__, &offsetX, &offsetY, &oFFsUnitType);
#ifdef DEBUG
      cerr << "PNG::getInfo offsetX=" << offsetX << " offsetY=" << offsetY
	   << " oFFsUnitType=" << oFFsUnitType << endl;
#endif
    }

  if (png_get_valid (pngPtr__, infoPtr__, PNG_INFO_pHYs))
    {
      png_get_pHYs (pngPtr__, infoPtr__, &resX, &resY, &pHYsUnitType);
#ifdef DEBUG
      cerr << "PNG::getInfo resX=" << resX << " resY=" << resY
	   << " pHYsUnitType=" << pHYsUnitType << endl;
#endif
    }
}

    

void PNG::load (const string& filename)
{
  FILE* fp = fopen (filename.c_str (), "r");
  if (!fp)
    {
      perror ("PNG::load file is not found");
      throw PNG::FileIsNotFound ();
    }


  const size_t PNG_BYTES_TO_CHECK = 8;
  png_byte header[PNG_BYTES_TO_CHECK];
  if (fread (header, 1, PNG_BYTES_TO_CHECK, fp) != PNG_BYTES_TO_CHECK)
    {
      fclose (fp);
      cerr << filename << " is too short to be png format." << endl;
      throw PNG::FileIsTooShort ();
    }

  png_init_io (pngPtr, fp);
  png_set_sig_bytes (pngPtr, PNG_BYTES_TO_CHECK);
  png_read_info (pngPtr, infoPtr);
  getInfo (pngPtr, infoPtr);
  
  //transformation
  //256paletteѴ
  if (colorType == PNG_COLOR_TYPE_GRAY && bitDepth < 8)
    png_set_expand (pngPtr);
  if (bitDepth == 16)
    png_set_strip_16 (pngPtr);
  if (bitDepth < 8)
    png_set_packing (pngPtr);
  if (colorType == PNG_COLOR_TYPE_GRAY ||
      colorType == PNG_COLOR_TYPE_GRAY_ALPHA)
    png_set_gray_to_rgb (pngPtr);

#if 0
  double screenGamma;
  const char* const gammaStr = getenv ("SCREEN_GAMMA");
  if (gammaStr != NULL)
    {
      screenGamma = atof (gammaStr);
    }
  else
    {
      screenGamma = 2.2;
      //screenGamms = 2.0;
      //screenGamms = 1.7;
      //screenGamms = 1.0;
    }

  if (png_get_gAMA (pngPtr, infoPtr, &gamma))
    png_set_gamma (pngPtr, screenGamma, gamma);
  else
    png_set_gamma (pngPtr, screenGamma, 0.50);
#endif
  
  if (png_get_valid (pngPtr, infoPtr, PNG_INFO_PLTE))
    {
      assert (numPalette > 0);
      if (numPalette > MAX_SCREEN_COLORS)
	png_set_dither (pngPtr, palette,
			numPalette, MAX_SCREEN_COLORS, hist, 1);
    }
  else
    {
      DEBUG_MESSAGE ("PNG::open dithering");
      //XXX
      //NOTE 6*7*6 = 252
      //6*7*6 = 5*7*6 + 6*6 + 6
      png_color stdColorCube[6*7*6];
      for (png_byte r = 0; r < 6; r++)
	{
	  for (png_byte g = 0; g < 7; g++)
	    {
	      for (png_byte b = 0; b < 6; b++)
		{
		  const png_byte index = r*7*6+g*6+b;
		  stdColorCube[index].red   = r;
		  stdColorCube[index].green = g;
		  stdColorCube[index].blue  = b;
		}
	    }
	}
      png_set_dither (pngPtr, stdColorCube,
		      6*7*6, MAX_SCREEN_COLORS,
		      NULL, 1);
      //???
      png_set_PLTE (pngPtr, infoPtr, pngPtr->palette, pngPtr->num_palette);
    }
  
  png_read_update_info (pngPtr, infoPtr);
  getInfo (pngPtr, infoPtr);


  rowPointers = new png_bytep[height];
  for (uint i = 0; i < height; i++)
    rowPointers[i] = new png_byte[rowbytes];
  png_read_image (pngPtr, rowPointers);


  png_read_end (pngPtr, endInfo);

  //??? should close after destroyed pngPtr,infoPtr,endInfo ?
  if (fclose (fp) != 0)
    {
      perror ("PNG::open: fclose");
      throw CanNotCloseFile ();
    }
}
