#ifndef PNG_H
#define PNG_H

#include <string>

#include <png.h>

using namespace std;

class PNG
{
public:
  static const int MAX_SCREEN_COLORS = 256;
private:
  //png 
  png_structp pngPtr;
  png_infop infoPtr;
  png_infop endInfo;
  
  //png info data
  png_uint_32 width;
  png_uint_32 height;
  int bitDepth;
  int colorType;
  int interlaceType;
  int compressionType;
  int filterType;
  png_byte channels;
  png_uint_32 rowbytes;
  png_bytep signature;

  int numText;
  png_textp textPtr;

  //optional png info data
  png_colorp palette;
  int numPalette;
  double gamma;
  int intent;
  png_color_8p sigBit;
  png_bytep trans;
  int numTrans;
  png_color_16p transValues;
  png_uint_16p hist;
  png_timep modTime;
  png_color_16p background;
  png_uint_32 offsetX;
  png_uint_32 offsetY;
  int oFFsUnitType;
  png_uint_32 resX;
  png_uint_32 resY;
  int pHYsUnitType;
  

  //image data
  png_bytep* rowPointers;

  //aux method
  static void pngErrorHandler (png_structp, png_const_charp);
  void getInfo (png_structp pngPtr__, png_infop infoPtr__);
  void load (const string& filename);
public:
  PNG (const string& filename);
  ~PNG ();

  //acess method
  const png_bytep* getRowPointers () const {return rowPointers;};
  png_byte getPixel (png_uint_32 x, png_uint_32 y) const
    {return (getRowPointers ()[y])[x];};//???
  
  png_uint_32 getWidth () const {return width;};
  png_uint_32 getHeight () const {return height;};
  int getBitDepth () const {return bitDepth;};
  int getColorType () const {return colorType;};
  int getInterlaceType () const {return interlaceType;};
  int getCompressionType () const {return compressionType;};
  int getFilterType () const {return filterType;};
  png_byte getChannels () const {return channels;};
  png_uint_32 getRowbytes () const {return rowbytes;};
  const png_bytep getSignature () const {return signature;};


  int getNumText () const {return numText;};
  const png_textp getTextPtr () const {return textPtr;};
  
  png_uint_32 getValid (png_uint_32 flag)
    {return png_get_valid (pngPtr, infoPtr, flag);};

  const png_colorp getPalette () const {return palette;};
  int getNumPalette () const {return numPalette;};
  double getGamma () const {return gamma;};
  int getIntent () const {return intent;};
  const png_color_8p getSigBit () const {return sigBit;};
  const png_bytep getTrans () const {return trans;};
  int getNumTrans () const {return numTrans;};
  const png_color_16p getTransValues () const {return transValues;};
  const png_uint_16p getHist () const {return hist;};
  const png_timep getModTime () const {return modTime;};
  const png_color_16p getBackground () const {return background;};
  png_uint_32 getOffsetX () const {return offsetX;};
  png_uint_32 getOffsetY () const {return offsetY;};
  int getOFFsUnitType () const {return oFFsUnitType;};
  png_uint_32 getResX () const {return resX;};
  png_uint_32 getResY () const {return resY;};
  int getPHYsUnitType () const {return pHYsUnitType;};

  
public:
  //exceptional classes
  class PNGError {};
  class FileIsNotFound:public PNGError {};
  class FileIsTooShort:public PNGError {};
  class CanNotAllocate:public PNGError {};
  class CanNotCloseFile:public PNGError {};
  
};
#endif // PNG_H
