#ifndef SherryCommand_H
#define SherryCommand_H

#include "SherryParameter.h"

class SherryCommand
{
public:
  static const int MAX_PARAMETERS = 14;

  //Sherry WRD$BMQ$N(Bcommand$B$,<h$k0z?t$N>pJs(B
  static const SherryParameter::ParameterType PARAMETER_TYPE[8][16][MAX_PARAMETERS];
  static const SherryParameter::ParameterType getParameterType (int command,
								int i)
    {return PARAMETER_TYPE[(command >> 4)&0x07][command & 0x0f][i];};

  typedef uchar CommandType;
  static const CommandType END_OF_DATA               = 0x00;
  static const CommandType START_PLAYING             = 0x01;
  
  static const CommandType RESERVED_COMMAND          = 0x10;
  
  static const CommandType ALLOCATE_COMMAND          = 0x20;
  static const CommandType NEW_PALETTE               = 0x21;
  static const CommandType DELETE_PALETTE            = 0x22;
  static const CommandType NEW_SCREEN                = 0x25;
  static const CommandType DELETE_SCREEN             = 0x26;
  static const CommandType NEW_SCREEN_AND_LOAD_PNG   = 0x27;
  
  static const CommandType TRANSFER_COMMAND          = 0x30;
  static const CommandType TRANSFER_PALETTE          = 0x31;
  static const CommandType TRANSFER_FULL_SCREEN      = 0x35;
  static const CommandType TRANSFER_PARTIAL_SCREEN   = 0x36;
  						     
  static const CommandType PALTTE_COMMAND            = 0x40;
  static const CommandType SET_PALETTE               = 0x41;
  static const CommandType MERGE_PALETTE             = 0x42;
  static const CommandType COPY_PALETTE              = 0x43;
  static const CommandType MASK_COPY_PALETTE         = 0x44;
  static const CommandType PARTIAL_COPY_PALETTE      = 0x45;
  						     
  static const CommandType DRAWING_COMMAND           = 0x50;
  static const CommandType DRAW_TEXT                 = 0x51;
  static const CommandType DRAW_BOX                  = 0x52;
  static const CommandType DRAW_VIRTIAL_LINE         = 0x53;
  static const CommandType DRAW_HORIZONTAL_LINE      = 0x54;
  static const CommandType DRAW_LINE                 = 0x55;
  
  static const CommandType COPY_COMMAND              = 0x60;
  static const CommandType COPY_AREA                 = 0x61;
  static const CommandType MASK_COPY                 = 0x62;
  static const CommandType PLANE_COPY                = 0x63;
  
  static const CommandType DEBUGGER_COMMAND          = 0x70;
  static const CommandType SOURCE_FILE_POSITION      = 0x71;
  static const CommandType DEBUGGER_PRIVATE          = 0x7f;
  
private:
  const uint32 deltaTime;//delta time
  const uint32 totalDeltaTime;//$B:G=i$+$i(Bdelta time$B$r2C;;$7$F$$$C$?$b$N!#(B
  const CommandType commandType;
  const bool skipBitFlag;
  
  SherryParameter** const sherryParameter;
  const int numberOfParameter;//sherryParameter$B$,3NJ]$5$l$?8D?t!#(B
  int nowNumberOfParameter;
public:
  SherryCommand (uint32 deltaTime__, uint32 totalDeltaTime__,
		 CommandType commandType__, int numberOfParameter__);
  ~SherryCommand ();

  void appendParameter (char charParameter);
  void appendParameter (uint8 uint8Parameter);
  void appendParameter (uint16 uint16Parameter);
  void appendParameter (uint32 uint32Parameter);
  
  void appendParameter (const char* charArray, int size);
  void appendParameter (const uint8* uint8Array, int size);
  void appendParameter (const uint16* uint16Array, int size);

  void appendParameter (const SherryParameter::ScreenParameter* screenArray,
			int size);
  void appendParameter (const SherryParameter::PaletteParameter* paletteArray,
			int size);
  void appendParameter (const uint8 paletteCopyMask[32]);
  void appendParameter (const SherryParameter::PaletteNumberParameter* paletteNumberArray,
			int size);

  //////////////////////////////////////////////////////////////access method
  uint32 getDeltaTime () const {return deltaTime;};
  uint32 getTotalDeltaTime () const {return totalDeltaTime;};
  CommandType getCommandType () const {return commandType;};
  bool isSkippable () const {return skipBitFlag;};
  const SherryParameter& getParameter (int i) const
    {
      assert (i >= 0);
      assert (i < numberOfParameter);
      assert (nowNumberOfParameter == numberOfParameter);
      assert (sherryParameter[i] != NULL);
      return *(sherryParameter[i]);
    };

};

#endif  //SherryCommand_H
