#include "SherryParameter.h"

SherryParameter::SherryParameter (char charParameter):
  parameterType (CHAR), sizeOfArray (0)
{
  value.charParameter = charParameter;
}

SherryParameter::SherryParameter (uint8 uint8Parameter):
  parameterType (UINT8), sizeOfArray (0)
{
  value.uint8Parameter = uint8Parameter;
}

SherryParameter::SherryParameter (uint16 uint16Parameter):
  parameterType (UINT16), sizeOfArray (0)
{
  value.uint16Parameter = uint16Parameter;
}

SherryParameter::SherryParameter (uint32 uint32Parameter):
  parameterType (UINT32), sizeOfArray (0)
{
  value.uint32Parameter = uint32Parameter;
}

  
SherryParameter::SherryParameter (const char* charArray, int size):
  parameterType (CHAR_ARRAY),sizeOfArray (size)
{
  value.charArray = new char[size];
  memcpy (value.charArray, charArray, size*sizeof (char));
}

SherryParameter::SherryParameter (const uint8* uint8Array, int size):
  parameterType (UINT8_ARRAY), sizeOfArray (size)
{
  value.uint8Array = new uint8[size];
  memcpy (value.uint8Array, uint8Array, size*sizeof (uint8));
}

SherryParameter::SherryParameter (const uint16* uint16Array, int size):
  parameterType (UINT16_ARRAY), sizeOfArray (size)
{
  value.uint16Array = new uint16[size];
  memcpy (value.uint16Array, uint16Array, size*sizeof (uint16));
}


SherryParameter::SherryParameter (const ScreenParameter* screenArray, int size):
  parameterType (SCREEN_ARRAY), sizeOfArray (size)
{
  value.screenArray = new ScreenParameter[size];
  for (int i = 0; i < size; i++)
    value.screenArray[i] = screenArray[i];
}

SherryParameter::SherryParameter (const PaletteParameter* paletteArray, int size):
  parameterType (PALETTE_ARRAY), sizeOfArray (size)
{
  value.paletteArray = new PaletteParameter[size];
  for (int i = 0; i < size; i++)
    value.paletteArray[i] = paletteArray[i];
}

SherryParameter::SherryParameter (const uint8 paletteCopyMask[32]):
  parameterType (PALETTE_COPY_MASK), sizeOfArray (32)
{
  value.uint8Array = new uint8[32];
  memcpy (value.uint8Array, paletteCopyMask, 32*sizeof (uint8));
}

SherryParameter::SherryParameter (const PaletteNumberParameter* paletteNumberArray, int size):
  parameterType (PALETTE_NUMBER_ARRAY), sizeOfArray (size)
{
  value.paletteNumberArray = new PaletteNumberParameter[size];
  for (int i = 0; i < size; i++)
    value.paletteNumberArray[i] = paletteNumberArray[i];
}


SherryParameter::~SherryParameter ()
{
  switch (parameterType)
    {
    case CHAR_ARRAY:
      delete[] value.charArray;
      break;
    case UINT8_ARRAY:
      delete[] value.uint8Array;
      break;
    case UINT16_ARRAY:
      delete[] value.uint16Array;
      break;
    case SCREEN_ARRAY:
      delete[] value.screenArray;
      break;
    case PALETTE_ARRAY:
      delete[] value.paletteArray;
      break;
    case PALETTE_COPY_MASK:
      assert (sizeOfArray == 32);
      delete[] value.uint8Array;
      break;
    case PALETTE_NUMBER_ARRAY:
      delete[] value.paletteNumberArray;
      break;
    default:
      //do nothing
      assert (sizeOfArray == 0);
      break;
    }
}
