#ifndef SherryWrd_H
#define SherryWrd_H

#include "IO/IO.h"
#include "SherryWrdFile.h"
#include "SherryPalette.h"
#include "PNG.h"

class SherryWrd:public IO
{
public:
  static const int MAX_COLORS = 256;
  static const int MAX_PLANES = 8;//2^8 = 256
  static const int MAX_VIRTUAL_SCREENS = 65536;
  static const int MAX_VIRTUAL_SCREEN_WIDTH = 65535;
  static const int MAX_VIRTUAL_SCREEN_HEIGHT = 65535;
  static const int MAX_VIRTUAL_PALETTES = 65536;
  static const int REAL_SCREEN_SIZE_X = 640;
  static const int REAL_SCREEN_SIZE_Y = 480;
  //Ⱦʸ8x16礭,ʸ16x16礭
  static const int CHAR_WIDTH = 8;
  static const int CHAR_HEIGHT = 16;
  
protected:
  string songParentDirectory;//song fileΤdirectory
  SherryWrdFile* sherryWrdFile;
  SherryWrdFile::Command command;//߼¹command

  //sherryWrdFilefileload롣
  //fileĤʤäꤷ顢sherryWrdFile = NULLˤ롣
  void newOpenSherryWrdFile (const string& songFileName);

  //PNGnew֤
  //fileĤʤä,throw PNGFileIsNotFound ();
  PNG* loadPNG (const string& fileName);
  class PNGFileIsNotFound {};
  
  
  //variablereset
  //over ride餽ǤƤӽФ٤
  virtual void resetVariables ();

  SherryPalette* virtualPalette[MAX_VIRTUAL_PALETTES];//ۥѥå
  SherryPalette realPalette; //¥ѥå

  SherryParameter::ScreenParameter* virtualScreenInfo[MAX_VIRTUAL_SCREENS];
  
  long getLeftUSec (uint32 sherryWrdNextTicks, int16 sherryWrdTimeBase,
		    uint32 songTicks, uint32 songTempo, uint16 songDivision)
    {
      //leftSongTicks = sherryWrdNextTicks*songDivision/sherryWrdTimeBase
      //                 -songTicks;
      //leftUSec = leftSongTicks*songTempo/songDivision;
      return
	static_cast<long> (static_cast<double> (sherryWrdNextTicks)
			   *static_cast<double> (songTempo)
			   /sherryWrdTimeBase
			   -static_cast<double> (songTicks)
			   *static_cast<double> (songTempo)
			   /static_cast<double> (songDivision));
    };
public:
  
  virtual void prePlay ();
  virtual void postPlay ();
  virtual void playLoop ();
  virtual void nonStopPlayLoop (bool waitKey);
  
protected:
  SherryWrd (State* state__, WidiController* widiController__,
	      Option* option__);
  virtual ~SherryWrd ();
  

  ///////////////////////////////////////////////////////////////command¹
  //ºݤ̿¹Ԥ롣
  //delta time==0ΤΤ³Ƽ¹ԡ
  virtual void executeCommand (SherryWrdFile::Command& command__,
			       long leftUSec,
			       bool& isRealPaletteChanged,
			       bool& isRealScreenChanged);
  //exceptional classes
  //to report runtime error
  class RunTimeError {};
  //ϰϳ
  class OutOfRange:public RunTimeError {};
  //񸻤γݤʤä
  class CanNotAllocate:public RunTimeError {};
  //˳ݤƤ񸻤ݤ褦Ȥ
  class AlreadyAllocated:public RunTimeError {};
  //褦Ȥ񸻤ϳݤƤʤä
  //ϡѤ褦Ȥ񸻤ݤƤʤä
  class NotAllocated:public RunTimeError {};
  class CanNotLoadPNG:public RunTimeError {};
  class UnsupportedPNGFormat:public RunTimeError {};
  
  
  //ºݤžʪФƤ
  list<uint16> transferPalettesList;
  class ScreenArea
  {
  public:
    const uint16 screen;
    const uint16 fromLeft;
    const uint16 fromTop;
    const uint16 toLeft;
    const uint16 toTop;
    const uint16 width;
    const uint16 height;
    ScreenArea (uint16 screen__,
		uint16 fromLeft__, uint16 fromTop__,
		uint16 toLeft__,   uint16 toTop__,
		uint16 width__, uint16 height__):
      screen (screen__),
      fromLeft (fromLeft__), fromTop (fromTop__),
      toLeft (toLeft__), toTop (toTop__),
      width (width__), height (height__)
      {};
    ~ScreenArea () {};
  };
  list<ScreenArea> transferScreensList;
  
  //ºݤž̿¹Ԥ롣
  //ºݤɽ̤ϹʤξΤߤι
  //NOTE allocatecommandˤդ뤳ȡ
  //(Ūˤ,transfer̿,delete̿Ȥʤ뤳Ȥͭ롣)
  virtual void transferPaletteAndScreen (bool isRealPaletteChanged,
					 bool isRealScreenChanged);
  virtual void realTransferPalette (uint16 paletteNumber);
  virtual void realTransferScreen (uint16 screen,
				   uint16 fromLeft, uint16 fromTop,
				   uint16 toLeft, uint16 toTop,
				   uint16 width, uint16 height) = 0;
  //žºݤɽƤʪup date롣
  virtual void updatePalette () = 0;
  virtual void updateScreen () = 0;
  
  virtual void execEndOfData ();
  virtual void execStartPlaying ();
			  
  virtual void execReservedCommand ();
			  
  virtual void execNewPalette (const uint16* palettes, int arraySize);
  virtual void execDeletePalette (const uint16* palettes, int arraySize);
  virtual void execNewScreen (const SherryParameter::ScreenParameter* screens,
			      int arraySize);
  //ºݤvirtual screenݤ롣ϰϤˤݾڤ롣
  virtual void doNewScreen (const SherryParameter::ScreenParameter* screens,
			    int arraySize) = 0;
  
  
  virtual void execDeleteScreen (const uint16* screens, int arraySize);
  //ºݤvirtual screen롣ϰϤˤݾڤ롣
  virtual void doDeleteScreen (const uint16* screens, int arraySize) = 0;
  virtual void execNewScreenAndLoadPng (uint16 screen, uint16 palette,
					const char* pngFileName,
					int arraySize);
  virtual void doNewScreenAndLoadPng (uint16 screen, uint16 palett,
				      const PNG* png) = 0;
			  
  virtual void execTransferPalette (uint16 palete);
  virtual void execTransferFullScreen (uint16 screen, uint16 left, uint16 top);
  virtual void execTransferPartialScreen (uint16 screen,
					  uint16 fromLeft, uint16 fromTop,
					  uint16 toLeft, uint16 toTop,
					  uint16 width, uint16 height);
			  
  virtual void execSetPalette (uint16 palette,
			       const SherryParameter::PaletteParameter* rgbs,
			       int arraySize);
  virtual void execMergePalette (uint16 fromPalette1, uint16 fromPalette2,
				 uint16 toPalette,
				 uint8 palette1In, uint8 palette1Bit,
				 uint8 palette1Out,
				 uint8 palette2In, uint8 palette2Bit,
				 uint8 paleete2Out,
				 uint8 transparency1, uint8 transparency2);
  virtual void execCopyPalette (uint16 fromPalette, uint16 toPalette);
  virtual void execMaskCopyPalette (uint16 fromPalette, uint16 toPalette,
				    const uint8* paletteCopyMask);
  virtual void execPartialCopyPalette (uint16 fromPalette, uint16 toPalette,
				       const SherryParameter::PaletteNumberParameter* paletteNumbers, int arraySize);
			  
  virtual void execDrawText (uint16 screen,
			     uint8 planeMask, uint8 mode,
			     uint8 foreground, uint8 background,
			     uint16 left, uint16 top,
			     const char* sjisStr, int arraySize);
  //ºݤʸ褹롣
  //ݾڤ롣
  virtual void doDrawText (uint16 screen,
			   uint8 planeMask, uint8 mode,
			   uint8 foreground, uint8 background,
			   uint16 left, uint16 top,
			   const char* eucStr, int eucLength) = 0;
  virtual void execDrawBox (uint16 screen, uint8 planeMask,
			    uint16 x1, uint16 y1,
			    uint16 x2, uint16 y2,
			    uint8 foreground);
  //ºݤ˥ܥå
  //Ȥݾڤ롣
  virtual void doDrawBox (uint16 screen, uint8 planeMask,
			  uint16 x1, uint16 y1,
			  uint16 x2, uint16 y2,
			  uint8 foreground) = 0;
  virtual void execDrawVirtialLine (uint16 screen, uint8 planeMask,
				    uint16 x1, uint16 y1, uint16 y2,
				    uint8 foreground);
  virtual void execDrawHorizontalLine (uint16 screen, uint8 planeMask,
				       uint16 x1, uint16 x2, uint16 y1,
				       uint8 foreground);
  virtual void execDrawLine (uint16 screen, uint8 planeMask,
			     uint16 x1, uint16 y1,
			     uint16 x2, uint16 y2,
			     uint8 foreground);
  //ºݤʬ
  //Ȥݾڤ롣
  virtual void doDrawLine (uint16 screen, uint8 planeMask,
			   uint16 x1, uint16 y1,
			   uint16 x2, uint16 y2,
			   uint8 foreground) = 0;


  
  virtual void execCopyArea (uint16 fromScreen, uint16 toScreen,
			     uint8 planeMask, uint8 transparencyFlag,
			     uint16 x1, uint16 y1,
			     uint16 x2, uint16 y2,
			     uint16 toX, uint16 toY);
  //ºݤ˥ԡԤ
  //Ȥݾڤ롣
  virtual void doCopyArea (uint16 fromScreen, uint16 toScreen,
			   uint8 planeMask, uint8 transparencyFlag,
			   uint16 x1, uint16 y1,
			   uint16 x2, uint16 y2,
			   uint16 toX, uint16 toY) = 0;
  
  virtual void execMaskCopy (uint16 fromScreen, uint16 toScreen,
			     uint8 planeMask, uint8 transparencyFlag,
			     uint16 x1, uint16 y1,
			     uint16 x2, uint16 y2,
			     uint16 toX, uint16 toY,
			     uint8 maskX, uint8 maskY,
			     const uint8* maskData, int arraySize);
  //ºݤ˥ԡԤ
  //Ȥݾڤ롣
  virtual void doMaskCopy (uint16 fromScreen, uint16 toScreen,
			   uint8 planeMask, uint8 transparencyFlag,
			   uint16 x1, uint16 y1,
			   uint16 x2, uint16 y2,
			   uint16 toX, uint16 toY,
			   uint8 maskX, uint8 maskY,
			   const uint8* maskData, int arraySize) = 0;
  
  virtual void execPlaneCopy (uint16 fromScreen, uint16 toScreen,
			      uint8 planeMask, uint8 transparencyFlag,
			      uint8 fromPlane, uint8 toPlane,
			      uint8 planeNumber,
			      uint16 x1, uint16 y1,
			      uint16 x2, uint16 y2,
			      uint16 toX, uint16 toY);
  //ºݤ˥ԡԤ
  //Ȥݾڤ롣
  virtual void doPlaneCopy (uint16 fromScreen, uint16 toScreen,
			    uint8 planeMask, uint8 transparencyFlag,
			    uint8 fromPlane, uint8 toPlane,
			    uint8 planeNumber,
			    uint16 x1, uint16 y1,
			    uint16 x2, uint16 y2,
			    uint16 toX, uint16 toY) = 0;
			  
  virtual void execSourceFilePosition (uint16 sourceFileNumber,
				       uint16 sourceFileLineNumber);
  virtual void execDebuggerPrivate (const char* privateData, int arraySize);
};
#endif // SherryWrd_H
