#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "PNG.h"
#include "IO/X/WidiToolkit/WdkWindow.h"

int main (int argc, char** argv)
{
  if (argc <= 1)
    return 0;

  const string filename (argv[1]);

  PNG png (filename);

  Display* display = XOpenDisplay (NULL);
  int screenNumber = DefaultScreen (display);
  int depth = DefaultDepth (display,screenNumber);
  Visual* visual = DefaultVisual (display,screenNumber);



  WdkWindow window (display, NULL, 0, 0, png.getWidth (), png.getHeight (),
		    0, depth, visual);
  
  Colormap colormap= XCreateColormap (display, window.getWindow (),
				      window.getVisual (), AllocAll);
  window.setColormap (colormap);
  window.selectInput (ExposureMask);
  window.map ();
  XFlush (display);

  XColor xcolor[PNG::MAX_SCREEN_COLORS];
  for (int i = 0; i < PNG::MAX_SCREEN_COLORS; i++)
    {
      xcolor[i].pixel = i;
      xcolor[i].red   = (png.getPalette ()[i].red)   * 257;
      xcolor[i].green = (png.getPalette ()[i].green) * 257;
      xcolor[i].blue  = (png.getPalette ()[i].blue)  * 257;
      xcolor[i].flags = DoRed | DoGreen | DoBlue;

      cerr << "IOXSherry::updatePalette"
	   << i << "-th color:"
	   << "pixel=" << xcolor[i].pixel
	   << " r=" << xcolor[i].red
	   << " g=" << xcolor[i].green
	   << " b=" << xcolor[i].blue
	   << endl;
    }
  XStoreColors (display, colormap, xcolor, PNG::MAX_SCREEN_COLORS);
  
  XImage* image = XCreateImage (display, visual, depth, ZPixmap,
				0, NULL,
				png.getWidth (), png.getHeight (), 8, 0);
  image->data = new char [image->bytes_per_line*png.getHeight ()];


  for (uint y = 0; y < png.getHeight (); y++)
    {
      for (uint x = 0; x < png.getWidth (); x++)
	{
	  XPutPixel (image, x, y, (png.getRowPointers ()[y])[x]);
	}
    }
  
  WdkGC gc (&window);
  for (;;)
    {
      XEvent event;
      XNextEvent (display, &event);
      XPutImage (display, window.getWindow (), gc.getGC (), image,
		 0, 0, 0, 0, png.getWidth (), png.getHeight ());
      XFlush (display);
    }

  delete[] image->data;
  image->data = NULL;
  XDestroyImage (image);
}
