#include "util/KanjiConvert.h"
#include "TextScreen.h"

//const uint TextScreen::TEXT_SIZE_X;
//const uint TextScreen::TEXT_SIZE_Y;
const char TextScreen::ESC;

void TextScreen::locate (uint x_, uint y_)
{
#if 0
  assert (1 <= x_);
  assert (x_ <= TEXT_SIZE_X);
  assert (1 <= y_);
  assert (y_ <= TEXT_SIZE_Y);
#endif
  if (x_ < 1) x_ = 1;
  if (x_ > TEXT_SIZE_X) x_ = TEXT_SIZE_X;
  if (y_ < 1) y_ = 1;
  if (y_ > TEXT_SIZE_Y) y_ = TEXT_SIZE_Y;
  
  x = x_;
  y = y_;
}

void TextScreen::printChar (char chr_)
{
  assert (isiso (chr_));
  checkPreChar (x, y);
  checkPostChar (x, y);
  textRam[x-1][y-1].set (chr_, color, CHARSET_ISO);
  right ();

  assert (checkScreen ());
}

void TextScreen::printChar2b (char c1, char c2)
{
  assert (iskanji (c1) || iskana (c1));
  if (iskanji (c1))
    {//$BA43Q4A;z(B
      assert (x < TEXT_SIZE_X);
      assert (checkScreen ());
	    
      checkPreChar (x, y);
      textRam[x-1][y-1].set (c1, color, CHARSET_KANJI1);
      right ();
      
      checkPostChar (x, y);
      textRam[x-1][y-1].set (c2, color, CHARSET_KANJI2);
      assert (textRam[x-1][y-1].getCharSet () == CHARSET_KANJI2);
      right ();

      if (!checkScreen ())
	{
	  cerr << "TextScreen::printChar2b x=" << x-2 << ":y=" << y << ":c1=" << hex << static_cast<int> (c1) << ":c2=" << static_cast<int> (c2) << dec << endl;
	  cerr << "c2.charset="
	       << static_cast<int> (textRam[x-1-1][y-1].getCharSet ()) << endl;
	}
      assert (checkScreen ());
    }
  else if (iskana (c1))
    {//$BH>3Q$+$J(B
      checkPreChar (x, y);
      checkPostChar (x, y);
      textRam[x-1][y-1].set (c2, color, CHARSET_KANA);
      right ();

      assert (checkScreen ());
    }
  else
    {//error!!!
      DEBUG_MESSAGE ("ERROR:TextScreen::printChar2b invalid parameter");
    }

}


void TextScreen::tcls (uint x1=1, uint y1=1,
		       uint x2 = TEXT_SIZE_X, uint y2 = TEXT_SIZE_Y,
		       int color = 0, char chr = ' ')
{
  assert (1 <= x1);
  assert (x1 <= TEXT_SIZE_X);
  assert (1 <= y1);
  assert (y1 <= TEXT_SIZE_Y);
  assert (1 <= x2);
  assert (x2 <= TEXT_SIZE_X);
  assert (1 <= y2);
  assert (y2 <= TEXT_SIZE_Y);
  assert (x1 <= x2);
  assert (y1 <= y2);
  assert (0 <= color);
  assert (color <= 47);
  assert (isiso (chr));
  
  for (uint y_ = y1; y_ <= y2; y_++)
    {
      for (uint x_ = x1; x_ <= x2; x_++)
	{
	  textRam[x_-1][y_-1].set (chr, color, CHARSET_ISO);
	}
    }

  //check 2byte char
  //??? needs this ???
  for (uint y_ = y1; y_ <= y2; y_++)
    {
      checkPreChar (x1, y_, color, chr);
      checkPostChar (x2, y_, color, chr);
    }

  assert (checkScreen ());
}


void TextScreen::scroll (uint x1 = 1, uint y1 = 1,
			 uint x2 = TEXT_SIZE_X, uint y2 = TEXT_SIZE_Y,
			 uint32 mode = 0, int color = 0, char chr = ' ')
{
  assert (1 <= x1);
  assert (x1 <= TEXT_SIZE_X);
  assert (1 <= y1);
  assert (y1 <= TEXT_SIZE_Y);
  assert (1 <= x2);
  assert (x2 <= TEXT_SIZE_X);
  assert (1 <= y2);
  assert (y2 <= TEXT_SIZE_Y);
  assert (x1 <= x2);
  assert (y1 <= y2);
  assert (mode == 0 || mode == 1 || mode == 2 || mode == 3);
  assert (0 <= color);
  assert (color <= 47);
  assert (isiso (chr));
  
  switch (mode)
    {
    case 0://up
      for (uint y_ = y1+1; y_ < y2; y_++)
	{
	  //check 2byte char
	  checkPreChar (x1, y_, color, chr);
	  if (textRam[x1-1][y_-1].getCharSet () == CHARSET_KANJI2)
	    textRam[x1-1][y_-1].set (chr, color, CHARSET_ISO);
	  if (textRam[x2-1][y_-1].getCharSet () == CHARSET_KANJI1)
	    textRam[x2-1][y_-1].set (chr, color, CHARSET_ISO);
	  checkPostChar (x2, y_, color, chr);
	  
	  //scroll
	  for (uint x_ = x1; x_ <= x2; x_++)
	    textRam[x_-1][y_-1].set (textRam[x_-1][y_-1+1]);
	}
      tcls (x1, y2, x2, y2, color, chr);
      break;
    case 1://down
      for (uint y_ = y2-1; y_ >= y1; y_--)
	{
	  //check 2byte char
	  checkPreChar (x1, y_, color, chr);
	  if (textRam[x1-1][y_-1].getCharSet () == CHARSET_KANJI2)
	    textRam[x1-1][y_-1].set (chr, color, CHARSET_ISO);
	  if (textRam[x2-1][y_-1].getCharSet () == CHARSET_KANJI1)
	    textRam[x2-1][y_-1].set (chr, color, CHARSET_ISO);
	  checkPostChar (x2, y_, color, chr);
	  
	  //scroll
	  for (uint x_ = x1; x_ <= x2; x_++)
	    textRam[x_-1][y_-1+1].set (textRam[x_-1][y_-1]);
	}
      tcls (x1, y1, x2, y1, color, chr);
      break;
    case 2://right
      for (uint y_ = y1; y_ <= y2; y++)
	{
	  //2byte char check
	  checkPreChar (x1, y_, color, chr);
	  if (textRam[x1-1][y_-1].getCharSet () == CHARSET_KANJI2)
	    textRam[x1-1][y_-1].set (chr, color, CHARSET_ISO);
	  if (textRam[x2-1][y_-1].getCharSet () == CHARSET_KANJI1)
	    textRam[x2-1][y_-1].set (chr, color, CHARSET_ISO);
	  checkPostChar (x2+1, y_, color, chr);
	  
	  //scroll
	  for (uint x_ = x2; x_ >= x1+1; x_--)
	    textRam[x_-1][y_-1].set (textRam[x_-1-1][y_-1]);
	}
      tcls (x1, y1, x1, y2, color, chr);
      break;
    case 3://left
      for (uint y_ = y1; y_ <= y2; y++)
	{
	  //2byte char check
	  checkPreChar (x1-1, y_, color, chr);
	  if (textRam[x1-1][y_-1].getCharSet () == CHARSET_KANJI2)
	    textRam[x1-1][y_-1].set (chr,color,CHARSET_ISO);
	  if (textRam[x2-1][y_-1].getCharSet ()==CHARSET_KANJI1)
	    textRam[x2-1][y_-1].set (chr,color,CHARSET_ISO);
	  checkPostChar (x2, y_, color, chr);
	  
	  //scroll
	  for (uint x_ = x1; x_ <= x2-1; x_++)
	    textRam[x_-1][y_-1].set (textRam[x_-1+1][y_-1]);
	}
      tcls (x2,y1,x2,y2,color,chr);
      break;
      
    default:
      DEBUG_MESSAGE ("ERROR:TextScreen::scroll mode is invalid!!!");
      break;
    }

  assert (checkScreen ());
}


void TextScreen::doEscapeSequence (const char*& ptr)
{
  //DEBUG_MESSAGE ("TextScreen::doEscapeSequence");
  assert (ptr!=NULL);
  assert (*ptr==ESC);
  ptr++;
  switch(*(ptr++))
    {
    case 'D': // (ESC D) get down the cursor
      if (y<TEXT_SIZE_Y)
	down ();
      else
	scroll ();
      break;
    case 'E': // (ESC E) carriage return and line feed
      nextLine ();
      break;
    case 'M': // (ESC M) get up the cursor
      if (1<y)
	up ();
      else
	scroll (1,1,TEXT_SIZE_X,TEXT_SIZE_Y,1);
      break;
    case 'R': // (ESC R) remove the line and scroll up
      tcls (1,y,TEXT_SIZE_X,y);
      scroll (1,y,TEXT_SIZE_X,TEXT_SIZE_Y);
      break;
    case '2': // erase the lines
      tcls (1,y,TEXT_SIZE_X,TEXT_SIZE_Y);
      break;
    case '3': // erase the lines
      tcls (1,1,TEXT_SIZE_X,y);
      break;
    case 'T': // erase the line
      tcls (1,y,TEXT_SIZE_X,y);
      break;
    case 'Y': 
    case '*': // clear screen
      tcls ();
      break;
    case '=': // (ESC=lc) move the cursor
      {
	const uint x_=*(ptr++)-0x20;
	const uint y_=*(ptr++)-0x20;
	locate (x_,y_);
	break;
      }
    case '[':
      switch(*(ptr++))
	{
	case '>':
	  switch(*(ptr++))
	    {
	    case '1':
	      switch(*(ptr++))
		{
		case 'h': // (ESC[>1h) hide function key
		  break;
		case 'l': // (ESC[>1l) show function key
		  break;
		}
	      break;
	    case '3':
	      switch(*(ptr++))
		{
		case 'h': // (ESC[>3h) set to 20 line mode
		  break;
		case 'l': // (ESC[>3l) set to 25 line mode
		  break;
		case 'n': // (ESC[>3n) set to 31 line mode
		  break;
		}
	      break;
	    case '5':
	      switch(*(ptr++))
		{
		case 'h': // (ESC[>5h) hide cursor
		  break;
		case 'l': // (ESC[>5l) show cursor
		  break;
		}
	      break;
	    }
	  break;
	case 's': // (ESC[s) save both cursor position and attribute
	  save ();
	  break;
	case 'u': // (ESC[u) recover both cursor position and attribute
	  restore ();
	  break;
	case '0': case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
	  {
	    int n;
	    for (n = *(ptr-1)-'0'; '0' <= *ptr && *ptr <= '9'; ptr++)
	      n=n*10+*ptr-'0';//should use strtol???
	    switch(*(ptr++))
	      {
	      case 'A': // 
		up (n);
		break;
	      case 'B':
		down (n);
		break;
	      case 'C':
		right (n);
		break;
	      case 'D':
		left (n);
		break;
	      case 'J':
	      case 'j': // small capital for some WRDs
		switch(n)
		  {
		  case 0:
		    //tclear(1, loc_y, TX_SIZE, TY_SIZE);// $B%+!<%=%k9T$N=hM}(B???
		    tcls (1,y,TEXT_SIZE_X,TEXT_SIZE_Y);
		    break;
		  case 1:
		    //tclear(1, 1, TX_SIZE, loc_y);// $B%+!<%=%k9T$N=hM}(B???
		    tcls (1,1,TEXT_SIZE_X,y);
		    break;
		  case 2: // clear whole screen and move cursor to home position
		    tcls ();
		    locate (1,1);
		    break;
		  }
		break;
	      case 'K':
		switch(n)
		  {
		  case 0:
		    tcls (x,y,TEXT_SIZE_X,y);
		    break;
		  case 1:
		    tcls (1,y,x,y);
		    break;
		  case 2:
		    tcls (1,y,TEXT_SIZE_X,y);
		    break;
		  }
		break;
	      case 'L':
		for(int i = 0; i < n; i++)
		  tcls (1,y+i,TEXT_SIZE_X,TEXT_SIZE_Y,0);
	      break;
	      case 'M':
		for(int i = 0; i < n; i++)
		  tcls (1,y,TEXT_SIZE_X,TEXT_SIZE_Y-i,1);
	      break;
	      case 'm': // 
		setColor (n);
		break;
	      case ';':
		{
		  unsigned m;
		  for (m = 0; '0' <= *ptr && *ptr <= '9'; ptr++)
		    m=m*10+*ptr-'0';//should use strtol ???
		  switch(*(ptr++))
		    {
		    case 'f':
		    case 'H':
		      locate (m,n);
		      break;
		    case 'm':
		      break;
		    }
		}
	      break;
	      }
	  }
	break;
	}
      break;
    case ')':
      switch(*(ptr++))
	{
	case '3': // (ESC)3) set to graphic character mode
	  break;
	case '0': // (ESC)0) set to kanji mode
	  break;
	}
      break;
    }

  assert (checkScreen ());
}

bool TextScreen::checkScreen ()
{
  for (uint y_ = 1; y_ < TEXT_SIZE_Y; y_++ )
    {
      for (uint x_ = 1; x_ < TEXT_SIZE_X; x_++)
	{
	  const OneChar& oneChar = getText (x_, y_);
	  
	  switch (oneChar.getCharSet ())
	    {
	    case CHARSET_ISO:
	      if (!isiso (oneChar.getChar ()))
		{
		  cerr << "Text::Screen:checkScreen CHARSET_ISO,but isiso failed" << endl;
		  return false;
		}
	      break;
	    case CHARSET_KANJI1:
	      if (!iskanji (oneChar.getChar ()))
		{
		  cerr << "Text::Screen:checkScreen CHARSET_KANJI1,but iskanji failed." << endl;
		  return false;
		}
	      if (x_ == TEXT_SIZE_X)
		{
		  cerr << "Text::Screen:checkScreen CHARSET_KANJI1,but x=TEXT_SIZE_X" << endl;
		  return false;
		}
	      x_++;
	      {
		const OneChar& oneChar2 = getText (x_, y_);
		if (oneChar2.getCharSet () != CHARSET_KANJI2)
		  {
		    cerr << "Text::Screen:checkScreen CHARSET_KANJI1's next char is not CHARSET_KANJI2" << endl;
		    cerr << "x=" << x_-1 << ":y=" << y_ << ":c1=" << hex << static_cast<uint> (oneChar.getChar ()) << ":c2=" << static_cast<uint>(oneChar2.getChar ()) << dec << ":c2.charset=" << static_cast<int>(oneChar2.getCharSet ())<< endl;
		    return false;
		  }
		if (oneChar2.getChar () & 0x80 == 0)
		  {
		    cerr << "Text::Screen:checkScreen CHARSET_KANJI2,but &0x80 == 0" << endl;
		    return false;
		  }
	      }
	      break;
	    case CHARSET_KANJI2:
	      {
		cerr << "Text::Screen:checkScreen only CHARSET_KANJI2 is found." << endl;
		return false;
	      }
	    break;
	    case CHARSET_KANA:
	      if (oneChar.getChar () & 0x80 == 0)
		{
		  cerr << "Text::Screen:checkScreen CHARSET_KANA,but &0x80 == 0 " << endl;
		  return false;
		}	      break;
	    }
	}
    }
  return true;
}
