#ifndef TextScreen_H
#define TextScreen_H

#include <algorithm>
#include <cassert>

#include "util/mytype.h"
#include "util/KanjiConvert.h"

using namespace std;

//text screen$B$NFbMF$HB0@-$r3P$($F$*$/%/%i%9(B
//escape sequence$BEy$N2r<a$b$9$k!#(B
class TextScreen
{
public:
  //static const uint TEXT_SIZE_X=80; //text screen size
  //static const uint TEXT_SIZE_Y=25;
#define TEXT_SIZE_X 80U//text screen size
#define TEXT_SIZE_Y 25U
  static const char ESC = '\x1b';//escape sequencer$B$N;O$^$j(B
  
  //0:off 1:on 2:$BG\3Q%b!<%I(B
  enum TextMode
  {
    TEXT_MODE_OFF = 0,
    TEXT_MODE_ON = 1,
    TEXT_MODE_DOULBE = 2
  };
  
  enum CharSet
  {
    CHARSET_ISO,//ascii
    CHARSET_KANJI1,//kannji 1byte$BL\(B
    CHARSET_KANJI2,//kannji 2byte$BL\(B
    CHARSET_KANA,//kana
  };
  class OneChar
  {
  protected:
    char chr;
    int color;
    CharSet charset;
    bool hasChanged;
  public:
    OneChar ():chr (' '),color (0),charset (CHARSET_ISO),hasChanged (false)
      {/*do nothing*/;};
    //$B=i4|2=(B
    void reset ()
      {chr = ' '; color = 0 ; charset = CHARSET_ISO; hasChanged = false;};
    void set (char chr_,int color_,CharSet charset_)
      {
	assert (TextScreen::checkColor (color_));
	if (chr != chr_ || color != color_ || charset != charset_)
	  {
	    hasChanged = true;
	    chr = chr_;
	    color = color_;
	    charset = charset_;
	  }
      };
    void set (OneChar oneChar)
      {
	if (chr != oneChar.chr || color != oneChar.color ||
	    charset != oneChar.charset)
	  {
	    hasChanged = true;
	    chr = oneChar.chr;
	    color = oneChar.color;
	    charset = oneChar.charset;
	  }
	assert (TextScreen::checkColor (color));
      };
    void refresh () {hasChanged = false;};

    char getChar () const {return chr;};
    int getColor () const {return color;};
    CharSet getCharSet () const {return charset;};
    bool isChanged () const {return hasChanged;};
  };
private:
  TextMode textMode;
  OneChar textRam[TEXT_SIZE_X][TEXT_SIZE_Y];

  //cursor$B0LCV(B,@LOCATE (y;x) @LOCATE (x,y)
  //!!!NOTE!!! $B:8>e$,(B(1,1)$B$G$"$k;v$KCm0U!#(B
  uint x;//1<=x<=TEXT_SIZE_X
  uint y;//1<=y<=TEXT_SIZE_Y
  //save$B$5$l$?(Bcursor$B0LCV(B
  uint savedX;
  uint savedY;
  
  //$B8=:_$N(Bcolor
  int color;
  //save$B$5$l$?(Bcolor
  int savedColor;
  /*
  color	0:default
	1:$B%O%$%i%$%H(B(98$B$G$O%b%N%/%m$N$_(B)
	2:virtical line
	4:$B2<@~(B($BF1>e(B)
	5:$BE@LG(B($BF1>e(B)
	7:$BH?E>(B($BF1>e(B)
	16,8:$B%7!<%/%l%C%H(B
	17:$B@V(B
	18:$B@D(B
	19:$B;g(B
	20:$BNP(B
	21:$B2+?'(B
	22:$B?e?'(B
	23:$BGr(B $BG;(B($BL@(B)
	30:$B9u(B $BC8(B($B0E(B)
	31:$B@V(B
	32:$BNP(B
	33:$B2+(B
	34:$B@D(B
	35:$B%^%<%s%@(B
	36:$B%7%"%s(B
	37:$BGr(B $BG;(B($BL@(B)
	40-47:30-37$B$NH?E>(B
	*/

public:
  TextScreen ():
    textMode (TEXT_MODE_ON),x (1),y (1),savedX (1),savedY (1),
    color (0),savedColor (0) {};
  ~TextScreen () {};
  //$BJQ?t=i4|2=(B
  void reset ()
    {
      textMode = TEXT_MODE_ON;
      x = 1;
      y = 1;
      savedX = 1;//???
      savedY = 1;//???
      color = 0;//???
      savedColor = 0;//???
      for (uint y_ = 1; y_ <= TEXT_SIZE_Y; y_++)
	{
	  for (uint x_ = 1; x_ <= TEXT_SIZE_X; x_++)
	    textRam[x_-1][y_-1].reset ();
	}
    }
  
  //cursor$B0\F0(B
  void locate (uint x_, uint y_);
  //$B2~9TI|5"=hM}(B
  void nextLine ()
    {
      x = 1;
      if (y == TEXT_SIZE_Y)
	scroll ();
      else
	down ();
    };
  void down (int dy=1)
    {
      y += dy;
      if (y < 1) y = 1;
      if (y > TEXT_SIZE_Y) y = TEXT_SIZE_Y;
    };
  void up (int dy = 1) {down (-dy);};
  void right (int dx = 1)
    {
      x += dx;
      if (x < 1) x = 1;
      if (x > TEXT_SIZE_X) x = TEXT_SIZE_X;
    };
  void left (int dx = 1) {right (-dx);};
  void refresh ()
    {
      for (uint y_ = 1; y_ <= TEXT_SIZE_Y; y_++)
	{
	  for (uint x_ = 1; x_ <= TEXT_SIZE_X; x_++)
	    {
	      textRam[x_-1][y_-1].refresh ();
	    }
	}
    };
  
  //$BJ8;zI=<(Ey(B
  //$B0lJ8;zI=<((B
  void printChar (char chr);
  //$B0lJ8;zI=<((B(2byte)
  void printChar2b (char c1, char c2);
  void ton (uint32 sw)
    {assert (sw == 0 || sw == 1 || sw == 2);
    textMode=static_cast<TextMode>(sw);};
  void setColor (int color_){assert (checkColor (color_)); color = color_;};
  void tcls
  (uint x1 = 1, uint y1 = 1,
   uint x2 = TEXT_SIZE_X, uint y2 = TEXT_SIZE_Y,
   int color = 0, char chr = ' ');
  void scroll (uint x1=1,uint y1=1,uint x2=TEXT_SIZE_X,uint y2=TEXT_SIZE_Y,
	       uint32 mode=0,int color=0,char chr=' ');
  void save (){savedX=x;savedY=y;savedColor=color;};
  void restore (){x=savedX;y=savedY;color=savedColor;};
  //ptr$B$+$i$N(Bescape sequence$B$r2r<a$9$k!#(B
  //ptr$B$r2r<a8e$N0LCV$K0\F0$7$FJV$9!#(B
  void doEscapeSequence (const char*& ptr);
  
  /////////////////////////////////////////////////////////////access method
  const OneChar& getText (int x_,int y_) const {return textRam[x_-1][y_-1];};
  uint getX () const {assert (1 <= x); assert (x <= TEXT_SIZE_X); return x;};
  uint getY () const {assert (1 <= y); assert (y <= TEXT_SIZE_Y); return y;};
  int getColor () const {return color;};
  TextMode getTextMode () const {return textMode;};
  //check color_ is valid
  static bool checkColor (int color_)
    {
      return (( 0 <= color_ && color_ <=  1) ||
	      ( 4 <= color_ && color_ <=  8) ||
	      (16 <= color_ && color_ <= 23) ||
	      (30 <= color_ && color_ <= 37) ||
	      (40 <= color_ && color_ <= 47)) ;
    };  
  //text$B$NFbMF$,(B2byte$BJ8;z$r$A$c$s$HI=<($7$F$$$k$+Ey$r%A%'%C%/!#(B
  bool checkScreen ();
protected:
  //needs those???
  void checkPreChar (uint x_, uint y_, int color = 0, char chr = ' ')
    {
      //2byte check ???
      assert (isiso (chr));
      if (1 < x_ &&
	  textRam[x_-1-1][y_-1].getCharSet () == CHARSET_KANJI1)
	textRam[x_-1-1][y_-1].set (chr, color, CHARSET_ISO);
    };
  void checkPostChar (uint x_, uint y_, int color = 0, char chr = ' ')
    {
      //2byte check ???
      assert (isiso (chr));
      if (x_ < TEXT_SIZE_X &&
	  textRam[x_-1+1][y_-1].getCharSet () == CHARSET_KANJI2)
	textRam[x_-1+1][y_-1].set (chr, color, CHARSET_ISO);
    };
};
#endif TextScreen_H
