#include <algorithm>

#include "util/KanjiConvert.h"
#include "util/schedYield.h"
#include "util/getABSTimeSpec.h"
#include "IO/ioutil/setSuffix.h"
#include "IO/ioutil/getDirectoryName.h"
#include "Wrd.h"

#ifndef HAVE_NANOSLEEP
#include "compat/nanosleep.h"
#endif // HAVE_NANOSLEEP

using namespace std;

const int Wrd::GRAPHIC_SIZE_X;
const int Wrd::GRAPHIC_SIZE_Y;
const int Wrd::MAX_TEXT_COLORS;
const int Wrd::MAX_GRAPHIC_DEPTH;
const int Wrd::MAX_GRAPHIC_COLORS;
const uint32 Wrd::WRD_TICKS_PER_MEASURE;
const int Wrd::MAX_PALETTE;
const int Wrd::Palette::MAX_RGB;
const int Wrd::WRD_USER_FONT_NUM;
const int Wrd::WRD_USER_FONT_CHAR_NUM;
const int Wrd::WRD_REGSAVE_SIZE_MAX;
const int Wrd::WRD_REG_SIZE;
const uint32 Wrd::FADING_TICKS_PER_MEASURE;
const int Wrd::MAX_FADING_COUNT;

Wrd::Wrd (State* state__, WidiController* widiController__,
	  Option* option__):
  IO (state__, widiController__, option__), wrdFile (NULL)
{
  resetVariables ();
}

Wrd::~Wrd ()
{
  delete wrdFile;
}



void Wrd::newOpenWrdFile (const char* midiFileName)
{
  const int fileNameLength=strlen (midiFileName);
  if (wrdFile!=NULL && fileNameLength > 3 &&
      strncmp (midiFileName,wrdFile->getFileName (),fileNameLength-3)==0 &&
      midiFileName[fileNameLength-4]=='.')
    return;//$B4{$KFI$_9~$_:Q!#(B
  
  delete wrdFile;
  wrdFile=NULL;
  
  wrdFile=new WrdFile ();
  char* wrdFileName=new char[fileNameLength+1];
  strcpy (wrdFileName,midiFileName);

  setSuffix (wrdFileName, "wrd");
  try
    {
#ifdef DEBUG
      cerr << "Wrd::newOpenWrdFile trying load " << wrdFileName << endl;
#endif
      wrdFile->load (wrdFileName);
    }
  catch (WrdFile::FileNotFound)
    {
#ifdef DEBUG
      cerr << "file " << wrdFileName
	   << " is not found.trying .WRD suffinx." << endl;
#endif
      setSuffix (wrdFileName,"WRD");
      try
	{
	  wrdFile->load (wrdFileName);
	}
      catch (WrdFile::FileNotFound)
	{
#ifdef DEBUG
	  cerr << "file " << wrdFileName<< " is not found." << endl;
	  cerr << "wrd file is not found" << endl;
#endif
	  delete wrdFile;
	  wrdFile=NULL;
	}
    }
  delete[] wrdFileName;
}


MagFile* Wrd::newLoadMag (const string& fileName_)
{
  string magName = fileName_;
  MagFile* mag = new MagFile (magName.c_str ());
  
  try
    {
      mag->load ();
    }
  catch (MagFile::MError)
    {
      delete mag;
      magName = songParentDirectory + fileName_;
      mag = new MagFile (magName.c_str ());
      try
	{
	  mag->load ();
	}
      catch (MagFile::MError)
	{
	  delete mag;
	  magName = path + fileName_;
	  mag = new MagFile (magName.c_str ());
	  try
	    {
	      mag->load ();
	    }
	  catch (MagFile::MError)
	    {
	      delete mag;
	      throw MagFileIsNotFound ();
	    }
	}
    }
  
  if(mag->header.ScreenMode & 0x80 == 256)
    {
      cerr <<  "ERROR:IOXWrd::execMAG 256 color MAG file not supported.\n" << endl;
      delete mag;
      throw MagFileIsNotFound ();
    }

  return mag;
}

void Wrd::loadMag (MagFile* mag, int32 x_, int32 y_, uint32 s, uint32 p)
{
  assert (mag != NULL);
  
  if(x_ == -1)//x
    x_ = mag->header.Left;
  x_ |= ((x_ & 0x7)? 0x7: 0);//8bit$B$G4]$a$k!#(B
  if(y_ == -1)//y
    y_ = mag->header.Top;
  
  const int setPalNum=(activePage == 0)? 18: 19;
  for (int i = 0; i < MAX_GRAPHIC_COLORS; i++)
    {
      //mag.Red,mag.Green,mag.Blue$B$N<h$jGd$kCM$O(B0-255
      //$B0lJ}(Bpalette$B$NJ}$O(B0-15$B$G$"$k;v$KCm0U!#(B
      //$B>-Mh$O=$@5$9$Y$-!#(BXXX(palette$B$NJ}$r(B255$B$K$9$k$Y$-!#(B)
      //NOTE:15*17=255
#if 0
      palette[17].rgb[i].set (mag->Red[i] >> 4, mag->Green[i] >> 4, mag->Blue[i] >> 4);
      palette[setPalNum].rgb[i].set (mag->Red[i] >> 4, mag->Green[i] >> 4, mag->Blue[i] >> 4);
      if (p == 0 || p == 2)
	palette[0].rgb[i].set (mag->Red[i] >> 4, mag->Green[i] >> 4, mag->Blue[i] >> 4);
#else
      palette[17].rgb[i].set (mag->Red[i]/17, mag->Green[i]/17, mag->Blue[i]/17);
      palette[setPalNum].rgb[i].set (mag->Red[i]/17, mag->Green[i]/17, mag->Blue[i]/17);
      if (p == 0 || p == 2)
	palette[0].rgb[i].set (mag->Red[i]/17, mag->Green[i]/17, mag->Blue[i]/17);
#endif
    }

  execMAG (*mag, x_, y_, s, p);
}


void Wrd::loadMag (const string& songFileName_)
{
  string magFileName_ = songFileName_;
  setSuffix (magFileName_, "mag");
  MagFile* mag = new MagFile (magFileName_.c_str ());

  try
    {
      mag->load ();
    }
  catch (MagFile::MError)
    {
      delete mag;
# ifdef DEBUG
      cerr << "file" << magFileName_ << " is not found.trying .MAG suffinx." << endl;
# endif
      setSuffix (magFileName_, "MAG");
      mag = new MagFile (magFileName_.c_str ());
      try
	{
	  mag->load ();
	}
      catch (MagFile::MError)
	{
# ifdef DEBUG
	  cerr << "file " << magFileName_ << " is not found." << endl;
	  cerr << "mag file is not found" << endl;
# endif
	  delete mag;
	  return;
	}
    }

  loadMag (mag, -1, -1, 1, 0);
  
  delete mag;
}

PH0* Wrd::newLoadPH0 (const string& fileName_)
{
  string ph0Name=fileName_;
  PH0* ph0 = new PH0;
  try
    {
      ph0->load (fileName_);
    }
  catch (PH0::PError)
    {
      try
	{
	  ph0->load (songParentDirectory + fileName_);
	}
      catch (PH0::PError)
	{
	  try
	    {
	      ph0->load (path + fileName_);
	    }
	  catch (PH0::PError)
	    {
	      delete ph0;
	      throw PH0FileIsNotFound();
	    }
	}
    }

  return ph0;
}

void Wrd::loadPH0FromSongFileName (const string& songFileName_)
{
  string ph0FileName_ = songFileName_;
  PH0* ph0 = NULL;
  try
    {
      setSuffix (ph0FileName_, "ph0");
      ph0 = newLoadPH0 (ph0FileName_);
    }
  catch (PH0FileIsNotFound)
    {
      setSuffix (ph0FileName_, "PH0");
      try
	{
	  ph0 = newLoadPH0 (ph0FileName_);
	}
      catch (PH0FileIsNotFound)
	{
	  delete ph0;
	  return;
	}
    }
  
  execPLOAD (ph0);
  delete ph0;
}


void Wrd::prePlay ()
{
  DEBUG_MESSAGE ("Wrd::prePlay called.");
  IO::prePlay ();
  resetVariables ();

  getDirectoryName (songParentDirectory, cacheState.songStatus.getFileName ());
  
  DEBUG_MESSAGE ("Wrd::prePlay trying load ph0.");
  loadPH0FromSongFileName (cacheState.songStatus.getFileName ());
  DEBUG_MESSAGE ("Wrd::prePlay trying load mag.");
  loadMag (cacheState.songStatus.getFileName ());
  DEBUG_MESSAGE ("Wrd::prePlay trying load wrd.");
  newOpenWrdFile (cacheState.songStatus.getFileName ().c_str ());
  
  //@startup$B$N<B9T(B
  if (wrdFile!=NULL)
    {
      resetVariables ();
      wrdCommand=wrdFile->getCommand ();

      //$B:G=i$K(B@STARTUP$B$,$J$$>l9g(B@STARTUP$B$^$G?J$a$k!#K\Ev$O(B.wrd$B$,4V0c$C$F$$$k(B?
      if (wrdCommand->getTokenType ()!=tSTARTUP)
	{
	  cerr << "WARNING:Wrd::prePlay At the begin of .wrd file,@STARTUP is not found." << endl;
	  while (wrdCommand->getTokenType ()!=tSTARTUP)
	    {
	      wrdCommand=wrdCommand->getNext ();
	      switch (wrdCommand->getTokenType ())
		{
		case tINVALID://falling through
		case tATMARK://falling through
		case tRETURN://falling through
		case tREM://falling through
		case tREMARK:
		    //do nothing
		    break;
		case tEND:
		  cerr << "ERROR:Wrd::prePlay @STARTUP command not found." << endl;
		  //$B<h$j4:$($::G=i$KLa$7$F$*$/(B
		  wrdCommand=wrdFile->getCommand ();
		  return;
		  break;
		default:
		  cerr << "WARNING:Wrd::prePlay before '@STARTUP',other command is found.Ignoring." << endl;
		    break;
		}
	    }
	}

      //@STARTUP$B<B9T(B
      DEBUG_MESSAGE ("Wrd::prePlay @STARTUP executing");
      if (wrdCommand->getTokenType ()==tSTARTUP)
	{
	  execSTARTUP (wrdCommand);//exec startup
	  bool isContinuing=wrdCommand->isContinuing ();
	  wrdCommand=wrdCommand->getNext ();
	  while (isContinuing)
	    {
	      if (wrdCommand->getTokenType ()==tLYRICS)
		execStartupLYRICS (wrdCommand,wmode);
	      else
		execute (wrdCommand);
	      isContinuing=wrdCommand->isContinuing ();
	      wrdCommand=wrdCommand->getNext ();
	    }
	}
    }
  DEBUG_MESSAGE ("Wrd::prePlay done.");
}

void Wrd::postPlay ()
{
  //@end$B$N<B9T(B
  //XXX
  wrdCommand=NULL;
  IO::postPlay ();
}

void Wrd::playLoop ()
{
  if (wrdFile==NULL)
    {//wrd file$B$,L5$+$C$?!#(B
      assert (wrdCommand==NULL);
      return;
    }
  
  state->lock ();
  state->getState (&cacheState);
  state->unlock ();

  bool isInLyrics=false;//$B2N;lI=<(9T$NESCf$+(B?
  const char* lyrics;//isInLyrics==true$B$N;~I=<($7$F$$$k0LCV$r3P$($F$*$/(B
  isFading=false;//fade$B<B9TCf$+(B?
  bool isWaiting=false;//@WAIT,@REST,@INKEY$B<B9TCf$G$GBT$C$F$$$k$+(B?
  
  for (;;)
    {
      //$B<!2s$N(Bcommand$B<B9T$^$G$NBT$A;~4V$r7W;;(B
      timespec timeSpec;
      state->lock ();
      state->getState (&cacheState);
      state->unlock ();
#if 0
      ulong timeOutUsec;
      if (isWaiting || deltaSteps==0)
	{
	  timeOutUsec=
	    (static_cast<long>(barWaitFor+offsetBarWaitFor)
	     -static_cast<long>(cacheState.songStatus.measures+1))
	    *WRD_TICKS_PER_MEASURE
	    +static_cast<long>(stepsWaitFor)
	    -static_cast<long>(cacheState.songStatus.ticksFromLastMeasure
			       *WRD_TICKS_PER_MEASURE
			       /cacheState.songStatus.getTicksPerMeasure ());
	}
      else
	{//deltaSteps>0
	  assert (deltaSteps>0);
	  timeOutUsec=
	    deltaSteps*WRD_TICKS_PER_MEASURE
	    *cacheState.songStatus.getDivision ()
	    /(cacheState.songStatus.getTicksPerMeasure ()
	      *cacheState.songStatus.getTempo ());
	}
      
      if (timeOutUsec<0) timeOutUsec=50000UL;
      timeOutUsec=min (timeOutUsec,100000UL);//$B2?$b$J$/$F$b(B0.1$BIC$G5/$-$k!#(B
      timeSpec.tv_sec=0;
      timeSpec.tv_nsec=timeOutUsec*1000;
      nanosleep (&timeSpec,NULL);

      state->lock ();
      state->getState (&cacheState);
      state->unlock ();
#else
      long timeOutUsec=
	(static_cast<long>(barWaitFor+offsetBarWaitFor)
	 -static_cast<long>(cacheState.songStatus.measures+1))
	*static_cast<long> (WRD_TICKS_PER_MEASURE)
	+static_cast<long>(stepsWaitFor)
	-static_cast<long>((cacheState.songStatus.ticksFromLastMeasure
			    *WRD_TICKS_PER_MEASURE
			    +cacheState.songStatus.getTicksPerMeasure ()-1)
			   /cacheState.songStatus.getTicksPerMeasure ());
      if (timeOutUsec>0)
	{
	  timeOutUsec=min (timeOutUsec,100000L);//$B2?$b$J$/$F$b(B0.1$BIC$G5/$-$k!#(B
	  timeSpec.tv_sec=0;
	  timeSpec.tv_nsec=timeOutUsec*1000L;
	  nanosleep (&timeSpec,NULL);

	  state->lock ();
	  state->getState (&cacheState);
	  state->unlock ();
	}
#endif
      while ((static_cast<long>(cacheState.songStatus.measures+1)
	      -static_cast<long>(barWaitFor+offsetBarWaitFor))
	     *static_cast<long> (WRD_TICKS_PER_MEASURE)
	     +static_cast<long>((cacheState.songStatus.ticksFromLastMeasure
				 *WRD_TICKS_PER_MEASURE
				 +cacheState.songStatus.getTicksPerMeasure ()-1)
				/cacheState.songStatus.getTicksPerMeasure ())
	     -static_cast<long>(stepsWaitFor)>=0)
	{
#if 0
	  //DEBUG_MESSAGE ("Wrd::playLoop while executing");
	  cerr << barWaitFor+offsetBarWaitFor << ":" << stepsWaitFor << ":"
	       << cacheState.songStatus.measures+1 << ":"
	       << (cacheState.songStatus.ticksFromLastMeasure
		   *WRD_TICKS_PER_MEASURE
		   +cacheState.songStatus.getTicksPerMeasure ()-1)
	    /cacheState.songStatus.getTicksPerMeasure ();
#endif
	  //command$B<B9T(B
	  execute (wrdCommand,isInLyrics,lyrics,isFading,isWaiting);
	  if (isFading)//fade$B$N=hM}(B
	    doFade ();
	  if (isWaiting) break;
	}
      
      //pause,$B1iAU=*N;$N=hM}(B
      //wrdCommand->getTokenType () == tEND$B$r<B9T$7$?8e$O(B
      //wrdCommand == NULL$B$H$J$k!#(B
      checkPause ();
      if (cacheState.songStatus.state==SongStatus::DONE || wrdCommand == NULL)
      //if (wrdCommand == NULL)
	break;
      schedYield ();
    }
}

void Wrd::nonstopPlayLoop (bool isWaitKey)
{
  bool isInLyrics=false;
  const char* lyrics;
  isFading=false;
  bool isWaiting=false;
  while (wrdCommand != NULL)
    {//wrd file$B$,L5$+$C$?!#(B
      if (isWaitKey)
	cin.get ();
      execute (wrdCommand,isInLyrics,lyrics,isFading,isWaiting);
    }
}


void Wrd::resetVariables ()
{
  version=0;
  deltaSteps=0;
  wmode=0;
  barWaitFor=1;
  stepsWaitFor=0;
  offsetBarWaitFor=0;
  textScreen.reset ();
  isGraphicOn=true;
  activePage=0;
  displayPage=0;
  for (int i = 0; i < MAX_PALETTE; i++)
    palette[i].reset ();
  barFading=0;
  stepsFading=0;
  ticksFading=0;
  isFading=false;
  fromPalette=0;
  toPalette=0;
  fadingSpeed=0;
  gmode=0x0F;
  path="./";
  loopCount=0;
  textDot=0;
  tmode=-1;
  tscrLines=0;
  lineMode=0;
  gscMode=0;
  fontMode=0;
  virtualVramPageNum=0;
  virtualVram=NULL;
}


void Wrd::execute (WrdCommand*& wrdCommand_,
		   bool& isInLyrics,const char*& lyrics,bool& isFading_,
		   bool& isWaiting_)
{
  isWaiting_ = false;
  bool needExecuteNewCommand = true;
  if (isInLyrics)
    {//$B2N;l$NI=<((B
      assert (lyrics != NULL);
      execLYRICS (lyrics,wmode);
      if (lyrics == NULL)
	{
	  isInLyrics = false;
	  if (wrdCommand_->isContinuing () == false)
	    needExecuteNewCommand = false;

	  wrdCommand_ = wrdCommand_->getNext ();
	}
    }

  bool isNeedTimeRecalc=true;//$B$3$l$O1x$$$J$!(B
  if (isInLyrics==false && needExecuteNewCommand)
    {
      bool isContinue=true;
      while (isContinue && wrdCommand_ != NULL)
	{
	  if (wrdCommand_->getTokenType ()==tLYRICS)
	    {
	      execLYRICS (wrdCommand_,lyrics,wmode);
	      if (lyrics!=NULL)
		{
		  isInLyrics=true;
		  break;
		}
	    }
	  else
	    {
	      execute (wrdCommand_);
	    }
		  
	  if (wrdCommand_->getTokenType ()==tWAIT ||
	      wrdCommand_->getTokenType ()==tREST ||
	      wrdCommand_->getTokenType ()==tINKEY)
	    {//$B;~4VBT$A$H$J$k$N$G!"(Bcommand$B<B9T%k!<%W$+$iH4$1=P$9!#(B
	      isWaiting_=true;
	      if (wrdCommand_->isContinuing ())
		isNeedTimeRecalc=false;
	      wrdCommand_=wrdCommand_->getNext ();
	      break;
	    }

#ifdef DEBUG
	  if (wrdCommand_->getTokenType () == tEND)
	    {
	      assert (wrdCommand_->isContinuing () == false);
	      assert (wrdCommand_->getNext () == NULL);
	    }
#endif
	  isContinue=wrdCommand_->isContinuing ();
	  wrdCommand_=wrdCommand_->getNext ();
	}
    }

  //$B<!$NBT$A;~4V$r7W;;!#(B
  if (isNeedTimeRecalc==true)
    calculateNextWaitFor (deltaSteps,wmode,&barWaitFor,&stepsWaitFor);
}
  

void Wrd::calculateNextWaitFor
(uchar deltaSteps_,uint wmode_,
 uint32* const barWaitFor_,uint32* const stepsWaitFor_)
{
  assert (wmode_==0 || wmode_==1);
  assert (barWaitFor_!=NULL);
  assert (stepsWaitFor_!=NULL);
  if (deltaSteps_==0)
    {//$B0l9T0l>.@a$N%b!<%I(B
      *stepsWaitFor_=0;
      (*barWaitFor_)++;
    }
  else
    {
      assert (deltaSteps_<WRD_TICKS_PER_MEASURE);
      *stepsWaitFor_+=deltaSteps_+1;
      if (*stepsWaitFor_>=WRD_TICKS_PER_MEASURE)
	{//$B7+>e$,$j(B
	  *barWaitFor_+=*stepsWaitFor_/WRD_TICKS_PER_MEASURE;
	  *stepsWaitFor_%=WRD_TICKS_PER_MEASURE;
	}
    }

  //#ifdef DEBUG
#if 0
  const uint32 steps=
    (cacheState.songStatus.ticks-cacheState.songStatus.lastMeasureTicks)
    *WRD_TICKS_PER_MEASURE/cacheState.songStatus.getTicksPerMeasure ();
  cerr << ":Wrd::calculateNextWaitFor"
       << ":bar=" << cacheState.songStatus.measures
       << ":steps=" << steps
       << ":barWaitFor=" << *barWaitFor_
       << ":stepsWaitFor=" << *stepsWaitFor_
       << ":"<< endl;
#endif
}


void Wrd::execute (WrdCommand*& wrdCommand_)
{
  if (option->wrdDoesCerrDebug)
    {
      int  i = 0;
      while (wrdCommand_->getCommandPtr ()[i] != '\n')
	i++;
      cerr.write (wrdCommand_->getCommandPtr (),i+1);
    }
  switch (wrdCommand_->getTokenType ())
    {
      /////////////////////////////////////////////////////// for internal use
    case tINVALID:
      execINVALID (wrdCommand_);
      break;            
    case tATMARK:
      execATMARK (wrdCommand_);
      break;
    case tLYRICS:
#ifdef DEBUG
      cerr << "Wrd::execute wrcCommand->getTokenType ()==tLYRICS !!!" << endl;
#endif
      break;
    case tRETURN:
      execRETURN (wrdCommand_);
      break;                        
      ///////////////////////////////////////////////////// for MIMPI
    case tCOLOR:
      execCOLOR (wrdCommand_);
      break;                         
    case tEND:
      execEND (wrdCommand_);
      break;                           
    case tESC:
      execESC (wrdCommand_);
      break;                           
    case tEXEC:
      execEXEC (wrdCommand_);
      break;                          
    case tFADE:
      execFADE (wrdCommand_);
      break;                          
    case tGCIRCLE:
      execGCIRCLE (wrdCommand_);
      break;                       
    case tGCLS:
      execGCLS (wrdCommand_);
      break;                          
    case tGINIT:
      execGINIT (wrdCommand_);
      break;                         
    case tGLINE:
      execGLINE (wrdCommand_);
      break;                         
    case tGMODE:
      execGMODE (wrdCommand_);
      break;                         
    case tGMOVE:
      execGMOVE (wrdCommand_);
      break;                         
    case tGON:
      execGON (wrdCommand_);
      break;                           
    case tGSCREEN:
      execGSCREEN (wrdCommand_);
      break;                       
    case tINKEY:
      execINKEY (wrdCommand_);
      break;                         
    case tLOCATE:
      execLOCATE (wrdCommand_);
      break;                        
    case tLOOP:
      execLOOP (wrdCommand_);
      break;                          
    case tMAG:
      execMAG (wrdCommand_);
      break;                           
    case tMIDI:
      execMIDI (wrdCommand_);
      break;                          
    case tOFFSET:
      execOFFSET (wrdCommand_);
      break;                        
    case tPAL:
      execPAL (wrdCommand_);
      break;                           
    case tPALCHG:
      execPALCHG (wrdCommand_);
      break;                        
    case tPALREV:
      execPALREV (wrdCommand_);
      break;                        
    case tPATH:
      execPATH (wrdCommand_);
      break;                          
    case tPLOAD:
      execPLOAD (wrdCommand_);
      break;                         
    case tREM:
      execREM (wrdCommand_);
      break;                           
    case tREMARK:
      execREMARK (wrdCommand_);
      break;                        
    case tREST:
      execREST (wrdCommand_);
      break;                          
    case tSCREEN:
      execSCREEN (wrdCommand_);
      break;                        
    case tSCROLL:
      execSCROLL (wrdCommand_);
      break;                        
    case tSTARTUP:
#ifdef DEBUG
      cerr << "Wrd::execute wrcCommand->getTokenType ()==tSTARTUP !!!" << endl;
#endif
      break;                       
    case tSTOP:
      execSTOP (wrdCommand_);
      break;                          
    case tTCLS:
      execTCLS (wrdCommand_);
      break;                          
    case tTON:
      execTON (wrdCommand_);
      break;                           
    case tWAIT:
      execWAIT (wrdCommand_);
      break;                          
    case tWMODE:
      execWMODE (wrdCommand_);
      break;                         
      ////////////////////////////////////////// for ensyutsukun mechanism
    case tFONTM:
      execFONTM (wrdCommand_);
      break;                         
    case tFONTP:
      execFONTP (wrdCommand_);
      break;                         
    case tFONTR:
      execFONTR (wrdCommand_);
      break;                         
    case tGSC:
      execGSC (wrdCommand_);
      break;                           
    case tLINE:
      execLINE (wrdCommand_);
      break;                          
    case tPAL_:	// ^PAL
      execPAL_ (wrdCommand_);
      break;
    case tREGSAVE:
      execREGSAVE (wrdCommand_);
      break;                       
    case tSCROLL_:// ^SCROLL
      execSCROLL_ (wrdCommand_);
      break;
    case tTEXTDOT:
      execTEXTDOT (wrdCommand_);
      break;                       
    case tTMODE:
      execTMODE (wrdCommand_);
      break;                         
    case tTSCRL:
      execTSCRL (wrdCommand_);
      break;                         
    case tVCOPY:
      execVCOPY (wrdCommand_);
      break;                         
    case tVSGET:
      execVSGET (wrdCommand_);
      break;                         
    case tVSRES:
      execVSRES (wrdCommand_);
      break;                         
    case tXCOPY:
      execXCOPY (wrdCommand_);
      break;
    }
}

void Wrd::execATMARK (WrdCommand*& wrdCommand_)
{
  execATMARK ();
}

void Wrd::execCOLOR (WrdCommand*& wrdCommand_)
{
  textScreen.setColor (wrdCommand_->parameter[0].getPositiveInt ());
  execCOLOR (textScreen.getColor ());
}

void Wrd::execEND (WrdCommand*& wrdCommand_)
{
  assert (wrdCommand->getNext () == NULL);
  assert (wrdCommand->isContinuing () == false);
  execEND ();
}

void Wrd::execESC (WrdCommand*& wrdCommand_)
{
  const char* const tmp=wrdCommand_->parameter[0].getStr ();
  char* escStr=new char[strlen (tmp)+3];
  escStr[0]=TextScreen::ESC;
  escStr[1]='[';
  strcpy (escStr+2,tmp);

  //escape sequence$B$N=hM}!#(B
  if (*escStr==TextScreen::ESC)
    doEscapeSequence (escStr);
  if (*escStr!='\0')
    printLYRICS (escStr,strlen (escStr));//$BI=<($9$Y$-J8;zNs$,;D$C$F$$$?!#(B
  refreshTextScreen (textScreen);
}

void Wrd::execEXEC (WrdCommand*& wrdCommand_)
{
  const char* filename=wrdCommand_->parameter[0].getStr ();
  execEXEC (filename);
}

void Wrd::doFade ()
{
  const int nowFadeCount=
    min (((static_cast<long>(cacheState.songStatus.measures+1)
      -static_cast<long>(barFading))
     *static_cast<long> (FADING_TICKS_PER_MEASURE)
     +static_cast<long>((cacheState.songStatus.ticksFromLastMeasure
			 *FADING_TICKS_PER_MEASURE
			 +cacheState.songStatus.getTicksPerMeasure ()-1)
			/cacheState.songStatus.getTicksPerMeasure ())
	  -static_cast<long>(stepsFading))/fadingSpeed,
	 static_cast<long> (MAX_FADING_COUNT));
  
  if (nowFadeCount > fadeCount)
    {
      fadeCount = nowFadeCount;
      for (int i = 0; i < MAX_GRAPHIC_COLORS; i++)
	{
	  const uchar newR=
	    (palette[fromPalette].rgb[i].getR ()*(MAX_FADING_COUNT-fadeCount)+
	     palette[toPalette  ].rgb[i].getR ()*fadeCount)
	    /MAX_FADING_COUNT;
	  const uchar newG=
	    (palette[fromPalette].rgb[i].getG ()*(MAX_FADING_COUNT-fadeCount)+
	     palette[toPalette  ].rgb[i].getG ()*fadeCount)
	    /MAX_FADING_COUNT;
	  const uchar newB=
	    (palette[fromPalette].rgb[i].getB ()*(MAX_FADING_COUNT-fadeCount)+
	     palette[toPalette  ].rgb[i].getB ()*fadeCount)
	    /MAX_FADING_COUNT;

	  palette[0].rgb[i].set (newR, newG, newB);
	}

      doFading (fadeCount, fromPalette, toPalette);

      if (fadeCount >= MAX_FADING_COUNT)
	{//fade$B=*N;!#(B
	  isFading = false;
	  barFading = 0;
	  stepsFading = 0;
	  fadeCount = 0;
	  fadingSpeed = 1;
	}
    }
}


//XXX
void Wrd::execFADE (WrdCommand*& wrdCommand_)
{
  fromPalette=wrdCommand_->parameter[0].getPositiveInt ();
  toPalette=wrdCommand_->parameter[1].getPositiveInt ();
  fadingSpeed=wrdCommand_->parameter[2].getEInt ();
  assert (0 <= fromPalette);
  assert (fromPalette < MAX_PALETTE);
  assert (0 <= toPalette);
  assert (toPalette < MAX_PALETTE);
  //assert (isFading == false);
  fadeCount = 0;
  
  if (fadingSpeed == -1) fadingSpeed = 1;//XXX
  
  if (fadingSpeed == 0)
    {
      palette[0].set (palette[toPalette]);
    }
  else //if (fadingSpeed > 0)
    {//$B<B9T$7$J$,$i(Bfading$B$9$k$N$GI,MW$JJ*$r3P$($F$*$/!#(B
      assert (fadingSpeed > 0);
      ticksFading=cacheState.songStatus.ticks;
      barFading=cacheState.songStatus.measures;
      stepsFading=ticksFading-cacheState.songStatus.lastMeasureTicks;
      isFading=true;

      palette[0].set (palette[fromPalette]);
    }
  
  execFADE (fromPalette,toPalette,fadingSpeed);
}

void Wrd::execGCIRCLE (WrdCommand*& wrdCommand_)
{
  uint32 value[6];
  wrdCommand_->getIntParameter (value,6);
  execGCIRCLE (value[0],value[1],value[2],value[3],value[4],value[5]);
}

void Wrd::execGCLS (WrdCommand*& wrdCommand_)
{
  uint32 value;
  wrdCommand_->getIntParameter (&value,1);
  execGCLS (value);
}

void Wrd::execGINIT (WrdCommand*& wrdCommand_)
{
  isGraphicOn=true;
  activePage=0;
  displayPage=0;
  execGINIT ();
}

void Wrd::execGLINE (WrdCommand*& wrdCommand_)
{
  uint32 value[7];
  wrdCommand_->getIntParameter (value,7);
  execGLINE (value[0],value[1],value[2],value[3],value[4],value[5],value[6]);
}

void Wrd::execGMODE (WrdCommand*& wrdCommand_)
{
  gmode=wrdCommand_->parameter[0].getPositiveInt ();
  //assert (0 <= gmode);
  assert (gmode < 16);
  execGMODE (gmode);
}

void Wrd::execGMOVE (WrdCommand*& wrdCommand_)
{
  uint32 value[9];
  wrdCommand_->getIntParameter (value, 9);
  execGMOVE (value[0], value[1], value[2], value[3], value[4], value[5], 
	     value[6], value[7], value[8]);
}

void Wrd::execGON (WrdCommand*& wrdCommand_)
{
  //0:off 1:on
  const bool oldIsGraphicOn=isGraphicOn;
  isGraphicOn=wrdCommand_->parameter[0].getPositiveInt ();
  execGON (oldIsGraphicOn, isGraphicOn);
}

void Wrd::execGSCREEN (WrdCommand*& wrdCommand_)
{
  const uint32 oldActivePage=activePage;
  const uint32 oldDisplayPage=displayPage;
  activePage=wrdCommand_->parameter[0].getPositiveInt ();
  displayPage=wrdCommand_->parameter[1].getPositiveInt ();
  execGSCREEN (oldActivePage, oldDisplayPage, activePage, displayPage);
}

void Wrd::execINKEY (WrdCommand*& wrdCommand_)
{
  barWaitFor=wrdCommand_->parameter[0].getPositiveInt ();
  execINKEY (barWaitFor);
}

void Wrd::execLOCATE (WrdCommand*& wrdCommand_)
{
  const uint x=wrdCommand_->parameter[0].getPositiveInt ();
  const uint y=wrdCommand_->parameter[1].getPositiveInt ();
  textScreen.locate (x,y);
  execLOCATE (textScreen.getX (),textScreen.getY ());
}

void Wrd::execLOOP (WrdCommand*& wrdCommand_)
{
  loopCount=wrdCommand_->parameter[0].getPositiveInt ();
  execLOOP (loopCount);
}

//XXX
void Wrd::execMAG (WrdCommand*& wrdCommand_)
{
  const char* tmp=wrdCommand_->parameter[0].getStr ();
  char* const fileName = new char[strlen (tmp)+1];
  strcpy (fileName,tmp);
  int32 value[2];
  wrdCommand_->getEIntParameter (value,2,1);
  uint32 sw[2];
  wrdCommand_->getIntParameter (sw,2,3);
  
  assert (-1 <= value[0]);
  assert (value[0] < GRAPHIC_SIZE_X);
  assert(-1 <= value[1]);
  assert (value[1] < GRAPHIC_SIZE_Y);
  assert (1 <= sw[0]);
  assert (sw[0] <= 16);
  assert (sw[1] == 0 || sw[1] == 1 || sw[1] == 2);

  MagFile* mag;
  const int nameLength = strlen (fileName);
  try
    {
      mag = newLoadMag (fileName);
    }
  catch (MagFileIsNotFound)
    {
      for (int i = 0; i < nameLength; i++)
	fileName[i] = tolower (fileName[i]);
      try
	{
	  mag = newLoadMag (fileName);
	}
      catch (MagFileIsNotFound)
	{
	  for (int i = 0; i < nameLength; i++)
	    fileName[i] = toupper (fileName[i]);
	  try
	    {
	      mag = newLoadMag (fileName);
	    }
	  catch (MagFileIsNotFound)
	    {
	      return;
	    }
	}
    }

  loadMag (mag, value[0], value[1], sw[0], sw[1]);
  
  delete mag;
}

void Wrd::execMIDI (WrdCommand*& wrdCommand_)
{
  //XXX
}

void Wrd::execOFFSET (WrdCommand*& wrdCommand_)
{
  offsetBarWaitFor=wrdCommand_->parameter[0].getPositiveInt ();
  execOFFSET (offsetBarWaitFor);
}

void Wrd::execPAL (WrdCommand*& wrdCommand_)
{
  uint32 palNum=wrdCommand_->parameter[0].getNo ();
  int32 rgb[Palette::MAX_RGB];
  wrdCommand_->getHexParameter (rgb, Palette::MAX_RGB, 1);
  palette[palNum].set (rgb);
  execPAL (palNum,rgb);
}

void Wrd::execPALCHG (WrdCommand*& wrdCommand_)
{
  //XXX
}

void Wrd::execPALREV (WrdCommand*& wrdCommand_)
{
  const uint32 p=wrdCommand_->parameter[0].getPositiveInt ();
  palette[p].reverse ();
  execPALREV (p);
}

void Wrd::execPATH (WrdCommand*& wrdCommand_)
{
  const char* tmp = wrdCommand_->parameter[0].getStr ();
  if (tmp == NULL)
    path = ".";
  else
    path = tmp;
  
  if (path == "*" || path == ".")
    path = ".";
  else if (path == "@")
    path=songParentDirectory;
  else
    {
      //delimiter$B$rJQ49$9$k!#(B'\\' -> '/'
      for (size_t i = 0; i < path.length (); i++)
	{
	  if (path[i] == '\\')
	    path[i] = '/';
	}
    }

  path += "/";
}

void Wrd::execPLOAD (WrdCommand*& wrdCommand_)
{
  const char* const tmp = wrdCommand_->parameter[0].getStr ();
  char* const ph0Name = new char[strlen (tmp)+1];
  strcpy (ph0Name, tmp);
  const int nameLength = strlen (ph0Name);

  PH0* ph0 = NULL;
  try
    {
      ph0 = newLoadPH0 (ph0Name);
    }
  catch (PH0FileIsNotFound)
    {
      for (int i = 0; i < nameLength; i++)
	ph0Name[i] = tolower (ph0Name[i]);
      try
	{
	  ph0 = newLoadPH0 (ph0Name);
	}
      catch (PH0FileIsNotFound)
	{
	  for (int i = 0; i < nameLength; i++)
	    ph0Name[i] = toupper (ph0Name[i]);
	  try
	    {
	      ph0 = newLoadPH0 (ph0Name);
	    }
	  catch (PH0FileIsNotFound)
	    {
	      return;
	    }
	}
    }

  assert (ph0 != NULL);
  execPLOAD (ph0);
  delete ph0;
}

void Wrd::execREM (WrdCommand*& wrdCommand_)
{
  execREMARK ();
}

void Wrd::execREMARK (WrdCommand*& wrdCommand_)
{
  execREMARK ();
}

void Wrd::execREST (WrdCommand*& wrdCommand_)
{
  const uint32 barOffsetWaitFor=wrdCommand_->parameter[0].getPositiveInt ();
  barWaitFor=cacheState.songStatus.measures+barOffsetWaitFor;
  stepsWaitFor=wrdCommand_->parameter[0].getPositiveInt ();
  execREST (barOffsetWaitFor,stepsWaitFor);
}

void Wrd::execSCREEN (WrdCommand*& wrdCommand_)
{
  //XXX ???
}

void Wrd::execSCROLL (WrdCommand*& wrdCommand_)
{
  uint32 value[7];
  wrdCommand_->getIntParameter (value,7);
  textScreen.scroll
    (value[0],value[1],value[2],value[3],value[4],value[5],value[6]);
  execSCROLL
    (value[0],value[1],value[2],value[3],value[4],value[5],value[6]);
  refreshTextScreen (textScreen);
}

void Wrd::execSTARTUP (WrdCommand*& wrdCommand_)
{
  version=wrdCommand_->parameter[0].getPositiveInt ();
  execSTARTUP (version);
}

void Wrd::execSTOP (WrdCommand*& wrdCommand_)
{
  execSTOP ();
}

void Wrd::execTCLS (WrdCommand*& wrdCommand_)
{
  uint32 value[6];
  wrdCommand_->getIntParameter (value,6);

  //needs more parameter check? ???
  if (value[0] <= 0) value[0] = 1;
  if (value[1] <= 0) value[1] = 1;

  textScreen.tcls (value[0],value[1],value[2],value[3],value[4],value[5]);
  execTCLS (value[0],value[1],value[2],value[3],value[4],value[5]);
  refreshTextScreen (textScreen);
}

void Wrd::execTON (WrdCommand*& wrdCommand_)
{
  //0:off 1:on
  const uint32 textMode=wrdCommand_->parameter[0].getPositiveInt ();

  textScreen.ton (textMode);
  execTON (textMode);
}

void Wrd::execWAIT (WrdCommand*& wrdCommand_)
{
  barWaitFor=wrdCommand_->parameter[0].getPositiveInt ();
  stepsWaitFor=wrdCommand_->parameter[1].getPositiveInt ();
  execWAIT (barWaitFor,stepsWaitFor);
}

void Wrd::execWMODE (WrdCommand*& wrdCommand_)
{
  deltaSteps=wrdCommand_->parameter[0].getPositiveInt ();
  wmode=wrdCommand_->parameter[1].getPositiveInt ();
  if (wmode==0)
    stepsWaitFor=0;
  execWMODE (deltaSteps,wmode);
}

void Wrd::execFONTM (WrdCommand*& wrdCommand_)
{
  fontMode=wrdCommand_->parameter[0].getEInt ();
  assert (fontMode==0 || fontMode==1);

  execFONTM (fontMode);
}

void Wrd::execFONTP (WrdCommand*& wrdCommand_)
{
  int32 value[4];
  wrdCommand_->getEIntParameter (value,4);

  execFONTP (value[0],value[1],value[2],value[3]);
}

void Wrd::execFONTR (WrdCommand*& wrdCommand_)
{
  const int32 fontNumber=wrdCommand_->parameter[0].getEInt ();
  assert (0<=fontNumber);
  assert (fontNumber<=15);
  int32 value[16];
  wrdCommand_->getEIntParameter (value,16,1);

  execFONTR (fontNumber,value);
}

void Wrd::execGSC (WrdCommand*& wrdCommand_)
{
  gscMode=wrdCommand_->parameter[0].getEInt ();
  assert (gscMode==0 || gscMode==1);
  execGSC (gscMode);
}

void Wrd::execLINE (WrdCommand*& wrdCommand_)
{
  lineMode=wrdCommand_->parameter[0].getEInt ();
  assert (0<=lineMode);
  assert (lineMode<=32);

  execLINE (lineMode);
}

void Wrd::execPAL_ (WrdCommand*& wrdCommand_)
{
  const int32 paletteNumber=wrdCommand_->parameter[0].getEInt ();
  const int32 rgb=wrdCommand_->parameter[1].getEInt ();

  execPAL_ (paletteNumber,rgb);
}

void Wrd::execREGSAVE (WrdCommand*& wrdCommand_)
{
  int32 value[17];
  wrdCommand_->getEIntParameter (value,17);
  assert (0<=value[0]);
  assert (value[0]<=9);

  execREGSAVE (value[0],value+1);
}

void Wrd::execSCROLL_ (WrdCommand*& wrdCommand_)
{
  const int32 x_=wrdCommand_->parameter[0].getEInt ();
  const int32 y_=wrdCommand_->parameter[1].getEInt ();
  assert (0<=x_);
  assert (x_<=39);
  assert (0<=y_);
  assert (y_<=399);
  
  execSCROLL_ (x_,y_);
}

void Wrd::execTEXTDOT (WrdCommand*& wrdCommand_)
{
  textDot=wrdCommand_->parameter[0].getEInt ();
  assert (textDot==0 || textDot==1);
  execTEXTDOT (textDot);
}

void Wrd::execTMODE (WrdCommand*& wrdCommand_)
{
  tmode=wrdCommand_->parameter[0].getEInt ();
  assert (tmode==0 || tmode==1 || tmode==2);
  execTMODE (tmode);
}

void Wrd::execTSCRL (WrdCommand*& wrdCommand_)
{
  tscrLines=wrdCommand_->parameter[0].getEInt ();
  //XXX$B0z?t%A%'%C%/(B

  execTSCRL (tscrLines);
}

void Wrd::execVCOPY (WrdCommand*& wrdCommand_)
{
  int32 value[9];
  wrdCommand_->getEIntParameter (value,9);

  execVCOPY (value[0],value[1],value[2],value[3],
	     value[4],value[5],value[6],value[7],value[8]);
}

void Wrd::execVSGET (WrdCommand*& wrdCommand_)
{
  int32 value[4];
  wrdCommand_->getEIntParameter (value,4);
  //$B2>A[2hLL$N3NJ](B XXX

  
  execVSGET (value);
}

void Wrd::execVSRES (WrdCommand*& wrdCommand_)
{
  //XXX$B2>A[2hLL$N2rJ|(B

  execVSRES ();
}

void Wrd::execXCOPY (WrdCommand*& wrdCommand_)
{
  int32 value[14];
  wrdCommand_->getEIntParameter (value,14);
  
  execXCOPY (value[0],value[1],value[2],value[3],
	     value[4],value[5],value[6],value[7],
	     value[8],value+9);
}

void Wrd::execLYRICS (const char*& lyrics,uint wmode_)
{
  assert (lyrics!=NULL);
  assert (wmode_==0 || wmode_==1);
  if (wmode_==0)
    {
      const int length=strlen (lyrics);
      if (length>0)
	{
	  if (lyrics[length-1]=='\n')
	    {
	      if (length>1)
		printLYRICS (lyrics,length-1);
	      nextLine ();
	    }
	  else
	    {
	      printLYRICS (lyrics,length);
	    }
	}
      lyrics=NULL;
    }
  else
    {//wmode_==1
      if (*lyrics=='_')
	{
	  lyrics++;
	}
      else if (*lyrics=='|')
	{
	  lyrics++;//skip '|'
	  //$BBP1~$9$k(B'|'$B$rC5$9!#(B
	  const char* nextBar=strchr (lyrics,'|');
	  while (nextBar!=NULL && *(nextBar-1)=='\\')
	    nextBar=strchr (nextBar+1,'|');
	  if (nextBar==NULL)
	    {//$BBP1~$9$k(B'|'$B$,8+$D$+$i$J$+$C$?!#9TKv$^$G$H$9$k!#(B
	      cerr << "ERROR:"
		   << "Wrd::execLYRICS (const char*& lyrics,uint wmode_):"
		   << "there is no matching '|'"
		   << endl;
	      //*nextBar=='\0'(end of string lyrics)
	      nextBar=lyrics+strlen (lyrics);
	    }
	  
	  //'\'$B$N=hM}(B
	  //nextBar$B$O4^$^$J$$!#(B*nextBar=='|' or *nextBar=='\0'
	  int length=nextBar-lyrics;
	  char* const realLyrics=new char[length+1];
	  strncpy (realLyrics,lyrics,length);
	  realLyrics[length]='\0';
	  char* tmp=realLyrics;
	  while ((tmp=strchr (tmp,'\\'))!=NULL)
	    {//$BK\Ev$O$b$C$HNI$$J}K!$,$"$k$N$@$m$&$1$l$I!D!#(B
	      memmove (tmp,tmp+1,length-(tmp-realLyrics+1)+1);//'\0'$B$bF0$+$9!#(B
	      length--;
	      tmp++;
	    }
	  
	  printLYRICS (realLyrics,length);
	  delete[] realLyrics;

	  //lyrics=nextBar+1$B$N$?$a$KJd@5(B
	  if (*nextBar=='\0')
	    nextBar--;
	  if (*nextBar=='\n')
	    nextBar--;
	  lyrics=nextBar+1;
	}
      else
	{
	  if (*lyrics=='\\')
	    {
	      lyrics++;
	      if (*lyrics!='\n' && *lyrics!='\0')
		{
		  printLYRICS1 (lyrics);
		}
	      else
		{
		  cerr << "ERROR:"
		       << "Wrd::execLYRICS (const char*& lyrics,uint wmode_):"
		       << "there is '\\' at end of line."
		       << endl;
		}
	    }
	  else
	    {
	      if (*lyrics!='\n' && *lyrics!='\0')
		printLYRICS1 (lyrics);
	    }
	}

      //$B9TKv$N=hM}(B
      if (*lyrics=='\n')
	{
	  nextLine ();
	  lyrics++;
	  assert (*lyrics=='\0');
	}
      if (*lyrics=='\0')
	lyrics=NULL;
    }

  refreshTextScreen (textScreen);
}

void Wrd::printLYRICS (const char* ptr,int size)
{
  const char* const from=ptr;
  const char* now=from;
  while (now-from<size)
    printLYRICS1 (now);
  assert (now-from==size);
}

void Wrd::printLYRICS1 (const char*& ptr)
{
  //escape sequence$B$N=hM}!#(B
  //XXX
  if (*ptr==TextScreen::ESC)
    doEscapeSequence (ptr);
  
  if (*ptr=='\0')
    {
      DEBUG_MESSAGE ("WARNING:Wrd::printLYRICS1 there is no character to print");
    }
  else if (static_cast<uchar> (*ptr)<0x20)
    {//control code
      DEBUG_MESSAGE ("Wrd::printLYRICS1 control code");
      assert (*ptr!='\0');
      assert (*ptr!='\n');
      assert (*ptr!='\r');
      switch (*ptr)
	{
	case '\t':
	  //XXX
	  break;
	default:
	  break;
	}
      ptr++;
    }
  else if (isiso (*ptr))
    {
      //DEBUG_MESSAGE ("Wrd::printLYRICS1 isiso");
      const uint x=textScreen.getX ();
      const uint y=textScreen.getY ();
      const int color=textScreen.getColor ();
      textScreen.printChar (*ptr);
      printChar (x,y,*ptr,color);
      ptr++;
    }
  else if ((iskanji (*ptr) || iskana (*ptr))
	   && *(ptr+1)!='\n' && *(ptr+1)!='\0')
    {
      //DEBUG_MESSAGE ("Wrd::printLYRICS1 iskana");
      const uint x_=textScreen.getX ();
      const uint y_=textScreen.getY ();
      const int color=textScreen.getColor ();
      textScreen.printChar2b (*ptr,*(ptr+1));
      printChar2b (x_,y_,*ptr,*(ptr+1),color);
      ptr+=2;
    }
  else
    {
      DEBUG_MESSAGE ("Wrd::printLYRICS1 invalid charactor found.");
      ptr++;
    }
}

void Wrd::execStartupLYRICS (WrdCommand*& wrdCommand_,uint wmode_)
{
  const char* lyrics;
  do
    {
      execLYRICS (wrdCommand_,lyrics,wmode_);
    }while (lyrics!=NULL);
  refreshTextScreen (textScreen);
}


void Wrd::doEscapeSequence (const char*& ptr)
{
  while (*ptr==TextScreen::ESC)
    textScreen.doEscapeSequence (ptr);//$BFbIt%G!<%?$N99?7(B
  doEscapeSequence ();//$BI=<($N99?7(B
}
