#include <iostream>
#include <fstream>
#include <cctype>
#include <cassert>
#include <stdlib.h>

#include "WrdFile.h"
#include "wrd.h"
#include "util/KanjiConvert.h"

#ifdef DEBUG
//# define PRINT_COMPILED_RESULTS
#endif

using namespace std;

WrdFile::WrdFile ():
  fileName (NULL),wrdBuffer (NULL),bufferSize (0),wrdCommand (NULL)
{
  //do nothing
}

WrdFile::~WrdFile ()
{
#ifdef DEBUG
  cerr << "DEBUG::~WrdFile" << endl;
#endif
  WrdCommand* tmp = wrdCommand;
  while (wrdCommand  !=  NULL)
    {
      assert (tmp != NULL);
      tmp = tmp->getNext ();
      delete wrdCommand;
      wrdCommand = tmp;
    }
  delete[] wrdBuffer;
  delete[] fileName;
}

void WrdFile::load (const char* fileName_)
{
  delete[] fileName;
  fileName = new char[strlen (fileName_)+1];
  strcpy (fileName, fileName_);
  
  //open file
  ifstream ifs (fileName, ios::in|ios::nocreate|ios::ate);
  if (!ifs)
    {
      cerr << "WrdFile::can not open " << fileName << endl;
      throw FileNotFound ();
    }
  //file size$B<hF@(B
  bufferSize = ifs.tellg ();
  //file$B$N@hF,$KLa$9!#(B
  ifs.seekg (0);

  //memory$B>e$KFI$_9~$_!#(B
  wrdBuffer = new char[bufferSize+1];
  ifs.read (wrdBuffer, bufferSize);
  wrdBuffer[bufferSize] = '\0';

  //close file
  ifs.close ();
  
  //file$B$rFbIt7A<0$KJQ49$9$k!#(B
  wrdCommand = compile (wrdBuffer, bufferSize);
}

const resword* WrdFile::getCommandType (const char* ptr)
{
  char str[MAX_WORD_LENGTH+1];
  str[0] = '@';
  int length;
  for (length = 1; length < MAX_WORD_LENGTH; length++)
    {
      if (!isalpha (*(ptr+length)))
	break;
      str[length] = tolower (*(ptr+length));
    }
  str[length] = '\0';
  return in_word_set (str, length);
}


WrdCommand* WrdFile::compile (const char* const buffer, int size)
{
  assert (buffer != NULL);
  const char* ptr = buffer;

  WrdCommand* const commandStart = new WrdCommand ();
  WrdCommand* command = commandStart;
  while (ptr-buffer < size)
    {
      command->setCommandPtr (ptr);
      if (*ptr == '@' || *ptr == '^')
	{//command for minpi or ensyutukunn
	  const resword* resWord = getCommandType (ptr);
	  if (resWord == NULL)
	    {
	      cerr << "unsupported command found." << endl;
	      cerr.write (ptr, 10) << endl;
	      command->setINVALID ();
	      ptr++;//skip++
	      //error recovery
	      while (*ptr != '@' && *(ptr-2) != '\r' && *(ptr-1) != '\n' &&
		     ptr-buffer <= size)
		ptr++;//$B<!$N(Bcommand$B$K=P2q$&$+9TF,$K9T$/$^$G?J$`!#(B

	      command->setContinue ();
	      if (*(ptr-3) != ';' && *(ptr-2) == '\r' && *(ptr-1) == '\n')
		command->offContinue ();

	      command->setNext (new WrdCommand ());
	      command = command->getNext ();
	      continue;
	    }
#ifdef PRINT_COMPILED_RESULTS
	  cerr << "DEBUG:WrdFile::compile token=" << resWord->name;
#endif
	  command->setTokenType (resWord->tokenType);
	  if (resWord->tokenType == tREM || resWord->tokenType == tREMARK)
	    {
	      //skip comment
	      while (*(ptr+1) != '\r' && *(ptr+2) != '\n' && ptr-buffer <= size)
		ptr++;//$B9TF,$^$G?J$a$k!#(B
	    }
	  else if (resWord->tokenType == tEND)
	    break;//$B=*$j(B
	  else
	    {
	      try
		{
		  compileCommand (ptr, command, resWord);
		}
	      catch (CompileError)
		{
		  command->setINVALID ();
#ifdef DEBUG
		  cerr << "WrdFile::errorOccured:WrdFile" << endl;//XXX
		  cerr.write (ptr-10, 20);//XXX
		  //$BK\Ev$O(Bio$B$K=PNO$9$Y$-!#(B
#endif
		  //error recovery
		  while (*ptr != '@' && *ptr != '\r' && *(ptr+1) != '\n' &&
			 ptr-wrdBuffer < bufferSize-1)
		    ptr++;
		}
	    }

	  skipWhiteSpace (ptr);
	  command->setContinue ();
	  if (ptr-buffer > size-1 ||
	      (*(ptr-1) != ';' && *ptr == '\r' && *(ptr+1) == '\n'))
	    command->offContinue ();
	  
	  if (*ptr == ';')
	    ptr++;

	  while ((*ptr == ' ' || *ptr == '\t' || *ptr == '\r' || *ptr == '\n') &&
		 !(*(ptr-2) == '\r' || *(ptr-1) == '\n') && ptr-buffer <= size)
	    ptr++;//$B<!$K2?$+J8;z$K=P2q$&$+9TF,$K9T$/$^$G?J$`!#(B

#ifdef PRINT_COMPILED_RESULTS
	  if (resWord->tokenType != tREM &&
	      resWord->tokenType != tREMARK &&
	      resWord->tokenType != tATMARK )
	    {
	      cerr << " (";
	      for (int i = 0;
		   resWord->parameterType[i].type != param_end;
		   i++)
		{
		  if (i > 0) cerr << ",";
		  switch (resWord->parameterType[i].type)
		    {
		    case param_int:
		      cerr << command->parameter[i].getPositiveInt ();
		      break;
		    case param_no:
		      cerr << "#" << command->parameter[i].getNo ();
		      break;
		    case param_hex:
		      cerr << hex << command->parameter[i].getHex () << dec;
		      break;
		    case param_eint:
		      cerr << command->parameter[i].getEInt ();
		      break;
		    case param_str:
		      cerr << command->parameter[i].getStr ();
		      break;
		    case param_not_specified:
		      break;
		    case param_end:
		      break;
		    }
		}
	      cerr << ")";
	    }
#endif
	}
      else
	{//Lyrics
	  //Lyrics$B$O9TKv$K(B';'(i.e ;\r\n$B$@$C$?$i(B)$B$,IU$$$F$$$l$P(B';'$B$ND>A0$^$G$r(B
	  //';'$B$,IU$$$F$$$J$+$C$?$i9TKv$r(B'\n'$B$r3P$($F$*$/!#(B(null-terminated)
	  //lyrics;\r\n -> lyrics\0
	  //lyrics\r\n  -> lyrics\n\0
	  //$B$H3P$($F$*$/!#(B
	  command->setTokenType (tLYRICS);
	  int length = 0;
	  //$B9TKv$^$G2?%P%$%HM-$k$+D4$Y$k!#(B
	  const char* const tmp = (char*) memchr (ptr, '\r', size-(ptr-buffer)+1);
	  if (tmp == NULL)
	    length = size-(ptr-buffer)+1;//data$B$N:G8e$^$G!#(B
	  else
	    length = tmp-ptr+1;//'\r'$B$^$G!#(B

	  //$B<B:]$K3P$($F$*$/!#(B
	  char* tmpstr = new char[length];
	  memcpy (tmpstr, ptr, length);
	  tmpstr[length-1] = '\n';//'\r'$B$r(B'\n'$B$K(B("\r\n"$B$r(B"\n\n$B$K(B")
	  command->offContinue ();
	  if (tmp !=  NULL && *(tmp-1) == ';')
	    {//';\r\n'
	      length -= 2;//';\n'$B$NJ,$r<N$F$k!#(B
	      command->setContinue ();
	    }
	  char* euc = new char[length*2];
	  int eucLength = convertSjis2Euc (tmpstr, length, euc);
	  delete[] tmpstr;
	  command->parameter = new Parameter[1];
	  command->parameter->storeStr (euc, eucLength);
	  delete[] euc;

	  if (tmp == NULL)
	    {//data$B$N:G8e$KC#$7$?!#(B
	      break;
	    }
	  else
	    {//'\r\n'$B$N<!$^$GHt$P$9!#(B
	      ptr = tmp+2;
	    }
#ifdef PRINT_COMPILED_RESULTS
	  cerr << "DEBUG:WrdFile::compile Lyrics="
	       << command->parameter[0].value.str;
#endif
	}
#ifdef PRINT_COMPILED_RESULTS
      if (command->isContinuing ())
	cerr << ";";
      cerr << endl;
#endif
      command->setNext (new WrdCommand ());
      command = command->getNext ();
    }

  if (command->getTokenType () != tEND)
    {
      command->setNext (new WrdCommand ());
      command = command->getNext ();
      command->setTokenType (tEND);
    }
  command->setNext (NULL);
  return commandStart;
}

void WrdFile::compileCommand
(const char*& ptr, WrdCommand*& command, const resword* resWord)
{
#ifdef DEBUG
  //cerr << "DEBUG::WrdFile::compileCommand token=" << resWord->name << endl;
#endif

  //$BFCJL$J=hM}$,I,MW$JJ*!#(B
  if (resWord->tokenType == tATMARK)
    {
      ptr++;
      while (*ptr != '@' && *ptr != '\r' && *(ptr+1) != '\n' &&
	     ptr-wrdBuffer < bufferSize-1)
	ptr++;
      return;
    }
  
  ptr += strlen (resWord->name);
  int parameters;
  for (parameters = 0;
       resWord->parameterType[parameters].type != param_end;
       parameters++)
    /*do nothing*/;//parameter$B$N8D?t$r?t$($k(B
  command->parameter = new Parameter[parameters];

  skipWhiteSpace (ptr);

  if (resWord->tokenType == tSTARTUP)
    {//startup$B$OFCJL$J=hM}$,I,MW!#(B
      if (checkOldtypeStartup (ptr, command, resWord))
	return;
    }
  
  if (*ptr != '(')
    throw CompileError ();
  ptr++;//skip '('

  if (resWord->tokenType == tPAL)
    {//pal$B$OFCJL$J=hM}$,I,MW!#(B
      compilePal (ptr, command, resWord);
      return;
    }
  if (resWord->tokenType == tLOCATE)
    {//locate$B$OFCJL$J=hM}$,I,MW!#(B
      compileLocate (ptr, command, resWord);
      return;
    }
  

  for (int i = 0; i < parameters; i++)
    {
      skipWhiteSpace (ptr);
      if (*ptr == ')')
	{//$B0z?t$,>JN,$5$l$F$$$?!#(B
	  for (; i < parameters; i++)
	    {
	      if (resWord->parameterType[i].isOmittalbe == param_noomit)
		{//$B>JN,=PMh$J$+$C$?!#(B
		  throw CompileError ();
		}
	      else
		storeParameter (command->parameter[i], 
				resWord->parameterType[i]);
	      continue;
	    }
	  break;
	}
      storeParameter (ptr, command->parameter[i], resWord->parameterType[i]);
      skipWhiteSpace (ptr);
      if (*ptr == ',' || *ptr == ';')//XXX for locate
	{
	  ptr++;
	  continue;
	}
      else if (*ptr == ')')
	continue;
      else
	throw CompileError ();
    }
  skipWhiteSpace (ptr);
  if (*ptr != ')')
    throw CompileError ();
  ptr++;//skip ')'
}


bool WrdFile::checkOldtypeStartup (const char*& ptr,WrdCommand*& command,
				   const resword* resWord)
{
  assert (ptr != NULL);
  assert (command != NULL);
  assert (resWord != NULL);
  assert (command->getTokenType () == tSTARTUP);
  
  skipWhiteSpace (ptr);
  if (*ptr == '(')//@STARTUP (version)$B$@$C$?!#(B
    return false;

  //@STARTUP$B$@$C$?!#(B
  command->parameter[0].type = param_int;
  command->parameter[0].value.positiveint = 399;//??? @STARTUP$B$N>l9g$N(Bversion$B$O(B?

  return true;
}


void WrdFile::compilePal
(const char*& ptr, WrdCommand*& command, const resword* resWord)
{
  skipWhiteSpace (ptr);
  if (*ptr == '#')
    {
      storeParameter (ptr, command->parameter[0], resWord->parameterType[0]);
      if (*ptr != ',')
	throw CompileError ();
      ptr++;
    }
  else
    storeParameter (command->parameter[0], resWord->parameterType[0]);

  for (int i = 1; i < 16; i++)
    {
      skipWhiteSpace (ptr);
      storeParameter (ptr, command->parameter[i], resWord->parameterType[i]);
      skipWhiteSpace (ptr);
      if (*ptr != ',')
	throw CompileError ();
      ptr++;
    }
  storeParameter (ptr, command->parameter[16], resWord->parameterType[16]);
  skipWhiteSpace (ptr);
  if (*ptr != ')')
    throw CompileError ();
  ptr++;//skip ')'
}


void WrdFile::compileLocate
(const char*& ptr, WrdCommand*& command, const resword* resWord)
{//compile @LOCATE (y;x),@LOCATE (x,y)
  
  storeParameter (ptr, command->parameter[0], resWord->parameterType[0]);

  bool isNeedSwap = true;
  if (*ptr != ';' && *ptr != ',')
    throw CompileError ();
  if (*ptr == ';')
    isNeedSwap = true;//@LOCATE(y;x)$B$@$C$?!#(B
  ptr++;//skip  ','or';'

  storeParameter (ptr, command->parameter[1], resWord->parameterType[1]);
  skipWhiteSpace (ptr);
  if (*ptr != ')')
    throw CompileError ();
  ptr++;//skip ')'
  
  if (isNeedSwap)
    {//@LOCATE(y;x)$B$@$C$?!#(B
      const int32 tmp = command->parameter[0].getPositiveInt ();
      command->parameter[0].value.positiveint = 
	command->parameter[1].getPositiveInt ();
      command->parameter[1].value.positiveint = tmp;
    }
}


void WrdFile::storeParameter
(const char*& ptr, Parameter& parameter, const ParameterType& parameterType)
{
  assert (parameterType.type != param_end);
  parameter.type = parameterType.type;
  skipWhiteSpace (ptr);
  if (*ptr == ',' || *ptr == ')')
    {//$B>JN,$5$l$F$$$?!#(B
      storeParameter (parameter, parameterType);
      return;
    }
  char* endPtr;
  switch (parameter.type)
    {
    case param_int:
      parameter.value.positiveint = strtol (ptr, &endPtr, 10);
      break;		
    case param_no:
      if (*ptr == '#')//XXX
	ptr++;
      parameter.value.no = strtol (ptr, &endPtr, 10);
      break;		
    case param_hex:
      parameter.value.hex = strtol (ptr, &endPtr, 16);
      break;		
    case param_eint:
      parameter.value.eint = strtol (ptr, &endPtr, 10);
      break;		
    case param_str:
      {
	const char* endPtr_ = ptr;
	while (*endPtr_ != ' '&&*endPtr_ != '\t'&&*endPtr_ != ','&&*endPtr_ != ')')
	  endPtr_++;
	//parameter.value.str=new char[endPtr_-ptr+1];
	//memcpy (parameter.value.str, ptr, endPtr_-ptr);
	//parameter.value.str[endPtr_-ptr]='\0';
	parameter.storeStr (ptr, endPtr_-ptr);
	ptr = endPtr_;
	return;
      }
      break;		
    case param_not_specified://$BD>2<$X(B
    case param_end:
#ifdef DEBUG
      cerr << "WrdFile::storeParameter" << endl;
      cerr << "no matched default value" << endl;
#endif
      break;
    }
  ptr = endPtr;
}

void WrdFile::storeParameter
(Parameter& parameter, const ParameterType& parameterType)
{
  assert (parameterType.isOmittalbe == param_omit);
  parameter.type = parameterType.type;
  switch (parameter.type)
    {
    case param_int:
      parameter.value.positiveint = parameterType.defaultValue;
      break;
    case param_no:
      parameter.value.no = parameterType.defaultValue;
      break;
    case param_hex:
      parameter.value.hex = parameterType.defaultValue;
      break;
    case param_eint:
      parameter.value.eint = parameterType.defaultValue;
      break;
    default:
#ifdef DEBUG
      cerr << "WrdFile::storeParameter" << endl;
      cerr << "no matched default value" << endl;
#endif
      break;
    }
}
