#ifndef WrdFile_H
#define WrdFile_H

#include <cstdio>
#include <cassert>

#include "util/mytype.h"

using namespace std;

// token type
enum WrdTokenType
{
  // for internal use
  tINVALID,	// invalid macro name
  tATMARK,	// `@' alone
  tLYRICS,	// print word
  tRETURN,      //return only
  // for MIMPI
  tCOLOR,
  tEND, 
  tESC, 
  tEXEC, 
  tFADE, 
  tGCIRCLE, 
  tGCLS, 
  tGINIT, 
  tGLINE, 
  tGMODE,
  tGMOVE, 
  tGON,
  tGSCREEN, 
  tINKEY, 
  tLOCATE, 
  tLOOP, 
  tMAG, 
  tMIDI,
  tOFFSET, 
  tPAL, 
  tPALCHG, 
  tPALREV, 
  tPATH, 
  tPLOAD, 
  tREM, 
  tREMARK, 
  tREST, 
  tSCREEN, 
  tSCROLL, 
  tSTARTUP, 
  tSTOP, 
  tTCLS, 
  tTON, 
  tWAIT, 
  tWMODE,
  // for ensyutsukun mechanism
  tFONTM,
  tFONTP,
  tFONTR,
  tGSC,
  tLINE,
  tPAL_,	// ^PAL
  tREGSAVE,
  tSCROLL_, 	// ^SCROLL
  tTEXTDOT,
  tTMODE,
  tTSCRL,
  tVCOPY,
  tVSGET,
  tVSRES,
  tXCOPY
};

enum Param_omit
{
  param_omit,
  param_noomit
};

enum Param_type
{
  param_int,		// positive number
  param_no,		// positive number with a preceding `#'
  param_hex,		// hexadecimal number
  param_eint,		// number (positive, negative and hexadecimal)
  param_str,		// strings
  param_not_specified,  // not specified
  param_end		// end of parameter
};

class ParameterType
{
public:
  Param_type type;
  Param_omit isOmittalbe;
  int defaultValue;
};

class Parameter
{
public:
  Param_type type;
  union
  {
    uint32 positiveint;//param_int
    uint32 no;//param_no
    int32 hex;//param_hex
    int eint;//param_eint
    char* str;//param_str
  }value;
  Parameter (){value.str=NULL;};
  void storeStr (const char* str,int size)
    {
      type=param_str;
      value.str=new char[size+1];
      memcpy (value.str,str,size);
      value.str[size]='\0';
    };
  ~Parameter ()
    {
      if (type==param_str)
	delete[] value.str;
    };//is this memory leak?
  uint32 getPositiveInt (){assert (type==param_int);return value.positiveint;};
  uint32 getNo () {assert (type==param_no);return value.no;};
  int32 getHex () {assert (type==param_hex);return value.hex;};
  int getEInt () {assert (type==param_eint);return value.eint;};
  const char* getStr () {assert (type==param_str);return value.str;};
};

class WrdCommand
{
private:
  const char* ptr;//command pointer in original wrd file
  WrdTokenType type;//command type
  bool isContinued;//$BA02s$N(Bcommand$B$+$i(B';'$B$GB3$$$F$$$k$+(B?
  WrdCommand* next;//next
public:
  Parameter* parameter;//parameter
  WrdCommand ():
    ptr (NULL),type (tINVALID),isContinued (false),next (NULL),parameter (NULL)
    {/*do nothing*/;};
  ~WrdCommand (){delete[] parameter;};

  //////////////////////////////////////////////////////////////acess method
  void setCommandPtr (const char* ptr_) {assert (ptr==NULL);ptr=ptr_;};
  void setTokenType (WrdTokenType type_) {assert (type==tINVALID);type=type_;};
  void setINVALID () {delete[] parameter; parameter = NULL; type = tINVALID;};
  void setContinue () {assert (isContinued==false);isContinued=true;};
  void offContinue () {isContinued=false;};
  void setNext (WrdCommand* next_) {assert (next==NULL);next=next_;};
  
  const char* getStr () const {return parameter[0].getStr ();};
  const char* getCommandPtr () const {assert (ptr!=NULL);return ptr;};
  WrdTokenType getTokenType () const {return type;};
  bool isContinuing () const {return isContinued;};
  WrdCommand* getNext () const {return next;};
  void getIntParameter (uint32* value,int size,int offset=0) const
    {
      assert (parameter!=NULL);
      for (int i=0;i<size;i++)
	value[i]=parameter[i+offset].getPositiveInt ();
    };
  void getHexParameter (int32* value,int size,int offset=0) const
    {
      assert (parameter!=NULL);
      for (int i=0;i<size;i++)
	value[i]=parameter[i+offset].getHex ();
    };
  void getEIntParameter (int32* value,int size,int offset=0) const
    {
      assert (parameter!=NULL);
      for (int i=0;i<size;i++)
	value[i]=parameter[i+offset].getEInt ();
    };
};

//#include "wrd.h"
struct resword;

//Wrd file$B$rFI$_9~$s$GFbIt7A<0$KJQ49$9$k%/%i%9(B
//$B:F@8$O<u$1;}$?$J$$!#:F@8ItJ,$O(Bclass Wrd$B$,<u$1;}$D!#(B
class WrdFile
{
public:
  //exception class
  class FileNotFound {};
  
  WrdFile ();
  ~WrdFile ();
  void load (const char* fileName_);
  WrdCommand* getCommand () {return wrdCommand;};
  const char* getFileName () const {return fileName;};
protected:
  char* fileName;
  char* wrdBuffer;//wrd file$B$r%a%b%j!<>e$KFI$_9~$s$@$b$N!#(B
  int bufferSize;//wrdBuffer$B$N%5%$%:(B
  WrdCommand* wrdCommand;//wrd file$B$rFbIt7A<0$KJQ49$7$?$b$N!#(B
  //WrdCommand$B$N(Bsingle linked list$B$GI=$5$l$k!#(B
  void skipWhiteSpace (const char*& ptr)
    {
      while (*ptr==' ' || *ptr=='\t')
	ptr++;
    };
  WrdCommand* compile (const char* const buffer, int size);
  const resword* getCommandType (const char* ptr);
  //exeptional class
  class CompileError {};
  //$B86B'$H$7$F0z?t$O=g$K(Bcommand->parameter[i]$B$K3JG<$5$l$k!#(B
  void compileCommand (const char*& ptr, WrdCommand*& command,
		       const resword* resWord);

  //@STARTUP (version) :mimpi ver 4
  //@STARTUP           :mimpi ver 3   $B$,$"$k$?$a!#(B
  //ver 3$B$@$C$?>l9g!"$=$N(B@STARTUP$B$r=hM}$7$F(Btrue$B$rJV$9!#(B
  //ver 4$B$@$C$?>l9g2?$b$;$:$K(Bfalse$B$rJV$9!#(B
  bool checkOldtypeStartup (const char*& ptr, WrdCommand*& command,
			    const resword* resWord);
  
  //@PAL (rgb1,...rgb16)$B$3$l$O2<$N(Bp=0$B$H$7$F3JG<$5$l$k!#(B
  //@PAL (#p,rgb1,...,rgb16)
  void compilePal (const char*& ptr, WrdCommand*& command,
		   const resword* resWord);
  //@LOCATE(y;x),@LOCATE(x,y),tLOCATE$B$N;~(B
  //command->parameter[0].value.positiveint=x
  //command->parameter[1].value.positiveint=y
  //$B$G3JG<$5$l$k!#(B
  void compileLocate (const char*& ptr, WrdCommand*& command,
		      const resword* resWord);
  void storeParameter (const char*& ptr, Parameter& parameter,
		       const ParameterType& resWord);
  void storeParameter (Parameter& parameter,
		       const ParameterType& parameterType);
};
#endif // WrdFile_H
