#include <cstdio>

#include "IOGFileSelector.h"
#include "util/calcTime.h"
#include "util/convertAbsolutePathName.h"

const string IOGFileSelector::WINDOW_TITLE ("Widi "VERSION":GFileSelector");
const string IOGFileSelector::HANDLE_BOX_TITLE ("Widi "VERSION": Controler");
const int IOGFileSelector::FILE;
const int IOGFileSelector::TIME;
const int IOGFileSelector::SONG_TITLE;
const int IOGFileSelector::CLIST_COLUMNS;

IOGFileSelector::IOGFileSelector (State* state__,
				  WidiController* widiController__,
				  Option* option__):
  IO (state__, widiController__, option__),
  isSliderDragged (false),
  fileSelection (NULL),
  isFileSortLess (true), isTimeSortLess (true), isTitleSortLess (true)
{
  gdk_threads_enter ();
  
  //top level 
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  //gtk_widget_set_usize (window, 400, 200);
  gtk_window_set_title (GTK_WINDOW (window), WINDOW_TITLE.c_str ());

  gtk_signal_connect_object (GTK_OBJECT (window), "destroy",
			     GTK_SIGNAL_FUNC ((&gtkObjectCallBack<IOGFileSelector COMMA &IOGFileSelector::doQuit>)),
			     static_cast<GtkObject*> (static_cast<gpointer> (this)));
  

  //VBOX
  vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);

  //menu
  GtkWidget* menuBar = gtk_menu_bar_new ();
  gtk_box_pack_start (GTK_BOX (vbox), menuBar, FALSE, FALSE, 0);
  gtk_widget_show (menuBar);
  
  menuItemFile = menuItemNewAppend ("File", GTK_MENU_BAR (menuBar));
  menuFile = menuNewSetSubmenu (GTK_MENU_ITEM (menuItemFile));

  menuItemOpen = menuItemNewAppend ("Open", GTK_MENU (menuFile));
  menuItemQuit = menuItemNewAppend ("Quit", GTK_MENU (menuFile));

  gtk_signal_connect_object (GTK_OBJECT (menuItemOpen), "activate",
			     GTK_SIGNAL_FUNC (&gtkObjectCallBack<IOGFileSelector COMMA &IOGFileSelector::openFileSelection>),
			     static_cast<GtkObject*> (static_cast<gpointer>(this)));
  gtk_signal_connect_object (GTK_OBJECT (menuItemQuit), "activate",
			     GTK_SIGNAL_FUNC (&gtkObjectCallBack<IOGFileSelector COMMA &IOGFileSelector::doQuit>),
			     reinterpret_cast<GtkObject*> (this));

  menuItemHelp = gtk_menu_item_new_with_label ("Help");
  gtk_menu_item_right_justify (GTK_MENU_ITEM (menuItemHelp));
  gtk_menu_bar_append (GTK_MENU_BAR (menuBar), menuItemHelp);
  gtk_widget_show (GTK_WIDGET (menuItemHelp));


  menuHelp = menuNewSetSubmenu (GTK_MENU_ITEM (menuItemHelp));

  menuItemHelpHelp = menuItemNewAppend ("Help", GTK_MENU (menuHelp));
  menuItemAbout = menuItemNewAppend ("About", GTK_MENU (menuHelp));

  gtk_signal_connect_object (GTK_OBJECT (menuItemHelpHelp), "activate",
			     GTK_SIGNAL_FUNC (&gtkObjectCallBack<IOGFileSelector COMMA &IOGFileSelector::doHelp>),
			     reinterpret_cast<GtkObject*>(this));
  gtk_signal_connect_object (GTK_OBJECT (menuItemAbout), "activate",
			     GTK_SIGNAL_FUNC (&gtkObjectCallBack<IOGFileSelector COMMA &IOGFileSelector::doAbout>),
			     reinterpret_cast<GtkObject*>(this));

  /////////////////////////////////////////////////scrolledwindow
  scrolledWindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledWindow),
				  GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), scrolledWindow, TRUE, TRUE, 0);
  gtk_widget_show (scrolledWindow);
  
  ////////////////////////////////////////////////////////////clist
  clist = gtk_clist_new (CLIST_COLUMNS);
  //gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_IN);
  gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_OUT);
  //gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_ETCHED_IN);
  //gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_ETCHED_OUT);

  clistSetColumn (GTK_CLIST (clist), 0, "File",  GTK_JUSTIFY_RIGHT, 120);
  clistSetColumn (GTK_CLIST (clist), 1, "Time",  GTK_JUSTIFY_RIGHT, 50);
  clistSetColumn (GTK_CLIST (clist), 2, "Title", GTK_JUSTIFY_LEFT,  150);

  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
  gtk_clist_column_titles_show (GTK_CLIST (clist));
  gtk_clist_column_titles_active (GTK_CLIST (clist));
  
  //gtk_box_pack_start (GTK_BOX (vbox), clist, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (scrolledWindow), clist);
  gtk_widget_show (GTK_WIDGET (clist));

  gtk_signal_connect (GTK_OBJECT (clist), "select_row",
		      GTK_SIGNAL_FUNC (selectRowCallback), this);
  gtk_signal_connect (GTK_OBJECT (clist), "click_column",
		      GTK_SIGNAL_FUNC (clickColumnCallback), this);
  
  //hbox
  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (GTK_WIDGET (hbox));
  
  //handle box
  handlebox = gtk_handle_box_new ();
  //gtk_window_set_title (GTK_WINDOW (handlebox), HANDLE_BOX_TITLE.c_str ());
  gtk_box_pack_start (GTK_BOX (hbox), handlebox, FALSE, FALSE, 0);
  gtk_widget_show (GTK_WIDGET (handlebox));

  //toolbar
  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
  gtk_widget_show (toolbar);
  gtk_widget_realize (toolbar);//for creating pixmap
  
  //buttons
  GdkColor* const gdkColor =
    &(gtk_widget_get_style (toolbar)->bg[GTK_STATE_NORMAL]);

  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "play", "play song", NULL,
			   newPixmap ("gfsplay.xpm", toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doPlay>), this);
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "pause", "pause", NULL,
			   newPixmap ("gfspause.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doPause>), this);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "prev", "previous song", NULL,
			   newPixmap ("gfsprev.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doPrev>), this);
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "rewind", "rewind", NULL,
			   newPixmap ("gfsrewind.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doRewind>), this);
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "back", "back", NULL,
			   newPixmap ("gfsback.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doBack>), this);
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "stop", "stop playing", NULL,
			   newPixmap ("gfsstop.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doStop>), this);
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "forward", "forward", NULL,
			   newPixmap ("gfsforward.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doForward>), this);
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "next", "next song", NULL,
			   newPixmap ("gfsnext.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::doNext>), this);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
  
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "open", "open/close file selection", NULL,
			   newPixmap ("gfsopen.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::openFileSelection>), this);
  
  gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), "clear", "clear the list", NULL,
			   newPixmap ("gfsclear.xpm",
				      toolbar->window, gdkColor),
			   GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::clearFileSelection>), this);
  
  adjustment = gtk_adjustment_new (0.0, 0.0, 61.0, 1.0, 1.0, 1.0);
  hscale = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
  gtk_scale_set_digits (GTK_SCALE (hscale), 2);
  
  //gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_DELAYED);
  gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_DISCONTINUOUS);
  //gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_draw_value (GTK_SCALE (hscale), TRUE);
  gtk_signal_connect (GTK_OBJECT (hscale),
		      "button_release_event",
		      GTK_SIGNAL_FUNC (&IOGFileSelector::doSliderButtonCallBack),
		      this);
  gtk_signal_connect (GTK_OBJECT (hscale),
		      "button_press_event",
		      GTK_SIGNAL_FUNC (&IOGFileSelector::doSliderButtonCallBack),
		      this);
  gtk_box_pack_start (GTK_BOX (hbox), hscale, TRUE, TRUE, 0);
  gtk_widget_show (hscale);

  timeLabel = gtk_label_new (" 00'00'00  ");
  gtk_box_pack_start (GTK_BOX (hbox), timeLabel, FALSE, FALSE, 0);
  gtk_widget_show (timeLabel);
  
  gtk_widget_show (window);

  gdk_threads_leave ();
}

IOGFileSelector::~IOGFileSelector ()
{
  if (fileSelection != NULL)
    gtk_widget_destroy (fileSelection);
  gtk_widget_destroy (window);
}


void IOGFileSelector::createFileSelection ()
{
  assert (fileSelection == NULL);

  fileSelection = gtk_file_selection_new ("Select Midi Files");
  gtk_signal_connect (GTK_OBJECT (fileSelection), "destroy",
		      GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::hideFileSelection>),
		      this);
  
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (fileSelection));

  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fileSelection)->ok_button),
                      "clicked",
		      GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::fileSelectionOkClicked>),
		      this);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fileSelection)->cancel_button),
		      "clicked",
		      GTK_SIGNAL_FUNC (&gtkCallBack<IOGFileSelector COMMA &IOGFileSelector::hideFileSelection>),
		      this);
}

void IOGFileSelector::showFileSelection ()
{
  assert (fileSelection != NULL);
  gtk_widget_show (GTK_WIDGET (fileSelection));
}

void IOGFileSelector::hideFileSelection ()
{
  assert (fileSelection != NULL);
  gtk_widget_hide (GTK_WIDGET (fileSelection));
}


void IOGFileSelector::openFileSelection ()
{
  if (fileSelection == NULL)
    createFileSelection ();

  if (GTK_WIDGET_VISIBLE (fileSelection))
    {
      cerr << "IOGFileSelection::openFileSelection fileSelection is visible" << endl;
      hideFileSelection ();
    }
  else
    {
      cerr << "IOGFileSelection::openFileSelection fileSelection is NOT visible" << endl;
      showFileSelection ();
    }
}

void IOGFileSelector::clearFileSelection ()
{
  songFileList.clear ();

  gtk_clist_clear (GTK_CLIST (clist));
}


void IOGFileSelector::appendCList (GtkCList* clist_,
				   const SongFileList::SongItem& songItem)
{
  gchar** text = new char*[CLIST_COLUMNS];
  text[FILE] = new gchar[(*songItem).getFileName ().size ()+1];
  strcpy (text[FILE], (*songItem).getFileName ().c_str ());
  
  const TimeInfo totalTimeInfo = calcTime ((*songItem).getPlayTime ());
#define MAX_TIME_LENGTH 16
  text[TIME] = new gchar[MAX_TIME_LENGTH];
  sprintf (text[TIME], "%3d'%02d'%02d",
	   totalTimeInfo.minute,
	   totalTimeInfo.second, totalTimeInfo.milisecond/10);
#undef MAX_TIME_LENGTH

  text[SONG_TITLE] = new gchar[(*songItem).getTitle ().size ()+1];
  strcpy (text[SONG_TITLE], (*songItem).getTitle ().c_str ());

  gtk_clist_append (clist_, text);

  for (int i = 0; i < CLIST_COLUMNS; i++)
    delete[] text[i];
  delete[] text;
}


void IOGFileSelector::fileSelectionOkClicked ()
{
  DEBUG_MESSAGE ("IOGFileSelector::fileSelectionOkClicked");
  const string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fileSelection));

  const uint oldSize = songFileList.size ();
  songFileList.searchAppend (filename);
  //check really appended?
  if (oldSize == songFileList.size ())
    {//not appended because of filename is not song file.
      DEBUG_MESSAGE ("IOGFileSelector::appendClist no need appending");
      return;
    }
  
  DEBUG_MESSAGE ("IOGFileSelector::appendClist really appending");
  appendCList (GTK_CLIST (clist), --songFileList.end ());
}


void IOGFileSelector::selectRow (gint row)
{
  gchar* text;
  gtk_clist_get_text (GTK_CLIST (clist), row, 0, &text);
#ifdef DEBUG
  cout << "IOGFileSelector::selectRow called. text =" << text << endl;
#endif

  gdk_threads_leave ();
  widiController->serverCommand->playWithFileName (text);
  gdk_threads_enter ();
  
  DEBUG_MESSAGE ("IOGFileSelector::selectRow all done.");
}

void IOGFileSelector::clickColumn (gint column)
{
  switch (column)
    {
    case FILE:
      if (isFileSortLess)
	songFileList.fileNameSortLess ();
      else
	songFileList.fileNameSortGreater ();
      isFileSortLess = !isFileSortLess;
      break;
    case TIME:
      if (isTimeSortLess)
	songFileList.playTimeSortLess ();
      else
	songFileList.playTimeSortGreater ();
      isTimeSortLess = !isTimeSortLess;
      break;
    case SONG_TITLE:
      if (isTitleSortLess)
	songFileList.titleSortLess ();
      else
	songFileList.titleSortGreater ();
      isTitleSortLess = !isTitleSortLess;
      break;
    default:
      DEBUG_MESSAGE ("IOGFileSelector::clickColumn invalid column clicked");
      break;
    }
  songFileList.unique ();

  gtk_clist_freeze (GTK_CLIST (clist));
  gtk_clist_clear (GTK_CLIST (clist));
  for (SongFileList::SongItem i = songFileList.begin ();
       i != songFileList.end (); i++)
    appendCList (GTK_CLIST (clist), i);
  gtk_clist_thaw (GTK_CLIST (clist));
  cout << "IOGFileSelector::clickColumn=" << column << endl;
}

void IOGFileSelector::prePlay ()
{
  IO::prePlay ();

  const gfloat totalTime = cacheState.songStatus.totalLogicalTicks/SongStatus::SONG_CTRL_RATE;
  TimeInfo totalTimeInfo = calcTime (cacheState.songStatus.totalLogicalTicks);
  char buff[32];
  sprintf (buff, "%3d'%02d'%02d  ",
  	   totalTimeInfo.minute,
	   totalTimeInfo.second,
	   totalTimeInfo.milisecond/10);

  string filename = cacheState.songStatus.getFileName ();
  string title = WINDOW_TITLE;
  title += " : ";
  title += filename;

  //$B:#$+$i1iAU$9$k%U%!%$%k$,EPO?$5$l$F$J$1$l$PEPO?$9$k!#(B
  SongFileList::SongItem i = songFileList.begin ();
  gint row = 0;
  //clist$B$O(Babsolute pathname$B$G4IM}!#$h$C$F!"(BsongFileList$B$b(Babsolute pathname
  //$B$rJ];}$7$F$$$k$N$G!"@dBP(Bpath$BL>$K$9$k!#(B
  convertAbsolutePathName (filename);
  for (; i != songFileList.end (); i++, row++)
    {
      if ((*i).getFileName () == filename)
	break;//found
    }

  gdk_threads_enter ();

  if (i == songFileList.end ())
    {//$BEPO?$5$l$F$$$J$+$C$?!#(B
      songFileList.searchAppend (filename);
      i = --songFileList.end ();
      //CList$B$K$b(Bfilename$B$rEPO?$9$k!#(B
      appendCList (GTK_CLIST (clist), i);
    }
#if 0
  gtk_clist_select_row (GTK_CLIST (clist), row, 0);
  if (gtk_clist_row_is_visible (GTK_CLIST (clist), row) == GTK_VISIBILITY_NONE)
  gtk_clist_moveto (GTK_CLIST (clist), row, 0, 0.0, 0.0);
#endif
  
  gtk_window_set_title (GTK_WINDOW (window), title.c_str ());
  gtk_label_set (GTK_LABEL (timeLabel), buff);
  //memory leak???
  adjustment = gtk_adjustment_new (0.0,
				   0.0, totalTime+1.0,
				   0.1, 1.0, 1.0);
  gtk_range_set_adjustment (GTK_RANGE (hscale), GTK_ADJUSTMENT (adjustment));
  gdk_threads_leave ();
  
  update ();
}


void IOGFileSelector::update ()
{
  const gfloat nowTime = cacheState.songStatus.logicalTicks/SongStatus::SONG_CTRL_RATE;
  gdk_threads_enter ();
  if (isSliderDragged == false)
    {
      gtk_adjustment_set_value (GTK_ADJUSTMENT (adjustment), nowTime);
      gtk_scale_draw_value (GTK_SCALE (hscale));
      //gdk_threads_wake ();
    }
  gdk_threads_leave ();
}







////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////for dynamic loading
///////////////////////////////////////////////////////////////newIO<T>
///////////////////////////////////////////////////////////////newMainIO<T>
///////////////////////////////////////////////////////////////infoIO<T>
////////////////////////////////////////////////////////////////////////////

#include "IO/InfoIO.h"

static IO* newIOGFileSelector (State* state, WidiController* widiController,
			       Option* option)
{
  return new IOGFileSelector (state, widiController, option);
}

InfoIO infoIOGFileSelector =
{
  newIOGFileSelector,
  NULL,
  true
};


