#ifndef IOGFileSelector_H
#define IOGFileSelector_H

#include <string>
#include <gtk/gtk.h>

#include "IO/IO.h"
#include "PlugIn/X/GTKBase/GTKBase.h"
#include "PlugIn/X/GTKBase/GtkCallBackTemplate.h"
#include "Widi/SongFileList.h"
#include "Widi/ServerCommand.h"

#if 0 // with this, something goes wrong, but without this deadlocks!
#define gdk_threads_enter()
#define gdk_threads_leave()
#endif
using namespace std;

class IOGFileSelector:public IO,public GTKBase
{
private:
  static const string WINDOW_TITLE;
  static const string HANDLE_BOX_TITLE;

  //GTK stuff
  GtkWidget* window;//top level window
  GtkWidget* vbox;//contains menu bar,clist and buttons.

  //////////////////////////////////////////////////////menu bar
  GtkWidget* menuBar;//menu bar contains, menuItemFile,menuItemHelp
  GtkWidget* menuItemFile;//File menu, contains menuFile
  GtkWidget* menuItemHelp;//Help menu, contains menuHelp
  
  GtkWidget* menuFile;//contains menuItemOpen,menuItemQuit
  GtkWidget* menuItemOpen;
  GtkWidget* menuItemQuit;
  
  GtkWidget* menuHelp;//contains menuItemHelpHelp,menuItemAbout
  GtkWidget* menuItemHelpHelp;
  GtkWidget* menuItemAbout;
  
  ///////////////////////////////////scrolled window which contains clist
  GtkWidget* scrolledWindow;

  //////////////////////////////////////////////////////////clist
  static const int FILE = 0;
  static const int TIME = 1;
  static const int SONG_TITLE = 2;

  static const int CLIST_COLUMNS = 3;

  GtkWidget* clist;//clist

  ///////////////////////////////////////////////////////buttons
  GtkWidget* hbox;//contains handle box and scale

  GtkWidget* handlebox;//contains toolbar
  GtkWidget* toolbar;//contains bottuns

  GtkObject* adjustment;
  GtkWidget* hscale;//show play time
  bool isSliderDragged;
  GtkWidget* timeLabel;

  /////////////file selection
  GtkWidget* fileSelection;


  //menu$B4XO"(B
  GtkWidget* menuItemNewAppend (const string& label, GtkMenuBar* gtkMenuBar)
    {
      GtkWidget* menuItem = gtk_menu_item_new_with_label (label.c_str ());
      gtk_menu_bar_append (gtkMenuBar, menuItem);
      gtk_widget_show (GTK_WIDGET (menuItem));
      return menuItem;
    };

  GtkWidget* menuItemNewAppend (const string& label, GtkMenu* gtkMenu)
    {
      GtkWidget* menuItem = gtk_menu_item_new_with_label (label.c_str ());
      gtk_menu_append (gtkMenu, menuItem);
      gtk_widget_show (GTK_WIDGET (menuItem));
      return menuItem;
    };

  GtkWidget* menuNewSetSubmenu (GtkMenuItem* gtkMenuItem)
    {
      GtkWidget* menu = gtk_menu_new ();
      gtk_menu_item_set_submenu (gtkMenuItem, menu);
      return menu;
    };

  //clist$B4XO"(B
  void clistSetColumn (GtkCList* gtkCList, gint column,
		       const string& title, GtkJustification justification,
		       gint width)
    {
      gtk_clist_set_column_auto_resize (gtkCList, column, TRUE);
      gtk_clist_set_column_resizeable (gtkCList, column, TRUE);
      gtk_clist_set_column_title (gtkCList, column, title.c_str ());//XXX
      gtk_clist_set_column_justification (gtkCList, column, justification);
      gtk_clist_set_column_width (gtkCList, column, width);
    };

  GtkWidget* newPixmap (const string& filename,
			GdkWindow* gdkWindow, GdkColor* background)
    {
      assert (gdkWindow != NULL);
      assert (background != NULL);
      string fullFileName = WIDI_DATA_DIR"/";
      fullFileName += filename;
      //cerr << "newPixmap = " << fullFileName << endl;
      GdkBitmap *mask;
      GdkPixmap* pixmap = gdk_pixmap_create_from_xpm (gdkWindow, &mask,
						      background,
						      fullFileName.c_str ());
      assert (pixmap != NULL);
      if (pixmap == NULL)
	return NULL;
      
      return gtk_pixmap_new (pixmap, mask);
    };
  

  /////////////////////////////////////////////////////////////call back
  void doQuit ()
    {gdk_threads_leave (); widiController->serverCommand->quit (); gdk_threads_enter ();};
  void doHelp ()
    {gdk_threads_leave (); cerr << "menuHelp" << endl; gdk_threads_enter ();};
  void doAbout ()
    {gdk_threads_leave (); cerr << "menuAbout" << endl; gdk_threads_enter ();};

  void doPlay ()
    {gdk_threads_leave (); widiController->serverCommand->play (); gdk_threads_enter ();};
  void doPause ()
    {gdk_threads_leave (); widiController->serverCommand->pause (); gdk_threads_enter ();};
  void doPrev ()
    {gdk_threads_leave (); widiController->serverCommand->prev (); gdk_threads_enter ();};
  void doRewind ()
    {gdk_threads_leave (); widiController->serverCommand->rewind (); gdk_threads_enter ();};
  void doBack ()
    {gdk_threads_leave (); widiController->serverCommand->back (); gdk_threads_enter ();};
  void doStop ()
    {gdk_threads_leave (); widiController->serverCommand->stop (); gdk_threads_enter ();};
  void doForward ()
    {gdk_threads_leave (); widiController->serverCommand->forward (); gdk_threads_enter ();};
  void doNext ()
    {gdk_threads_leave (); widiController->serverCommand->next (); gdk_threads_enter ();};
  static gint doSliderButtonCallBack (GtkWindow* hscale_,
				      GdkEventButton* event,
				      gpointer me)
    {
      DEBUG_MESSAGE ("IOGFileSelector::doSliderButtonCallBack");
      IOGFileSelector* const me_ = static_cast<IOGFileSelector*>(me);

      if (event->type == GDK_BUTTON_RELEASE && me_->isSliderDragged)
	{//button release
	  me_->doSliderButtonRelease ();
	}
      else if (event->window == GTK_RANGE (me_->hscale)->slider &&
	       event->type == GDK_BUTTON_PRESS)
	{//button press
	  me_->doSliderButtonPress ();
	}
      
      return FALSE;
      //return TRUE;
    };
  
  void doSliderButtonPress ()
    {
      DEBUG_MESSAGE ("IOGFileSelector::doSliderButtonPress");
      isSliderDragged = true;
    };
  void doSliderButtonRelease ()
    {
      DEBUG_MESSAGE ("IOGFileSelector::doSliderButtonRelease");
      GtkAdjustment* tmp = gtk_range_get_adjustment (GTK_RANGE (hscale));
      assert (tmp != NULL);
      const gfloat value = tmp->value;
      
      gdk_threads_leave ();
      //SongStatus::SONG_CTRL_RATE$B$r$+$1$k$N$O(Bhscale$B$OICC10L$G$"$k$+$i!#(B
      widiController->serverCommand->setPosition (static_cast<uint32> (value*SongStatus::SONG_CTRL_RATE));
      gdk_threads_enter ();
      
      isSliderDragged = false;
    };

  void createFileSelection ();
  void showFileSelection ();
  void hideFileSelection ();
  void openFileSelection ();
  void clearFileSelection ();
  void fileSelectionOkClicked ();

  static void selectRowCallback (GtkWidget* widget,
				 gint row, gint /*column*/,
				 GdkEventButton* /*event*/,
				 gpointer me)
    {static_cast<IOGFileSelector*>(me)->selectRow (row);};
  void selectRow (gint row);
  static void clickColumnCallback (GtkWidget* widget,
				   gint column,
				   gpointer me)
    {static_cast<IOGFileSelector*>(me)->clickColumn (column);};
  void clickColumn (gint column);

  void createWidget ();
  //sort$B$9$k;~$K9_=g$+>:=g$+$r3P$($F$*$/(B
  bool isFileSortLess;
  bool isTimeSortLess;
  bool isTitleSortLess;
  //
  SongFileList songFileList;
  void appendCList (GtkCList* clist_, const SongFileList::SongItem& songItem);
  
  //IO
  void prePlay ();
  void update ();
public:
  IOGFileSelector (State* state__, WidiController* widiController__,
		   Option* option__);
  ~IOGFileSelector ();
};
#endif// IOGFileSelector_H
