/*
    Signal.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef Signal_H
#define Signal_H
#include <cassert>

#include "ConnectionImpl.h"

using namespace std;

class Signal0
{
private:
  typedef ConnectionImpl0* signalPtr;
  signalPtr first;
  signalPtr last;
public:
  Signal0 ():first (NULL),last (NULL) {/*do nothing*/};
  ~Signal0 ()
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now=first->next;
	  delete first;
	  first=now;
	}
    };
  void connect (ConnectionImpl0* connectionImpl_)
    {
      assert (connectionImpl_!=NULL);
      if (last==NULL)
	{
	  first=connectionImpl_;
	  last=first;
	}
      else
	{
	  last->next=connectionImpl_;
	  last=last->next;
	  last->next=NULL;
	}
      last->next=NULL;
    };
  template <class T>
  void connect (T* t,void (T::*method) ())
    {
      connect (new Connection0<T> (t,method));
    };
  void operator ()()
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now->call ();
	  now=now->next;
	}
    };
};

template <class E>
class Signal1
{
private:
  typedef ConnectionImpl1<E>* signalPtr;
  signalPtr first;
  signalPtr last;
public:
  Signal1 ():first (NULL),last (NULL) {/*do nothing*/};
  ~Signal1 ()
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now=first->next;
	  delete first;
	  first=now;
	}
    };
  void connect (ConnectionImpl1<E>* connectionImpl_)
    {
      assert (connectionImpl_!=NULL);
      if (last==NULL)
	{
	  first=connectionImpl_;
	  last=first;
	}
      else
	{
	  last->next=connectionImpl_;
	  last=last->next;
	  last->next=NULL;
	}
      last->next=NULL;
    };
  template <class T,class X>
  void connect (T* t,void (T::*method) (X))
    {
      connect (new Connection1<T,X> (t,method));
    };
  void operator ()(E e)
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now->call (e);
	  now=now->next;
	}
    };
};

template <class E,class F>
class Signal2
{
private:
  typedef ConnectionImpl2<E,F>* signalPtr;
  signalPtr first;
  signalPtr last;
public:
  Signal2 ():first (NULL),last (NULL) {/*do nothing*/};
  ~Signal2 ()
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now=first->next;
	  delete first;
	  first=now;
	}
    };
  void connect (ConnectionImpl2<E,F>* connectionImpl_)
    {
      assert (connectionImpl_!=NULL);
      if (last==NULL)
	{
	  first=connectionImpl_;
	  last=first;
	}
      else
	{
	  last->next=connectionImpl_;
	  last=last->next;
	  last->next=NULL;
	}
      last->next=NULL;
    };
  template <class T,class X,class Y>
  void connect (T* t,void (T::*method) (X,Y))
    {
      connect (new Connection2<T,X,Y> (t,method));
    };
  void operator ()(E e,F f)
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now->call (e,f);
	  now=now->next;
	}
    };
};



template <class E,class F,class G>
class Signal3
{
private:
  typedef ConnectionImpl3<E,F,G>* signalPtr;
  signalPtr first;
  signalPtr last;
public:
  Signal3 ():first (NULL),last (NULL) {/*do nothing*/};
  ~Signal3 ()
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now=first->next;
	  delete first;
	  first=now;
	}
    };
  void connect (ConnectionImpl3<E,F,G>* connectionImpl_)
    {
      assert (connectionImpl_!=NULL);
      if (last==NULL)
	{
	  first=connectionImpl_;
	  last=first;
	}
      else
	{
	  last->next=connectionImpl_;
	  last=last->next;
	  last->next=NULL;
	}
      last->next=NULL;
    };
  template <class T,class X,class Y,class Z>
  void connect (T* t,void (T::*method) (X,Y,Z))
    {
      connect (new Connection3<T,X,Y,Z> (t,method));
    };
  void operator ()(E e,F f,G g)
    {
      signalPtr now=first;
      while (now!=NULL)
	{
	  now->call (e,f,g);
	  now=now->next;
	}
    };
};
#endif// Signal_H
