/*
    WdkGC.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#include <cassert>

#include "WdkWindow.h"
#include "WdkPixmap.h"
#include "WdkGC.h"

using namespace std;

WdkGC::WdkGC (WdkWindow* window):
  display (window->getDisplay ())
{
  gc=XCreateGC (display, window->getWindow (), 0, 0);
}

WdkGC::WdkGC (WdkPixmap* pixmap):
  display (pixmap->getDisplay ())
{
  gc=XCreateGC (display, pixmap->getPixmap (), 0, 0);
}


WdkGC::~WdkGC ()
{
  XFreeGC (display, gc);
}


void WdkGC::setTile (WdkPixmap& pixmap)
{
  XSetTile (display, gc, pixmap.getPixmap ());
}

void WdkGC::setTile (WdkPixmap* pixmap)
{
  assert (pixmap!=NULL);
  XSetTile (display, gc, pixmap->getPixmap ());
}

void WdkGC::setStipple (WdkPixmap& pixmap)
{
  XSetStipple (display, gc, pixmap.getPixmap ());
}

void WdkGC::setStipple (WdkPixmap* pixmap)
{
  assert (pixmap!=NULL);
  XSetStipple (display, gc, pixmap->getPixmap ());
}

void WdkGC::setClipMask (WdkPixmap& bitmap)
{
  assert (bitmap.getDepth () == 1);
  XSetClipMask (display, gc, bitmap.getPixmap ());
}
