/*
    WdkGC.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WdkGC_H
#define WdkGC_H
#include <X11/Xlib.h>

#include "WdkTypes.h"
class WdkWindow;
class WdkPixmap;

class WdkGC
{
private:
  Display* display;
  GC gc;
public:
  WdkGC (WdkWindow* window);
  WdkGC (WdkPixmap* window);
  ~WdkGC ();

  //////////////////////////////////////////////////////////////access method
  Display* getDisplay () const {return display;};
  GC getGC () const {return gc;};
  

  ////////////////////////////////////////////////////////////wapper of Xlib
  void copy (WdkGC* src, ulong valuemask)
    {XCopyGC (display,  src->getGC (),  valuemask, gc);};
  void copy (WdkGC& src, ulong valuemask)
    {XCopyGC (display, src.getGC (), valuemask, gc);};
  void change (ulong valuemask, XGCValues* values)
    {XChangeGC (display, gc, valuemask, values);};
  Status getGCValues (ulong valuemask, XGCValues* valuesReturn)
    {return XGetGCValues (display, gc, valuemask, valuesReturn);};
  void setState (ulong foreground, ulong background, 
		 int function, ulong planeMask)
    {XSetState (display, gc, foreground, background, function, planeMask);};
  void setFunction (int function)
    {XSetFunction (display, gc, function);};
  void setPlaneMask (ulong planeMask)
    {XSetPlaneMask (display, gc, planeMask);};
  void setForeground (ulong foreground)
    {XSetForeground (display, gc, foreground);};
  void setForeground (WdkColor& color)
    {XSetForeground (display, gc, color.getPixel ());};
  void setForeground (WdkColor* color)
    {XSetForeground (display, gc, color->getPixel ());};
  void setBackground (ulong background)
    {XSetBackground (display, gc, background);};
  void setBackground (WdkColor& color)
    {XSetBackground (display, gc, color.getPixel ());};
  void setBackground (WdkColor* color)
    {XSetBackground (display, gc, color->getPixel ());};
  void setLineAttributes (ulong lineWidth, int lineStyle, 
			  int capStyle, int joinStyle)
    {XSetLineAttributes (display, gc,
			 lineWidth, lineStyle, capStyle, joinStyle);};
  void setDashes (int dashOffset, char dashList[], int n)
    {XSetDashes (display, gc, dashOffset, dashList, n);};
  void setFillStyle (int fillStyle)
    {XSetFillStyle (display, gc, fillStyle);};
  void setFillRule (int fillRule)
    {XSetFillRule (display, gc, fillRule);};
  void setTile (WdkPixmap& pixmap);
  void setTile (WdkPixmap* pixmap);
  void setStipple (WdkPixmap& pixmap);
  void setStipple (WdkPixmap* pixmap);
  void setTSOrigin (int tsX, int tsY)
    {XSetTSOrigin (display, gc, tsX, tsY);};
  void setClipMask (WdkPixmap& bitmap);
  void resetClipMask ()
    {XSetClipMask (display, gc, None);};
  void setClipRectangles (int x, int y, XRectangle rectangles[], 
			  int n, int order)
    {XSetClipRectangles (display, gc, x, y, rectangles, n, order);};
  void setClipOrigin (int x, int y)
    {XSetClipOrigin (display, gc, x, y);};
};

#endif // WdkGC_H
