/*
   WdkPixmap.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cassert>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "WdkWindow.h"
#include "WdkPixmap.h"

using namespace std;

WdkPixmap::WdkPixmap (WdkWindow* window, uint width_, uint height_):
  display (window->getDisplay ()),
  width (width_),height (height_),depth (window->getDepth ())
{
  assert (window != NULL);
  pixmap=XCreatePixmap (display, window->getWindow (), width,height,depth);
}

WdkPixmap::WdkPixmap (WdkWindow* window, uint width_, uint height_, uint depth_):
  display (window->getDisplay ()),
  width (width_),height (height_),depth (depth_)
{
  assert (window != NULL);
  pixmap=XCreatePixmap (display, window->getWindow (),width,height,depth);
}

WdkPixmap::WdkPixmap (WdkPixmap* pixmap_, uint width_, uint height_):
  display (pixmap_->display),
  width (width_), height (height_), depth (pixmap_->depth)
{
  assert (pixmap != 0);
  pixmap=XCreatePixmap (display, pixmap_->pixmap, width, height, depth);
}

WdkPixmap::WdkPixmap (WdkPixmap* pixmap_, uint width_, uint height_, uint depth_):
  display (pixmap_->display),
  width (width_), height (height_), depth (depth_)
{
  assert (pixmap != 0);
  pixmap=XCreatePixmap (display, pixmap_->pixmap, width, height, depth);
}



WdkPixmap::WdkPixmap (WdkWindow* window, char* filename):
  display (window->getDisplay ()),
  depth (1)
{
  //cerr << "WdkPixmap::WdkPixma=" << filename << endl;
  assert (window != 0);
  assert (filename != NULL);
  int dummyX;
  int dummyY;
  const int ret=XReadBitmapFile (display, window->getWindow (), filename,
				 &width, &height, &pixmap, &dummyX, &dummyY);
  if (ret != BitmapSuccess)
    {
      cerr << "WdkPixmap::WdkPixmap:can not create bitmap.";
      switch (ret)
	{
	case BitmapOpenFailed:
	  cerr << ":BitmapOpenFailed" << endl;
	  break;
	case BitmapFileInvalid:
	  cerr << "BitmapFileInvalid" << endl;
	  break;
	case BitmapNoMemory:
	  cerr << "BitmapNoMemory" << endl;
	  break;
	default:
	  //not reached
	  break;
	}
    }
}

WdkPixmap::WdkPixmap (WdkPixmap* pixmap_, char* filename):
  display (pixmap_->display),
  depth (1)
{
  assert (pixmap != 0);
  assert (filename != NULL);
  int dummyX;
  int dummyY;
  if (XReadBitmapFile (display, pixmap_->pixmap, filename,
		       &width, &height, &pixmap, &dummyX, &dummyY) != BitmapSuccess)
    cerr << "WdkPixmap::WdkPixmap:can not create bitmap." << endl;
}

WdkPixmap::WdkPixmap (WdkWindow* window, char* data, uint width_, uint height_):
  display (window->getDisplay ()),
  width (width_), height (height_), depth (1)
{
  assert (window != 0);
  assert (data != NULL);
  pixmap=XCreateBitmapFromData (display, window->getWindow (), data, width, height);
}

WdkPixmap::WdkPixmap (WdkPixmap* pixmap_, char* data, uint width_, uint height_):
  display (pixmap_->display),
  width (width_), height (height_), depth (1)
{
  assert (pixmap != 0);
  assert (data != NULL);
  pixmap=XCreateBitmapFromData (display, pixmap_->pixmap, data, width, height);
}

WdkPixmap::~WdkPixmap ()
{
  XFreePixmap (display, pixmap);
}
