/*
    WdkPixmap.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WdkPixmap_H
#define WdkPixmap_H
#include <X11/Xlib.h>

#include "util/mytype.h"
#include "WdkGC.h"
#include "WdkWindow.h"

class WdkPixmap
{
private:
  Display* display;
  Pixmap pixmap;
  uint width;
  uint height;
  int depth;
public:
  //for create child pixmap
  //some attribures copied form parent's
  WdkPixmap (WdkWindow* window, uint width_, uint height_);
  //for create bitmap
  WdkPixmap (WdkWindow* window, uint width_, uint height_,  uint depth_);
  WdkPixmap (WdkPixmap* window, uint width_, uint height_);
  //for create bitmap
  WdkPixmap (WdkPixmap* window, uint width_, uint height_, uint depth_);
  WdkPixmap (WdkWindow* window, char* filename);
  WdkPixmap (WdkPixmap* window, char* filename);
  WdkPixmap (WdkWindow* window, char* data, uint width, uint height);
  WdkPixmap (WdkPixmap* window, char* data, uint width, uint height);
  virtual ~WdkPixmap ();

  //////////////////////////////////////////////////////////acess method
  Display* getDisplay () const {return display;};
  uint getWidth () const {return width;};
  uint getHeight () const {return height;};
  int getDepth () const {return depth;};
  Pixmap getPixmap () const {return pixmap;};
  
  ///////////////////////////////////////////////////////////wrapper of Xlib
  //drawing
  void copyArea (WdkWindow* destWindow,  WdkGC* gc,
		 int srcX,  int srcY,  uint widht,  uint height,  int destX,
		 int destY)
    {XCopyArea (display, pixmap, destWindow->getWindow (), gc->getGC (),
		srcX, srcY, widht, height, destX, destY);};
  void copyArea (WdkWindow* destWindow,  WdkGC& gc,
		 int srcX,  int srcY,  uint widht,  uint height,  int destX,
		 int destY)
    {XCopyArea (display, pixmap, destWindow->getWindow (), gc.getGC (),
		srcX, srcY, widht, height, destX, destY);};
  void copyArea (WdkPixmap* destPixmap,  WdkGC* gc,
		 int srcX,  int srcY,  uint widht,  uint height,  int destX,
		 int destY)
    {XCopyArea (display, pixmap, destPixmap->getPixmap (), gc->getGC (),
		srcX, srcY, widht, height, destX, destY);};
  void copyArea (WdkPixmap* destPixmap,  WdkGC& gc,
		 int srcX,  int srcY,  uint widht,  uint height,  int destX,
		 int destY)
    {XCopyArea (display, pixmap, destPixmap->getPixmap (), gc.getGC (),
		srcX, srcY, widht, height, destX, destY);};
  void drawPoint (WdkGC& gc, int x, int y)
    {XDrawPoint (display, pixmap, gc.getGC (), x, y);};
  void drawPoint (WdkGC* gc, int x, int y)
    {assert (gc != NULL);XDrawPoint (display, pixmap, gc->getGC (), x, y);};
  void drawPoints (WdkGC& gc, XPoint* points, int npoints, int mode)
    {XDrawPoints (display, pixmap, gc.getGC (), points, npoints, mode);};
  void drawPoints (WdkGC* gc, XPoint* points, int npoints, int mode)
    {assert (gc != NULL);assert (points != NULL);
    XDrawPoints (display, pixmap, gc->getGC (), points, npoints, mode);};
  void drawLine (WdkGC& gc, int x1, int y1, int x2, int y2)
    {XDrawLine (display, pixmap, gc.getGC (), x1, y1, x2, y2);};
  void drawLine (WdkGC* gc, int x1, int y1, int x2, int y2)
    {XDrawLine (display, pixmap, gc->getGC (), x1, y1, x2, y2);};
  void drawLines (WdkGC& gc, XPoint* points, int npoints, int mode)
    {XDrawLines (display, pixmap, gc.getGC (), points, npoints, mode);};
  void drawSegments (WdkGC& gc, XSegment* segments, int nsegments)
    {XDrawSegments (display, pixmap, gc.getGC (), segments, nsegments);};
  void drawArc (WdkGC* gc,
		int x, int y, uint width, uint height, int angle1, int angle2)
    {XDrawArc (display, pixmap, gc->getGC (), x, y, width, height, angle1, angle2);};
  void drawArc (WdkGC& gc,
		int x, int y, uint width, uint height, int angle1, int angle2)
    {XDrawArc (display, pixmap, gc.getGC (), x, y, width, height, angle1, angle2);};
  void drawArcs (WdkGC* gc, XArc* arcs, int narcs)
    {XDrawArcs (display, pixmap, gc->getGC (), arcs, narcs);};
  void drawArcs (WdkGC& gc, XArc* arcs, int narcs)
    {XDrawArcs (display, pixmap, gc.getGC (), arcs, narcs);};
  void fillRectangle (WdkGC& gc, int x, int y, uint width, uint height)
    {XFillRectangle (display, pixmap, gc.getGC (), x, y, width, height);};
  void fillRectangle (WdkGC* gc, int x, int y, uint width, uint height)
    {XFillRectangle (display, pixmap, gc->getGC (), x, y, width, height);};
  void fillRectangles (WdkGC& gc, XRectangle* rectangles, int nrectangles)
    {XFillRectangles (display, pixmap, gc.getGC (), rectangles, nrectangles);};
  void fillPlygon (WdkGC& gc, XPoint* points, int npoints, int shape, int mode)
    {XFillPolygon (display, pixmap, gc.getGC (), points, npoints, shape, mode);};
  void fillArc (WdkGC* gc, 
		int x, int y, uint width, uint height, int angle1, int angle2)
    {XFillArc (display, pixmap, gc->getGC (), x, y, width, height, angle1, angle2);};
  void fillArc (WdkGC& gc, 
		int x, int y, uint width, uint height, int angle1, int angle2)
    {XFillArc (display, pixmap, gc.getGC (), x, y, width, height, angle1, angle2);};
  void fillArcs (WdkGC* gc, XArc* arcs, int narcs)
    {XFillArcs (display, pixmap, gc->getGC (), arcs, narcs);};
  void fillArcs (WdkGC& gc, XArc* arcs, int narcs)
    {XFillArcs (display, pixmap, gc.getGC (), arcs, narcs);};
  void drawString (XFontSet fontSet, WdkGC& gc, int x, int y, 
		   char* string, int numBytes)
    {XmbDrawString (display, pixmap, fontSet, gc.getGC (), x, y,
		    string, numBytes);};
  void drawString (XFontSet fontSet, WdkGC* gc, int x, int y, 
		   char* string, int numBytes)
    {XmbDrawString (display, pixmap, fontSet, gc->getGC (), x, y,
		    string, numBytes);};
  
  void drawImageString (XFontSet fontSet, WdkGC& gc, int x, int y, 
			char* string, int numBytes)
    {XmbDrawImageString (display, pixmap, fontSet, gc.getGC (), x, y,
			 string, numBytes);};
  void drawImageString (XFontSet fontSet, WdkGC* gc, int x, int y, 
			char* string, int numBytes)
    {XmbDrawImageString (display, pixmap, fontSet, gc->getGC (), x, y,
			 string, numBytes);};
};
#endif// WdkPixmap_H
