/*
    WdkWindow.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "WdkWindow.h"
#include "WdkPixmap.h"

using namespace std;

WdkWindow::WdkWindow (Display* display_, WdkWindow* parent_,
		      int x, int y, uint width, uint height,
		      uint borderWidth, 
		      int depth_,Visual* visual_):
  display (display_),
  depth (depth_), visual (visual_)
{
  assert (display!=NULL);
  if (parent_==NULL)
    parent=DefaultRootWindow (display);
  else
    parent=parent_->window;
  
  if (depth==0)
    depth=DefaultDepth (display, DefaultScreen (display));
  if (visual==NULL)
    visual=DefaultVisual (display, DefaultScreen (display));
  
  XSetWindowAttributes dummy;
#if 1
  window=XCreateWindow (display, parent, x, y, width, height, borderWidth, 
			depth, InputOutput, visual, 0, &dummy);
#else
  window=XCreateWindow (display, parent, x, y, width, height, 1, 
			depth, InputOutput, visual, 0, &dummy);
  //cerr << "x=" << x << ":y=" << y << ":width=" << width << ":height=" << height << endl;
#endif
}

WdkWindow::WdkWindow (WdkWindow* parent_, 
		      int x, int y, uint width, uint height):
  display (parent_->display), 
  depth (parent_->depth), visual (parent_->visual)
{
  assert (parent_!=0);
  parent=parent_->window;
  XSetWindowAttributes dummy;
#if 1
  window=XCreateWindow (display, parent, x, y, width, height, 0, 
			CopyFromParent, CopyFromParent, CopyFromParent, 
			0, &dummy);
#else
  window=XCreateWindow (display, parent, x, y, width, height, 1, 
			CopyFromParent, CopyFromParent, CopyFromParent, 
			0, &dummy);
  //cerr << "x=" << x << ":y=" << y << ":width=" << width << ":height=" << height << endl;
#endif
}


WdkWindow::~WdkWindow ()
{
  XDestroySubwindows (display, window);
  XDestroyWindow (display, window);
}


void WdkWindow::setBackgroundPixmap (WdkPixmap& backgroundPixmap)
{
  XSetWindowBackgroundPixmap (display, window, backgroundPixmap.getPixmap ());
};

void WdkWindow::setBackgroundPixmap (WdkPixmap* backgroundPixmap)
{
  XSetWindowBackgroundPixmap (display, window, backgroundPixmap->getPixmap ());
};

void WdkWindow::copyArea (WdkPixmap* srcPixmap, WdkGC* gc, 
			  int srcX, int srcY, uint width, uint height, 
			  int destX, int destY)
{
  XCopyArea (display, srcPixmap->getPixmap (), window, gc->getGC (), 
	     srcX, srcY, width, height, destX, destY);
}
