/*
    WdkWindow.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WdkWindow_H
#define WdkWindow_H
#include <cassert>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "util/mytype.h"
#include "WdkGC.h"

class WdkPixmap;

class WdkWindow
{
private:
  Display* display;
  Window parent;
  Window window;
  int depth;
  Visual* visual;
public:
  //for create toplevel window
  //default root window if parent_==0
  //defualt depth if depth_==0
  //defualt visual if visual_==NULL
  WdkWindow (Display* display_, WdkWindow* parent_,
	      int x, int y, uint width, uint height,
	      uint borderWidth, int depth_, Visual* visual_);
  //for create child window
  //some attribures copied form parent's
  WdkWindow (WdkWindow* parent_, int x, int y, uint width, uint height);
  virtual ~WdkWindow ();

  //////////////////////////////////////////////access method
  Display* getDisplay () const {return display;};
  Window getWindow () const {return window;};
  int getDepth () const {return depth;};
  Visual* getVisual () const {return visual;};
  
  ///////////////////////////////////////////////////////////wrapper of Xlib
  //window attributes
  void selectInput (ulong eventMask)
    {XSelectInput (display, window, eventMask);};
  void changeAttributes (ulong valueMask, XSetWindowAttributes* attributes)
    {XChangeWindowAttributes (display, window, valueMask, attributes);};
  //???$B$3$l$O;D$C$F$$$k$N$,NI$$$N$+$J$!!)(B
  void setBackground (ulong backgroundPixel)
    {XSetWindowBackground (display, window, backgroundPixel);};
  void setBackground (WdkColor& color)
    {XSetWindowBackground (display, window, color.getPixel ());};
  void setBackgroundPixmap (Pixmap backgroundPixmap)
    {XSetWindowBackgroundPixmap (display, window, backgroundPixmap);};
  void setBackgroundPixmap (WdkPixmap& backgroundPixmap);
  void setBackgroundPixmap (WdkPixmap* backgroundPixmap);
  //void setBorder (ulong borderPixel)
  //  {XSetWindowBorder (display, window, borderPixel);};
  void setBorder (WdkColor& color)
    {XSetWindowBorder (display, window, color.getPixel ());};
  void setBorderPixmap (Pixmap borderPixmap)
    {XSetWindowBorderPixmap (display, window, borderPixmap);};
  void setColormap (Colormap colormap)
    {XSetWindowColormap (display, window, colormap);};
  void map (){XMapWindow (display, window);};
  void unmap (){XUnmapWindow (display, window);};

  //WM Hints
  void storeName (char* windowName)
    {XStoreName (display, window, windowName);};
  //void setIconName (const char* iconName){XSetIconName (display, window, const_cast<char*>(iconName));};//??? $B$3$l$G(Bdiscards const$B$N(Bwarning$B$O=P$J$/$J$k$,(B
  void setIconName (const char* iconName)
    {XSetIconName (display, window, iconName);};//XXX
  void setClassHint (XClassHint* classHint)
    {XSetClassHint (display, window, classHint);};
  void getClassHint (XClassHint* classHint)
    {
      assert (classHint != NULL);
      XGetClassHint (display, window, classHint);
    };
  void setCommand (char** argv, int argc)
    {
      assert (argv != NULL);
      XSetCommand (display, window, argv, argc);
    };
  void setWMHints (XWMHints* wmhints)
    {
      assert (wmhints != NULL);
      XSetWMHints (display, window, wmhints);
    };
  XWMHints* getWMHints ()
    {return XGetWMHints (display, window);};
  void setWMNormalHints (XSizeHints* hints)
    {
      assert (hints != NULL);
      XSetWMNormalHints (display, window, hints);
    };
  void setWMSizeHints (XSizeHints* hints, Atom property)
    {
      assert (hints != NULL);
      XSetWMSizeHints (display, window, hints, property);
    };
  //drawing
  void drawPoint (WdkGC& gc, int x, int y)
    {XDrawPoint (display, window, gc.getGC (), x, y);};
  void drawPoints (WdkGC& gc, XPoint* points, int npoints, int mode)
    {XDrawPoints (display, window, gc.getGC (), points, npoints, mode);};
  void drawLine (WdkGC& gc, int x1, int y1, int x2, int y2)
    {XDrawLine (display, window, gc.getGC (), x1, y1, x2, y2);};
  void drawLine (WdkGC* gc, int x1, int y1, int x2, int y2)
    {XDrawLine (display, window, gc->getGC (), x1, y1, x2, y2);};
  void drawLines (WdkGC& gc, XPoint* points, int npoints, int mode)
    {XDrawLines (display, window, gc.getGC (), points, npoints, mode);};
  void drawLines (WdkGC* gc, XPoint* points, int npoints, int mode)
    {XDrawLines (display, window, gc->getGC (), points, npoints, mode);};
  void drawSegments (WdkGC& gc, XSegment* segments, int nsegments)
    {XDrawSegments (display, window, gc.getGC (), segments, nsegments);};
  void drawArc (WdkGC& gc, 
		int x, int y, uint width, uint height, int angle1, int angle2)
    {XDrawArc (display, window, gc.getGC (),
	       x, y, width, height, angle1, angle2);};
  void drawArcs (WdkGC& gc, XArc* arcs, int narcs)
    {XDrawArcs (display, window, gc.getGC (), arcs, narcs);};
  void fillRectangle (WdkGC& gc, int x, int y, uint width, uint height)
    {XFillRectangle (display, window, gc.getGC (), x, y, width, height);};
  void fillRectangle (WdkGC* gc, int x, int y, uint width, uint height)
    {XFillRectangle (display, window, gc->getGC (), x, y, width, height);};
  void fillRectangles (WdkGC& gc, XRectangle* rectangles, int nrectangles)
    {XFillRectangles (display, window, gc.getGC (), rectangles, nrectangles);};
  void fillPolygon (WdkGC* gc, XPoint* points, int npoints,
		    int shape, int mode)
    {XFillPolygon (display, window, gc->getGC (),
		   points, npoints, shape, mode);};
  void fillPolygon (WdkGC& gc,
		    XPoint* points, int npoints, int shape, int mode)
    {XFillPolygon (display, window, gc.getGC (),
		   points, npoints, shape, mode);};
  void fillArc (WdkGC& gc, 
		int x, int y, uint width, uint height, int angle1, int angle2)
    {XFillArc (display, window, gc.getGC (),
	       x, y, width, height, angle1, angle2);};
  void fillArcs (WdkGC& gc, XArc* arcs, int narcs)
    {XFillArcs (display, window, gc.getGC (), arcs, narcs);};
  void drawString (XFontSet fontSet, WdkGC& gc, int x, int y, 
		   char* string, int numBytes)
    {XmbDrawString (display, window, fontSet, gc.getGC (),
		    x, y, string, numBytes);};
  void drawString (XFontSet fontSet, WdkGC* gc, int x, int y, 
		   char* string, int numBytes)
    {XmbDrawString (display, window, fontSet, gc->getGC (),
		    x, y, string, numBytes);};
  void drawImageString (XFontSet fontSet, WdkGC& gc, int x, int y, 
			char* string, int numBytes)
    {XmbDrawImageString (display, window, fontSet, gc.getGC (),
			 x, y, string, numBytes);};
  void drawImageString (XFontSet fontSet, WdkGC* gc, int x, int y, 
			char* string, int numBytes)
    {XmbDrawImageString (display, window, fontSet, gc->getGC (),
			 x, y, string, numBytes);};
  void copyArea (WdkWindow* destWindow, WdkGC* gc, 
		 int srcX, int srcY, uint width, uint height,
		 int destX, int destY)
    {XCopyArea (display, window, destWindow->window, gc->getGC (), 
		srcX, srcY, width, height, destX, destY);};
  void moveArea (WdkGC* gc, 
		 int srcX, int srcY, uint width, uint height,
		 int destX, int destY)
    {XCopyArea (display, window, window, gc->getGC (), 
		srcX, srcY, width, height, destX, destY);};
  void copyArea (WdkPixmap* srcPixmap, WdkGC* gc, 
		 int srcX, int srcY, uint width, uint height,
		 int destX, int destY);
  //clear
  void clear () {XClearWindow (display, window);};
  void clearArea (int x, int y, uint width, uint height, Bool exposure)
    {XClearArea (display, window, x, y, width, height, exposure);};

  //configuration
  void move (int x, int y)
    {XMoveWindow (display, window, x, y);};
  void resize (uint width, uint height)
    {XResizeWindow (display, window, width, height);};
  void moveResize (int x, int y, uint width, uint height)
    {XMoveResizeWindow (display, window, x, y, width, height);};
  void reparent (WdkWindow& parent_, int x, int y)
    {XReparentWindow (display, window, parent_.getWindow (), x, y);};
  void reparent (WdkWindow* parent_, int x, int y)
    {XReparentWindow (display, window, parent_->getWindow (), x, y);};
};
#endif// WdkWindow_H
