#ifndef WdkXImage_H
#define WdkXImage_H

#include "WdkWindow.h"
#include "WdkPixmap.h"

class WdkXImage
{
private:
  Display* const display;
  XImage* const ximage;
  //ximage->data$B$r(Bnew$B$G3d$jEv$F$?$+(B?(true)
  //$B0?$O(Bxlib$B$,<+F0E*$K3d$jEv$F$?$+(B?(false)
  const bool isDataAllocated;

public:
  WdkXImage (Display* display__, Visual* visual,
	     unsigned int depth, int format, int offset,
	     unsigned int width, unsigned int height):
    display (display__),
    ximage (XCreateImage (display, visual, depth, format, offset, None,
			  width, height, 8, 0)),
    isDataAllocated (true)
    {
      const size_t length = ximage->bytes_per_line * width;
      ximage->data = new char[length];
      memset (ximage->data, 0, length);
    };
  WdkXImage (WdkXImage* parentXImage,
	     int x, int y,
	     unsigned int subimageWidth, unsigned int subimageHeight):
    display (parentXImage->getDisplay ()),
    ximage (XSubImage (parentXImage->getXImage (),
		       x, y, subimageWidth, subimageHeight)),
    isDataAllocated (false)
    {};
  WdkXImage (WdkWindow* window,
	     int x, int y, unsigned int width, unsigned int height,
	     unsigned long planeMask, int format):
    display (window->getDisplay ()),
    ximage (XGetImage (display, window->getWindow (),
		       x, y, width, height, planeMask, format)),
    isDataAllocated (false)
    {};
  WdkXImage (WdkPixmap* pixmap,
	     int x, int y, unsigned int width, unsigned int height,
	     unsigned long planeMask, int format):
    display (pixmap->getDisplay ()),
    ximage (XGetImage (display, pixmap->getPixmap (),
		       x, y, width, height, planeMask, format)),
    isDataAllocated (false)
    {;};
  ~WdkXImage ()
    {
      if (isDataAllocated)
	{
	  delete[] ximage->data;
	  ximage->data = NULL;
	}
      XDestroyImage (ximage);
    };

  //acess method
  Display* getDisplay () const {return display;};
  XImage* getXImage () const {return ximage;};
  
  //wrapper for Xlib
  unsigned long getPixel (int x, int y) const
    {return XGetPixel (ximage, x, y);};
  void putPixel (int x, int y, unsigned long pixel)
    {XPutPixel (ximage, x, y, pixel);};
  void addPixel (long value) {XAddPixel (ximage, value);};
  
  void putImage (WdkPixmap* pixmap, WdkGC* gc,
		 int srcX, int srcY, int destX, int destY,
		 unsigned int width, unsigned int height)
    {XPutImage (display, pixmap->getPixmap (), gc->getGC (), ximage,
		srcX, srcY, destX, destY, width, height);};
  void putImage (WdkPixmap* pixmap, WdkGC& gc,
		 int srcX, int srcY, int destX, int destY,
		 unsigned int width, unsigned int height)
    {XPutImage (display, pixmap->getPixmap (), gc.getGC (), ximage,
		srcX, srcY, destX, destY, width, height);};
  void putImage (WdkPixmap& pixmap, WdkGC& gc,
		 int srcX, int srcY, int destX, int destY,
		 unsigned int width, unsigned int height)
    {XPutImage (display, pixmap.getPixmap (), gc.getGC (), ximage,
		srcX, srcY, destX, destY, width, height);};
  void putImage (WdkPixmap& pixmap, WdkGC* gc,
		 int srcX, int srcY, int destX, int destY,
		 unsigned int width, unsigned int height)
    {XPutImage (display, pixmap.getPixmap (), gc->getGC (), ximage,
		srcX, srcY, destX, destY, width, height);};
  void putImage (WdkWindow* window, WdkGC* gc,
		 int srcX, int srcY, int destX, int destY,
		 unsigned int width, unsigned int height)
    {XPutImage (display, window->getWindow (), gc->getGC (), ximage,
		srcX, srcY, destX, destY, width, height);};
  void getSubImage (WdkWindow* window,
		    int x, int y, unsigned int width, unsigned int height,
		    unsigned long planeMask, int format, int destX, int destY)
    {
      XImage* ret = XGetSubImage (display, window->getWindow (),
				  x, y, width, height,
				  planeMask, format, ximage, destX, destY);
      assert (ret == ximage);
    };
  void getSubImage (WdkPixmap* pixmap,
		    int x, int y, unsigned int width, unsigned int height,
		    unsigned long planeMask, int format, int destX, int destY)
    {
      XImage* ret = XGetSubImage (display, pixmap->getPixmap (),
				  x, y, width, height,
				  planeMask, format, ximage, destX, destY);
      assert (ret == ximage);
    };
};
#endif // WdkXImage_H
