/*
    WtkClickableLabel.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>
#include <cassert>

#include "WtkClickableLabel.h"

using namespace std;

const char* const WtkClickableLabel::defaultResource="";

WtkClickableLabel::WtkClickableLabel (Display* display, const char* className_,
				      const char* name_,
				      int x_,int y_,uint width_,uint height_,
				      int depth,Visual* visual,
				      Colormap colormap_,
				      char* resourceFileName):
  WtkLabel (display,className_,name_,x_,y_,width_,height_,
	    depth,visual,colormap_,resourceFileName)
{
  initialize ();
}

WtkClickableLabel::WtkClickableLabel (Display* display, const char* className_,
				      const char* name_,
				      int x_, int y_,
				      uint width_, uint height_,
				      int depth, Visual* visual,
				      Colormap colormap_,
				      XrmDatabase displayRdb_):
  WtkLabel (display,className_,name_,x_,y_,width_,height_,
	    depth,visual,colormap_,displayRdb_)
{
  initialize ();
}

WtkClickableLabel::WtkClickableLabel (WtkWidget* parent,
				      const char* className_,
				      const char* name_,
				      int x_, int y_,
				      uint width_, uint height_):
  WtkLabel (parent,className_,name_,x_,y_,width_,height_)
{
  initialize ();
}

WtkClickableLabel::~WtkClickableLabel ()
{
  
}

void WtkClickableLabel::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  window->selectInput (ExposureMask | ButtonPressMask | ButtonReleaseMask);
  getResource ();
  setResource ();
}


void WtkClickableLabel::buttonPressed (XEvent* event)
{
  lockEventState ();
  eventState.isButtonPressed[event->xbutton.button]=true;
  clicked (event);
  unlockEventState ();
}
