/*
    WtkEventHandloer.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cassert>
#include <pthread.h>
#include <X11/Xatom.h>

#include "WtkEventHandler.h"
#include "util/PThreadAttribute.h"
#include "util/schedYield.h"

using namespace std;

WtkEventHandler::WtkEventHandler (Display* display_):
  display (display_), subWindows (),
  threadLock (), eventHappend (), isThreadCreated (false)
{
  //do nothing
}

WtkEventHandler::~WtkEventHandler ()
{
  cancelEventLoopThread ();
}

Bool WtkEventHandler::eventPredicate
(Display* /*display*/, XEvent* event, char* data)
{
  //trick!!!!
  //Does this work well?
  map<Window, WtkWidget*, less>* subWindows_ = reinterpret_cast<map<Window, WtkWidget*, less>*> (data);
  
  return (subWindows_->find (event->xany.window)!=subWindows_->end ());
}

void WtkEventHandler::nextEvent (XEvent* retEvent, WtkWidget* retWtkWidget)
{
  XIfEvent (display, retEvent, eventPredicate, reinterpret_cast <char*> (&subWindows));//this is a trick
  retWtkWidget=subWindows[retEvent->xany.window];
}

Bool WtkEventHandler::checkEvent (XEvent* retEvent, WtkWidget* retWtkWidget)
{
  if (XCheckIfEvent (display, retEvent, eventPredicate,
		     (char*)&subWindows/*this is a trick*/))
    {
      retWtkWidget=subWindows[retEvent->xany.window];
      return True;
    }
  else
    {
      retEvent=NULL;
      retWtkWidget=NULL;
      return False;
    }
}

void* WtkEventHandler::startEventLoopThread (WtkEventHandler* me)
{
  //DEBUG_MESSAGE ("WtkEventHandler::startEvetnLoopThread");
  me->eventLoop ();
  return NULL;
}


void WtkEventHandler::createEventLoopThread ()
{
  threadLock.lock ();
  if (isThreadCreated==false)
    {
      PThreadAttribute threadAttribute;
      pthread_create (&thread,  &threadAttribute.getPThreadAttribute (),
		      WtkEventHandler::startEventLoopThread, this);
    }
  isThreadCreated=true;
  threadLock.unlock ();
}

#define WTK_EVENT_HANDLER_LOOP_END_ATOM XA_ATOM //XA_ATOM$B$H$$$&;v$K0UL#$O$J$$(B($B%"%H%`$J$i2?$G$bNI$$!#(B)
#define WTK_EVENT_HANDLER_LOOP_CANCEL_MESSAGE "WtkEventLoopCancel"

void WtkEventHandler::eventLoop ()
{
  //DEBUG_MESSAGE ("now in event Loop");
  assert (display!=NULL);

#ifdef LOCALLY_SCHEDLUED_THREAD
#if 0
# ifdef __FreeBSD__
  //!!!NOTE!!! pthread_getprio, pthread_setprio are not based on pthread1003.1c
  int prio = pthread_getprio (pthread_self ());
  if (errno == 0)
    {
      prio -= 1;
      const int ret = pthread_setprio (pthread_self (), prio);
      if (ret != 0)
	{
	  //error
	  DEBUG_MESSAGE ("WtkEventHandler::eventLoop can not set schedParam");
	}
    }
  else
    {//error
      DEBUG_MESSAGE ("WtkEventHandler::eventLoop can not get schedParam");
    }
# else //__FreeBSD__
  //XXX (not implemented)
# endif
#endif
#endif

  for (;;)
    {
      XEvent event;

#ifdef LOCALLY_SCHEDULED_THREAD
      if (!XCheckIfEvent (display, &event, &WtkEventHandler::eventPredicate,
			  reinterpret_cast<char*> (&subWindows)) ||
	  subWindows.find (event.xany.window) == subWindows.end ())//???
	{//no events are found.
	  //cerr << "sleeping" << endl;
	  //sleep (0);
	  schedYield ();
	  continue;
	}
#else
      XIfEvent (display, &event,
		&WtkEventHandler::eventPredicate,
		reinterpret_cast<char*> (&subWindows));
#endif

      //cerr << "XIfEvent done." << endl;
      assert (subWindows.find (event.xany.window) != subWindows.end ());
      //cerr << "subWindows" << endl;
      WtkWidget* widget=subWindows[event.xany.window];
      //cerr << "event switching" << endl;
      switch (event.type)
	{
	case ButtonPress:
	  widget->buttonPressed (&event);
	  break;
	case ButtonRelease:
	  widget->buttonReleased (&event);
	  break;
	case MotionNotify:
#if 0
	  while (XCheckTypedWindowEvent (display,
					 widget->getWindow ()->getWindow (),
					 ButtonMotionMask, &event))
	    /*do nothing*/;
#else
	  while (XCheckTypedEvent (display, ButtonMotionMask, &event))
	    /*do nothing*/;
#endif
	  widget->pointerMotion (&event);
	  break;
	case EnterNotify:
	  if (!XCheckTypedWindowEvent (display,
				       widget->getWindow ()->getWindow (),
				       LeaveWindowMask, &event))
	    widget->enterNotify (&event);
	  break;
	case LeaveNotify:
	  widget->leaveNotify (&event);
	  break;
	case KeyPress:
	  widget->keyPressed (&event);
	  break;
	case KeyRelease:
	  widget->keyReleased (&event);
	  break;
	case Expose:
	  //???
	  {
	    int left=event.xexpose.x;
	    int top=event.xexpose.y;
	    int right=left+event.xexpose.width;
	    int buttom=top+event.xexpose.height;
	    while (XCheckTypedWindowEvent (display,
					   widget->getWindow ()->getWindow (),
					   ExposureMask, &event))
	      {
		left=min (left, event.xexpose.x);
		top=min (top, event.xexpose.y);
		right=max (right, event.xexpose.x+event.xexpose.width);
		buttom=max (buttom, event.xexpose.y+event.xexpose.height);
	      }
	    event.xexpose.x=left;
	    event.xexpose.y=top;
	    event.xexpose.width=right-left;
	    event.xexpose.height=buttom-top;
	  }
	widget->exposed (&event);
	  break;
	case MapNotify:
	  break;
	case ClientMessage:
	  //check the client message is valid
	  if (event.xclient.message_type==WTK_EVENT_HANDLER_LOOP_END_ATOM &&
	      event.xclient.format==8 &&
	      strcmp (event.xclient.data.b,
		      WTK_EVENT_HANDLER_LOOP_CANCEL_MESSAGE)==0)
	    pthread_exit (NULL);
	  else
	    cerr << "WtkEventHandler::eventLoop:unknown XClientMessageEvent recieved.";
	  break;
	default:
	  //do nothing
	  break;
	}
      eventHappend.broadcast ();
    }
}

void WtkEventHandler::cancelEventLoopThread ()
{
  //DEBUG_MESSAGE ("WtkEventHandler::cancelEventLoopThread");
  threadLock.lock ();
  if (isThreadCreated)
    {
      XEvent clientMessage;
      clientMessage.xany.type=ClientMessage;
      clientMessage.xany.display=display;
      clientMessage.xany.window=(*(subWindows.begin ())).first;
      clientMessage.xclient.message_type=WTK_EVENT_HANDLER_LOOP_END_ATOM;
      clientMessage.xclient.format=8;
      strcpy (clientMessage.xclient.data.b,
	      WTK_EVENT_HANDLER_LOOP_CANCEL_MESSAGE);
      DEBUG_MESSAGE ( "WtkEventHandler::cancelEventLoopThread:now sending ClientMessage event.");
      XSendEvent (display, clientMessage.xany.window, True, ExposureMask, &clientMessage);
      DEBUG_MESSAGE ( "WtkEventHandler::cancelEventLoopThread now flushing");
      XFlush (display);
      DEBUG_MESSAGE ( "WtkEvenHandler::cancelEventLoopThread now joining");
      if (pthread_join (thread, NULL)!=0)
	cerr << "WtkEventHandler:cancelEventLoopThread:pthread_join returned non-zero status." << endl;
      DEBUG_MESSAGE ( "WtkEventHandler::cancelEventLoopThread:joined");
      isThreadCreated=false;
    }
  threadLock.unlock ();
}

#undef WTK_EVENT_HANDLER_LOOP_END_ATOM
#undef WTK_EVENT_HANDLER_LOOP_CANCEL_MESSAGE
