/*
    WtkEventHandler.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef WtkEventHandler_H
#define WtkEventHandler_H
#include <map>

#include "WtkWidget.h"
#include "util/CondVar.h"

using namespace std;

class WtkEventHandler
{
private:
  Display* display;
  class less
  {
  public:
    bool operator () (Window x,Window y) const {return x<y;};
  };
  map<Window,WtkWidget*,less> subWindows;
  static Bool eventPredicate (Display* display,XEvent* event,char* data);

  Mutex threadLock;
  CondVar eventHappend;
  
  bool isThreadCreated;
  pthread_t thread;//$B$3$l$O$^$:$$(B?
  static void* startEventLoopThread (WtkEventHandler* me);//for thread creating
  void eventLoop ();//$B<B:]$N%$%Y%s%HBT$A%k!<%W(B
public:
  WtkEventHandler (Display* display_);
  ~WtkEventHandler ();
  void addWidget (WtkWidget* widget)
    {subWindows[widget->getWindow ()->getWindow ()]=widget;};
  void addWidget (WtkWidget* widget,WdkWindow* wdkWindow)
    {subWindows[wdkWindow->getWindow ()]=widget;};
  void removeWidget (WtkWidget* widget)
    {subWindows.erase (widget->getWindow ()->getWindow ());};
  bool isIn (WtkWidget* widget)
    {return subWindows.find (widget->getWindow ()->getWindow ())!=subWindows.end ();};

  void nextEvent (XEvent* retEvent,WtkWidget* retWtkWidget);//blocking
  Bool checkEvent (XEvent* retEvent,WtkWidget* retWtkWidget);//non-blocking

  void createEventLoopThread ();//$B$3$l$G%$%Y%s%H$N4F;k%k!<%W(Bthread$B$r@8@.$9$k!#(B
  void cancelEventLoopThread ();//event loop$B$N(Bthread$B$r=*N;$5$;$k!#(B
};
#endif // WtkEventHandler_H
