/*
    WtkLabel.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cassert>
#include <cstdio>

#include "WtkLabel.h"

using namespace std;

const char* const WtkLabel::defaultResource=
"*Justify: CENTER"
"*Text: ";


WtkLabel::WtkLabel (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName),
  label (NULL),justify (JUSTIFY_CENTER),labelX (0),labelY (0)
{
  logical.x=0;
  logical.y=0;
  logical.width=1;
  logical.height=1;
  initialize ();
}

WtkLabel::WtkLabel (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_),
  label (NULL),justify (JUSTIFY_CENTER)
{
  initialize ();
}

WtkLabel::WtkLabel (WtkWidget* parent,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_),
  label (NULL),justify (JUSTIFY_CENTER)
{
  initialize ();
}

WtkLabel::~WtkLabel ()
{
  delete[] label;
}

void WtkLabel::initialize ()
{
  logical.x=0;
  logical.y=0;
  logical.width=0;
  logical.height=0;

  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();
}

Justify WtkLabel::getResouceJustify (char* resourceName,char* resourceClass)
{
  const char* justify=getResource (resourceName,resourceClass);
  if (strcmp (justify,"LEFT")==0)
    return JUSTIFY_LEFT;
  else if (strcmp (justify,"RIGHT")==0)
    return JUSTIFY_RIGHT;
  else
    return JUSTIFY_CENTER;
}


void WtkLabel::getResource ()
{
  const char* tmpLabel=getResource ("text","Text");
  if (tmpLabel!=NULL)
    setLabel (tmpLabel);

  justify=getResouceJustify ("justify","Justify");
}

void WtkLabel::setLabel (const char* toBeSet)
{
  lockData ();
  if (label!=NULL && strcmp (label,toBeSet)==0)
    {
      unlockData ();
      return;//no change
    }
  
  delete[] label;
  label=new char[strlen (toBeSet)+1];
  memcpy (label,toBeSet,strlen (toBeSet)+1);
  overWrite ();
  unlockData ();

  //decorate ();//??? need this?
}

void WtkLabel::setLabel (const int value)
{
#define MAX_CHAR 256
  char buff[MAX_CHAR];
  sprintf (buff,"%d",value);
  setLabel (buff);
#undef MAX_CHAR
}


void WtkLabel::setJustify (Justify justify_)
{
  lockData ();
  justify=justify_;
  overWrite ();
  unlockData ();
}

void WtkLabel::calculateLocate ()
{
  if (label==NULL) return;
  
  XRectangle ink;
  XmbTextExtents (style.fontSet,label,strlen (label),&ink,&logical);
  labelY=height-logical.height-style.bottomOffset;
  switch (justify)
    {
    case JUSTIFY_RIGHT:
      labelX=width-logical.width-style.rightOffset;
      break;
    case JUSTIFY_CENTER:
      labelX=(width-logical.width)/2;
      break;
    case JUSTIFY_LEFT:
      labelX=style.leftOffset+2;
      break;
    }
}

void WtkLabel::overWrite ()
{
  const int tmpX=labelX;
  const int tmpY=labelY;
  const uint tmpWidth=logical.width;
  const uint tmpHeight=logical.height;
  calculateLocate ();
  //???need more clipping???
  gc->setForeground (style.text);
  window->clearArea (tmpX,tmpY,tmpWidth,tmpHeight,False);
  window->drawString (style.fontSet,gc,labelX,labelY-logical.y,
		      label,strlen (label));
}


void WtkLabel::exposed (XEvent* /*event*/)
{
  //cerr << "label:exposed" << endl;
  if (label!=NULL)
    {
      lockData ();
      gc->setForeground (style.text);
      window->drawString (style.fontSet,gc,labelX,labelY-logical.y,
			  label,strlen (label));
      unlockData ();
    }
  decorate ();
}
