/*
    WtkLevel.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "WtkLevel.h"

const char* const WtkLevel::defaultResource="\n";

WtkLevel::WtkLevel (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName),
  level (0.0),levelWidth (width-style.topOffset-style.bottomOffset)
{
  initialize ();
}

WtkLevel::WtkLevel (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_),
  level (0.0),levelWidth (width-style.topOffset-style.bottomOffset)
{
  initialize ();
}

WtkLevel::WtkLevel (WtkWidget* parent,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_),
  level (0.0),levelWidth (width-style.topOffset-style.bottomOffset)
{
  initialize ();
}

WtkLevel::~WtkLevel ()
{
  //do nothing
}

void WtkLevel::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
}

void WtkLevel::setLevel (double level_)
{
  if (level_<0) level_=0;
  if (level_>1) level_=1;
  const uint newLevelWidth=static_cast<uint>(levelWidth*level_+0.5);
  lockData ();
  const uint oldLevelWidth=static_cast<uint>(levelWidth*level+0.5);
  if (oldLevelWidth>newLevelWidth)
    window->clearArea
      (newLevelWidth+style.leftOffset,style.topOffset,
       oldLevelWidth-newLevelWidth,height-style.topOffset-style.bottomOffset,
       False);
  else if (oldLevelWidth<newLevelWidth)
    {
      gc->setForeground (style.fg);
      window->fillRectangle
	(gc,
	 oldLevelWidth+style.leftOffset,style.topOffset,
	 newLevelWidth-oldLevelWidth,height-style.topOffset-style.bottomOffset);
    }

  level=level_;
  unlockData ();
}

void WtkLevel::exposed (XEvent* /*event*/)
{
  lockData ();
  gc->setForeground (style.fg);
  const uint nowLevelWidth=static_cast<uint>(levelWidth*level+0.5);
  window->fillRectangle
    (gc,
     style.leftOffset,style.topOffset,
     nowLevelWidth,height-style.topOffset-style.bottomOffset);
  unlockData ();
  decorate ();
}
