/*
    WtkMaxedLevel.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#include <cstdio>
#include <cassert>

#include "WtkMaxedLevel.h"

using namespace std;

const char* const WtkMaxedLevel::defaultResource="";

WtkMaxedLevel::WtkMaxedLevel
(Display* display,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_,
 int depth,Visual* visual,Colormap colormap_,
 char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
             depth,visual,colormap_,resourceFileName),
  level (0.0),maxLevel (0.0),
  levelWidth (width-style.topOffset-style.bottomOffset)
{
  initialize ();
}

WtkMaxedLevel::WtkMaxedLevel
(Display* display,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_,
 int depth,Visual* visual,Colormap colormap_,
 XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
             depth,visual,colormap_,displayRdb_),
  level (0.0), maxLevel (0.0),
  levelWidth (width-style.topOffset-style.bottomOffset)
{
  initialize ();
}

WtkMaxedLevel::WtkMaxedLevel
(WtkWidget* parent,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_),
  level (0.0),maxLevel (0.0),
  levelWidth (width-style.topOffset-style.bottomOffset)
{
  initialize ();
}

WtkMaxedLevel::~WtkMaxedLevel ()
{
  //do nothing
}

void WtkMaxedLevel::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
}

void WtkMaxedLevel::setLevel (double level_,double maxLevel_)
{
  if (level_<0) level_=0;
  if (level_>1) level_=1;
  if (maxLevel_<level_) maxLevel_=level_;
  if (maxLevel_>1) maxLevel_=1;
  const uint newLevelWidth=static_cast<uint> (levelWidth*level_+0.5);
  const uint newMaxLevelWidth=static_cast<uint> (levelWidth*maxLevel_+0.5);
  lockData ();
  gc->setForeground (style.fg);
  //level
  const uint oldLevelWidth=static_cast<uint> (levelWidth*level+0.5);
  if (oldLevelWidth>newLevelWidth)
    window->clearArea
      (newLevelWidth+style.leftOffset,style.topOffset,
       oldLevelWidth-newLevelWidth,height-style.topOffset-style.bottomOffset,
       False);
  else if (oldLevelWidth<newLevelWidth)
    window->fillRectangle
      (gc,oldLevelWidth+style.leftOffset,style.topOffset,
       newLevelWidth-oldLevelWidth,height-style.topOffset-style.bottomOffset);
  //maxed bar
  const uint oldMaxLevelWidth=static_cast<uint> (levelWidth*maxLevel+0.5);
  if (oldMaxLevelWidth>newLevelWidth && oldMaxLevelWidth!=newMaxLevelWidth)
    window->clearArea
      (oldMaxLevelWidth+style.leftOffset,style.topOffset,
       1,height-style.topOffset-style.bottomOffset,False);
  window->drawLine
    (gc,
     newMaxLevelWidth+style.leftOffset,style.topOffset,
     newMaxLevelWidth+style.leftOffset,height-style.bottomOffset);
  
  level=level_;
  maxLevel=maxLevel_;
  unlockData ();
}

void WtkMaxedLevel::exposed (XEvent* /*event*/)
{
  lockData ();
  gc->setForeground (style.fg);
  const uint nowLevelWidth=static_cast<uint> (levelWidth*level+0.5);
  window->fillRectangle
    (gc,style.leftOffset,style.topOffset,
     nowLevelWidth,height-style.topOffset-style.bottomOffset);
  const uint maxLevelWidth=static_cast<uint> (levelWidth*maxLevel+0.5);
  window->drawLine
    (gc,
     maxLevelWidth+style.leftOffset,style.topOffset,
     maxLevelWidth+style.leftOffset,height-style.topOffset-style.bottomOffset);
  unlockData ();
  decorate ();
}
