/*
    WtkMessage.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "WtkMessage.h"

const char* const WtkMessage::defaultResource="";

WtkMessage::WtkMessage (Display* display,const char* className_,const char* name_,
			int x_,int y_,uint width_,uint height_,
			int depth,Visual* visual,Colormap colormap_,
			char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName),
  maxLines (0),readpos (0),leftLines (0)
{
  initialize ();
}

WtkMessage::WtkMessage (Display* display,const char* className_,const char* name_,
			int x_,int y_,uint width_,uint height_,
			int depth,Visual* visual,Colormap colormap_,
			XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_),
  maxLines (0),readpos (0),leftLines (0)
{
  initialize ();
}

WtkMessage::WtkMessage (WtkWidget* parent,const char* className_,const char* name_,
			int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_),
  maxLines (0),readpos (0),leftLines (0)
{
  initialize ();
}

WtkMessage::~WtkMessage ()
{
  //do nothing
}

void WtkMessage::initialize ()
{
  memset (messageBuffer,0,sizeof (messageBuffer));
  XrmCombineDatabase(XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();
}

void WtkMessage::getResource ()
{
  fontSetExtents=XExtentsOfFontSet (style.fontSet);
  maxLines=(height-style.topOffset-style.bottomOffset)/(fontSetExtents->max_logical_extent.height);
  if (maxLines==0) maxLines=1;
  top=style.topOffset-fontSetExtents->max_logical_extent.y;
  charHeight=fontSetExtents->max_logical_extent.height;
}

void WtkMessage::coutMessage (const char* message)
{
  lockData ();
  if (++leftLines>maxLines)
    {//$B0lHV2<$^$GC#$7$?!#(B
      leftLines=maxLines;
      messageBuffer[readpos][0]='\0';
      ++readpos%=MAX_MESSAGE_NUMBER;
      //$B0lJ8;z>e$K(Bscroll
      window->moveArea
	(gc,
	 style.leftOffset,style.topOffset+charHeight,
	 width-style.leftOffset-style.rightOffset,height-style.topOffset-style.bottomOffset-charHeight,
	 style.leftOffset,style.topOffset);
    }

  strcpy (messageBuffer[(readpos+leftLines-1)%MAX_MESSAGE_NUMBER],message);
  gc->setForeground (style.fg);
  //$B:G2<9T$K?7$?$J%a%C%;!<%8$rI=<(!#(B
  window->clearArea (style.leftOffset,style.topOffset+charHeight*(leftLines-1),
		     width-style.leftOffset-style.rightOffset,charHeight,False);
#define LEFT_PADDING 3
  window->drawString (style.fontSet,gc,
		      LEFT_PADDING+style.leftOffset,top+charHeight*(leftLines-1),
		      messageBuffer[(readpos+leftLines-1)%MAX_MESSAGE_NUMBER],
		      strlen (messageBuffer[(readpos+leftLines-1)%MAX_MESSAGE_NUMBER]));
  unlockData ();
}


void WtkMessage::exposed (XEvent* /*event*/)
{
  lockData ();
  gc->setForeground (style.fg);
  for (int i=0;i<leftLines;i++)
    window->drawString (style.fontSet,gc,
			LEFT_PADDING+style.leftOffset,top+charHeight*(i),
			messageBuffer[(readpos+i)%MAX_MESSAGE_NUMBER],
			strlen (messageBuffer[(readpos+i)%MAX_MESSAGE_NUMBER]));
  unlockData ();
  decorate ();
}
#undef LEFT_PADDING
