/*
    WtkMeter.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "WtkMeter.h"

const char* const WtkMeter::defaultResource="\n";


WtkMeter::WtkMeter (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName),
  meter (0.0),realWidth (width-style.leftOffset-style.rightOffset),
  oldLeft (static_cast<uint>((realWidth+1.0)/2)+style.leftOffset),
  oldRight (static_cast<uint>((realWidth+1.0)/2)+style.leftOffset)
{
  initialize ();
}

WtkMeter::WtkMeter (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_),
  meter (0.0),realWidth (width-style.leftOffset-style.rightOffset),
  oldLeft (static_cast<uint>((realWidth+1.0)/2)+style.leftOffset),
  oldRight (static_cast<uint>((realWidth+1.0)/2)+style.leftOffset)
{
  initialize ();
}

WtkMeter::WtkMeter (WtkWidget* parent,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_),
  meter (0.0),realWidth (width-style.leftOffset-style.rightOffset),
  oldLeft (static_cast<uint>((realWidth+1.0)/2)+style.leftOffset),
  oldRight (static_cast<uint>((realWidth+1.0)/2)+style.leftOffset)
{
  initialize ();
}

WtkMeter::~WtkMeter ()
{
  //do nothing
}

void WtkMeter::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  realMeter=static_cast<uint>((realWidth+1.0)/2)+style.leftOffset;//$B=i4|$K$OCf1{!#(B
}

void WtkMeter::drawMeter (uint left,uint right)
{
#if 0
  cout << "WtkMeter::drawMeter:left=" << left
       << ":right=" << right
       << ":oldLeft=" << oldLeft
       << ":oldRight=" << oldRight
       << endl;
#endif
  assert (left<=right);
  if (left<style.leftOffset) left=style.leftOffset;
  if (right>width-style.rightOffset)right=width-style.rightOffset;
  if (left==oldLeft && right==oldRight) return;//no need update
  
  
  gc->setForeground (style.fg);
  if (oldLeft<left)
    window->clearArea (oldLeft,style.topOffset,
		       left-oldLeft,height-style.topOffset-style.bottomOffset,
		       False);
  if (right<oldRight)
    window->clearArea
      (right,style.topOffset,
       oldRight-right+1,height-style.topOffset-style.bottomOffset,
       False);
  window->fillRectangle (gc,left,style.topOffset,
			 right-left+1,height-style.
			 topOffset-style.bottomOffset);
  oldLeft=left;
  oldRight=right;
}


void WtkMeter::setMeter (double meter_)
{
  if (meter_<-1) meter=-1;
  if (meter_>1) meter_=1;
  lockData ();
  if (meter==meter_)
    {
      unlockData ();
      return;//no need updating
    }
  
  meter=meter_;
  const uint center=static_cast<uint>((realWidth*(0.0+1)+1.0)/2)
    +style.leftOffset;
  realMeter=static_cast<uint>((realWidth*(meter_+1)+1.0)/2)
    +style.leftOffset;;

  uint left=center;
  uint right=realMeter;
  if (realMeter<center)
    {
      left=realMeter;
      right=center;
    }
  
  drawMeter (left,right);
  unlockData ();
}

void WtkMeter::exposed (XEvent* /*event*/)
{
  lockData ();
  gc->setForeground (style.fg);
#if 0
  cerr << "WtkMeter::exposed:oldLeft=" << oldLeft
       << ":oldRight=" << oldRight
       << ":meter=" << meter
       << endl;
#endif
  window->fillRectangle (gc,oldLeft,style.topOffset,
			 oldRight-oldLeft+1,height-style.
			 topOffset-style.bottomOffset);
  unlockData ();
  decorate ();
}
