/*
    WtkOnOffLabel.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>
#include <cassert>

#include "WtkOnOffLabel.h"

using namespace std;

const char* const WtkOnOffLabel::defaultResource = "*.OffBackground: Gray60\n";

WtkOnOffLabel::WtkOnOffLabel (Display* display, const char* className_,
			      const char* name_, 
			      int x_, int y_, uint width_, uint height_, 
			      int depth, Visual* visual, Colormap colormap_,
			      char* resourceFileName):
  WtkClickableLabel (display, className_, name_, x_, y_, width_, height_,
		     depth, visual, colormap_, resourceFileName),
  isOn (true)
{
  //do needed
  
  initialize ();
}

WtkOnOffLabel::WtkOnOffLabel (Display* display,
			      const char* className_, const char* name_,
			      int x_, int y_, uint width_, uint height_,
			      int depth, Visual* visual, Colormap colormap_,
			      XrmDatabase displayRdb_):
  WtkClickableLabel (display, className_, name_, x_, y_, width_, height_,
		     depth, visual, colormap_, displayRdb_),
  isOn (true)
{
  initialize ();
}

WtkOnOffLabel::WtkOnOffLabel (WtkWidget* parent,
			      const char* className_, const char* name_,
			      int x_, int y_, uint width_, uint height_):
  WtkClickableLabel (parent, className_, name_, x_, y_, width_, height_),
  isOn (true)
{
  initialize ();
}

WtkOnOffLabel::~WtkOnOffLabel ()
{
  
}

void WtkOnOffLabel::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource), &privateRdb,
		      False);
  getResource ();
  setResource ();
}

void WtkOnOffLabel::getResource ()
{
  offStyle.setColormap (window->getDisplay (),style.getColormap ());
  offStyle.allocColor (window->getDisplay (),BACKGROUND,
		       getResource ("offBackground","OffBackground"));
}

void WtkOnOffLabel::set (bool isOn_)
{
  lockData ();
  if (isOn == isOn_)
    {//no need chage
      unlockData ();
      return;
    }
  isOn = isOn_;

  if (isOn_)
    window->setBackground (style.bg);
  else
    window->setBackground (offStyle.bg);

  window->clear ();
  overWrite ();
  unlockData ();
  
  decorate ();
}


void WtkOnOffLabel::toggle ()
{
  lockData ();

  isOn = !isOn;
  if (isOn)
    window->setBackground (style.bg);
  else
    window->setBackground (offStyle.bg);

  window->clear ();
  overWrite ();
  unlockData ();

  decorate ();
}
