/*
    WtkRange.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkRange_H
#define WtkRange_H

#include "WtkWidget.h"
#include "WtkEventHandler.h"
#include "Signal.h"

class WtkRange:public WtkWidget
{
protected:
  static const char* const defaultResource;

  //rangeθĤbuttonդƤꤹ롣
  //ʤ鱦դդ
  Direction direction;
  WtkStyle activeStyle;
  WdkWindow* forward;
  bool isForwardActive;
  bool isForwardUp;
  uint forwardWidth;
  uint forwardHeight;
  WdkWindow* back;
  bool isBackActive;
  bool isBackUp;
  uint backWidth;
  uint backHeight;
  WdkWindow* slider;
  bool isSliderActive;
  int sliderX;
  int sliderY;
  uint sliderWidth;
  uint sliderHeight;
  int sliderMotionOffset;//sliderdragslider򲡤Ƥ֡
  bool isSliderDragged;

  //barθ߽ĲǤɤ餫ޤ롣
  //ʤbutton
  //ĸʤbuttonι⤵
  uint thickness;
  uint minSliderSize;//sliderκǾ礭
  double unitStep;//button򲡤sliderư

  double limit;//Τ礭
  double position;//ɽƤ롣
  double scope;//ɽƤϰϤ礭
  //ǥСɽƤϰϤ
  //[0,total]Τ[value,value+scope]Ǥ롣

  
  virtual void widgetExposed (XEvent* event);//expose event롣
  virtual void widgetButtonPressed (XEvent* event);
  virtual void widgetButtonReleased (XEvent* event);
  virtual void widgetPointerMotion (XEvent* event);
  virtual double calcPosition (XEvent* event);
  
  virtual void updateSlider ();
  virtual void sliderExposed (XEvent* event);//expose event롣
  virtual void sliderEnterNotify (XEvent* event);
  virtual void sliderLeaveNotify (XEvent* event);

  virtual void forwardExposed (XEvent* event);//expose event롣
  virtual void forwardButtonPressed (XEvent* event);
  virtual void forwardButtonReleased (XEvent* event);
  virtual void forwardEnterNotify (XEvent* event);
  virtual void forwardLeaveNotify (XEvent* event);
  virtual void forwardDecorate ();

  virtual void backExposed (XEvent* event);//expose event롣
  virtual void backButtonPressed (XEvent* event);
  virtual void backButtonReleased (XEvent* event);
  virtual void backEnterNotify (XEvent* event);
  virtual void backLeaveNotify (XEvent* event);
  virtual void backDecorate ();

  
  void initialize ();//non virtual
  void getResource ();//resource manager non virtual
  char* getResource (char* resourceName,char* resourceClass)
    {return WtkWidget::getResource (resourceName,resourceClass);};
  int getResourceValue(char* resourceName,char* resourceClass)
    {return WtkWidget::getResourceValue(resourceName,resourceClass);};
  Direction getResourceDirection (char* resourceName,char* resourceClass);
  
  void setResource ();//resourceꡣ non virtual
  //virtual void decorate ();
  virtual void decorate (WdkWindow* wdkWindow,
			 uint windowWidth,uint windowHeight,
			 WtkStyle& style,ShadowType shadowType)
    {WtkWidget::decorate (wdkWindow,windowWidth,windowHeight,style,shadowType);};
  virtual void decorate (WdkWindow* wdkWindow,
			 uint windowWidth,uint windowHeight,
			 bool isActive,bool isUp);

  //θ
  enum SHIFT
  {
    SHIFT_UP,
    SHIFT_DOWN,
    SHIFT_LEFT,
    SHIFT_RIGHT
  };
  virtual void decorateTriangle (WdkWindow* wdkWindow,
				 uint windowWidth,uint windowHeight,
				 bool isActive,bool isUp,SHIFT shift);

  virtual void unlockedSetLimit (double limit_,bool isNeedScrolled);
  virtual void unlockedSetScope (double scope_,bool isNeedScrolled);
  virtual void unlockedSetPosition (double position_,bool isNeedScrolled);
public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkRange (Display* display,const char* className_,const char* name_,
	    int x_,int y_,uint width_,uint height_,
	    int depth,Visual* visual,Colormap colormap_,
	    char* resourceFileName);
  WtkRange (Display* display,const char* className_,const char* name_,
	    int x_,int y_,uint width_,uint height_,
	    int depth,Visual* visual,Colormap colormap_,
	    XrmDatabase displayRdb_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkRange (WtkWidget* parent,const char* className_,const char* name_,
	    int x_,int y_,uint width_,uint height_);
  virtual ~WtkRange ();

  //scrolled (double limit,double position,double scope)
  //limit Τϰ 
  //position ߥ饤ؤƤ 
  //scope 饤Ĺϰϡ
  //ΤϰϤ[0,limit],sliderμϰϤ[position,position+scope]
  Signal3<double,double,double> scrolled;
  
  virtual void registerEventHandler (WtkEventHandler* eventHandler);
  virtual void exposed (XEvent* event);//expose event롣
  virtual void buttonPressed (XEvent* event);
  virtual void buttonReleased (XEvent* event);
  virtual void pointerMotion (XEvent* event);
  virtual void enterNotify (XEvent* event);
  virtual void leaveNotify (XEvent* event);

  virtual void setLimit (double limit_) {unlockedSetLimit (limit_,true);};
  virtual void setScope (double scope_) {unlockedSetScope (scope_,true);};
  virtual void setPosition (double position_)
    {unlockedSetPosition (position_,true);};
  virtual double getLimit () const
    {lockData ();const double ret=limit;unlockData ();return ret;};
  virtual double getScope () const
    {lockData ();const double ret=scope;unlockData ();return ret;};
  virtual double getPosition () const
    {lockData ();const double ret=position;unlockData ();return ret;};
};
#endif// WtkRange_H
