/*
    WtkViewPort.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>
#include <cassert>

#include "WtkViewPort.h"

using namespace std;

const char* const WtkViewPort::defaultResource="";

WtkViewPort::WtkViewPort (Display* display,const char* className_,const char* name_,
			  int x_,int y_,uint width_,uint height_,
			  uint virtualWidth_,uint virtualHeight_,
			  int depth,Visual* visual,Colormap colormap_,
			  char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName),
  viewPort (NULL),virtualWidth (virtualWidth_),virtualHeight (virtualHeight_),
  viewPortX (0),viewPortY (0)
{
  //do needed
  initialize ();
}

WtkViewPort::WtkViewPort (Display* display,const char* className_,const char* name_,
			  int x_,int y_,uint width_,uint height_,
			  uint virtualWidth_,uint virtualHeight_,
			  int depth,Visual* visual,Colormap colormap_,
			  XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_),
  viewPort (NULL),virtualWidth (virtualWidth_),virtualHeight (virtualHeight_),
  viewPortX (0),viewPortY (0)
{
  //do needed
  initialize ();
}

WtkViewPort::WtkViewPort (WtkWidget* parent,const char* className_,const char* name_,
			  int x_,int y_,uint width_,uint height_,
			  uint virtualWidth_,uint virtualHeight_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_),
  viewPort (NULL),virtualWidth (virtualWidth_),virtualHeight (virtualHeight_),
  viewPortX (0),viewPortY (0)
{
  //do needed
  
  initialize ();
}

WtkViewPort::~WtkViewPort ()
{
  delete viewPort;
}

void WtkViewPort::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();
}

void WtkViewPort::getResource ()
{
  if (virtualWidth==0)
    virtualWidth=getResourceValue ("virtualWidth","VirtualWidth");
  if (virtualWidth<width) virtualWidth=width;
  if (virtualHeight==0)
    virtualHeight=getResourceValue ("virtualHeight","VirtualHeight");
  if (virtualHeight<height) virtualHeight=height;

  if (parent!=NULL)
    viewPort=new WdkWindow (parent->getWindow (),x,y,width,height);
  else
    viewPort=new WdkWindow (window->getDisplay (),NULL,x,y,width,height,0,
			    window->getDepth (),window->getVisual ());
  viewPort->setColormap (style.getColormap ());
  //viewPort->selectInput (ExposureMask);//???
  gc->setForeground (style.fg);
  gc->setBackground (style.bg);
  viewPort->setBackground (style.bg);
  window->selectInput (ExposureMask);
  window->reparent (viewPort,0,0);
  window->resize (virtualWidth,virtualHeight);
  viewPortX=0;
  viewPortY=0;
  window->map ();
}

void WtkViewPort::scroll (int viewPortX_,int viewPortY_)
{
  if (viewPortX_<0) viewPortX_=0;
  if (viewPortX_>(int) virtualWidth-(int) width)
    viewPortX_=virtualWidth-width;
  if (viewPortY_<0) viewPortY_=0;
  if (viewPortY_>(int) virtualHeight-(int) height)
    viewPortY_=virtualHeight-height;

  if (viewPortX!=viewPortX_ || viewPortY!=viewPortY_)
    {
      viewPortX=viewPortX_;
      viewPortY=viewPortY_;
      window->move (-viewPortX,-viewPortY);
    }
}


void WtkViewPort::exposed (XEvent* /*event*/)
{
  //redraw
  decorate ();
}

void WtkViewPort::registerEventHandler (WtkEventHandler* eventHandler)
{
  eventHandler->addWidget (this,viewPort);
}

void WtkViewPort::printResource () const
{
  WtkWidget::printResource ();
  cout << name << ".virtualWidth=" << virtualWidth << endl;
  cout << name << ".virtualHeight=" << virtualHeight << endl;
}
