/*
    WtkViewPort.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkViewPort_H
#define WtkViewPort_H

#include "WtkWidget.h"
#include "WtkEventHandler.h"

class WtkViewPort:public WtkWidget
{
protected:
  WdkWindow* viewPort;
  //for scrolling
  uint virtualWidth;
  uint virtualHeight;
  int viewPortX;
  int viewPortY;

  ///////////////////////////////
  static const char* const defaultResource;
  void initialize ();//non virtual
  void getResource ();//resource manager non virtual
  char* getResource (char* resourceName,char* resouceClass)
    {return WtkWidget::getResource (resourceName,resouceClass);};
  int getResourceValue(char* resourceName,char* resourceClass)
    {return WtkWidget::getResourceValue(resourceName,resourceClass);};
  void setResource (){};//resourceꡣ non virtual
  void decorate ()
    {WtkWidget::decorate (viewPort,width,height,style,style.shadowType);};
public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkViewPort (Display* display,const char* className_,const char* name_,
	       int x_,int y_,uint width_,uint height_,
	       uint virtualWidth_,uint virtualHeight_,
	       int depth,Visual* visual,Colormap colormap_,
	       char* resourceFileName);
  WtkViewPort (Display* display,const char* className_,const char* name_,
	       int x_,int y_,uint width_,uint height_,
	       uint virtualWidth_,uint virtualHeight_,
	       int depth,Visual* visual,Colormap colormap_,
	       XrmDatabase displayRdb_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkViewPort (WtkWidget* parent,const char* className_,const char* name_,
	       int x_,int y_,uint width_,uint height_,
	       uint virtualWidth_,uint virtualHeight_);
  virtual ~WtkViewPort ();

  virtual void map (){viewPort->map ();};
  virtual void unmap (){viewPort->unmap ();};
  virtual void registerEventHandler (WtkEventHandler* eventHandler);
  virtual void exposed (XEvent* event);//expose event롣

  
  virtual uint getVirtualWidth () const {return virtualWidth;};
  virtual uint getVirtualHeight () const {return virtualHeight;};
  virtual void scroll (int viewPortX_,int viewPorty_);
  virtual void scrollX (int viewPortX_){scroll (viewPortX_,viewPortY);};
  virtual void scrollY (int viewPortY_){scroll (viewPortX,viewPortY_);};
  virtual void printResource () const;
};
#endif // WtkViewPort_H
