/*
    WtkWidget.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkWidget_H
#define WtkWidget_H

#include <X11/Xlib.h>
#include <X11/Xresource.h>

#include "util/Mutex.h"
#include "WdkWindow.h"
#include "WdkTypes.h"
#include "WtkStyle.h"

class WtkWidget
{
protected:
  WtkWidget* parent;//==NULLλTOPLEVEL_WINDOWǤ롣
  WdkWindow* window;
  WdkGC* gc;
  int x;
  int y;
  uint width;
  uint height;
  mutable Mutex dataLock;
  void lockData () const {dataLock.lock ();};
  void unlockData ()const {dataLock.unlock ();};
  
  //for resource
  XrmDatabase displayRdb;//userresource
  XrmDatabase privateRdb;//app¦Ѱդdefaultresource
  static const char* const defaultResource;//!!!non virtual
  char* className;
  char* name;
  WtkStyle style;
  //XrmGetResource˻Ȥ٤̾Ϣ뤵롣
  char* getResourceName (const char* parent,const char* me);
  //resouce̾ФƤ
  void setResourceName (const char* parentClassName_,const char* ClassName_,
			const char* parentName_,const char* name_);
  //ºݤresourceͤ
  char* getResource (char* resourceName,char* resouceClass);
  int getResourceValue(char* resourceName,char* resourceClass);
  void getResource ();//resource manager !!!non virtual
  void getWindowSize ();//resource manager !!!non virtual
  void setResource ();//resourceꡣ !!!non virtual
  //getResource (),setResource ()򤽤줾ҥ饹overrice
  //constructorǸƤ٤O.K.
  //ΰ٤ΤvirtualǤʤ

  void initialize ();// !!!non virtual
  //shadow(¾)
  virtual void decorate (){decorate (window,width,height,style,style.shadowType);};
  virtual void decorate (WdkWindow* wdkWindow,
			 uint windowWidth,uint windowHeight,
			 WtkStyle& style,ShadowType shadowType);
  virtual void decorate (WdkWindow* wdkWindow,
			 uint windowWidth,uint windowHeight,
			 WtkStyle& nowStyle,
			 WdkColor* leftTopColor,WdkColor* topColor,
			 WdkColor* rightBottomColor,WdkColor* bottomColor);
  ////for event handling
  Mutex eventStateLock;
  EventState eventState;//eventξ֤ФƤ
  EventState cacheEventState;//cache
  void lockEventState (){eventStateLock.lock ();};
  void unlockEventState (){eventStateLock.unlock ();};
  void resetEventState ()
    {lockEventState ();eventState.reset ();unlockEventState ();};
  
public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkWidget (Display* display,const char* className_,const char* name_,
	     int x_,int y_,uint width_,uint height_,
	     int depth,Visual* visual,Colormap colormap_,
	     char* resourceFileName);
  WtkWidget (Display* display,const char* className_,const char* name_,
	     int x_,int y_,uint width_,uint height_,
	     int depth,Visual* visual,Colormap colormap_,
	     XrmDatabase rdb_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkWidget (WtkWidget* parent,const char* className_,const char* name_,
	     int x_,int y_,uint width_,uint height_);
  virtual ~WtkWidget ();
  //container classevent handling򤹤뤿ᡣ
  virtual void addSubWindow (WtkWidget* wtkWidget)
    {if (parent!=NULL) parent->addSubWindow (wtkWidget);};
  virtual void map () {window->map ();};
  virtual void unmap () {window->unmap ();};

  virtual void update (){/*do nothing*/};
  
  //for event handling
  //eventStateƤ򹹿롣
  void updateEventState (EventState& srcEventState);
  //eventStateƤФ
  void getEventState (EventState& destEventState)
    {
      lockEventState ();
      destEventState=eventState;
      eventState.reset ();
      unlockEventState ();
    };
  //EventState򹹿롣
  virtual void buttonPressed (XEvent* event);
  virtual void buttonReleased (XEvent* event);
  virtual void pointerMotion (XEvent* /*event*/){/*do nothing*/};
  virtual void enterNotify (XEvent* event);
  virtual void leaveNotify (XEvent* event);
  virtual void keyPressed (XEvent* /*event*/){/*do nothing yet*/};
  virtual void keyReleased (XEvent* /*event*/){/*do nothing yet*/};
  virtual void exposed (XEvent* /*event*/){decorate ();};

  
  WdkWindow* getWindow () const {return window;};
  uint getWidth () const {return width;};
  uint getHeight () const {return height;};
  int getX () const {return x;};
  int getY () const {return y;};
  virtual void printResource () const;
  virtual const char* getInstanceName () const {return name;};
  //for debug
  void rmPutFileDatabase (char* fileName){XrmPutFileDatabase (displayRdb,fileName);};

  void storeName (char* windowName) {window->storeName (windowName);};
  void setIconName (char* iconName){window->setIconName (iconName);};
};
#endif// WtkWidget_H
