#include <cstdio>

#include <X11/Xlocale.h>

#include "WtkEventHandler.h"
#include "WtkClickableLabel.h"

class Numeric
{
private:
  static const double UNIT_STEP = 0.1;
  double skew;
  WtkClickableLabel* clickableLabel;
  void setLabel (double value)
    {
      if (skew != value)
	{
	  skew = value;
	  char buff[16];
	  sprintf (buff,"%01.01f", skew);
	  clickableLabel->setLabel (buff);
	}
    };
  void clicked (const XEvent* event)
    {
      assert (event != NULL);
      assert (event->type == ButtonPress);
      switch (event->xbutton.button)
	{
	case 1:
	  setLabel (skew - UNIT_STEP);
	  break;
	case 3:
	  setLabel (skew + UNIT_STEP);
	  break;
	default:
	  //do nothing
	  break;
	}
    };
  
public:
  Numeric (WtkClickableLabel* clickableLabel_):
    skew (1.0), clickableLabel (clickableLabel_)
    {
      assert (clickableLabel != NULL);
      clickableLabel->clicked.connect (this, &Numeric::clicked);
      skew = 0.0;
      setLabel (1.0);
    };
  ~Numeric (){};

};

const double Numeric::UNIT_STEP;

main ()
{
  Display* display=XOpenDisplay (NULL);
  int screen=DefaultScreen (display);
  Colormap colormap=DefaultColormap (display,screen);
  int depth=DefaultDepth (display,screen);
  Visual* visual=DefaultVisual (display,screen);

  XrmInitialize ();
  WtkWidget widget (display, "Test", "test",
		    0, 0, 640, 400, depth, visual, colormap,
		    "testWtkWidget.ad");
  WtkWidget child (&widget, "Child", "child", 10, 10, 300, 300);
  WtkClickableLabel clickableLabel (&widget,
				    "ClickableLabel", "clickableLabel",
				    200, 200, 80, 80);
  Numeric numeric (&clickableLabel);
  clickableLabel.map ();
  child.map ();
  widget.map ();
  
  WtkEventHandler eventHandler (display);
  eventHandler.addWidget (&widget);
  eventHandler.addWidget (&child);
  eventHandler.addWidget (&clickableLabel);
  eventHandler.createEventLoopThread ();
  XFlush (display);
  
  getchar ();
}


