#include <unistd.h>
#include <cstdio>

#include <X11/Xlocale.h>

#include "WtkEventHandler.h"
#include "WtkOnOffLabel.h"

class OnOff
{
private:
  WtkOnOffLabel* onOffLabel;
  void clicked (const XEvent* event)
    {
      assert (event != NULL);
      assert (event->type == ButtonPress);
      cerr << "button " << event->xbutton.button << " clicked" << endl;
      switch (event->xbutton.button)
	{
	case 1:
	  onOffLabel->on ();
	  break;
	case 2:
	  onOffLabel->toggle ();
	  break;
	case 3:
	  onOffLabel->off ();
	  break;
	default:
	  //do nothing
	  break;
	}
    };
  
public:
  OnOff (WtkOnOffLabel* onOffLabel_):
    onOffLabel (onOffLabel_)
    {
      assert (onOffLabel != NULL);
      onOffLabel->clicked.connect (this, &OnOff::clicked);
    };
  ~OnOff (){};
};

main ()
{
  Display* display=XOpenDisplay (NULL);
  int screen=DefaultScreen (display);
  Colormap colormap=DefaultColormap (display,screen);
  int depth=DefaultDepth (display,screen);
  Visual* visual=DefaultVisual (display,screen);

  XrmInitialize ();
  WtkWidget widget (display, "Test", "test",
		    0, 0, 640, 400, depth, visual, colormap,
		    "testWtkWidget.ad");
  WtkWidget child (&widget, "Child", "child", 10, 10, 300, 300);
  WtkOnOffLabel onOffLabel (&widget,
			    "OnOffLabel", "onOffLabel",
			    200, 200, 80, 80);
  onOffLabel.setLabel ("test TEST test");
  
  OnOff onOff (&onOffLabel);
  onOffLabel.map ();
  child.map ();
  widget.map ();
  
  WtkEventHandler eventHandler (display);
  eventHandler.addWidget (&widget);
  eventHandler.addWidget (&child);
  eventHandler.addWidget (&onOffLabel);
  eventHandler.createEventLoopThread ();
  XFlush (display);

#if 1
  for (;;)
    {
      usleep (500*1000);
      onOffLabel.toggle ();
      XFlush (display);
    }
#endif
  
  getchar ();
}
