/*
    IOXWrd.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef IOXWrd_H
#define IOXWrd_H

#include <iostream>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>

#include "PlugIn/X/XBase/XBase.h"
#include "PlugIn/Wrd/Wrd.h"
#include "WidiToolkit/WdkPixmap.h"
#include "WidiToolkit/WdkTypes.h"

using namespace std;

class IOXWrd:public XBase,public Wrd
{
protected:
  //default resouce
  XrmDatabase displayRdb;
  XrmDatabase privateRdb;
  static const char* defaultResource;
  static const char resourceFileName[];
  static const char* className;
  static const char* resourceName;
  //escape sequencercolorTextColor?ֹ롣
  static const int color2PixelIndex[47+1];
  void concatinate (char* dest, const char* first, const char* second);
  char* getResource (XrmDatabase privateRdb, XrmDatabase displayRdb,
		     const char* className, const char* classResource,
		     const char* name, const char* resourceName);
  
  void setupResource (Display* display_, const char* resourceFileName_,
		      const char* defaultResource_,
		      const char* className_, const char* resourceName_,
		      char* colorName_[MAX_TEXT_COLORS],
		      XFontSet* fontSet_);
  
  int depth;//depth of screen
  Visual* visual;//visual type no need???
  int visualClass;//visual class(=PseudColor,DirectColor,TrueColor...)
  WdkWindow* window;
  Colormap colormap;
  
  XFontSet fontSet;
  char* colorName[MAX_TEXT_COLORS];//textο̾(Xresource)
  WdkColor tcolor[MAX_TEXT_COLORS];//textο

  //r,g,b,e
  WdkGC* gc;//graphicѤgc(plane maskset롣)
  WdkGC* copyGc;//pixmap -> windowȥԡ뤿gc(plane maskϴط̵)
  static const int MAX_PLANE_NUM = 4;
  //static const int MAX_GRAPHIC_COLORS = (1 << MAX_PLANE_NUM);//defined in class Wrd
  ulong basePixel;//base pixel
  ulong planePixel[MAX_PLANE_NUM];//plane pixel
  ulong graphicPixel[MAX_GRAPHIC_COLORS];//graphicѤο
  WdkPixmap* pixmap[2];//graphic
  WdkPixmap* virtualScreen;//for ensyutu kunn mechanism
  int nOfVirtualScreen;//number of virtual screen

  class CanNotAllocateColors {};
  //γݤ˼ԤCanNotAllocateColors򥹥롣
  void allocateColors (Display* display_,Colormap colormap_,
		       WdkColor tcolor_[MAX_TEXT_COLORS],
		       char* colorName[MAX_TEXT_COLORS],
		       ulong* basePixel_, ulong planePixel_[MAX_PLANE_NUM],
		       ulong graphicPixel_[MAX_GRAPHIC_COLORS]);
  virtual void handleXEvent ();//exposure eventˤ̤褹롣
  virtual void checkPause ();
public:
  IOXWrd (State* state__, WidiController* widiController__,
	  Option* option__);
  virtual ~IOXWrd ();
  virtual void initialize ();//Ƽ
  virtual void release ();//Ƽ
  virtual void prePlay ();
  virtual void postPlay ();
  
protected:
  virtual void execNextLine ();
  virtual void execATMARK ();
  virtual void execCOLOR (uint32 color_);
  virtual void execEND ();
  virtual void execEXEC (const char* filename);
  virtual void doFading (int fadeCount_, uint32 fromPalette, uint32 toPalette);
  virtual void execFADE (uint32 p1,uint32 p2,int speed);
  virtual void execGCIRCLE (uint32 x,uint32 y,uint r,uint32 p1,uint32 sw,uint32 p2);
  virtual void execGCLS (uint32 sw);
  virtual void execGINIT ();
  virtual void execGLINE
  (uint32 x1,uint32 y1,uint32 x2,uint32 y2,uint32 p1,uint32 sw,uint32 p2);
  virtual void execGMODE (uint32 sw);
  virtual void execGMOVE (uint32 x1,uint32 y1,uint32 x2,uint32 y2,uint32 xd,uint32 yd,uint32 vs,uint32 vd,uint32 sw);
  virtual void execGON (bool oldIsGrapnicOn_,bool isGraphicOn_);
  virtual void execGSCREEN (uint32 oldActivePage_, uint32 oldDisplayPage_, uint32 activePage_,uint32 displayPage_);
  virtual void execINKEY (uint32 barWaitFor_);
  virtual void execLOCATE (uint32 x_,uint32 y_);
  virtual void execLOOP (uint32 loop);//XXX
  virtual void execMAG
  (MagFile& mag, int32 x,int32 y,uint32 s,uint32 p);
  virtual void execOFFSET (uint32 offsetBarWaitFor_);

  virtual void realUpdatePal ();
  virtual void updatePal (uint page);
  virtual void execPAL (uint32 page,int32 rgb[16]);
  virtual void execPALREV (uint32 p);
  virtual void execPLOAD (PH0* ph0);
  virtual void execREMARK ();
  virtual void execREST (uint32 barOffsetWaitFor,uint32 stepsWaitFor);
  virtual void execSCROLL (uint32 x1,uint32 y1,uint32 x2,uint32 y2,uint32 mode,uint32 color,uint32 chr);
  virtual void execSTARTUP (uint32 versoin_);
  virtual void execSTOP ();
  virtual void execTCLS
  (uint32 x1,uint32 y1,uint32 x2,uint32 y2,uint32 color,uint32 chr);
  virtual void execTON (uint32 textMode_);
  virtual void execWAIT (uint32 barWaitFor_,uint32 steptsWaitFor_);
  virtual void execWMODE (uint32 deltaSteps_,uint32 wmode_);
  virtual void execFONTM (int32 fontMode_);
  virtual void execFONTP (uint32 num,uint32 x_,uint32 y_,uint32 attribute);
  virtual void execFONTR (int32 fontNumber,int32 fontData[16]);
  virtual void execGSC (int32 mode);
  virtual void execLINE (int32 mode);
  virtual void execPAL_ (int32 paletteNumber,int32 rgb);
  virtual void execREGSAVE (int32 num,int32 value[WRD_REGSAVE_SIZE_MAX]);
  virtual void execSCROLL_ (int32 x_,int32 y_);
  virtual void execTEXTDOT (int32 textDot_);//XXX
  virtual void execTMODE (int32 tmode_);
  virtual void execTSCRL (int32 lines);
  virtual void execVCOPY (int32 sx1,int32 sy1,int32 sx2,int32 sy2,int32 tx,int32 ty,int32 ss,int32 ts,int32 mode);
  virtual void execVSGET (int32 page[4]);//XXX 4Ƚ񤯤ΤϷʤ
  virtual void execVSRES ();
  virtual void execXCOPY (int32 sx1,int32 sy1,int32 sx2,int32 sy2,int32 tx,int32 ty,int32 ss,int32 ts,int32 method,int32 opt[5]);
  //!!!NOTE!!
  //printChar,printChar2bx_,y_@LOCATEλˤƥȤΰ֤
  //뤳Ȥա
  virtual void printChar (uint x_,uint y_,char c,int color_);
  virtual void printChar2b (uint x_,uint y_,char c1,char c2,int color_);
  virtual void doEscapeSequence ();
  virtual void doRefreshTextScreen (TextScreen& textScreen_);

  WdkColor& getTColor (int color_)
    {
      assert (TextScreen::checkColor (color_));
      return tcolor[color2PixelIndex[color_]];
    };

  //
  //!!!NOTE!!! ʲx_,y_,width,heightɽwindowˤ
  //textlocateˤ֤Ǥ̵ա
  uint getGraphicXFromTextX (uint textX) const
    {
      assert (1 <= textX);
      assert (textX <= TEXT_SIZE_X);
      return (textX-1)*CHAR_SIZE_X;
    };
  uint getGraphicYFromTextY (uint textY) const
    {
      assert (1 <= textY);
      assert (textY <= TEXT_SIZE_Y);
      return (textY-1)*CHAR_SIZE_Y;
    };
  bool isReversed (int color_)
    {
      assert (TextScreen::checkColor (color_));
      return (color_ == 7) || (40 <= color_ && color_ <= 47);
    };

  //ɬפʤڡ򥦥ɡ˥ԡƥƥȤľ
  //x_,y_,width,heightgraphicɸǤ롣
  virtual void checkCopyWindow (int x_, int y_, int width, int height);
  //x_,y_,width,heightgraphicɸǤ롣
  virtual void redrawText (int x_, int y_, int width, int height,
			   TextScreen& textScreen_);
  //textɸ(leftText,topText)-(rightText,downText)
  //ˤƥȤ褹롣
  virtual void drawText (TextScreen& textScreen_, bool doesForceRedraw = false,
			 uint leftText = 1, uint topText = 1,
			 uint rightText = TEXT_SIZE_X,
			 uint downText = TEXT_SIZE_Y,
			 bool doesNeedRestoreBackground = true);
  //ƥȺɸǤա
  virtual void drawString (uint x_, uint y_,
			   char* string, int numBytes,
			   WdkColor& color_, bool isReverse,
			   bool doesNeedRestoreBackground = true);
};
#endif// IOXWrd_H
