/*
    IOXlib.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>

#include "util/schedYield.h"
#include "util/getABSTimeSpec.h"
#include "util/calcTime.h"
#include "IOXlib.h"

using namespace std;

const double IOXlib::MAX_SKEW;
const double IOXlib::MIN_SKEW;
const double IOXlib::SKEW_UNIT;

IOXlib::IOXlib (State* state__, WidiController* widiController__,
		Option* option__):
  MainIO (state__, widiController__, option__),
  XBase (),
  eventHandler (NULL),
  topLevelWindow (NULL),

  infoWindow (NULL),
  filename (NULL),title (NULL),
  tpqn (NULL),bpm (NULL),bar (NULL),step (NULL),
  hyousiNow (NULL),hyousi (NULL),key (NULL),
  cc (NULL),bb (NULL),sf (NULL),mi (NULL),time (NULL),events (NULL),
  sequencerMode (NULL),skew (NULL), skewUp (NULL), skewDown (NULL),
  instrument (NULL),
  
  mainWindow (NULL),
  viewPort0 (NULL),viewPort1 (NULL),viewPort2 (NULL),
  verticalRange (NULL),horizontalRange (NULL),
  play (NULL),pause (NULL),
  prev (NULL),rewind (NULL),back (NULL),stop (NULL),forward (NULL),next (NULL),
  open (NULL),songPosition (NULL),
  containerPart (NULL),scroll0 (NULL),scroll1 (NULL),scroll2 (NULL),
  
  lcdWindow (NULL),lcd (NULL),
  
  messageWindow (NULL),
  
  keyboardWindow (NULL)
{
  //do nothing
}

void IOXlib::deleteAll ()
{
  if (eventHandler != NULL) eventHandler->cancelEventLoopThread ();
  delete eventHandler;

  delete filename;
  delete title;
  delete tpqn;             
  delete bpm;              
  delete bar;              
  delete step;             
  delete hyousiNow;
  delete hyousi;           
  delete key;              
  delete cc;               
  delete bb;               
  delete sf;               
  delete mi;               
  delete time;
  delete events;
  delete sequencerMode;
  delete skew;
  delete skewUp;
  delete skewDown;
  delete instrument;       
  delete infoWindow;

  delete containerPart;
  delete viewPort0;
  delete viewPort1;
  delete viewPort2;
  delete play;   
  delete pause;  
  delete prev;
  delete rewind;
  delete back;   
  delete stop;   
  delete forward;
  delete next;
  delete open;
  delete songPosition;
  delete verticalRange;
  delete horizontalRange;
  delete scroll0;
  delete scroll1;
  delete scroll2;
  delete mainWindow;
  
  delete lcd;
  delete lcdWindow;

  delete messageWindow;

  delete keyboardWindow;
  
  delete topLevelWindow;
}


IOXlib::~IOXlib ()
{
  deleteAll ();
  //XCloseDisplay (display);
}

void IOXlib::initialize () throw (Error)
{
  char resourceFileName[]=WIDI_DATA_DIR"/WidiXlib.ad";
  
  display=XOpenDisplay (NULL);//XXX
  screenNumber=DefaultScreen (display);
  colormap=DefaultColormap (display,screenNumber);
  depth=DefaultDepth (display,screenNumber);
  visual=DefaultVisual (display,screenNumber);


  eventHandler=new WtkEventHandler (display);
  topLevelWindow=new WtkWidget (display,"Widi.TopLevel","widi.topLevel",
				-1,-1,0,0,depth,visual,colormap,
				resourceFileName);
  

  infoWindow=new WtkWidget (topLevelWindow,"InfoWindow","infoWindow",
			    -1,-1,0,0);
  infoWindow->map ();

  filename=new WtkLabel (infoWindow,"Filename","filename",-1,-1,0,0);
  title=new WtkLabel (infoWindow,"Title","title",-1,-1,0,0);
  tpqn=new WtkLabel (infoWindow,"Tpqn","tpqn",-1,-1,0,0);
  bpm=new WtkLabel (infoWindow,"Bpm","bpm",-1,-1,0,0);              
  bar=new WtkLabel (infoWindow,"Bar","bar",-1,-1,0,0);              
  step=new WtkLabel (infoWindow,"Step","step",-1,-1,0,0);             
  hyousiNow=new WtkLabel (infoWindow,"HyousiNow","hyousiNow",-1,-1,0,0);
  hyousi=new WtkLabel (infoWindow,"Hyousi","hyousi",-1,-1,0,0);           
  key=new WtkLabel (infoWindow,"Key","key",-1,-1,0,0);              
  cc=new WtkLabel (infoWindow,"Cc","cc",-1,-1,0,0);               
  bb=new WtkLabel (infoWindow,"Bb","bb",-1,-1,0,0);               
  sf=new WtkLabel (infoWindow,"Sf","sf",-1,-1,0,0);               
  mi=new WtkLabel (infoWindow,"Mi","mi",-1,-1,0,0);               

  time=new WtkLabel (infoWindow,"Time","time",-1,-1,0,0);             
  events=new WtkLabel (infoWindow,"Events","events",-1,-1,0,0);
  sequencerMode=new WtkLabel (infoWindow,"SequencerMode","sequencerMode",-1,-1,0,0);
  skew=new WtkLabel (infoWindow,"Skew","skew",-1,-1,0,0);
  skewUp=new WtkButton (infoWindow,"SkewUp","skewUp",-1,-1,0,0);
  skewUp->clicked.connect (this, &IOXlib::doSkewUp);
  skewDown=new WtkButton (infoWindow,"SkewDown","skewDown",-1,-1,0,0);
  skewDown->clicked.connect (this, &IOXlib::doSkewDown);
  instrument=new WtkLabel (infoWindow,"Instrument","instrument",-1,-1,0,0);

  filename->map ();
  title->map ();
  tpqn->map ();             
  bpm->map ();              
  bar->map ();              
  step->map ();             
  hyousiNow->map ();
  hyousi->map ();           
  key->map ();              
  cc->map ();               
  bb->map ();               
  sf->map ();               
  mi->map ();                 
  time->map ();
  events->map ();
  sequencerMode->map ();
  skew->map ();
  skewUp->map ();
  skewDown->map ();
  instrument->map ();       


  mainWindow=new WtkWidget (topLevelWindow,"MainWindow","mainWindow",
			    -1,-1,0,0);
  viewPort0=new WtkViewPort (mainWindow,"ViewPort0","viewPort0",-1,-1,0,0,0,0);
  viewPort1=new WtkViewPort (mainWindow,"ViewPort1","viewPort1",-1,-1,0,0,0,0);
  viewPort2=new WtkViewPort (viewPort0,"ViewPort2","viewPort2",-1,-1,0,0,0,0);
  play=new WtkButton (mainWindow,"Play","play",-1,-1,0,0);
  pause=new WtkButton (mainWindow,"Pause","pause",-1,-1,0,0);
  prev=new WtkButton (mainWindow,"Prev","prev",-1,-1,0,0);
  rewind=new WtkButton (mainWindow,"Rewind","rewind",-1,-1,0,0);
  back=new WtkButton (mainWindow,"Back","back",-1,-1,0,0);
  stop=new WtkButton (mainWindow,"Stop","stop",-1,-1,0,0);
  forward=new WtkButton (mainWindow,"Forward","forward",-1,-1,0,0);
  next=new WtkButton (mainWindow,"Next","next",-1,-1,0,0);
  open=new WtkButton (mainWindow,"Open","open",-1,-1,0,0);
  songPosition=new WtkLockedRange (mainWindow,"SongPosition","songPosition",
				   -1,-1,0,0);
  play->   clicked.connect (this,&IOXlib::doPlay);
  pause->  clicked.connect (this,&IOXlib::doPause);
  prev->   clicked.connect (this,&IOXlib::doPrev);
  rewind-> clicked.connect (this,&IOXlib::doRewind);
  back->   clicked.connect (this,&IOXlib::doBack);
  stop->   clicked.connect (this,&IOXlib::doStop);
  forward->clicked.connect (this,&IOXlib::doForward);
  next->   clicked.connect (this,&IOXlib::doNext);
  open->   clicked.connect (this,&IOXlib::doOpen);
  songPosition->scrolled.connect (this,&IOXlib::doSongPosition);
  //slider$B$ND9$5$r:G>.$K$7$F$*$/!#(B
  songPosition->setScope (0.0,false);
  songPosition->setScope (songPosition->getScope (),false);
  verticalRange=new WtkRange (mainWindow,"VerticalRange","verticalRange",
			      -1,-1,0,0);
  
  horizontalRange=new WtkRange (mainWindow,"HorizontalRange","horizontalRange",
				-1,-1,0,0);
  containerPart=new WtkContainerPart2 (mainWindow,"Part","part",-1,-1,0,0,
				       widiController,
				       viewPort0,viewPort1,viewPort2);
  scroll0=new Scroll (viewPort0);
  verticalRange->setLimit (viewPort0->getVirtualHeight ());
  verticalRange->setScope (viewPort0->getHeight ());
  verticalRange->scrolled.connect (scroll0,&Scroll::scrollY);
  scroll1=new Scroll (viewPort1);
  scroll2=new Scroll (viewPort2);
  horizontalRange->setLimit (viewPort1->getVirtualWidth ());
  horizontalRange->setScope (viewPort1->getWidth ());
  horizontalRange->scrolled.connect (scroll1,&Scroll::scrollX);
  horizontalRange->scrolled.connect (scroll2,&Scroll::scrollX);
  mainWindow->map ();
  viewPort0->map ();
  viewPort1->map ();
  viewPort2->map ();
  play->map ();   
  pause->map ();  
  prev->map ();
  rewind->map ();     
  back->map ();   
  stop->map ();   
  forward->map ();
  next->map ();
  open->map ();
  songPosition->map ();
  verticalRange->map ();
  horizontalRange->map ();
  containerPart->map ();
  resetLevelInfo ();

  
  lcdWindow=new WtkWidget (topLevelWindow,"LcdWindow","lcdWindow",
			   -1,-1,0,0);
  lcdWindow->map ();
  lcd=new WtkLCD (lcdWindow,"Lcd","lcd",-1,-1,0,0);
  lcd->map ();
  
  messageWindow=new WtkMessage (topLevelWindow,"MessageWindow","messageWindow",
				-1,-1,0,0);
  messageWindow->map ();

  keyboardWindow=new WtkContainerKeyboard(viewPort2,"Keyboard","keyboard",
					  -1,-1,0,0);
  keyboardWindow->map ();
  
  eventHandler->addWidget (topLevelWindow);
  eventHandler->addWidget (infoWindow);
  eventHandler->addWidget (mainWindow);
  eventHandler->addWidget (lcdWindow);
  eventHandler->addWidget (messageWindow);

  eventHandler->addWidget (lcd);
  
  eventHandler->addWidget (filename);
  eventHandler->addWidget (title);             
  eventHandler->addWidget (tpqn);             
  eventHandler->addWidget (bpm);              
  eventHandler->addWidget (bar);              
  eventHandler->addWidget (step);             
  eventHandler->addWidget (hyousiNow);
  eventHandler->addWidget (hyousi);           
  eventHandler->addWidget (key);              
  eventHandler->addWidget (cc);               
  eventHandler->addWidget (bb);               
  eventHandler->addWidget (sf);               
  eventHandler->addWidget (mi);               
  eventHandler->addWidget (time);
  eventHandler->addWidget (events);
  eventHandler->addWidget (sequencerMode);
  eventHandler->addWidget (skew);
  eventHandler->addWidget (skewUp);
  eventHandler->addWidget (skewDown);
  eventHandler->addWidget (instrument);       

  viewPort0->registerEventHandler (eventHandler);
  viewPort1->registerEventHandler (eventHandler);
  viewPort2->registerEventHandler (eventHandler);
  eventHandler->addWidget (play);   
  eventHandler->addWidget (pause);  
  eventHandler->addWidget (prev);
  eventHandler->addWidget (rewind);
  eventHandler->addWidget (back);   
  eventHandler->addWidget (stop);   
  eventHandler->addWidget (forward);
  eventHandler->addWidget (next);
  eventHandler->addWidget (open);
  songPosition->registerEventHandler (eventHandler);
  verticalRange->registerEventHandler (eventHandler);
  horizontalRange->registerEventHandler (eventHandler);
  containerPart->registerEventHandler (eventHandler);

  keyboardWindow->registerEventHandler (eventHandler);
  
  eventHandler->createEventLoopThread ();

  
  topLevelWindow->map ();

  //topLevelWindow->rmPutFileDatabase ("result.ad");
  //XFlush ();

  //DEBUG_MESSAGE ("IOXlib::initialize done.");
  MainIO::initialize ();
  //DEBUG_MESSAGE ("MainIO::initialize done.");
  XSync (False);

  //printResource ();
}


void IOXlib::resetLevelInfo ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	levelInfo[port][channel].reset ();
    }
}

void IOXlib::reset ()
{
  //XXX
  //should be locked
  MainIO::reset ();
  resetLevelInfo ();
  containerPart->reset ();
  lcd->reset ();
}

void IOXlib::release ()
{
  deleteAll ();

  eventHandler=NULL;  
               
  filename=NULL;
  title=NULL;
  tpqn=NULL;          
  bpm=NULL;           
  bar=NULL;           
  step=NULL;          
  hyousiNow=NULL;
  hyousi=NULL;        
  key=NULL;           
  cc=NULL;            
  bb=NULL;            
  sf=NULL;            
  mi=NULL;            
  time=NULL;
  events=NULL;
  sequencerMode=NULL;
  skew=NULL;
  skewUp=NULL;
  skewDown=NULL;
  instrument=NULL;    
  infoWindow=NULL;    
               
  containerPart=NULL; 
  viewPort0=NULL;
  viewPort1=NULL;
  viewPort2=NULL;  
  play=NULL;   
  pause=NULL;  
  prev=NULL;
  rewind=NULL;
  back=NULL;   
  stop=NULL;   
  forward=NULL;
  next=NULL;
  open=NULL;
  songPosition=NULL;
  verticalRange=NULL; 
  horizontalRange=NULL;
  scroll0=NULL;
  scroll1=NULL;
  scroll2=NULL;        
  mainWindow=NULL;    
               
  lcd=NULL;           
  lcdWindow=NULL;     
               
  messageWindow=NULL; 
               
  keyboardWindow=NULL;
               
  topLevelWindow=NULL;


  XCloseDisplay (display);//???

  MainIO::release ();
}

void IOXlib::update ()
{
  //window$B$N99?7(B
  updateInfoWindow ();
  updateMainWindow ();
  updateLCDWindow ();
  updateKeyboardWindow ();
  
  XFlush ();
  //XSync (False);
}

void IOXlib::updateInfoWindow ()
{
  char buff[1024];//TTT

  title->setLabel (cacheState.songStatus.getTitle ().c_str ());

  sprintf (buff,"TPQN=%4d",cacheState.songStatus.getDivision ());
  tpqn->setLabel (buff);
  sprintf (buff,"bpm=%4d",getBPM (cacheState.songStatus.getTempo ()));
  bpm->setLabel (buff);
  //XXX
  int measures=cacheState.songStatus.measures;
  int steps=cacheState.songStatus.ticksFromLastMeasure;
  const int ticksPerMeasure=cacheState.songStatus.getTicksPerMeasure ();
#if 0
  if (steps<0)
    {
      measures--;
      steps+=ticksPerMeasure;
    }
  if (steps>ticksPerMeasure)
    {
      measure+=step/ticksPerMeasure;
      steps%=ticksPerMeasure;
    }
#endif
  sprintf (buff,"bar=%4d/%4d",measures,cacheState.songStatus.totalMeasures);
  bar->setLabel (buff);
  sprintf (buff,"step=%4d/%4d",steps,ticksPerMeasure);
  step->setLabel (buff);
  const int nn=cacheState.songStatus.getNn ();
  const int dd=cacheState.songStatus.getDd ();
  const int nnNow=(steps*nn-1)/ticksPerMeasure+1;
  sprintf (buff,"%d/%d",nnNow,dd);
  hyousiNow->setLabel (buff);
  sprintf (buff,"%d/%d",nn,dd);
  hyousi->setLabel (buff);
  sprintf (buff,"key=%d",cacheState.songStatus.getKey ());
  key->setLabel (buff);
  sprintf (buff,"cc=%d",cacheState.songStatus.getCc ());
  cc->setLabel (buff);
  sprintf (buff,"bb=%d",cacheState.songStatus.getBb ());
  bb->setLabel (buff);
  sprintf (buff,"sf=%d",cacheState.songStatus.getSf ());
  sf->setLabel (buff);
  sprintf (buff,"mi=%d",cacheState.songStatus.getMi ());
  mi->setLabel (buff);

  const TimeInfo timeInfo=calcTime (cacheState.songStatus.logicalTicks);
  const TimeInfo totalTimeInfo=calcTime (cacheState.songStatus.totalLogicalTicks);
  sprintf (buff,"%3d'%02d'%02d/%3d'%02d'%02d",
	   timeInfo.minute,timeInfo.second,timeInfo.milisecond/10,
  	   totalTimeInfo.minute,totalTimeInfo.second,totalTimeInfo.milisecond/10);
  time->setLabel (buff);

  sprintf (buff,"%6d/%6d",cacheState.songStatus.events,cacheState.songStatus.totalEvents);
  events->setLabel (buff);

  sprintf (buff,"%s %s %s %s",
	   (cacheState.sequencerStatus.deviceMode&SequencerStatus::DEVICE_GM_ON?"GM":"  "),
	   (cacheState.sequencerStatus.deviceMode&SequencerStatus::DEVICE_SC88_ON?"88":"  "),
	   (cacheState.sequencerStatus.deviceMode&SequencerStatus::DEVICE_GS_ON?"GS":"  "),
	   (cacheState.sequencerStatus.deviceMode&SequencerStatus::DEVICE_XG_ON?"XG":"  "));
  sequencerMode->setLabel (buff);

  skewValueLock.lock ();
  skewValue = cacheState.songStatus.getSkew ();
  skewValueLock.unlock ();
  sprintf (buff,"skew=%1.1f",cacheState.songStatus.getSkew ());
  skew->setLabel (buff);
  
  sprintf (buff,"instrum=%s",cacheState.sequencerStatus.displayedLetter);
  instrument->setLabel (buff);
}

void IOXlib::prePlay ()
{
  IO::prePlay ();
  state->lock ();
  state->getState (&cacheState);
  state->unlock ();

  string windowTitle = "Widi "VERSION":";
  windowTitle += cacheState.songStatus.getFileName ();
  topLevelWindow->storeName (windowTitle.c_str ());
  
  topLevelWindow->setIconName (cacheState.songStatus.getFileName ().c_str ());
  filename->setLabel (cacheState.songStatus.getFileName ().c_str ());
  title->setLabel (cacheState.songStatus.getTitle ().c_str ());
  //slider$B$ND9$5$r:G>.$K$7$F$*$/!#(B
  songPosition->setScope (0.0,false);
  songPosition->setPosition (0.0,false);
  if (cacheState.songStatus.totalLogicalTicks>0)
    {//songPositoin$B$O(BWtkRange$B$J$N$GI=<($9$k(Bslider$B$,(Bpropotional
      //$B$J$N$G$=$ND9$5$ND4@0$r$9$kI,MW$,$"$k!#(B
      //setScope (0)$B$GD9$5$r:G>.$K$9$k!#(B
      //limit$B$r:G8e$^$G=PMh$kMM$K$=$NJ,$@$1?-$P$9!#(B
      songPosition->setScope (0.0,false);
      songPosition->setLimit (cacheState.songStatus.totalLogicalTicks+songPosition->getScope (),false);
      songPosition->setScope (0.0,false);
      songPosition->setLimit (cacheState.songStatus.totalLogicalTicks+songPosition->getScope (),false);
      songPosition->setPosition (0.0,false);
#ifdef DEBUG
      cerr << "IOXlib::prePlay totalticks=" << cacheState.songStatus.totalLogicalTicks << ":limit=" << songPosition->getLimit () << ":scope=" << songPosition->getScope () << endl;
#endif
    }
}


void IOXlib::playLoop ()
{
  uint32 lastTicks = 0;
  uint timedOutCount = 0;//$B2?2sO"B3$G(Btimed out$B$7$?$+(B?
  for (;;)
    {
#define TIMEDOUT 100000000L //100000000nsec=0.1sec
      //$B2?$bJQ2=$,L5$/$F$b(BTIMEDOUT$B$4$H$KI=<($r99?7(B
      timespec timeSpec;
      getABSTimeSpec (&timeSpec,0,TIMEDOUT);

      //$B>uBV$NJQ2=BT$A(B
      int ret = 0;
      state->lock ();
      while ((lastTicks == state->songStatus.ticks) &&
	     (state->songStatus.state != SongStatus::DONE) &&
	     (ret != ETIMEDOUT))
	ret=state->timedWaitChange (&timeSpec);
      state->getState (&cacheState);
      state->unlock ();

      if (ret == ETIMEDOUT && lastTicks == cacheState.songStatus.ticks)
	{
	  //for pausing.pausing$B$J$i2hLL$N;~4V$N7P2a$r;_$a$k!#(BXXX
	  timedOutCount++;

	  //10$B$G3d$C$F$$$k$N$O(BTIMEDOUT=0.1 sec,rate$B$,(Bhz$BC10L$J$N$G!#(B
	  cacheState.songStatus.machineTicks +=
	    timedOutCount*cacheState.songStatus.rate/10;
	  cacheState.songStatus.logicalTicks +=
	    timedOutCount*SongStatus::SONG_CTRL_RATE/10;
	  cacheState.songStatus.ticks +=
	    timedOutCount*cacheState.songStatus.getDivision ()/
	    cacheState.songStatus.getTempo ()/10;
	}
      else
	{
	  timedOutCount = 0;
	}
      lastTicks = cacheState.songStatus.ticks;
      
      //$B6qBNE*$J=hM}!#(B
      update ();
      checkPause ();
      if (cacheState.songStatus.state == SongStatus::DONE)
	break;
      schedYield ();
    }
  resetLevelInfo ();
#undef TIMEDOUT
}

void IOXlib::updateMainWindow ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	//levelInfo[port][channel].getLevel (cacheState, port, channel);
	levelInfo[port][channel].getLevel2 (cacheState, port, channel);
    }
  //TTT
  containerPart->update (cacheState,levelInfo);

  if (cacheState.songStatus.totalLogicalTicks>0)
    songPosition->setPosition (cacheState.songStatus.logicalTicks,false);
}

void IOXlib::doSkewUp (const XEvent* /*xevent*/)
{
  //XXX cacheState$B$K(Baccess$B$9$k$N$O(Bthread safe$B$G$J$$!#(B
  DEBUG_MESSAGE ("IOXlib::doSkewUp called.");
  double skew_;
  skewValueLock.lock ();
  skew_ = skewValue;
  skewValueLock.unlock ();

  if (skew_ == MAX_SKEW) return;//no need change
  skew_ += SKEW_UNIT;
  if (skew_ > MAX_SKEW) skew_ = MAX_SKEW;

  widiController->serverCommand->setSkew (skew_);
#ifdef DEBUG
  cerr << "IOXlib::doSkewUp skew= " << skew_ << endl;
#endif

}

void IOXlib::doSkewDown (const XEvent* /*xevent*/)
{
  //XXX cacheState$B$K(Baccess$B$9$k$N$O(Bthread save$B$G$J$$!#(B
  DEBUG_MESSAGE ("IOXlib::doSkewDown called.");
  double skew_;
  skewValueLock.lock ();
  skew_ = skewValue;
  skewValueLock.unlock ();

  if (skew_ == MIN_SKEW) return;//no need change
  skew_ -= SKEW_UNIT;
  if (skew_ < MIN_SKEW) skew_ = MIN_SKEW;

  widiController->serverCommand->setSkew (skew_);
}


void IOXlib::doPlay (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doPlay clicked");
  widiController->serverCommand->play ();
  DEBUG_MESSAGE ("IOXlib::doPlay all done");
}

void IOXlib::doPause (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doPause clicked");
  widiController->serverCommand->pause ();
  DEBUG_MESSAGE ("IOXlib::doPause all done");
}

void IOXlib::doPrev (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doPrev clicked");
  widiController->serverCommand->prev ();
  reset ();//XXX clitical!!! should be locked
}

void IOXlib::doRewind (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doRewind clicked");
  widiController->serverCommand->rewind ();
  reset ();//XXX clitical!!! should be locked
}

void IOXlib::doBack (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doBack clicked");
  widiController->serverCommand->back ();
  DEBUG_MESSAGE ("IOXlib::doBack done");
}

void IOXlib::doStop (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doStop clicked");
  widiController->serverCommand->stop ();
  DEBUG_MESSAGE ("IOXlib::doStop done.");
}

void IOXlib::doForward (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doForward clicked");
  widiController->serverCommand->forward ();
  DEBUG_MESSAGE ("IOXlib::doForward done");
}

void IOXlib::doNext (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doNext clicked");
  widiController->serverCommand->next ();
  DEBUG_MESSAGE ("IOXlib::doNext done");
}

void IOXlib::doOpen (const XEvent* event)
{
  DEBUG_MESSAGE ("IOXlib::doOpen clicked");
}

void IOXlib::doSongPosition (double limit,double position,double scope)
{
  DEBUG_MESSAGE ("IOXlib::doSongPosition'ed");
  widiController->serverCommand->setPosition (position);
  DEBUG_MESSAGE ("IOXlib::doSongPosition done");
}

void IOXlib::updateLCDWindow ()
{
  lcd->update (cacheState);
}


void IOXlib::coutMessage (const char* message)
{
  messageWindow->coutMessage (message);
  XFlush ();
  //cerr << "DEBUG:IOXlib::coutMessage=" << message << endl;
}



void IOXlib::errorMessage (const char* errorMessage)
{normalMessage (errorMessage);};
void IOXlib::errorMessage (const char* errorMessage,int size)
{normalMessage (errorMessage,size);};


void IOXlib::printResource () const
{
  filename->printResource ();
  title->printResource ();
  tpqn->printResource ();             
  bpm->printResource ();              
  bar->printResource ();              
  step->printResource ();
  hyousiNow->printResource ();           
  hyousi->printResource ();           
  key->printResource ();              
  cc->printResource ();               
  bb->printResource ();               
  sf->printResource ();               
  mi->printResource ();               
  time->printResource ();
  events->printResource ();
  sequencerMode->printResource ();
  skew->printResource ();
  skewUp->printResource ();
  skewDown->printResource ();
  instrument->printResource ();       
  infoWindow->printResource ();

  containerPart->printResource ();
  viewPort0->printResource ();
  viewPort1->printResource ();
  viewPort2->printResource ();

  play->printResource ();   
  pause->printResource ();  
  prev->printResource ();
  rewind->printResource ();
  back->printResource ();   
  stop->printResource ();   
  forward->printResource ();
  next->printResource ();
  open->printResource ();
  songPosition->printResource ();
  verticalRange->printResource ();
  horizontalRange->printResource ();
  mainWindow->printResource ();
  
  lcd->printResource ();
  lcdWindow->printResource ();

  messageWindow->printResource ();

  keyboardWindow->printResource ();
  
  topLevelWindow->printResource ();
  
}

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////for dynamic loading
///////////////////////////////////////////////////////////////newIO<T>
///////////////////////////////////////////////////////////////newMainIO<T>
///////////////////////////////////////////////////////////////infoIO<T>
////////////////////////////////////////////////////////////////////////////

#include "IO/InfoIO.h"
static IO* newIOXlib (State* state, WidiController* widiController,
		      Option* option)
{
  return new IOXlib (state, widiController, option);
}

static MainIO* newMainIOXlib (State* state,WidiController* widiController,
			      Option* option)
{
  return new IOXlib (state, widiController, option);
}

InfoIO infoIOXlib=
{
  newIOXlib,
  newMainIOXlib,
  true
};


