/*
    IOXlib.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef IOXlib_H
#define IOXlib_H

#include "IO/MainIO.h"
#include "IO/ioutil/LevelInfo.h"
#include "PlugIn/X/XBase/XBase.h"
#include "WidiToolkit/WtkLabel.h"
#include "WidiToolkit/WtkEventHandler.h"
#include "WidiToolkit/WtkMessage.h"
#include "WidiToolkit/WtkLockedRange.h"
#include "WidiToolkit/WtkViewPort.h"
#include "WidiToolkit/WtkButton.h"
#include "WtkContainerKeyboard.h"
#include "WtkContainerPart2.h"
#include "WtkLCD.h"

class IOXlib:public MainIO,public XBase
{
protected:
  /////////////////////////////////////////interface$BI=<(MQ!#(B
  virtual void initialize () throw (Error);//$B3F<o=i4|2=!#(B
  virtual void release ();//$B3F<o8e;OKv!#(B
  virtual void reset ();//$B1iAUA0$N3F<o=i4|2=(B
  void deleteAll ();//$BA4It(Bdelete$B$9$k!#(B
  
  virtual void update ();
  virtual void playLoop ();
  virtual void coutMessage (const char* message);
  virtual void cerrMessage (const char* message){coutMessage (message);};
  virtual void cerrLoop () {/*do nothing*/};
  //cerr$B$O(Bcout$B$K$7$F$7$^$&$N$G(BcerrLoop$B$OITMW!#(B

  /////////////////////////////////////////////////////////for X
  int depth;
  Visual* visual;
  Colormap colormap;
  
  WtkEventHandler* eventHandler;
  WtkWidget* topLevelWindow;

  WtkWidget* infoWindow;
  void updateInfoWindow ();
  WtkLabel* filename;
  WtkLabel* title;
  WtkLabel* tpqn;
  WtkLabel* bpm;
  WtkLabel* bar;
  WtkLabel* step;
  WtkLabel* hyousiNow;
  WtkLabel* hyousi;
  WtkLabel* key;
  WtkLabel* cc;
  WtkLabel* bb;
  WtkLabel* sf;
  WtkLabel* mi;
  WtkLabel* time;
  WtkLabel* events;
  WtkLabel* sequencerMode;
  WtkLabel* skew;
  WtkButton* skewUp;
  WtkButton* skewDown;
  //XXX$B$3$NJU$O$I$&$$$&?t;z$,$$$$$+$A$c$s$H$-$a$kI,MW$,$"$k!#(B
  //$BKt!"$3$l$OK\Ev$O(BSongStatus$B$K$"$k$Y$-?t;z!#(B
  static const double MAX_SKEW = 10.0;
  static const double MIN_SKEW =  0.1;
  static const double SKEW_UNIT = 0.1;
  //skewValue,skewValueLock are ad hoc objects.
  //should be fixed.
  double skewValue;//$B8=:_$N(Bskew$B$NCM$r3P$($F$*$/!#(B
  Mutex skewValueLock;//skewValue$B$N(Block 
  void doSkewUp (const XEvent* xevent);
  void doSkewDown (const XEvent* xevent);
  WtkLabel* instrument;       
  
  WtkWidget* mainWindow;
  void updateMainWindow ();
  LevelInfo levelInfo[MIDI::MAX_PORT][MIDI::MAX_CHANNEL];
  void resetLevelInfo ();
  WtkViewPort* viewPort0;//main$B$r(Bscroll$B$9$kJ*!#(B
  WtkViewPort* viewPort1;//$BI=<(9`L\$r<($9ItJ,$r(Bscroll$B$9$kJ*!#(B
  WtkViewPort* viewPort2;//key board$BEy$r(Bscroll$B$9$kJ*!#(B
  WtkRange* verticalRange;
  WtkRange* horizontalRange;
  WtkButton* play;
  WtkButton* pause;
  WtkButton* prev;
  WtkButton* rewind;
  WtkButton* back;
  WtkButton* stop;
  WtkButton* forward;
  WtkButton* next;     
  WtkButton* open;
  WtkLockedRange* songPosition;
  //$B$=$l$>$l$N(Bbutton$B$,2!$5$l$?;~$K=hM}$r$9$k!#(B
  void doPlay (const XEvent* event);
  void doPause (const XEvent* event);
  void doPrev (const XEvent* event);
  void doRewind (const XEvent* event);
  void doBack (const XEvent* event);
  void doStop (const XEvent* event);
  void doForward (const XEvent* event);
  void doNext (const XEvent* event);
  void doOpen (const XEvent* event);
  void doSongPosition (double limit,double position,double scope);
  WtkContainerPart2* containerPart;
  class Scroll
  {
  public:
    WtkViewPort* viewPort;
    Scroll (WtkViewPort* viewPort_):viewPort (viewPort_){/*do nothing*/};
    ~Scroll (){/*do nothing*/};
    void scrollX (double /*limit*/,double position,double /*scope*/)
      {viewPort->scrollX ((int) position);};
    void scrollY (double /*limit*/,double position,double /*scope*/)
      {viewPort->scrollY ((int) position);};
  };
  Scroll* scroll0;
  Scroll* scroll1;
  Scroll* scroll2;

  
  WtkWidget* lcdWindow;
  void updateLCDWindow ();
  WtkLCD* lcd;
  
  WtkMessage* messageWindow;

  WtkContainerKeyboard* keyboardWindow;
  void updateKeyboardWindow (){keyboardWindow->update (cacheState);};
  
public:
  IOXlib (State* state__, WidiController* widiController__,
	  Option* option__);
  ~IOXlib ();
  virtual void prePlay ();
  virtual void errorMessage (const char* errorMessage);
  virtual void errorMessage (const char* errorMessage,int size);
  void printResource () const;
};
#endif // IOXlib_H
