/*
    WtkContainerKeyboard.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>

#include "WtkContainerKeyboard.h"

using namespace std;

const char* const WtkContainerKeyboard::defaultResource="";

WtkContainerKeyboard::WtkContainerKeyboard
(Display* display,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_,
 int depth,Visual* visual,Colormap colormap_,
 char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName)
{
  initialize ();
}

WtkContainerKeyboard::WtkContainerKeyboard
(Display* display,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_,
 int depth,Visual* visual,Colormap colormap_,
 XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_)
{
  initialize ();
}

WtkContainerKeyboard::WtkContainerKeyboard
(WtkWidget* parent,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_)
{
  initialize ();
}

WtkContainerKeyboard::~WtkContainerKeyboard ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	delete keyboard[port][channel];
    }
}

void WtkContainerKeyboard::initialize ()
{
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();

  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  char resourceName[20];
	  sprintf (resourceName, "keyboard%d",
		   port*MIDI::MAX_CHANNEL+channel+1);
	  keyboard[port][channel]=new WtkKeyboard (this, "Keyboard",
						   resourceName,-1,-1,0,0);
	}
    }
}

void WtkContainerKeyboard::update (const State& nowState)
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	keyboard[port][channel]->update (nowState.part[port][channel],
					 nowState.songStatus);
    }
}

void WtkContainerKeyboard::registerEventHandler (WtkEventHandler* eventHandler)
{
  eventHandler->addWidget (this);
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	eventHandler->addWidget (keyboard[port][channel]);
    }
}

void WtkContainerKeyboard::map ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	keyboard[port][channel]->map ();
    }
  WtkWidget::map ();
}

void WtkContainerKeyboard::unmap ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	keyboard[port][channel]->unmap ();
    }
  WtkWidget::unmap ();
}

void WtkContainerKeyboard::printResource () const
{
  WtkWidget::printResource ();
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	keyboard[port][channel]->printResource ();
    }
}
