/*
    WtkContainerPart.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkContainerPart_H
#define WtkContainerPart_H

#include "IO/ioutil/LevelInfo.h"
#include "WtkPart.h"

class WtkContainerPart:WtkWidget
{
private:
  static const char* const defaultResource;

  WtkLabel* channel;
  WtkLabel* instrument;
  WtkLabel* program;
  WtkLabel* bankMSB;
  WtkLabel* bankLSB;
  
  WtkLabel* level;
  WtkLabel* volume;
  WtkLabel* expression;
  WtkLabel* modulation;
  WtkLabel* reverbSendLevel;
  WtkLabel* panPot;
  WtkLabel* pitchBend;
  
  WtkLabel* chorusSendLevel;
  WtkLabel* delaySendLevel;
  WtkLabel* damperPedal;
  WtkLabel* sostenute;
  WtkLabel* softPedal;
  WtkLabel* portament;
  
  WtkLabel* vibrate;
  WtkLabel* vibrateRate;
  WtkLabel* vibrateDepth;
  WtkLabel* vibrateDelay;
  WtkLabel* tvfCutOff;
  WtkLabel* tvfResonance;
  WtkLabel* envelope;
  WtkLabel* envelopeAttackTime;
  WtkLabel* envelopeDecayTime;
  WtkLabel* envelopeReleaseTime;
  
  WtkLabel* port;
  
  WtkPart* part[State::MAX_PARTS];

  void initialize ();//non virtual
  void getResource (){};//resource manager$B$+$i@_Dj$r<hF@(B non virtual
  void setResource (){};//resource$B$r@_Dj!#(B non virtual
public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkContainerPart (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    char* resourceFileName);
  WtkContainerPart (Display* display,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_,
		    int depth,Visual* visual,Colormap colormap_,
		    XrmDatabase displayRdb_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkContainerPart (WtkWidget* parent,const char* className_,const char* name_,
		    int x_,int y_,uint width_,uint height_);
  virtual ~WtkContainerPart ();

  virtual void map ();
  virtual void unmap ();
  //XXX
  virtual void update (Part* part,LevelInfo level[State::MAX_PARTS]);//$B$3$l$GFbIt$N>uBV$r99?7$9$k!#(B
  virtual void exposed (XEvent* event){decorate ();};//expose event$B$r=hM}$9$k!#(B
  //event handler$B$K(Bwidget$B$rEPO?$9$k!#(B
  virtual void registerEventHandler (WtkEventHandler* eventHandler);
};
#endif// WtkContainerPart_H
