/*
    WtkContainerPart2.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>

#include "WtkContainerPart2.h"

using namespace std;

const char* const WtkContainerPart2::defaultResource="";

WtkContainerPart2::WtkContainerPart2 (Display* display,
				      const char* className_,
				      const char* name_,
				      int x_, int y_,
				      uint width_, uint height_,
				      int depth,
				      Visual* visual, Colormap colormap_,
				      char* resourceFileName,
				      WidiController* widiController_,
				      WtkViewPort* viewPort0_,
				      WtkViewPort* viewPort1_,
				      WtkViewPort* viewPort2_):
  WtkWidget (display, className_, name_, x_, y_, width_, height_, 
	     depth, visual, colormap_, resourceFileName),
  widiController (widiController_),
  viewPort0 (viewPort0_),viewPort1 (viewPort1_),viewPort2 (viewPort2_)
{
  assert (widiController != NULL);
  initialize ();
}

WtkContainerPart2::WtkContainerPart2 (Display* display,
				      const char* className_,
				      const char* name_,
				      int x_, int y_,
				      uint width_, uint height_,
				      int depth,
				      Visual* visual, Colormap colormap_,
				      XrmDatabase displayRdb_, 
				      WidiController* widiController_,
				      WtkViewPort* viewPort0_,
				      WtkViewPort* viewPort1_,
				      WtkViewPort* viewPort2_):
  WtkWidget (display, className_, name_, x_, y_, width_, height_,
	     depth, visual, colormap_, displayRdb_),
  widiController (widiController_),
  viewPort0 (viewPort0_),viewPort1 (viewPort1_),viewPort2 (viewPort2_)
{
  initialize ();
}

WtkContainerPart2::WtkContainerPart2 (WtkWidget* parent,
				      const char* className_,
				      const char* name_,
				      int x_, int y_,
				      uint width_, uint height_,
				      WidiController* widiController_,
				      WtkViewPort* viewPort0_,
				      WtkViewPort* viewPort1_,
				      WtkViewPort* viewPort2_):
  WtkWidget (parent, className_, name_, x_, y_, width_, height_),
  widiController (widiController_),
  viewPort0 (viewPort0_), viewPort1 (viewPort1_), viewPort2 (viewPort2_)
{
  initialize ();
}

WtkContainerPart2::~WtkContainerPart2 ()
{
  delete masterVolumeLabel;
  delete masterVolume;
  delete noteLabel;
  delete note;
  //delete masterPanLabel;
  //delete masterPan;
  
  delete channel;//channel 1-32         
  delete instrument;                    
  delete program;                       
  delete bankMSB;                       
  delete bankLSB;                       
                               
  delete level;                         
  delete volume;                        
  delete expression;                    
  delete modulation;                    
  delete reverbSendLevel;               
  delete panPot;                        
  delete pitchBend;                     
                               
  delete keyboard;

  delete chorusSendLevel;               
  delete delaySendLevel;                
  delete damperPedal;                   
  delete sostenute;                     
  delete softPedal;                     
  delete portament;                     

  delete vibrate;
  delete vibrateRate;                   
  delete vibrateDepth;                  
  delete vibrateDelay;                  
  delete tvfCutOff;                     
  delete tvfResonance;                  
  delete envelope;
  delete envelopeAttackTime;             
  delete envelopeDecayTime;             
  delete envelopeReleaseTime;              
                               
  delete port;//$B=PNO$7$F$$$k%]!<%HHV9f!#(B

  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	delete part[port__][channel__];
    }
}

void WtkContainerPart2::initialize ()
{
  assert (widiController != NULL);
  assert (viewPort0 != NULL);
  assert (viewPort1 != NULL);
  assert (viewPort2 != NULL);

  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();

  masterVolumeLabel=new WtkLabel (this,"MasterVolumeLabel","masterVolumeLabel",
				  -1,-1,0,0);
  masterVolume=new WtkLevel (this,"MasterVlume","masterVolume",-1,-1,0,0);
  noteLabel=new WtkLabel (this,"NoteLabel","noteLabel",-1,-1,0,0);
  note=new WtkMaxedLevel (this,"Note","note",-1,-1,0,0);
  
  channel=new WtkLabel (this,"Channel","channel",-1,-1,0,0);//channel//1-32
  instrument=new WtkLabel (this,"Instrument","instrument",-1,-1,0,0);
  program=new WtkLabel (this,"Program","program",-1,-1,0,0);
  bankMSB=new WtkLabel (this,"BankMSB","bankMSB",-1,-1,0,0);
  bankLSB=new WtkLabel (this,"BankLSB","bankLSB",-1,-1,0,0);
                               
  level=new WtkLabel (this,"Level","level",-1,-1,0,0);
  volume=new WtkLabel (this,"Volume","volume",-1,-1,0,0);
  expression=new WtkLabel (this,"Exporession","expression",-1,-1,0,0);
  modulation=new WtkLabel (this,"Modulation","modulation",-1,-1,0,0);
  reverbSendLevel=new WtkLabel (this,"ReverbSendLevel","reverbSendLevel",-1,-1,0,0);
  panPot=new WtkLabel (this,"PanPot","panPot",-1,-1,0,0);
  pitchBend=new WtkLabel (this,"PitchBend","pitchBend",-1,-1,0,0);
                               
  keyboard=new WtkLabel (viewPort1,"Keyboard","keyboard",-1,-1,0,0);

  chorusSendLevel=new WtkLabel (viewPort1,"ChorusSendLevel","chorusSendLevel",-1,-1,0,0);
  delaySendLevel=new WtkLabel (viewPort1,"DelaySendLevel","delaySendLevel",-1,-1,0,0);
  damperPedal=new WtkLabel (viewPort1,"DamperPedal","damperPedal",-1,-1,0,0);
  sostenute=new WtkLabel (viewPort1,"Sostenute","sostenute",-1,-1,0,0);
  softPedal=new WtkLabel (viewPort1,"SoftPedal","softPedal",-1,-1,0,0);
  portament=new WtkLabel (viewPort1,"Portament","portament",-1,-1,0,0);
                               
  vibrate=new WtkLabel (viewPort1,"Vibrate","vibrate",-1,-1,0,0);
  vibrateRate=new WtkLabel (viewPort1,"VibrateRate","vibrateRate",-1,-1,0,0);
  vibrateDepth=new WtkLabel (viewPort1,"VibrateDepth","vibrateDepth",-1,-1,0,0);
  vibrateDelay=new WtkLabel (viewPort1,"VibrateDelay","vibrateDelay",-1,-1,0,0);
  tvfCutOff=new WtkLabel (viewPort1,"TvfCutOff","tvfCutOff",-1,-1,0,0);
  tvfResonance=new WtkLabel (viewPort1,"TvfResonance","tvfResonance",-1,-1,0,0);
  envelope=new WtkLabel (viewPort1,"Envelope","envelope",-1,-1,0,0);
  envelopeAttackTime=new WtkLabel (viewPort1,"EnvelopeAttackTime","envelopeAttackTime",-1,-1,0,0);
  envelopeDecayTime=new WtkLabel (viewPort1,"EnvelopeDecayTime","envelopeDecayTime",-1,-1,0,0);
  envelopeReleaseTime=new WtkLabel (viewPort1,"EnvelopeReleaseTime","envelopeReleaseTime",-1,-1,0,0);
                               
  port=new WtkLabel (viewPort1,"Port","port",-1,-1,0,0);//$B=PNO$7$F$$$k%]!<%HHV9f!#(B

  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	{
	  char resourceName[10];
	  sprintf (resourceName, "part%02d",
		   port__*MIDI::MAX_CHANNEL+channel__+1);
	  part[port__][channel__] = new WtkPart2 (viewPort0,
						  "Part", resourceName,
						  -1, -1, 0, 0, viewPort2);
	  part[port__][channel__]->setPortChannel (port__, channel__);
	  part[port__][channel__]->setWidiController (widiController);
	}
    }
  reset ();
}

//XXX
void WtkContainerPart2::update (State& state,
				LevelInfo levelInfo[MIDI::MAX_PORT][MIDI::MAX_CHANNEL])
{
  masterVolume->setLevel (static_cast<double>(state.sequencerStatus.masterVolume)/
			  static_cast<double> (SequencerStatus::MAX_MASTER_VOLUME));
  int notesOn=0;
  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	{
	  part[port__][channel__]->update (&state.part[port__][channel__],
					   levelInfo[port__][channel__].nowLevel);//XXX
	  notesOn += state.part[port__][channel__].notesOnNumber;
	}
    }
  
  char notesOnBuff[16];
  sprintf (notesOnBuff,"Note:%2d",notesOn);
  noteLabel->setLabel (notesOnBuff);
#define DELAY (state.songStatus.rate*70/100) //0.7 second
#define MAX_NOTE_ON 64.0
  if (maxNotesOn<=notesOn ||
      state.songStatus.machineTicks-maxNotesOnMachineTicks>DELAY)
    {
      maxNotesOn=notesOn;
      maxNotesOnMachineTicks=state.songStatus.machineTicks;
    }
  note->setLevel (notesOn/MAX_NOTE_ON,maxNotesOn/MAX_NOTE_ON);
#undef MAX_NOTE_ON
#undef DELAY
}

void WtkContainerPart2::reset ()
{
  maxNotesOn=0;
  maxNotesOnMachineTicks=0;
  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_PORT; channel__++)
	part[port__][channel__]->reset ();
    }
}


void WtkContainerPart2::registerEventHandler (WtkEventHandler* eventHandler)
{
  eventHandler->addWidget (this);

  eventHandler->addWidget (masterVolumeLabel);//channel 1-32
  eventHandler->addWidget (masterVolume);//channel 1-32         
  eventHandler->addWidget (noteLabel);
  eventHandler->addWidget (note);
  
  eventHandler->addWidget (channel);//channel 1-32         
  eventHandler->addWidget (instrument);
  eventHandler->addWidget (program);
  eventHandler->addWidget (bankMSB);
  eventHandler->addWidget (bankLSB);
                               
  eventHandler->addWidget (level);
  eventHandler->addWidget (volume);
  eventHandler->addWidget (expression);
  eventHandler->addWidget (modulation);
  eventHandler->addWidget (reverbSendLevel);
  eventHandler->addWidget (panPot);
  eventHandler->addWidget (pitchBend);
                               
  eventHandler->addWidget (keyboard);

  eventHandler->addWidget (chorusSendLevel);
  eventHandler->addWidget (delaySendLevel);
  eventHandler->addWidget (damperPedal);
  eventHandler->addWidget (sostenute);
  eventHandler->addWidget (softPedal);
  eventHandler->addWidget (portament);

  eventHandler->addWidget (vibrate);
  eventHandler->addWidget (vibrateRate);
  eventHandler->addWidget (vibrateDepth);
  eventHandler->addWidget (vibrateDelay);
  eventHandler->addWidget (tvfCutOff);
  eventHandler->addWidget (tvfResonance);
  eventHandler->addWidget (envelope);
  eventHandler->addWidget (envelopeAttackTime);
  eventHandler->addWidget (envelopeDecayTime);
  eventHandler->addWidget (envelopeReleaseTime);
                               
  eventHandler->addWidget (port);//$B=PNO$7$F$$$k%]!<%HHV9f!#(B
  
  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	part[port__][channel__]->registerEventHandler (eventHandler);
    }
}


 
void WtkContainerPart2::map ()
{
  masterVolumeLabel->map ();
  masterVolume->map ();
  noteLabel->map ();
  note->map ();
  
  channel->map ();//channel 1-32         
  instrument->map ();
  program->map ();
  bankMSB->map ();
  bankLSB->map ();
                               
  level->map ();
  volume->map ();
  expression->map ();
  modulation->map ();
  reverbSendLevel->map ();
  panPot->map ();
  pitchBend->map ();

  keyboard->map ();
                               
  chorusSendLevel->map ();
  delaySendLevel->map ();
  damperPedal->map ();
  sostenute->map ();
  softPedal->map ();
  portament->map ();
                               
  vibrate->map ();
  vibrateRate->map ();
  vibrateDepth->map ();
  vibrateDelay->map ();
  tvfCutOff->map ();
  tvfResonance->map ();
  envelope->map ();
  envelopeAttackTime->map ();
  envelopeDecayTime->map ();
  envelopeReleaseTime->map ();
                               
  port->map ();//$B=PNO$7$F$$$k%]!<%HHV9f!#(B

  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	part[port__][channel__]->map ();
    }
  

  WtkWidget::map ();
}



void WtkContainerPart2::unmap ()
{
  masterVolumeLabel->unmap ();
  masterVolume->unmap ();
  noteLabel->unmap ();
  note->unmap ();
  
  channel->unmap ();//channel 1-32         
  instrument->unmap ();
  program->unmap ();
  bankMSB->unmap ();
  bankLSB->unmap ();
                               
  level->unmap ();
  volume->unmap ();
  expression->unmap ();
  modulation->unmap ();
  reverbSendLevel->unmap ();
  panPot->unmap ();
  pitchBend->unmap ();

  keyboard->unmap ();
  
  chorusSendLevel->unmap ();
  delaySendLevel->unmap ();
  damperPedal->unmap ();
  sostenute->unmap ();
  softPedal->unmap ();
  portament->unmap ();
                               
  vibrate->unmap ();
  vibrateRate->unmap ();
  vibrateDepth->unmap ();
  vibrateDelay->unmap ();
  tvfCutOff->unmap ();
  tvfResonance->unmap ();
  envelope->unmap ();
  envelopeAttackTime->unmap ();
  envelopeDecayTime->unmap ();
  envelopeReleaseTime->unmap ();
                               
  port->unmap ();//$B=PNO$7$F$$$k%]!<%HHV9f!#(B

  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	part[port__][channel__]->unmap ();
    }
  WtkWidget::unmap ();
}

void WtkContainerPart2::printResource () const
{
  WtkWidget::printResource ();

  masterVolumeLabel->printResource ();
  masterVolume->printResource ();
  noteLabel->printResource ();
  note->printResource ();
  
  channel->printResource ();//channel 1-32         
  instrument->printResource ();                    
  program->printResource ();                       
  bankMSB->printResource ();                       
  bankLSB->printResource ();                       
                               
  level->printResource ();                         
  volume->printResource ();                        
  expression->printResource ();                    
  modulation->printResource ();                    
  reverbSendLevel->printResource ();               
  panPot->printResource ();                        
  pitchBend->printResource ();

  keyboard->printResource ();                     
                               
  chorusSendLevel->printResource ();               
  delaySendLevel->printResource ();                
  damperPedal->printResource ();                   
  sostenute->printResource ();                     
  softPedal->printResource ();                     
  portament->printResource ();                     

  vibrate->printResource ();
  vibrateRate->printResource ();                   
  vibrateDepth->printResource ();                  
  vibrateDelay->printResource ();                  
  tvfCutOff->printResource ();                     
  tvfResonance->printResource ();                  
  envelope->printResource ();
  envelopeAttackTime->printResource ();             
  envelopeDecayTime->printResource ();             
  envelopeReleaseTime->printResource ();              
                               
  port->printResource ();//$B=PNO$7$F$$$k%]!<%HHV9f!#(B

  for (int port__ = 0; port__ < MIDI::MAX_PORT; port__++)
    {
      for (int channel__ = 0; channel__ < MIDI::MAX_CHANNEL; channel__++)
	part[port__][channel__]->printResource ();
    }
}



