/*
    WtkContainerPart2.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkContainerPart2_H
#define WtkContainerPart2_H

#include "IO/ioutil/LevelInfo.h"
#include "WidiToolkit/WtkViewPort.h"
#include "WidiToolkit/WtkMaxedLevel.h"
#include "WtkPart2.h"

class WtkContainerPart2:WtkWidget
{
private:
  static const char* const defaultResource;

  //$B$3$l$G1iAU>uBV$r@)8f$9$k!#(B
  WidiController* const widiController;
  
  WtkViewPort* viewPort0;//$BI=<(9`L\$r<($9ItJ,$r(Bscroll$B$9$kJ*!#(B
  WtkViewPort* viewPort1;//$BI=<(9`L\$r<($9ItJ,$r(Bscroll$B$9$kJ*!#(B
  WtkViewPort* viewPort2;//key board$BEy$r(Bscroll$B$9$kJ*!#(B

  WtkLabel* masterVolumeLabel;
  WtkLevel* masterVolume;
  int maxNotesOn;
  double maxNotesOnMachineTicks;
  WtkLabel* noteLabel;
  WtkMaxedLevel* note;

  WtkLabel* channel;
  WtkLabel* instrument;
  WtkLabel* program;
  WtkLabel* bankMSB;
  WtkLabel* bankLSB;
  
  WtkLabel* level;
  WtkLabel* volume;
  WtkLabel* expression;
  WtkLabel* modulation;
  WtkLabel* reverbSendLevel;
  WtkLabel* panPot;
  WtkLabel* pitchBend;
  
  WtkLabel* keyboard;

  WtkLabel* chorusSendLevel;
  WtkLabel* delaySendLevel;
  WtkLabel* damperPedal;
  WtkLabel* sostenute;
  WtkLabel* softPedal;
  WtkLabel* portament;
  
  WtkLabel* vibrate;
  WtkLabel* vibrateRate;
  WtkLabel* vibrateDepth;
  WtkLabel* vibrateDelay;
  WtkLabel* tvfCutOff;
  WtkLabel* tvfResonance;
  WtkLabel* envelope;
  WtkLabel* envelopeAttackTime;
  WtkLabel* envelopeDecayTime;
  WtkLabel* envelopeReleaseTime;
  
  WtkLabel* port;

  //resource name$B$O(B(port*MIDI::MAX_CHANNEL+channel)$B$NHV9f$r3d$jEv$F$k!#(B
  WtkPart2* part[MIDI::MAX_PORT][MIDI::MAX_CHANNEL];
  
  void initialize ();//non virtual
  void getResource (){};//resource manager$B$+$i@_Dj$r<hF@(B non virtual
  void setResource (){};//resource$B$r@_Dj!#(B non virtual
public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkContainerPart2 (Display* display,const char* className_,const char* name_,
		     int x_,int y_,uint width_,uint height_,
		     int depth,Visual* visual,Colormap colormap_,
		     char* resourceFileName,
		     WidiController* widiController_,
		     WtkViewPort* viewPort0_,WtkViewPort* viewPort1_,
		     WtkViewPort* viewPort2_);
  WtkContainerPart2 (Display* display,const char* className_,const char* name_,
		     int x_,int y_,uint width_,uint height_,
		     int depth,Visual* visual,Colormap colormap_,
		     XrmDatabase displayRdb_,
		     WidiController* widiController_,
		     WtkViewPort* viewPort0_,WtkViewPort* viewPort1_,
		     WtkViewPort* viewPort2_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkContainerPart2 (WtkWidget* parent,
		     const char* className_, const char* name_,
		     int x_, int y_, uint width_, uint height_,
		     WidiController* widiController_,
		     WtkViewPort* viewPort0_, WtkViewPort* viewPort1_,
		     WtkViewPort* viewPort2_);
  virtual ~WtkContainerPart2 ();

  virtual void map ();
  virtual void unmap ();
  //XXX
  //$B$3$l$GFbIt$N>uBV$r99?7$9$k!#(B
  virtual void update (State& state,
		       LevelInfo level[MIDI::MAX_PORT][MIDI::MAX_CHANNEL]);
  virtual void reset ();//$BJQ?t$r1iAUA0$K=i4|2=$9$k!#(B
  virtual void exposed (XEvent* /*event*/){decorate ();};//expose event$B$r=hM}$9$k!#(B
  //event handler$B$K(Bwidget$B$rEPO?$9$k!#(B
  virtual void registerEventHandler (WtkEventHandler* eventHandler);
  virtual void printResource () const;
};
#endif// WtkContainerPart2_H
