/*
    WtkKeyboard.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "WtkKeyboard.h"

const char* const WtkKeyboard::defaultResource=
"";

const bool WtkKeyboard::isBlack[OCTAVE]=
{false,true,false,true,false,false,true,false,true,false,true,false};
//0    1    2     3    4    |5     6    7     8    9     10   11

WtkKeyboard::WtkKeyboard
(Display* display,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_,
 int depth,Visual* visual,Colormap colormap_,
 char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName)
{
  initialize ();
}

WtkKeyboard::WtkKeyboard
(Display* display,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_,
 int depth,Visual* visual,Colormap colormap_,
 XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_)
{
  initialize ();
}

WtkKeyboard::WtkKeyboard
(WtkWidget* parent,const char* className_,const char* name_,
 int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_)
{
  initialize ();
}

WtkKeyboard::~WtkKeyboard ()
{
  delete pixmap;
}

void WtkKeyboard::initialize ()
{
  resetCacheNote ();
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();
}


void WtkKeyboard::resetCacheNote ()
{
  for (int i=0;i<MAX_OCTAVES*OCTAVE;i++)
    cacheNote[i]=false;
}

void WtkKeyboard::getResource ()
{
  octaveWidth=getResourceValue ("octaveWidth","OctaveWidth");
  octaveHeight=getResourceValue ("octaveHeight","OctaveHeight");
  keyWidth=octaveWidth/7;//7=$BGr80$N?t(B
  octaveWidth=keyWidth*7;
  keyHeight=octaveHeight;
  blackKeyWidth=getResourceValue ("blackKeyWidth","BlackKeyWidth");
  blackKeyHeight=getResourceValue ("blackKeyHeight","BlackKeyHeight");

  whiteKeyColor.allocNamed (window->getDisplay (),style.getColormap (),
			    getResource ("whiteKeyColor","whiteKeyColor"));
  blackKeyColor.allocNamed (window->getDisplay (),style.getColormap (),
			    getResource ("blackKeyColor","blackKeyColor"));


  position[0].x=0;
  position[0].y=0;
  position[1].x=keyWidth-blackKeyWidth/2;
  position[1].y=0;
  position[2].x=keyWidth;
  position[2].y=0;
  position[3].x=keyWidth*2-blackKeyWidth/2;
  position[3].y=0;
  position[4].x=keyWidth*2;
  position[4].y=0;
  position[5].x=keyWidth*3;
  position[5].y=0;
  position[6].x=keyWidth*4-blackKeyWidth/2;
  position[6].y=0;
  position[7].x=keyWidth*4;
  position[7].y=0;
  position[8].x=keyWidth*5-blackKeyWidth/2;
  position[8].y=0;
  position[9].x=keyWidth*5;
  position[9].y=0;
  position[10].x=keyWidth*6-blackKeyWidth/2;
  position[10].y=0;
  position[11].x=keyWidth*6;
  position[11].y=0;

  octaves=width/octaveWidth;
  if (octaves<=0)octaves=1;
  if (octaves>MAX_OCTAVES) octaves=MAX_OCTAVES;

  /*
  cerr << "WtkKeyboard::keyWidth=" << keyWidth << endl;
  cerr << "WtkKeyboard::octaveWidth=" << octaveWidth << endl;
  cerr << "WtkKeyboard::octaves=" << octaves << endl;
  cerr << "WtkKeyboard::OCTAVE" << OCTAVE << endl;
  cerr << "WtkKeyboard::max_notes" << octaves*OCTAVE << endl;
  */
}


void WtkKeyboard::setResource ()
{
  pixmap=new WdkPixmap (window,octaveWidth,octaveHeight);

  /////////////////////////////////////////////make background pixmap

  //pixmap$B$N(Bclear
  gc->setForeground (whiteKeyColor);
  pixmap->fillRectangle (gc,0,0,octaveWidth,octaveHeight);
  //$BGr80$NOH$rIA$/(B
  gc->setForeground (style.bg);
  for (int i=0;i<OCTAVE;i++)
    {
      if (!isBlack[i])
	pixmap->drawLine (gc,position[i].x,position[i].y,
			  position[i].x,position[i].y+octaveHeight);
    }
  pixmap->drawLine (gc,0,octaveHeight-1,octaveWidth-1,octaveHeight-1);

  //$B9u80$rIA$/(B
  gc->setForeground (blackKeyColor);
  for (int i=0;i<OCTAVE;i++)
    {
      if (isBlack[i])
	pixmap->fillRectangle (gc,position[i].x,position[i].y,
			       blackKeyWidth,blackKeyHeight);
    }

  window->setBackgroundPixmap (pixmap);
}


void WtkKeyboard::noteOn (int note)
{
  const int octaveOffset=note/OCTAVE-1;
  const int keyOffset=note%OCTAVE;

#if 0
  cerr << "WtkKeyboard::noteOn note=" << note << endl;
  cerr << "WtkKeyboard::noteOn noctaveOffset=" << octaveOffset << endl;
  cerr << "WtkKeyboard::noteOn keyOffset=" << keyOffset << endl;
  cerr << "WtkKeyboard::noteOn octaveWidth=" << octaveWidth << endl;
  cerr << "WtkKeyboard::noteOn keyWidth=" << keyWidth << endl;
#endif
  
  if (!isBlack[keyOffset])
    {//$BGr80$@$C$?!#(B
      window->fillRectangle (gc,
			     octaveOffset*octaveWidth+position[keyOffset].x+1,
			     position[keyOffset].y+blackKeyHeight,
			     keyWidth-1,keyHeight-blackKeyHeight);
    }
  else
    {//$B9u80$@$C$?!#(B
      window->fillRectangle (gc,
			     octaveOffset*octaveWidth+position[keyOffset].x,
			     position[keyOffset].y,
			     blackKeyWidth,blackKeyHeight);
    }
}

void WtkKeyboard::noteOff (int note)
{
  const int octaveOffset=note/OCTAVE-1;
  const int keyOffset=note%OCTAVE;
  if (!isBlack[keyOffset])
    {//$BGr80$@$C$?!#(B
      window->clearArea (octaveOffset*octaveWidth+position[keyOffset].x,
			 position[keyOffset].y+blackKeyHeight,
			 keyWidth,keyHeight-blackKeyHeight,False);
    }
  else
    {//$B9u80$@$C$?!#(B
      window->clearArea (octaveOffset*octaveWidth+position[keyOffset].x,
			 position[keyOffset].y,
			 blackKeyWidth,blackKeyHeight,False);
    }
}

void WtkKeyboard::update (const Part& nowPart,const SongStatus& nowSongStatus)
{
  lockData ();
  //GC$B$N@_Dj(B
  for (int i=0;i<Part::MAX_NOTES;i++)
    {
#define DELAY (nowSongStatus.rate*20/100)//0.2 second
      bool isNoteOn=(nowPart.note[i] ||
		     (nowSongStatus.machineTicks-nowPart.noteOnMachineTicks[i]<DELAY));
#undef DELAY
      gc->setForeground (style.fg);
      if (isNoteOn!=cacheNote[i])
	{//$B=q$-49$($NI,MW$,$"$C$?!#(B
	  if (isNoteOn)
	    {//key$B$,2!$5$l$?(B(note on)
	      noteOn (i%(octaves*OCTAVE));
	      cacheNote[i%(octaves*OCTAVE)]=true;//$B8=:_$N>uBV$r3P$($F$*$/!#(B
	    }
	  else
	    {//key$B$,N%$5$l$?(B(note off)
	      noteOff (i);
	      cacheNote[i]=false;//$B8=:_$N>uBV$r3P$($F$*$/!#(B
	    }
	}
    }
  unlockData ();
}

void WtkKeyboard::exposed (XEvent* /*event*/)
{
  lockData ();
  gc->setForeground (style.fg);
  for (int i=0;i<octaves*OCTAVE;i++)
    {
      if (cacheNote[i])//key$B$,2!$5$l$F$$$k!#(B(note is on)
	noteOn (i);
    }
  unlockData ();
}
