/*
    WtkKeyboard.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkKeyboard_H
#define WtkKeyboard_H

#include "Sequencer/Part.h"
#include "WidiToolkit/WdkPixmap.h"
#include "WidiToolkit/WtkWidget.h"


#define OCTAVE 12 //1 octave$B$O(B12$B2;$+$i$J$k(B
#define MAX_OCTAVES 11//note$BHV9f$O(B0-127$B$J$N$G:GBg(B11octavem$B$^$G!#(B

class WtkKeyboard:public WtkWidget
{
protected:
  static const char* const defaultResource;
  bool cacheNote[MAX_OCTAVES*OCTAVE];
  void resetCacheNote ();
  int octaves;//$BI=<($5$l$F$$$k%-!<$N%*%/%?!<%V?t!#2?%*%/%?!<%VI=<(=PMh$k$+(B?
  uint octaveWidth;//1 octave$B$N%-!<%\!<%I$N(Bwidth
  uint octaveHeight;//keyboard$B$N(Bheight
  uint keyWidth;//key$B$N(Bwidth$B!#$3$l$O(BoctaveWidth$B$+$i7W;;$5$l$k!#(B
  uint keyHeight;//=octaveHeight
  uint blackKeyWidth;//$B9u80$N(Bwidth
  uint blackKeyHeight;//$B9u80$N(Bheight
  WdkColor whiteKeyColor;//$BGr80$N?'(B
  WdkColor blackKeyColor;//$B9u80$N?'(B
  //note on$B$N?'$O(Bforeground$B$r;H$&(B
  static const bool isBlack[OCTAVE];
  struct Position
  {
    int x;
    int y;
  };
  Position position[OCTAVE];//note on$B$rIA$/$H$-$N(Boffset
  
  
  WdkPixmap* pixmap;//background$B$KD%$k(Bpixmap
  void noteOn (int note);
  void noteOff (int note);
  
  
  void initialize ();//$B=i4|2=(B
  void getResource ();//resource manager$B$+$i@_Dj$r<hF@(B non virtual
  char* getResource (char* resourceName,char* resouceClass)
    {return WtkWidget::getResource (resourceName,resouceClass);};
  void setResource ();//resource$B$r@_Dj!#(B non virtual

public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkKeyboard (Display* display,const char* className_,const char* name_,
	       int x_,int y_,uint width_,uint height_,
	       int depth,Visual* visual,Colormap colormap_,
	       char* resourceFileName);
  WtkKeyboard (Display* display,const char* className_,const char* name_,
	       int x_,int y_,uint width_,uint height_,
	       int depth,Visual* visual,Colormap colormap_,
	       XrmDatabase displayRdb_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkKeyboard (WtkWidget* parent,const char* className_,const char* name_,
	       int x_,int y_,uint width_,uint height_);
  virtual ~WtkKeyboard ();

  virtual void update (const Part& nowPart,const SongStatus& nowSongStatus);
  virtual void exposed (XEvent* event);//expose event$B$r=hM}$9$k!#(B
};
#endif// WtkKeyboard_H
