/*
    WtkLCD.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "WtkLCD.h"

const char* const WtkLCD::defaultResource=
"";

const char WtkLCD::defaultLetters[MAX_DISPLAYABLE_LETTERS+1]=
"- WIDI  Canvas -";

WtkLCD::WtkLCD (Display* display,const char* className_,const char* name_,
		int x_,int y_,uint width_,uint height_,
		int depth,Visual* visual,Colormap colormap_,
		char* resourceFileName):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,resourceFileName)
{
  initialize ();
}

WtkLCD::WtkLCD (Display* display,const char* className_,const char* name_,
		int x_,int y_,uint width_,uint height_,
		int depth,Visual* visual,Colormap colormap_,
		XrmDatabase displayRdb_):
  WtkWidget (display,className_,name_,x_,y_,width_,height_,
	     depth,visual,colormap_,displayRdb_)
{
  initialize ();
}

WtkLCD::WtkLCD (WtkWidget* parent,const char* className_,const char* name_,
		int x_,int y_,uint width_,uint height_):
  WtkWidget (parent,className_,name_,x_,y_,width_,height_)
{
  initialize ();
}

WtkLCD::~WtkLCD ()
{
  delete pixmap;
  for (int i=0;i<96;i++)
    delete letters[i];
}

void WtkLCD::resetInfo ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_PORT; channel++)
	levelInfo[port][channel].reset ();
    }
  lcdInfo.reset ();
  oldLCDInfo.reset ();
}


void WtkLCD::initialize ()
{
  strcpy (nowLetters,defaultLetters);
  resetInfo ();
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();
}

void WtkLCD::getResource ()
{
  lcdWidth=getResourceValue ("lcdWidth","LcdWidth");
  lcdHeight=getResourceValue ("lcdHeight","LcdHeight");
  lcdHorizontalDistance=getResourceValue ("lcdHorizontalDistance",
					  "LcdHorizontalDistance");
  lcdVerticalDistance=getResourceValue ("lcdVerticalDistance",
					"LcdVerticalDistance");
  
  letterWidth=getResourceValue ("letterWidth","LetterWidth");
  letterHeight=getResourceValue ("letterHeight","LetterHeight");
  
  lcdOffColor.allocNamed (window->getDisplay (),style.getColormap (),
			  getResource ("lcdOffColor","LcdOffColor"));

  //lcd$B$N0LCV$r7hDj!#(B
  for (int i=0;i<MIDI::MAX_CHANNEL;i++)
    {
      letterPosition[i].x=(letterWidth+lcdHorizontalDistance)*i;
      letterPosition[i].y=0;
    }
  
  for (int x=0;x<LCDInfo::LCD_COLS;x++)
    {
      for (int y=0;y<LCDInfo::LCD_LINES;y++)
	{
	  lcdPosition[y][x].x=(lcdWidth+lcdHorizontalDistance)*x;
	  lcdPosition[y][x].y=(lcdHeight+lcdVerticalDistance)*y+
	    letterHeight+lcdVerticalDistance;
	}
    }
}

#include "IO/ioutil/lcdchars.h"
void WtkLCD::setResource ()
{
  pixmap=new WdkPixmap (window,width,height);
  /////////////////////////////////////////////make background pixmap
  gc->setForeground (style.bg);
  pixmap->fillRectangle (gc,0,0,width,height);
  gc->setForeground (lcdOffColor);
  for (int i=0;i<MIDI::MAX_CHANNEL;i++)
    pixmap->fillRectangle (gc,letterPosition[i].x,letterPosition[i].y,
			   letterWidth,letterHeight);
  for (int x=0;x<LCDInfo::LCD_COLS;x++)
    {
      for (int y=0;y<LCDInfo::LCD_LINES;y++)
	pixmap->fillRectangle (gc,lcdPosition[y][x].x,lcdPosition[y][x].y,
			       lcdWidth,lcdHeight);
    }
  window->setBackgroundPixmap (pixmap);

  //make letters for LCD
  const int dotWidth=letterWidth/5;
  const int dotHeight=letterHeight/7;
#ifdef DEBUG
  //cerr << "dotWidth=" << dotWidth  << ",dotHeight=" << dotHeight << endl;
#endif
  for (int i=0;i<LCD_LETTERS;i++)
    {
      letters[i]=new WdkPixmap (window,letterWidth,letterHeight);
      gc->setForeground (lcdOffColor);
      letters[i]->fillRectangle (gc,0,0,letterWidth,letterHeight);
      //draw dots
      gc->setForeground (style.fg);
      for(int y=0;y<7;y++)
	{
	  for(int x=0;x<5;x++)
	    {
	      if(lcdchars[i*7+y]&(1<<x))//lcdchars is defined in IO/lcdchars.h
		letters[i]->fillRectangle (gc,x*dotWidth+1,y*dotHeight+1,dotWidth,dotHeight);
	    }
	}
    }
}

void WtkLCD::displayLetter ()
{
  for (int i=0;i<MAX_DISPLAYABLE_LETTERS;i++)
    {
      if (32<=nowLetters[i]) //ascii
	window->copyArea (letters[nowLetters[i]-0x20],gc,
			  0,0,letterWidth,letterHeight,
			  letterPosition[i].x,letterPosition[i].y);
    }
}


void WtkLCD::update (State& nowState)
{
#define UNIT_TIME (nowState.songStatus.rate*30/100) //300ms
#define DISPLAY_TIME (nowState.songStatus.rate*240/100) //2.4 sec
  //$BI=<($9$kJ8;zNs$r5a$a$k!#(B
  const double machineTicks=nowState.songStatus.machineTicks-nowState.sequencerStatus.letterMachineTicks;
  const int scrollNum=(int) (machineTicks/UNIT_TIME);
  char displayedLetters[MAX_DISPLAYABLE_LETTERS+1];
  strcpy (displayedLetters,defaultLetters);
  const int letterLength=strlen (nowState.sequencerStatus.displayedLetter);
  if (letterLength<=MAX_DISPLAYABLE_LETTERS)
    {//16$BJ8;z0J2<!#0lDj;~4VI=<((B
      if (machineTicks<DISPLAY_TIME)
	{//$B$^$@I=<($7$F$$$k!#(B
	  strcpy (displayedLetters,nowState.sequencerStatus.displayedLetter);
	  for (int i=letterLength;i<MAX_DISPLAYABLE_LETTERS;i++)
	    displayedLetters[i]=' ';
	  displayedLetters[MAX_DISPLAYABLE_LETTERS+1]='\0';
	}
    }
  else
    {//16$B$h$jBg!#(Bscroll$B$7$FI=<(!#(B
      if (scrollNum<letterLength+MAX_DISPLAYABLE_LETTERS+1)
	{//scroll$B$N:GCf!#(B
	  if (scrollNum<=MAX_DISPLAYABLE_LETTERS)
	    {
	      strcpy (displayedLetters,defaultLetters+scrollNum);
	      const int offset=MAX_DISPLAYABLE_LETTERS-scrollNum;
	      displayedLetters[offset]='<';
	      for (int i=offset+1;i<MAX_DISPLAYABLE_LETTERS;i++)
		displayedLetters[i]=nowState.sequencerStatus.displayedLetter[i-offset-1];
	    }
	  else
	    {
	      strncpy (displayedLetters,
		       nowState.sequencerStatus.displayedLetter+
		       scrollNum-MAX_DISPLAYABLE_LETTERS,
		       MAX_DISPLAYABLE_LETTERS);
	      const int offset=letterLength-(scrollNum-MAX_DISPLAYABLE_LETTERS);
	      if (offset<MAX_DISPLAYABLE_LETTERS)
		{
		  displayedLetters[offset]='<';
		  for (int i=offset+1;i<MAX_DISPLAYABLE_LETTERS;i++)
		    displayedLetters[i]=defaultLetters[i-offset-1];
		}
	    }
	  displayedLetters[MAX_DISPLAYABLE_LETTERS+1]='\0';
	}
    }
#undef UNIT_TIME
#undef DISPLAY_TIME
  
  
  lockData ();
  if (strcmp (nowLetters,displayedLetters)!=0)
    {//$BI=<(J8;zNs$KJQ99$,$"$C$?$N$G!"99?7$9$k!#(B
      strcpy (nowLetters,displayedLetters);
      displayLetter ();
    }
  
    
  //lcd$B$N>uBV$r5a$a$k!#(B
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  levelInfo[port][channel].getLevel (nowState, port, channel);
	  //levelInfo[port][channel].getLevel2 (nowState, port, channel);
	}
    }
  lcdInfo.update (nowState, levelInfo);
  
  //lcd$B$N>uBV$r99?7$9$k!#(B
  gc->setForeground (style.fg);
  for (int x=0;x<LCDInfo::LCD_COLS;x++)
    {
      for (int y=0;y<LCDInfo::LCD_LINES;y++)
	{
	  if (lcdInfo.isOn[y][x]!=oldLCDInfo.isOn[y][x])
	    {//$BJQ2=$5$;$kI,MW$,$"$C$?!#(B
	      if (lcdInfo.isOn[y][x])
		window->fillRectangle (gc,
				       lcdPosition[y][x].x,
				       lcdPosition[y][x].y,
				       lcdWidth,lcdHeight);
	      else
		window->clearArea (lcdPosition[y][x].x,lcdPosition[y][x].y,
				   lcdWidth,lcdHeight,False);
	    }
	}
    }
  
  oldLCDInfo=lcdInfo;//$B8=:_$N>uBV$r3P$($F$*$/!#(B
  unlockData ();
}

void WtkLCD::exposed (XEvent* /*event*/)
{
  lockData ();
  gc->setForeground (style.fg);

  displayLetter ();
  for (int x=0;x<LCDInfo::LCD_COLS;x++)
    {
      for (int y=0;y<LCDInfo::LCD_LINES;y++)
	{
	  if (lcdInfo.isOn[y][x])
	    window->fillRectangle (gc,lcdPosition[y][x].x,lcdPosition[y][x].y,
				   lcdWidth,lcdHeight);
	}
    }

  unlockData ();
}



