/*
    WtkPart2.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>
#include "IO/ioutil/LevelInfo.h"
#include "WtkPart2.h"

using namespace std;

const char* const WtkPart2::defaultResource="";

WtkPart2::WtkPart2 (Display* display,
		    const char* className_, const char* name_,
		    int x_, int y_, uint width_, uint height_,
		    int depth, Visual* visual, Colormap colormap_,
		    char* resourceFileName, WtkViewPort* viewPort_):
  WtkWidget (display, className_, name_, x_, y_, width_, height_,
	     depth, visual, colormap_, resourceFileName),
  widiController (NULL),  cachePart (), viewPort (viewPort_)
{
  initialize (className_, name_);
}

WtkPart2::WtkPart2 (Display* display,
		    const char* className_, const char* name_,
		    int x_, int y_, uint width_, uint height_,
		    int depth, Visual* visual, Colormap colormap_,
		    XrmDatabase displayRdb_, WtkViewPort* viewPort_):
  WtkWidget (display, className_, name_, x_, y_, width_, height_,
	     depth, visual, colormap_, displayRdb_),
  widiController (NULL), cachePart (), viewPort (viewPort_)
{
  initialize (className_, name_);
}

WtkPart2::WtkPart2 (WtkWidget* parent,
		    const char* className_, const char* name_,
		    int x_, int y_, uint width_, uint height_,
		    WtkViewPort* viewPort_):
  WtkWidget (parent, className_, name_, x_, y_, width_, height_),
  widiController (NULL), cachePart (),viewPort (viewPort_)
{
  initialize (className_, name_);
}

WtkPart2::~WtkPart2 ()
{
  delete channel;
  delete instrument;
  delete program;
  delete bankMSB;
  delete bankLSB;
  
  delete level;
  delete volume;
  delete expression;
  delete modulation;
  delete reverbSendLevel;
  delete panPot;
  delete pitchBend;
  
  delete chorusSendLevel;
  delete delaySendLevel;
  delete damperPedal;
  delete sostenute;
  delete softPedal;
  delete portament;
  
  delete vibrateRate;
  delete vibrateDepth;
  delete vibrateDelay;
  delete tvfCutOff;
  delete tvfResonance;
  delete envelopeAttackTime;
  delete envelopeDecayTime;
  delete envelopeReleaseTime;
  
  delete viewPortWidget;
  delete port;
}

void WtkPart2::doMute (const XEvent* xevent)
{
  assert (playChannel != CHANNEL_IS_NOT_SET);
  if (playChannel == CHANNEL_IS_NOT_SET)
    return;
  
  assert (xevent != NULL);
  assert (xevent->type == ButtonPress);
  cerr << "button " << xevent->xbutton.button << " clicked" << endl;
  switch (xevent->xbutton.button)
    {
    case 1:
      {//$B;XDj$5$l$?%A%c%s%M%k$N(Bmute$B>uBV$r(Btoggle$B$9$k!#(B
	Sequencer::Muting muting;
	muting.port = playPort;
	muting.channel = playChannel;
	widiController->serverCommand->toggleMute (&muting, 1);
	break;
      }
    case 2:
      {//$B;XDj$5$l$?%A%c%s%M%k$N(Bmute$B>uBV$r(Bmute$B$r2r=|$9$k!#(B
	Sequencer::Muting muting;
	muting.port = playPort;
	muting.channel = playChannel;
	muting.isMute = false;
	widiController->serverCommand->setMute (&muting, 1);
	break;
      }
    case 3:
      {//$B;XDj$5$l$?%A%c%s%M%k0J30A4$F$N(Bmute$B>uBV$r(Btogglew$B$9$k!#(B
	const int size = MIDI::MAX_PARTS-1;//for simplicity
	Sequencer::Muting muting[size];
	for (int i = 0; i < playChannel; i++)
	  {
	    muting[i].port = (i/MIDI::MAX_CHANNEL);
	    muting[i].channel = (i % MIDI::MAX_CHANNEL);
	  }
	for (int i = playChannel; i < size; i++)
	  {
	    muting[i].port = (i+1)/MIDI::MAX_CHANNEL;
	    muting[i].channel = (i+1) % MIDI::MAX_CHANNEL;
	  }
	widiController->serverCommand->toggleMute (muting, size);
	break;
      }
    default:
      //do nothing
      break;
    }
}


void WtkPart2::initialize (const char* className_,const char* name_)
{
  assert (viewPort!=NULL);
  XrmCombineDatabase (XrmGetStringDatabase (defaultResource),&privateRdb,False);
  getResource ();
  setResource ();

  //cachePart.reset ();
  playChannel = CHANNEL_IS_NOT_SET;
  channel=new WtkOnOffLabel (this,"Channel","channel",-1,-1,0,0);
  channel->clicked.connect (this, &WtkPart2::doMute);

  instrument=new WtkLabel (this,"Instrument","instrument",-1,-1,0,0);
  program=new WtkLabel (this,"Program","program",-1,-1,0,0);
  bankMSB=new WtkLabel (this,"BankMSB","bankMSB",-1,-1,0,0);
  bankLSB=new WtkLabel (this,"BankLSB","bankLSB",-1,-1,0,0);

  level=new WtkLevel (this,"Level","level",-1,-1,0,0);
  volume=new WtkLevel (this,"Volume","volume",-1,-1,0,0);
  expression=new WtkLevel (this,"Expression","expression",-1,-1,0,0);
  modulation=new WtkLevel (this,"Modulation","modulation",-1,-1,0,0);
  reverbSendLevel=new WtkLevel (this,"ReverbSendLevel","reverbSendLevel",-1,-1,0,0);
  panPot=new WtkMeter (this,"PanPot","panPot",-1,-1,0,0);
  pitchBend=new WtkMeter (this,"PitchBend","pitchBend",-1,-1,0,0);
                                  

  viewPortWidget=new WtkWidget (viewPort,className_,name_,-1,-1,0,0);
  chorusSendLevel=new WtkLabel (viewPortWidget,"ChorusSendLevel","chorusSendLevel",-1,-1,0,0);
  delaySendLevel=new WtkLabel (viewPortWidget,"DelaySendLevel","delaySendLevel",-1,-1,0,0);
  damperPedal=new WtkLabel (viewPortWidget,"DamperPedal","damperPedal",-1,-1,0,0);
  sostenute=new WtkLabel (viewPortWidget,"Sostenute","sostenute",-1,-1,0,0);
  softPedal=new WtkLabel (viewPortWidget,"SoftPedal","softPedal",-1,-1,0,0);
  portament=new WtkLabel (viewPortWidget,"Portament","portament",-1,-1,0,0);
                                  
  vibrateRate=new WtkLabel (viewPortWidget,"VibrateRate","vibrateRate",-1,-1,0,0);
  vibrateDepth=new WtkLabel (viewPortWidget,"VibrateDepth","vibrateDepth",-1,-1,0,0);
  vibrateDelay=new WtkLabel (viewPortWidget,"VibrateDelay","vibrateDelay",-1,-1,0,0);
  tvfCutOff=new WtkLabel (viewPortWidget,"TvfCutOff","tvfCutOff",-1,-1,0,0);
  tvfResonance=new WtkLabel (viewPortWidget,"TvfResonance","tvfResonance",-1,-1,0,0);
  envelopeAttackTime=new WtkLabel (viewPortWidget,"EnvelopeAttackTime","envelopeAttackTime",-1,-1,0,0);
  envelopeDecayTime=new WtkLabel (viewPortWidget,"EnvelopeDecayTime","envelopeDecayTime",-1,-1,0,0);
  envelopeReleaseTime=new WtkLabel (viewPortWidget,"EnvelopeReleaseTime","envelopeReleaseTime",-1,-1,0,0);
                                  
  port=new WtkLabel (viewPortWidget,"PortNumber","portNumber",-1,-1,0,0);//$B=PNO$7$F$$$k%]!<%HHV9f!#(B
}

void WtkPart2::reset ()
{
  cachePart.reset ();
  cacheLevel=0;
}


void WtkPart2::setPortChannel (MIDI::PORT port__, uchar channel__)
{
  assert (playChannel == CHANNEL_IS_NOT_SET);
  playPort = port__;
  playChannel = channel__;
  channel->setLabel (playPort*MIDI::MAX_CHANNEL+playChannel+1);
}

void WtkPart2::setWidiController (WidiController* widiController_)
{
  assert (widiController == NULL);
  assert (widiController_ != NULL);
  widiController = widiController_;
}

void WtkPart2::update (Part* part, double level_)
{
  cachePart = *part;
  cacheLevel = level_;
  
  //channel;
  channel->set (!part->isMuted);
  instrument->setLabel (cachePart.instrument);
  program->setLabel (cachePart.program);
  bankMSB->setLabel (cachePart.bankMSB);
  bankLSB->setLabel (cachePart.bankLSB);
                                  
  level->setLevel (cacheLevel/MAX_LEVEL);
  volume->setLevel (cachePart.channelVolume/static_cast<double> (Part::MAX_CHANNEL_VOLUME));
  expression->setLevel (cachePart.expression/static_cast<double> (Part::MAX_EXPRESSION));
  modulation->setLevel (cachePart.modulationWheel/static_cast<double> (Part::MAX_MODULATION));
  reverbSendLevel->setLevel (cachePart.reverbSendLevel/static_cast<double> (Part::MAX_REVERB_SEND_LEVEL));
  panPot->setMeter ((static_cast<double> (cachePart.pan)-Part::CENTER_PAN)/Part::MAX_PAN);
  pitchBend->setMeter (cachePart.pitchBend/static_cast<double> (Part::MAX_PITCH_BEND));
                                  
  chorusSendLevel->setLabel (cachePart.chorusSendLevel);
  delaySendLevel->setLabel (cachePart.delaySendLevel);
  damperPedal->setLabel (cachePart.damperPedal);
  sostenute->setLabel (cachePart.sostenute);
  softPedal->setLabel (cachePart.softPedal);
  portament->setLabel (cachePart.portamentSwitch);
                                  
  vibrateRate->setLabel (cachePart.vibrateRate);
  vibrateDepth->setLabel (cachePart.vibrateDepth);
  vibrateDelay->setLabel (cachePart.vibrateDelay);
  tvfCutOff->setLabel (cachePart.tvfCutOff);
  tvfResonance->setLabel (cachePart.tvfResonance);
  envelopeAttackTime->setLabel (cachePart.envelopeAttackTime);
  envelopeDecayTime->setLabel (cachePart.envelopeDecayTime);
  envelopeReleaseTime->setLabel (cachePart.envelopeReleaseTime);
                                  
  char buff[5];
  sprintf (buff, "%c%d", 'A'+cachePart.port, cachePart.RxChannel);
  port->setLabel (buff);
}

void WtkPart2::registerEventHandler (WtkEventHandler* eventHandler)
{
  eventHandler->addWidget (this);
  
  eventHandler->addWidget (channel);
  eventHandler->addWidget (instrument);
  eventHandler->addWidget (program);
  eventHandler->addWidget (bankMSB);
  eventHandler->addWidget (bankLSB);
                                
  eventHandler->addWidget (level);
  eventHandler->addWidget (volume);
  eventHandler->addWidget (expression);
  eventHandler->addWidget (modulation);
  eventHandler->addWidget (reverbSendLevel);
  eventHandler->addWidget (panPot);
  eventHandler->addWidget (pitchBend);
                                
  eventHandler->addWidget (viewPortWidget);
  eventHandler->addWidget (chorusSendLevel);
  eventHandler->addWidget (delaySendLevel);
  eventHandler->addWidget (damperPedal);
  eventHandler->addWidget (sostenute);
  eventHandler->addWidget (softPedal);
  eventHandler->addWidget (portament);
                                
  eventHandler->addWidget (vibrateRate);
  eventHandler->addWidget (vibrateDepth);
  eventHandler->addWidget (vibrateDelay);
  eventHandler->addWidget (tvfCutOff);
  eventHandler->addWidget (tvfResonance);
  eventHandler->addWidget (envelopeAttackTime);
  eventHandler->addWidget (envelopeDecayTime);
  eventHandler->addWidget (envelopeReleaseTime);
                                
  eventHandler->addWidget (port);//$B=PNO$7$F$$$k%]!<%HHV9f!#(B
}

void WtkPart2::map ()
{
  channel->map ();
  instrument->map ();
  program->map ();
  bankMSB->map ();
  bankLSB->map ();
                                
  level->map ();
  volume->map ();
  expression->map ();
  modulation->map ();
  reverbSendLevel->map ();
  panPot->map ();
  pitchBend->map ();
                                
  viewPortWidget->map ();
  chorusSendLevel->map ();
  delaySendLevel->map ();
  damperPedal->map ();
  sostenute->map ();
  softPedal->map ();
  portament->map ();
                                
  vibrateRate->map ();
  vibrateDepth->map ();
  vibrateDelay->map ();
  tvfCutOff->map ();
  tvfResonance->map ();
  envelopeAttackTime->map ();
  envelopeDecayTime->map ();
  envelopeReleaseTime->map ();
                                
  port->map ();

  WtkWidget::map () ;
}

void WtkPart2::unmap ()
{
  
  channel->unmap ();
  instrument->unmap ();
  program->unmap ();
  bankMSB->unmap ();
  bankLSB->unmap ();
                                
  level->unmap ();
  volume->unmap ();
  expression->unmap ();
  modulation->unmap ();
  reverbSendLevel->unmap ();
  panPot->unmap ();
  pitchBend->unmap ();
                                
  viewPortWidget->map ();
  chorusSendLevel->unmap ();
  delaySendLevel->unmap ();
  damperPedal->unmap ();
  sostenute->unmap ();
  softPedal->unmap ();
  portament->unmap ();
                                
  vibrateRate->unmap ();
  vibrateDepth->unmap ();
  vibrateDelay->unmap ();
  tvfCutOff->unmap ();
  tvfResonance->unmap ();
  envelopeAttackTime->unmap ();
  envelopeDecayTime->unmap ();
  envelopeReleaseTime->unmap ();
                                
  port->unmap ();

  WtkWidget::unmap () ;
}


void WtkPart2::printResource () const
{
  WtkWidget::printResource ();
  channel->printResource ();
  instrument->printResource ();
  program->printResource ();
  bankMSB->printResource ();
  bankLSB->printResource ();

  level->printResource ();
  volume->printResource ();
  expression->printResource ();
  modulation->printResource ();
  reverbSendLevel->printResource ();
  panPot->printResource ();
  pitchBend->printResource ();

  chorusSendLevel->printResource ();
  delaySendLevel->printResource ();
  damperPedal->printResource ();
  sostenute->printResource ();
  softPedal->printResource ();
  portament->printResource ();

  vibrateRate->printResource ();
  vibrateDepth->printResource ();
  vibrateDelay->printResource ();
  tvfCutOff->printResource ();
  tvfResonance->printResource ();
  envelopeAttackTime->printResource ();
  envelopeDecayTime->printResource ();
  envelopeReleaseTime->printResource ();

  viewPortWidget->printResource ();
  port->printResource ();
}
