/*
    WtkPart2.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef WtkPart2_H
#define WtkPart2_H

#include "Sequencer/Part.h"
#include "Widi/ServerCommand.h"
#include "Widi/WidiController.h"

#include "WidiToolkit/WtkEventHandler.h"
#include "WidiToolkit/WtkOnOffLabel.h"
#include "WidiToolkit/WtkLevel.h"
#include "WidiToolkit/WtkMeter.h"
#include "WidiToolkit/WtkViewPort.h"

class WtkPart2:public WtkWidget
{
protected:
  static const char* const defaultResource;

  WidiController* widiController;
  Part cachePart;
  double cacheLevel;

  WtkViewPort* viewPort;//scroll$B$5$;$k0Y$N(Bview port
  WtkWidget*  viewPortWidget;

  MIDI::PORT playPort;//port$B$r3P$($F$*$/!#(B
  int playChannel;//channel$B$r3P$($F$*$/(B
  static const int CHANNEL_IS_NOT_SET = -1;

  //(port*MIDI::MAX_CHANNEL+channel+1)$B$rI=<((B
  WtkOnOffLabel* channel;//channel 1-MIDI::MAX_PARTS
  void doMute (const XEvent* xevent);
  
  WtkLabel* instrument;
  WtkLabel* program;
  WtkLabel* bankMSB;
  WtkLabel* bankLSB;

  WtkLevel* level;
  WtkLevel* volume;
  WtkLevel* expression;
  WtkLevel* modulation;
  WtkLevel* reverbSendLevel;
  WtkMeter* panPot;
  WtkMeter* pitchBend;

  WtkLabel* chorusSendLevel;
  WtkLabel* delaySendLevel;
  WtkLabel* damperPedal;
  WtkLabel* sostenute;
  WtkLabel* softPedal;
  WtkLabel* portament;

  WtkLabel* vibrateRate;
  WtkLabel* vibrateDepth;
  WtkLabel* vibrateDelay;
  WtkLabel* tvfCutOff;
  WtkLabel* tvfResonance;
  WtkLabel* envelopeAttackTime;
  WtkLabel* envelopeDecayTime;
  WtkLabel* envelopeReleaseTime;

  //$B=PNO$7$F$$$k%]!<%HHV9f!#(B
  //('A'+port)($B%"%k%U%!%Y%C%H(B)+channel($B?t;z(B)$BHV9f$rI=<($9$k(B
  WtkLabel* port;

  void initialize (const char* className_, const char* name_);//non-virtual
  void getResource (){/*do nothing*/};//resource manager$B$+$i@_Dj$r<hF@(B non virtual
  void setResource (){/*do nothgin*/};//resource$B$r@_Dj!#(B non virtual
public:
  //for create toplevel
  //get from resource manager if x<0,y<0,width==0,height==0
  //ignored if resourceFileName==NULL
  WtkPart2 (Display* display,const char* className_,const char* name_,
	    int x_,int y_,uint width_,uint height_,
	    int depth,Visual* visual,Colormap colormap_,
	    char* resourceFileName,WtkViewPort* viewPort_);
  WtkPart2 (Display* display,const char* className_,const char* name_,
	    int x_,int y_,uint width_,uint height_,
	    int depth,Visual* visual,Colormap colormap_,
	    XrmDatabase displayRdb_,WtkViewPort* viewPort_);
  //for create child of some widget
  //get from resource manager if x,y,width,height==0
  WtkPart2 (WtkWidget* parent,const char* className_,const char* name_,
	    int x_,int y_,uint width_,uint height_,WtkViewPort* viewPort_);
  virtual ~WtkPart2 ();


  void reset ();//$BJQ?t$r=i4|2=$9$k!#(B
  //port$BHV9f$H(Bchannel$BHV9f$r@_Dj$9$k!#(B
  void setPortChannel (MIDI::PORT port__, uchar channel__);
  //WtkPart2$B$,A`:n$9$k(BWidiController$B$r@_Dj$9$k!#(B
  void setWidiController (WidiController* widiController_);
  virtual void update (Part* part,double level_);//$B$3$l$GFbIt$N>uBV$r99?7$9$k!#(B
  virtual void exposed (XEvent* /*event*/){decorate ();};//expose event$B$r=hM}$9$k!#(B
  //event handler$B$K(Bwidget$B$rEPO?$9$k!#(B
  virtual void registerEventHandler (WtkEventHandler* eventHandler);
  virtual void map ();
  virtual void unmap ();
  virtual void printResource () const;
};
#endif// WtkPart2_H
