#ifndef AlsaClient_H
#define AlsaClient_H

#include <iostream>
#include <cassert>
#include <sys/asoundlib.h>

using namespace std;

//currently AlsaClient uses alib.
//In future, throwing away alib, and be optimized by direct use of kernel api
//and direct management of buffer...
class AlsaClient
{
public:
  //exceptional classes
  class AError {};
  class InvalidParameter:public AError {};
  class CanNotOpenDevice:public AError {};
  class CanNotGetClientID:public AError {};
  class IoctlError:public AError {};


private:
  void* handle;
  int clientNumber;
  
  void checkResult (int result)
    {
      if (result < 0)
	{
	  cerr << "AlsaClient::blockMode:" << snd_strerror (result) << endl;
	  throw AError ();
	}
    };
  
  void setOutputEventHeader (snd_seq_event_t* event,
			     snd_seq_tick_time_t ticks,
			     int srcPort, int queue,
			     int destClient, int destPort, int channel)
    {
      assert (event != NULL);
      memset (event, 0, sizeof (snd_seq_event_t));
      event->flags = SND_SEQ_TIME_STAMP_TICK | SND_SEQ_TIME_MODE_ABS;
      event->time.tick = ticks;
      //event->source.queue;//don't care
      //event->source.client;//don't care
      event->source.port = srcPort;
      //event->source.channel;//don't care???
      event->dest.queue = queue;
      event->dest.client = destClient;
      event->dest.port = destPort;
      event->dest.channel = channel;
    };

  void setOutputEventHeader (snd_seq_event_t* event,
			     snd_seq_real_time_t real,
			     int srcPort, int queue,
			     int destClient, int destPort, int channel)
    {
      assert (event != NULL);
      event->flags = SND_SEQ_TIME_STAMP_REAL | SND_SEQ_TIME_MODE_ABS;
      event->time.real = real;
      //event->source.queue;//don't care
      //event->source.client;//don't care
      event->source.port = srcPort;
      //event->source.channel;//don't care???
      event->dest.queue = queue;
      event->dest.client = destClient;
      event->dest.port = destPort;
      event->dest.channel = channel;
    };

  
  int doNote (snd_seq_event_t* event,
	      unsigned char note, unsigned char velocity,
	      unsigned int duration);
  int AlsaClient::doNoteOn (snd_seq_event_t* event,
			    unsigned char note, unsigned char velocity);
  int doNoteOff (snd_seq_event_t* event,
		 unsigned char note, unsigned char velocity);
  int doKeyPressure (snd_seq_event_t* event,
		     unsigned char note, unsigned char keyPressure);
  int doController (snd_seq_event_t* event,
		    unsigned char controll, unsigned char value);
  int doProgramChange (snd_seq_event_t* event,
		       unsigned char program);
  int doChannelPressure (snd_seq_event_t* event,
			 unsigned char channelPressure);
  int doPitchBend (snd_seq_event_t* event,
		   unsigned char lsb, unsigned char msb);
  int doSysEx (snd_seq_event_t* event,
	       int length, const char* buff);
  int doEcho (snd_seq_event_t* event);
  
  
  
  

public:
  AlsaClient (int mode);
  ~AlsaClient ();

  
  ////////////////////////////////////////////////////////////system info
  void getPVersion (int* version);
  void getSystemInfo(snd_seq_system_info_t * info);
  
  ////////////////////////////////////////////////////////////client
  int getClientId () const {return clientNumber;};
  int blockMode (bool enable);
  
  ////////////////////////////////////////////////////////////queue
  //int getOwnedQueue () const {return ownedQueue;};//XXX
  
  int useQueue (int queue);

  int ownQueue (int queue);
  int lockQueue (int queue);
  int freeQueue (int queue);
  
  int setQueueClientInfo (snd_seq_queue_client_t* info);
  int setQueueClientInfo (int queue, bool used,
			  int low = -1/*unchange*/, int high = -1/*unchange*/);
  int getQueueClientInfo (int queue, snd_seq_queue_client_t* info);
  int getQueueClientInfo (snd_seq_queue_client_t* info);
  
  int getQueueInfo (snd_seq_queue_info_t* info);
  int setQueueInfo (int queue, bool own, bool lock, bool used);
  int setQueueInfo (int queue, int tempo, int ppq);
  int setQueueInfo (snd_seq_queue_info_t* info);

  
  ////////////////////////////////////////////////////////////subscribe
  int outputSubscribe (int port, int queue, int destClient, int destPort,
		       bool exclusive = false, bool realtime = false);
  int outputUnsubscribe (int port, int queue, int destClient, int destPort);
  
  int inputSubscribe (int srcClient, int srcPort, int queue, int port,
		      bool exclusive = false, bool realtime = false);
  int inputUnsubscribe (int srcClient, int srcPort, int queue, int port);

  

  ////////////////////////////////////////////////////////////client info
  int getClientInfo (snd_seq_client_info_t* clientInfo);
  int getClientInfo (int client, snd_seq_client_info_t* clientInfo);
  int setClientInfo (//64 is from snd_seq_client_info_t
		     char name[64],
		     unsigned int filter,
		     //8 is from snd_seq_client_info_t
		     unsigned char multicastFilter[8],
		     //32 is from snd_seq_client_info_t
		     unsigned char eventFilter[32]);


  ////////////////////////////////////////////////////////////port
  //portֹϼưŪ˳꿶롣
  //portξportInfo㤨롣
  int createPort (snd_seq_port_info_t* portInfo);
  int createPort (const char* name, int capability, int type,
		  int midiChannels, int synthVoices, int use,
		  snd_seq_port_info_t* portInfo);
  int deletePort (int port);
  int getPortInfo (int client, int port, snd_seq_port_info_t* portInfo);
  int getPortInfo (int port, snd_seq_port_info_t* portInfo);
  int setPortInfo (int port,
		   //64 is from snd_seq_port_info_t
		   char name[64],
		   unsigned int capability, unsigned int type,
		   int midiChannels, int synthVoices);
  
  
  ////////////////////////////////////////////////write event
  int eventOutput (snd_seq_event_t* event);

  //???
  //SND_SEQ_EVENT_SYSTEM
  int systemEvent ();
  
  //SND_SEQ_EVENT_NOTE       
  int note (unsigned char note, unsigned char velocity, unsigned int duration,
	    snd_seq_tick_time_t ticks,
	    int srcPort, int queue, int destClient, int destPort, int channel);
  int note (unsigned char note, unsigned char velocity, unsigned int duration,
	    snd_seq_real_time_t real,
	    int srcPort, int queue, int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_NOTEON     
  int noteOn (unsigned char note, unsigned char velocity,
	      snd_seq_tick_time_t ticks,
	      int srcPort, int queue,
	      int destClient, int destPort, int channel);
  int noteOn (unsigned char note, unsigned char velocity,
	      snd_seq_real_time_t real,
	      int srcPort, int queue,
	      int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_NOTEOFF    
  int noteOff (unsigned char note, unsigned char velocity,
	       snd_seq_tick_time_t ticks,
	       int srcPort, int queue,
	       int destClient, int destPort, int channel);
  int noteOff (unsigned char note, unsigned char velocity,
	       snd_seq_real_time_t real,
	       int srcPort, int queue,
	       int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_KEYPRESS   
  int keyPressure (unsigned char note, unsigned char keyPressure,
		   snd_seq_tick_time_t ticks,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  int keyPressure (unsigned char note, unsigned char keyPressure,
		   snd_seq_real_time_t real,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  //controlChange
  //SND_SEQ_EVENT_CONTROLLER 
  int controller (unsigned char controll, unsigned char value,
		  snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int controller (unsigned char controll, unsigned char value,
		  snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_PGMCHANGE  
  int programChange (unsigned char program,
		     snd_seq_tick_time_t ticks,
		     int srcPort, int queue,
		     int destClient, int destPort, int channel);
  int programChange (unsigned char program,
		     snd_seq_real_time_t real,
		     int srcPort, int queue,
		     int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_CHANPRESS  
  int channelPressure (unsigned char channelPressure,
		       snd_seq_tick_time_t ticks,
		       int srcPort, int queue,
		       int destClient, int destPort, int channel);
  int channelPressure (unsigned char channelPressure,
		       snd_seq_real_time_t real,
		       int srcPort, int queue,
		       int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_PITCHBEND  
  int pitchBend (unsigned char lsb, unsigned char msb,
		 snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int pitchBend (unsigned char lsb, unsigned char msb,
		 snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
#if 0
  //???
  //SND_SEQ_EVENT_CONTROL14
  int control14 (snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int control14 (snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
#endif
#if 0
  //???
  //SND_SEQ_EVENT_NONREGPARAM
  int nonRegProg (snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int nonRegProg (snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  
  //???
  //SND_SEQ_EVENT_REGPARAM
  int regProg (snd_seq_tick_time_t ticks,
	       int srcPort, int queue,
	       int destClient, int destPort, int channel);
  int regProg (snd_seq_real_time_t real,
	       int srcPort, int queue,
	       int destClient, int destPort, int channel);
#endif

#if 0
  //SND_SEQ_EVENT_SONGPOS/* Song Position Pointer with LSB and MSB values */
  //0xF2 lsb msb
  int songPosition (unsigned char lsb, unsgined char msb,
		    snd_seq_tick_time_t ticks,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
  int songPosition (unsigned char lsb, unsgined char msb,
		    snd_seq_real_time_t real,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_SONGSEL		/* Song Select with song ID number */
  //0xF3 dd
  int songSelct (unsgined char songNumber,
		 snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int songSelct (unsgined char songNumber,
		 snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_CLOCK		 midi Real Time Clock message */
  //0xF8
  int timingClock (snd_seq_tick_time_t ticks,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  int timingClock (snd_seq_real_time_t real,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_START		 midi Real Time Start message */
  //0xFA
  int start (snd_seq_tick_time_t ticks,
	     int srcPort, int queue,
	     int destClient, int destPort, int channel);
  int start (snd_seq_real_time_t real,
	     int srcPort, int queue,
	     int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_CONTINUE		/* midi Real Time Continue message */
  //0xFB
  int eventContinue (snd_seq_tick_time_t ticks,
		     int srcPort, int queue,
		     int destClient, int destPort, int channel);
  int eventContinue (snd_seq_real_time_t real,
		     int srcPort, int queue,
		     int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_STOP		 midi Real Time Stop message */
  //0xFC
  int eventStop (snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int eventStop (snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_QFRAME		/* Midi time code quarter frame */
  //0xF1 mX
  int qframe (unsgined char messageType, unsgined char value,
	      snd_seq_tick_time_t ticks,
	      int srcPort, int queue,
	      int destClient, int destPort, int channel);
  int qframe (unsgined char frameCount,
	      unsigned char second, unsigned char minuite,
	      unsigned char hour,
	      snd_seq_real_tick_t ticks,
	      int srcPort, int queue,
	      int destClient, int destPort, int channel);
  int qframe (unsgined char messageType, unsgined char value,
	      snd_seq_real_time_t real,
	      int srcPort, int queue,
	      int destClient, int destPort, int channel);
  int qframe (unsgined char frameCount,
	      unsigned char second, unsigned char minuite,
	      unsigned char hour,
	      snd_seq_real_time_t real,
	      int srcPort, int queue,
	      int destClient, int destPort, int channel);
#endif
  
#if 0
  //SND_SEQ_EVENT_TEMPO   /* (SMF) Tempo event */
  int tempo (int tempo, snd_seq_tick_time_t ticks,
	     int srcPort, int queue, int destClient, int destPort,
	     int channel = 0/*don't care*/);
  int tempo (int tempo, snd_seq_real_time_t real,
	     int srcPort, int queue, int destClient, int destPort,
	     int channel = 0/*don't care*/);
  //SND_SEQ_EVENT_TIMESIGN/* SMF Time Signature event */
  int timeSignature (snd_seq_tick_time_t ticks,
		     int srcPort, int queue,
		     int destClient, int destPort, int channel);
  int timeSignature (snd_seq_real_time_t real,
		     int srcPort, int queue,
		     int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_KEYSIGN /* SMF Key Signature event */
  int keySignature (snd_seq_tick_time_t ticks,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
  int keySignature (snd_seq_real_time_t real,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
#endif
  //SND_SEQ_EVENT_SYSEX   /* system exclusive data (variable length) */
  int sysEx (int length, const char* buff,
	     snd_seq_tick_time_t ticks,
	     int srcPort, int queue, int destClient, int destPort, int channel);
  int sysEx (int length, const char* buff,
	     snd_seq_real_time_t real,
	     int srcPort, int queue, int destClient, int destPort, int channel);

#if 0
  //SND_SEQ_EVENT_HEARTBEAT/* "active sensing" event */
  //0xFE
  int heartBeat (snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int heartBeat (snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
#endif
  //SND_SEQ_EVENT_ECHO/* echo event */
  int echoEvent (snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int echoEvent (snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
#if 0
  /* system status messages */
  //SND_SEQ_EVENT_CLIENT_START  /* new client has connected */
  int clientStart (snd_seq_tick_time_t ticks,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  int clientStart (snd_seq_real_time_t real,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_CLIENT_EXIT   /* client has left the system */
  int clientExit (snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int clientExit (snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_CLIENT_CHANGE /* client status/info has changed */
  int clientChange (snd_seq_tick_time_t ticks,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
  int clientChange (snd_seq_real_time_t real,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);

  //SND_SEQ_EVENT_PORT_START    /* new port was created */
  int portStart (snd_seq_tick_time_t ticks,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  int portStart (snd_seq_real_time_t real,
		 int srcPort, int queue,
		 int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_PORT_EXIT	    /* port was deleted from system */
  int portExit (snd_seq_tick_time_t ticks,
		int srcPort, int queue,
		int destClient, int destPort, int channel);
  int portExit (snd_seq_real_time_t real,
		int srcPort, int queue,
		int destClient, int destPort, int channel);
  //SND_SEQ_EVENT_PORT_CHANGE   /* port status/info has changed */
  //???
  int portChange (snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int portChange (snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
#endif

#if 0
  /* synthesizer events */	
  //???
  //SND_SEQ_EVENT_SAMPLE	     /* sample select */
  int sampleEvent (snd_seq_tick_time_t ticks,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  int sampleEvent (snd_seq_real_time_t real,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  //???
  //SND_SEQ_EVENT_SAMPLE_START   /* voice start */
  int sampleStart (snd_seq_tick_time_t ticks,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  int sampleStart (snd_seq_real_time_t real,
		   int srcPort, int queue,
		   int destClient, int destPort, int channel);
  //???
  //SND_SEQ_EVENT_SAMPLE_STOP    /* voice stop */
  int sampleStop (snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int sampleStop (snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  //???
  //SND_SEQ_EVENT_SAMPLE_FREQ    /* playback frequency */
  int sampleFreq (snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int sampleFreq (snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  //???
  //SND_SEQ_EVENT_SAMPLE_VOLUME  /* volume and balance */
  int sampleVolume (snd_seq_tick_time_t ticks,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
  int sampleVolume (snd_seq_real_time_t real,
		    int srcPort, int queue,
		    int destClient, int destPort, int channel);
  //???
  //SND_SEQ_EVENT_SAMPLE_LOOP    /* sample loop */
  int sampleLoop (snd_seq_tick_time_t ticks,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  int sampleLoop (snd_seq_real_time_t real,
		  int srcPort, int queue,
		  int destClient, int destPort, int channel);
  //???
  //SND_SEQ_EVENT_SAMPLE_POSITION/* sample position */
  int samplePosition (snd_seq_tick_time_t ticks,
		      int srcPort, int queue,
		      int destClient, int destPort, int channel);
  int samplePosition (snd_seq_real_tick_time_t real,
		      int srcPort, int queue,
		      int destClient, int destPort, int channel);
#endif

  int flushOutput ();
  int drainOutput ();
  
  ////////////////////////////////////////////////////////////timer
  void startTimer (int destQueue);
  void stopTimer (int destQueue, snd_seq_tick_time_t ticks);
  void stopTimer (int destQueue, snd_seq_real_time_t real);

  ////////////////////////////////////////////////////////////event
  ////////////////////////////////////////////////////////////memory
#if 0
  static snd_seq_event_t* newEvent ()
    {
      return snd_seq_create_event ();
    };
#endif
  static void deleteEvent (snd_seq_event_t* ev)
    {
      snd_seq_free_event (ev);
    };
  static int eventLength (snd_seq_event_t* event)
    {
      return snd_seq_event_length (event);
    };
  
  ////////////////////////////////////////////////////////////read event
  //ɤ߽ФeventdeleteEventǲ뤳ȡ
  //֤̤ͤbuffer˻ĤäƤcellο(i.e. eventο)
  int getEvent (snd_seq_event_t** event);
  int drainInput ();


  ////////////////////////////////////////////////////////////
  void sync (snd_seq_real_time_t real,
	     int srcPort, int destQueue, int destPort);
};
#endif // AlsaClient_H
