#include "SequencerALSA.h"
#include "util/getEnvValue.h"

const int SequencerALSA::CONTROL_RATE;

const int SequencerALSA::DEFAULT_DEST_QUEUE_A;
const int SequencerALSA::DEFAULT_DEST_CLIENT_A;
const int SequencerALSA::DEFAULT_DEST_PORT_A;
const int SequencerALSA::DEFAULT_DEST_QUEUE_B;
const int SequencerALSA::DEFAULT_DEST_CLIENT_B;
const int SequencerALSA::DEFAULT_DEST_PORT_B;
const char SequencerALSA::DEST_QUEUE_A[];
const char SequencerALSA::DEST_CLIENT_A[];
const char SequencerALSA::DEST_PORT_A[];
const char SequencerALSA::DEST_QUEUE_B[];
const char SequencerALSA::DEST_CLIENT_B[];
const char SequencerALSA::DEST_PORT_B[];

SequencerALSA::SequencerALSA (State* globalState, Option* option__):
  SequencerImple (globalState, option__),
  destQueueA (getEnvValue (DEST_QUEUE_A, DEFAULT_DEST_QUEUE_A)),
  destClientA (getEnvValue (DEST_CLIENT_A, DEFAULT_DEST_CLIENT_A)),
  destPortA (getEnvValue (DEST_PORT_A, DEFAULT_DEST_PORT_A)),
  isPortASubscribing (false),
  isPortAUsable (false),
  destQueueB (getEnvValue (DEST_QUEUE_B, DEFAULT_DEST_QUEUE_B)),
  destClientB (getEnvValue (DEST_CLIENT_B, DEFAULT_DEST_CLIENT_B)),
  destPortB (getEnvValue (DEST_PORT_B, DEFAULT_DEST_PORT_B)),
  isPortBSubscribing (false),
  isPortBUsable (false)
{
  currentRealTime.tv_sec = 0;
  currentRealTime.tv_nsec = 0;
  
  for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
    {
      destQueueClient[MIDI::PORT_A][channel].set (destQueueA,
						  destClientA, destPortA,
						  channel);
      destQueueClient[MIDI::PORT_B][channel].set (destQueueB,
						  destClientB, destPortB,
						  channel);
    }
  
  //////////////////////////////////////////////////open alsa sequencer
  try
    {
      //alsaClient = new AlsaClient (SND_SEQ_OPEN_OUT);
      alsaClient = new AlsaClient (SND_SEQ_OPEN);
    }
  catch (AlsaClient::AError&)
    {
      throw CanNotOpenAlsaDevice ();
    }
  unsigned char multicastFilter[8];
  memset (multicastFilter, 0, sizeof (multicastFilter));
  unsigned char eventFilter[32];
  memset (eventFilter, 0, sizeof (eventFilter));
  snd_seq_set_bit (SND_SEQ_EVENT_ECHO, eventFilter);
  alsaClient->setClientInfo ("SequencerALSA", SND_SEQ_FILTER_USE_EVENT,
			     multicastFilter, eventFilter);
  alsaClient->blockMode (true);

  //////////////////////////////////////////////////create port
  snd_seq_port_info_t portInfo;
  int tmp = alsaClient->createPort ("SequencerALSA",
				    SND_SEQ_PORT_CAP_OUT |
				    SND_SEQ_PORT_CAP_IN |
				    SND_SEQ_PORT_CAP_SUBSCRIPTION,
				    SND_SEQ_PORT_TYPE_MIDI_GENERIC,
				    16, 0, 0, &portInfo);
  if (tmp < 0)
    {
      DEBUG_MESSAGE ("SequencerALSA::SequencerALSA create port failed.");
      delete alsaClient;
      throw CanNotCreatePort ();
    }
  srcPort = portInfo.port;

  /////////////////////////////////////////////////////////set queue client
#if 1
  //XXX log water mark$B$H(Bhigh water mark$B$NCM(B
  const int lowWaterMark = 1;
  const int highWaterMark = 0xfffffff;
  tmp = alsaClient->setQueueClientInfo (destQueueA, true,
					lowWaterMark, highWaterMark);
  if (tmp < 0)
    {
      DEBUG_MESSAGE ("SequencerALSA::SequencerALSA "
		     "setQueueClientInfo failed.");
      delete alsaClient;
      throw CanNotCreatePort ();
    }
  if (destQueueB != destQueueA)
    {
      tmp =  alsaClient->setQueueClientInfo (destQueueB, true,
					     lowWaterMark, highWaterMark);
      if (tmp < 0)
	{
	  DEBUG_MESSAGE ("SequencerALSA::SequencerALSA "
			 "setQueueClientInfo failed.");
	  delete alsaClient;
	  throw CanNotCreatePort ();
	} 
    }
#else
  //setup queue
  snd_seq_queue_client_t queueClientInfo;
  queueClientInfo.queue = destQueueA;
  queueClientInfo.used = 1;
  queueClientInfo.low = 0;
  queueClientInfo.high = 500-100;//???
  tmp = alsaClient->setQueueClientInfo (&queueClientInfo);
  if (tmp < 0)
    {
      DEBUG_MESSAGE ("SequencerALSA::SequencerALSA "
		     "setQueueClientInfo failed.");
      delete alsaClient;
      throw CanNotCreatePort ();
    } 
#endif

#if 0
  //////////////////////////////////////////////////////set queue info
  snd_seq_queue_info_t queueInfo;
  queueInfo.queue = 1;   /* queue we're using */
  queueInfo.ppq = 120;
  queueInfo.tempo = 500000;      /* don't change */
  tmp = alsaClient->setQueueInfo (&queueInfo);
  if (tmp < 0)
    {
      DEBUG_MESSAGE ("SequencerALSA::SequencerALSA setQueueInfo failed.");
      exit(1);
    }
#endif
  
			     
  
  
  /////////////////////////////////////////////////subscribeing
  initializePort (srcPort, destQueueA, destClientA, destPortA,
		  &isPortASubscribing, &isPortAUsable);
  //A$B$H(BB$BF1$8$@$C$?$i:$$k$N$G$=$N(Bcheck
  if (destClientB != destClientA || destPortB != destPortA)
    initializePort (srcPort, destQueueB, destClientB, destPortB,
		    &isPortBSubscribing, &isPortBUsable);
  
  //$B$I$N(Bport$B$b;H$($J$+$C$?$i(Berror$B$r(Bthrow
  if (isPortAUsable == false && isPortBUsable == false)
    {
      delete alsaClient;
      throw CanNotFindUsablePort ();
    }
}

SequencerALSA::~SequencerALSA ()
{
  //XXX B$B$r;H$C$F$$$?$J$i(B
  releasePort (srcPort, destQueueB, destClientB, destPortB,
	       isPortBSubscribing);
  releasePort (srcPort, destQueueA, destClientA, destPortA,
	       isPortASubscribing);
}

//kernel client$B$@$C$?$i(Bsubscribe$B$9$k!#(B
//user client$B$@$C$?$i(Bsubscribe$B=PMh$k$J$i(Bsubscribe$B$9$k!#(B
//$B=PMh$J$1$l$P$=$l$G$bNI$7!#(B
void SequencerALSA::initializePort (int srcPort__, int destQueue,
				    int destClient, int destPort,
				    bool* isSubscribing, bool* isUsable)
{
  *isSubscribing = false;
  *isUsable = false;

  snd_seq_client_info_t clientInfo;
  int ret = alsaClient->getClientInfo (destClient, &clientInfo);
  if (ret < 0)
    {
      cerr << "SequencerALSA::initializePort getClientInfo cliet="
	   << destClient << ":" << snd_strerror (ret) << endl;
      return;
    }

  switch (clientInfo.type)
    {
    case NO_CLIENT:
      cerr << "SequencerALSA::initializePort cliet=" << destClient
	   << " is not found." << endl;
      return;
      break;
    case USER_CLIENT:
      //do nothing
      break;
    case KERNEL_CLIENT:
      //do nothing
      break;
    }

  snd_seq_port_info_t portInfo;
  ret = alsaClient->getPortInfo (destClient, destPort, &portInfo);
  if (ret < 0)
    {
      cerr << "SequencerALSA::initializePort getPortInfo cliet="
	   << destClient << ":port=" << destPort << ":"
	   << snd_strerror (ret) << endl;
      return;
    }
  if ((portInfo.capability & SND_SEQ_PORT_CAP_OUT) == 0)
    {
      cerr << "SequencerALSA::initializePort getPortInfo cliet="
	   << destClient << ":port=" << destPort
	   << " does not have out capability" << endl;
      return;
    }
  *isUsable = true;
  
  if ((portInfo.capability & SND_SEQ_PORT_CAP_SUBSCRIPTION) != 0)
    {
      ret = alsaClient->outputSubscribe (srcPort__,
					 destQueue, destClient, destPort,
					 false, true);
      if (ret < 0)
	{
	  cerr << "SequencerALSA::initializePort outputSubscribe srcPort="
	       << srcPort__ << " destCliet="
	       << destClient << ":port=" << destPort
	       << " failed." << snd_strerror (ret) << endl;
	  return;
	}
      *isSubscribing = true;
    }
}

//subscribe$B$7$F$$$?$J$i(Bunsubscribe$B$9$k!#(B
void SequencerALSA::releasePort (int srcPort__,
				 int destQueue, int destClient, int destPort,
				 bool isSubscribing)
{
  if (isSubscribing == false)
    return;//this port is not subscribing. so nothing to be done.
  
  //unsubscribing
  int ret = alsaClient->outputUnsubscribe (srcPort__,
					   destQueue, destClient, destPort);
  if (ret < 0)
    {
      cerr << "SequencerALSA::initializePort outputUnsubscribe cliet="
	   << destClient << ":port=" << destPort
	   << " failed." << snd_strerror (ret) << endl;
    }
}

bool SequencerALSA::checkPortChannel (MIDI::PORT port, uchar channel)
{
  assert (channel<MIDI::MAX_CHANNEL);
  //note:0<= port$B$O$$$D$b??(B
  switch (port)
    {
    case MIDI::PORT_A:
      return isPortAUsable;
      break;
    case MIDI::PORT_B:
      return isPortBUsable;
      break;
    default:
      DEBUG_MESSAGE ("SequencerALSA::checkPortChannel port is out of range."
		     " ERROR!!!");
      return false;
      break;
    }
  
  return false;
}

void SequencerALSA::dumpBuf ()
{
  alsaClient->flushOutput ();
}

void SequencerALSA::reset ()
{
  //XXX
  /////////////////////////////sequencer$B$N=i4|2=(B
  alsaClient->drainOutput ();
  sync ();
  stopTimer ();//no need???
  startTimer ();//no need???
  allNoteOffSoundOff ();
  //NOTE:0xBn 0x79 0x0 reset all controller
  //port a, port b$B$,;H$($k$+$I$&$+$OD4$Y$J$/$FNI$$!#(B
  //noteOff$BEy$G$d$C$F$$$k$+$i(B
  for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
    {
      controlChange (MIDI::PORT_A, channel, 0x79, 0);
    }
  for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
    {
      controlChange (MIDI::PORT_B, channel, 0x79, 0);
    }

  dumpBuf ();
  sync ();
  traceReset ();
}

  
//midi event$B$r<B9T$9$k!#(B
void SequencerALSA::doNoteOff (int port, uchar channel,
			       uchar note, uchar velocity)
{
  alsaClient->noteOff (note, velocity, currentRealTime, srcPort,
		       destQueueClient[port][channel].getQueue (),
		       destQueueClient[port][channel].getDestClient (),
		       destQueueClient[port][channel].getDestPort (),
		       destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doNoteOn (int port, uchar channel,
			      uchar note, uchar velocity)
{
  alsaClient->noteOn (note, velocity, currentRealTime, srcPort,
		      destQueueClient[port][channel].getQueue (),
		      destQueueClient[port][channel].getDestClient (),
		      destQueueClient[port][channel].getDestPort (),
		      destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doPolyphonicKeyPressure (int port, uchar channel,
					     uchar note, uchar keyPressure)
{
  alsaClient->keyPressure (note, keyPressure, currentRealTime, srcPort,
			   destQueueClient[port][channel].getQueue (),
			   destQueueClient[port][channel].getDestClient (),
			   destQueueClient[port][channel].getDestPort (),
			   destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doControlChange (int port, uchar channel,
				     uchar control, uchar value)
{
  alsaClient->controller (control, value, currentRealTime, srcPort,
			  destQueueClient[port][channel].getQueue (),
			  destQueueClient[port][channel].getDestClient (),
			  destQueueClient[port][channel].getDestPort (),
			  destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doProgramChange (int port, uchar channel, uchar  program)
{
  alsaClient->programChange (program, currentRealTime, srcPort,
			     destQueueClient[port][channel].getQueue (),
			     destQueueClient[port][channel].getDestClient (),
			     destQueueClient[port][channel].getDestPort (),
			     destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doChannelPressure (int port, uchar channel,
				       uchar channelPressure)
{
  alsaClient->channelPressure (channelPressure, currentRealTime, srcPort,
			       destQueueClient[port][channel].getQueue (),
			       destQueueClient[port][channel].getDestClient (),
			       destQueueClient[port][channel].getDestPort (),
			       destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doPitchBendChange (int port, uchar channel,
				       uchar lsb, uchar msb)
{
  alsaClient->pitchBend (lsb, msb, currentRealTime, srcPort,
			 destQueueClient[port][channel].getQueue (),
			 destQueueClient[port][channel].getDestClient (),
			 destQueueClient[port][channel].getDestPort (),
			 destQueueClient[port][channel].getDestChannel ());
}

void SequencerALSA::doExclusive (const uchar* udata, uint32 size,
				 int port = -1, int channel = -1)
{
  //XXX exclusibe$B$N=hM}$O$I$&$9$Y$-(B?
  //
  if (port == -1)
    port = 0;
  if (channel == -1)
    channel = 0;
  alsaClient->sysEx (size, reinterpret_cast<const char*> (udata),
		     currentRealTime, srcPort,
		     destQueueClient[port][channel].getQueue (),
		     destQueueClient[port][channel].getDestClient (),
		     destQueueClient[port][channel].getDestPort (),
		     destQueueClient[port][channel].getDestChannel ());
}


void SequencerALSA::startTimer ()
{
  traceStartTimer ();
  alsaClient->startTimer (destQueueA);
  if (destQueueB != destQueueA)
    alsaClient->startTimer (destQueueB);
  
  //XXX send alsa event of start timer
}

void SequencerALSA::stopTimer ()
{
  //XXX send alsa event of stop timer
  sync ();
  alsaClient->stopTimer (destQueueA, currentRealTime);
  if (destQueueB != destQueueA)
    alsaClient->stopTimer (destQueueB, currentRealTime);
  traceStopTimer ();
}

void SequencerALSA::sync ()
{
  //DEBUG_MESSAGE ("SequencerALSA::syncing");
  int destPort = destPortA;
  if (isPortAUsable == false)
    destPort = destPortB;
#if 1
  alsaClient->sync (currentRealTime, srcPort, destQueueA, destPort);
#else
  if (currentRealTime.tv_sec != 0)
    {
      const int GUARD_NSEC = 100*1000*1000;//0.1 sec
      snd_seq_real_time_t waitTime = currentRealTime;
      waitTime.tv_nsec -= GUARD_NSEC;
      if (waitTime.tv_nsec < 0)
	{
	  waitTime.tv_sec -= 1;
	  waitTime.tv_nsec += 1000000000;
	}
      alsaClient->sync (waitTime, srcPort, destQueueA, destPort);
    }
  else
    {
      alsaClient->sync (currentRealTime, srcPort, destQueueA, destPort);
    }
#endif
  //DEBUG_MESSAGE ("SequencerALSA::synced");
}


////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////for dynamic loading
///////////////////////////////////////////////////////////newSequencer<T>
//////////////////////////////////////////////////////////infoSequencer<T>
////////////////////////////////////////////////////////////////////////////
#include "Sequencer/InfoSequencer.h"

static Sequencer* newSequencerALSA (State* state__, Option* option__)
{
  return new SequencerALSA (state__, option__);
}

InfoSequencer infoSequencerALSA =
{
  newSequencerALSA
};

