#ifndef MIDI_H
#define MIDI_H

#include "util/mytype.h"

//MIDI$B4XO"$NDj?t$rDj5A$9$k%/%i%9(B
class MIDI
{
public:
  ///////////////////////////////////////////////////////MIDI event constants
  //note off
  //0x8n note velocity
  static const uint NOTE_OFF = 0x80;

  //note on
  //0x9n note velocity(velocity==0$B$N;~$O(Bnote off)
  static const uint NOTE_ON = 0x90;

  //polyphonic key pressure
  //0xAn note pressure
  static const uint KEY_PRESSURE = 0xA0;
  
  //control change & mode messate
  //0xBn control value
  static const uint CONTROL_CHANGE = 0xB0;

  //program change
  //0xCn program
  static const uint PROGRAM_CHANGE = 0xC0;

  //channel pressure
  //0xDn pressure
  static const uint CHANNEL_PRESSURE = 0xD0;

  //pitch bend change
  //0xEn LSB MSB
  static const uint PITCH_BEND = 0xE0;

  //SysEx Event
  //0xF0 ...
  static const uint SYS_EXLUSIVE = 0xF0;


  ////////////////////////////////////////////////////////////////////ID
  static const uchar UNIVERSAL_REALTIME    = 0x7F;//universal realtime message
  static const uchar UNIVERSAL_NONREALTIME = 0x7E;//universal non-realtime message
  static const uchar ROLAND_ID             = 0x41;//Roland maker ID
  static const uchar DT1                   = 0x12;//command id
  static const uchar YAMAHA_ID             = 0x43;//Yamaha maker ID
  static const uchar EOX                   = 0xF7;//end of exclusive


  ////////////////////////////////////////////////////////reset sequencer
  static const uchar GM_SYSTEM_ON_DATA[];//GM reset
  static const int GM_SYSTEM_ON_LENGTH;
  static const uchar GS_RESET_DATA[];//GS reset
  static const int GS_RESET_LENGTH;
  static const uchar SYSTEM_MODE_SET_DATA[];//88 system mode on
  static const int SYSTEM_MODE_SET_LENGTH;
  static const uchar XG_SYSTEM_ON_DATA[];//xg system on
  static const int XG_SYSTEM_ON_LENGTH;
  
  ///////////////////////////////////////////////////////////////channel
  static const int MAX_CHANNEL = 16;//1port 16channel

  typedef int PORT;
  static const PORT PORT_A = 0;
  static const PORT PORT_B = 1;
  static const PORT MAX_PORT = 2;
  static const int MAX_PARTS = MAX_CHANNEL*MAX_PORT;
};
#endif// MIDI_H
