/*
    SequencerMIDI.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstdio>
#include <sys/ioctl.h>
#include <iostream>

#include "util/schedYield.h"
#include "Sequencer/MIDI.h"
#include "Sequencer/instrument.h"
#include "SequencerMIDI.h"

using namespace std;

SequencerMIDI::SequencerMIDI (State* state_, int midiDeviceNum,
			      Option* option_):
  SequencerOSS (state_, option_), midiDevice (midiDeviceNum)
{
  if (midiDevice < 0 || getNRMidis () <= midiDevice)
    {
      cerr << "SequencerMIDI::SequencerMIDI:midiDeviceNumber is invalid.\n"
	   << "midiDeviceNumber=" << midiDevice
	   << ":NRMidis=" << getNRMidis ()
	   << ".assuming 0." << endl;
      midiDevice = 0;
    }
  assert (0 <= midiDevice);
  assert (midiDevice < getNRMidis ());
}

SequencerMIDI::~SequencerMIDI ()
{
  //do nothing
}


void SequencerMIDI::reset () throw (Error)
{
  if (ioctl(seqfd, SNDCTL_SEQ_RESET)==-1)//sequencerν
    {
      perror ("seqfd,SNDCTL_SEQ_RESET");
      Error error ("can not reset the midi device");
      throw (error);
    }
  allNoteOffSoundOff ();
  ////////////////////////////reset all controllers
  char reset[3] = {0xB0,0x79,0x00};
  sendAllChannel (reset);
  dumpBuf ();
  sync ();
  
  traceReset ();
}

void SequencerMIDI::sync () throw (Error)
{
  if (ioctl (seqfd,SNDCTL_SEQ_SYNC) == -1)
    {
#ifdef DEBUG
      perror ("SequencerMIDI::sync seqfd,SNDCTL_SEQ_SYNC");
#endif
      if (errno == EINTR)
	return;
      perror ("SequencerMIDI::sync seqfd,SNDCTL_SEQ_SYNC");
      Error error ("can not sync the midi device");
      throw (error);
    }
}

//////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////handle MIDI Event
//////////////////////////////////////////////////////////////////////////
bool SequencerMIDI::checkPortChannel (MIDI::PORT port, uchar channel)
{
  //assert (channel<MAX_CHANNEL);
  //note:0<= portϤĤ⿿
  switch (option->portChangeType)
    {
    case Option::IGNORE:
      if (port == MIDI::PORT_A && channel < MIDI::MAX_CHANNEL)
	return true;
      break;
    case Option::SERIAL:
      if ((port == MIDI::PORT_A || port == MIDI::PORT_B) &&
	  channel < MIDI::MAX_CHANNEL)
	{
	  if(cacheState.sequencerStatus.port != port)
	    portChange (port);
	  return true;
	}
      break;
    case Option::USE_ROLAND_SYS_EX:
      if ((port == MIDI::PORT_A || port == MIDI::PORT_B) &&
	  channel < MIDI::MAX_CHANNEL)
	{
	  if(cacheState.sequencerStatus.port != port)
	    /*not implemented*/;
	  //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!̤
	  return true;
	}
      break;
    }
  
  return false;
}

//port==0 port_A
//port==1 port_B
void SequencerMIDI::portChange (int port)
{
  //DEBUG_MESSAGE ("SequencerMIDI::portChange port change meta Event found.ingnoring.sorry not supported yet.");
  switch (option->portChangeType)
    {
    case Option::IGNORE:
      //do nothing
      break;
    case Option::SERIAL:
      //0xF5 pp
      //pp<=1 port A,pp>=2 port B
      midiOut (0xF5);
      midiOut ((uchar) port+1);
      break;
    case Option::USE_ROLAND_SYS_EX:
      //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!̤
      break;
    }
  tracePortChange (port);
}

void SequencerMIDI::doNoteOff (int port, uchar channel,
			       uchar note, uchar velocity)
{

  midiOut (MIDI::NOTE_OFF|channel);
  midiOut (note);
  midiOut (velocity);
}

void SequencerMIDI::doNoteOn (int port, uchar channel,
			      uchar note, uchar velocity)
{
  midiOut (MIDI::NOTE_ON|channel);
  midiOut (note);
  midiOut (velocity);
}

void SequencerMIDI::doPolyphonicKeyPressure (int port, uchar channel,
					     uchar note, uchar keyPressure)
{
  midiOut (MIDI::KEY_PRESSURE|channel);
  midiOut (note);
  midiOut (keyPressure);
}


void SequencerMIDI::doControlChange (int port, uchar channel,
				     uchar control, uchar value)
{
  midiOut (MIDI::CONTROL_CHANGE|channel);
  midiOut (control);
  midiOut (value);
}

void SequencerMIDI::doProgramChange (int port, uchar channel, uchar program)
{
  midiOut (MIDI::PROGRAM_CHANGE|channel);
  midiOut (program);
}

void SequencerMIDI::doChannelPressure (int port, uchar channel, uchar pressure)
{
  midiOut (MIDI::CHANNEL_PRESSURE|channel);
  midiOut (pressure);
}

void SequencerMIDI::doPitchBendChange (int port, uchar channel,
				       uchar lsb, uchar msb)
{
  midiOut (MIDI::PITCH_BEND|channel);
  midiOut (lsb);
  midiOut (msb);
}

void SequencerMIDI::doExclusive (const uchar* udata, uint32 size,
				 int port = -1, int channel=-1)
{
  //channel=-1̵ѹ
  //ǥͥˤäexclusiveϤݡѹ롣
  //ߤϼס
  
  midiOut (udata, size);
}

