/*
    SequencerMIDI.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SequencerMIDI_H
#define SequencerMIDI_H

#include <cassert>

#include "Sequencer/OSS/SequencerOSS.h"

using namespace std;

class SequencerMIDI:public SequencerOSS
{
private:
  int midiDevice;
  //void midiOut (char data){SEQ_MIDIOUT (midiDevice,data);};
  void midiOut (uchar udata) {SEQ_MIDIOUT (midiDevice, udata);};
  void midiOut (const char* data, int size)
    {
      assert (size > 0);
      for (int i = 0; i < size; i++)
	midiOut (data[i]);
    };
  void midiOut (const uchar* data, uint32 size)
    {
      assert (size > 0);
      for (uint32 i = 0; i < size; i++)
	midiOut (data[i]);
    };
  void sendAllChannel (char data[3])//XXX$B$3$l$O%]!<%H$b9M$($J$$$H$$$1$J$$!#(B
    {
      switch (option->portChangeType)
	{
	case Option::SERIAL:
	  portChange (MIDI::PORT_A);
	  for (int channel = 0;
	       channel < MIDI::MAX_CHANNEL;
	       channel++)
	    {
	      midiOut (data, 3);
	      data[0]++;
	    }
	  portChange (MIDI::PORT_B);
	  data[0] -= MIDI::MAX_CHANNEL;
	  for (int channel = 0;
	       channel < MIDI::MAX_CHANNEL;
	       channel++)
	    {
	      midiOut (data, 3);
	      data[0]++;
	    }
	  portChange (MIDI::PORT_A);
	  break;
	case Option::IGNORE://$BD>2<$X(B
	case Option::USE_ROLAND_SYS_EX://$BD>2<$X(B
	default:
	  for (int channel = 0;
	       channel < MIDI::MAX_CHANNEL;
	       channel++)
	    {
	      midiOut (data, 3);
	      data[0]++;
	    }
	  break;
	}
    };
  virtual bool checkPortChannel (MIDI::PORT port, uchar channel);
  
public:
  SequencerMIDI (State* state_, int midiDeviceNum, Option* option_);
  ~SequencerMIDI ();
  virtual void dumpBuf () {SEQ_DUMPBUF ();traceDumpBuf ();};
  virtual void reset () throw (Error);
  virtual void startTimer ()//$B$3$l$O%?%$%^!<$r(B0$B$K$9$k!#(B
    {traceStartTimer ();SEQ_START_TIMER ();};
  virtual void waitTimer (ulong waitForMachineTicks)
    {traceWaitTimer (waitForMachineTicks);
    SEQ_WAIT_TIME (machineTicks);SEQ_DUMPBUF ();};
  virtual void deltaTimer (ulong waitForMachineTicks)
    {traceDeltaTimer (waitForMachineTicks);SEQ_DELTA_TIME(machineTicks);};
  
  //port=0 port_a
  //port=1 port_b$B$K@_Dj(B
  virtual void portChange (int port);
  //sync
  virtual void sync () throw (Error);

protected:
  //
  //MIDI event$B$r(Bdevice$B$KAw?.$9$k!#(B($B$=$l$KAjEv$9$kF0:n$r(Bemulate$B$9$k!#(B)
  //$BKt!"(Bstate$B$r99?7$9$k!#(B
  //
  virtual void doNoteOff (int port, uchar channel, uchar note, uchar velocity);
  virtual void doNoteOn (int port, uchar channel, uchar note, uchar velocity);
  virtual void doPolyphonicKeyPressure (int port, uchar channel,
					uchar note, uchar keyPressure);
  virtual void doControlChange (int port, uchar channel,
				uchar control, uchar value);
  virtual void doProgramChange (int port, uchar channel, uchar program);
  virtual void doChannelPressure (int port, uchar channel, uchar pressure);
  virtual void doPitchBendChange (int port, uchar channel,
				  uchar lsb, uchar msb);
  virtual void doExclusive (const uchar* udata, uint32 size,
			    int port = -1, int channel = -1);


};
#endif// SequencerMIDI_H
