/*
    SequencerOSS.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <cstdio>

#include <iostream>

#include "SequencerOSS.h"

using namespace std;

SEQ_DEFINEBUF (32768);
//SEQ_DEFINEBUF (38400);

int seqfd = -1;
const char SequencerOSS::SEQUENCER_DEVICE[];
int SequencerOSS::nOfSynths = -1;
int SequencerOSS::nOfMidis = -1;

void seqbuf_dump ()
{
  //cout << "seqbuf_dump is called."  << endl;
  if (_seqbufptr)
    {
      if (write (seqfd, _seqbuf, _seqbufptr) == -1)
	{
	  SequencerOSS::perrorSequencerDevice ("write ");
	  exit (-1);
	}
    }
  _seqbufptr = 0;
}

int SequencerOSS::open ()
{
  assert (seqfd == -1);//XXX
  seqfd = std::open (SEQUENCER_DEVICE, O_WRONLY);
  if (seqfd == -1)
    {
      perrorSequencerDevice ("SequencerOSS::open:can not open ");
      throw CanNotOpenDevSequencer ();
    }
  else
    {
      //sequencer$B$N=i4|2=(B
      if (ioctl(seqfd,SNDCTL_SEQ_RESET) == -1)
	{
	  perrorSequencerDevice ("SequencerOSS::open:can not reset ");
	  throw CanNotResetDevice ();
	}
      nOfSynths = getNRSynthsPrivate ();
      nOfMidis = getNRMidisPrivate ();
    }
  return seqfd;
}

int SequencerOSS::close ()
{
  assert (seqfd != -1);//XXX
  int ret = std::close (seqfd);
  if (ret == -1)
    {
      perrorSequencerDevice ("SequencerOSS::close:can not close ");
      throw CanNotCloseDevSequencer ();
    }
  seqfd = -1;
  return ret;
}

int SequencerOSS::getNRSynthsPrivate ()
{
  int ret;
  if (ioctl (seqfd, SNDCTL_SEQ_NRSYNTHS, &ret) == -1)
    {
      perror ("SequencerOSS::getNRSynthsPrivate:SNDCTL_SEQ_NRSYNTHS");
      throw CanNotGetNRSynths ();
    }
  return ret;
}

int SequencerOSS::getSynthInfo (int synthNumber, synth_info* synthInfo)
{
  assert (synthInfo != NULL);
  if (0 <= synthNumber && synthNumber < nOfSynths)
    {
      synthInfo->device = synthNumber;
      int ret=ioctl (seqfd, SNDCTL_SYNTH_INFO, synthInfo);
      if (ret == -1)
	{
	  perror ("SequencerOSS::getSynthsInfo:SNDCTL_SYNTH_INFO");
	  throw CanNotGetSynthInfo ();
	}
      return ret;
    }

  return -1;
}

int SequencerOSS::getNRMidisPrivate ()
{
  int ret;
  if (ioctl (seqfd, SNDCTL_SEQ_NRMIDIS, &ret) == -1)
    {
      perror ("SequencerOSS::getNRMidisPrivate:SNDCTL_SEQ_NRSYNTHS");
      throw CanNotGetNRMidis ();
    }
  return ret;
}

int SequencerOSS::getMidiInfo (int midiNumber,midi_info* midiInfo)
{
  assert (midiInfo != NULL);
  if (0 <= midiNumber && midiNumber < nOfMidis)
    {
      midiInfo->device = midiNumber;
      const int ret = ioctl (seqfd, SNDCTL_MIDI_INFO, midiInfo);
      if (ret == -1)
	{
	  perror ("SequencerOSS::getMidiInfo SNDCTOL_MIDI_INFO");
	  throw CanNotGetMidiInfo ();
	}
      return ret;
    }

  return -1;
}


SequencerOSS::SequencerOSS (State* state_, Option* option_):
  SequencerImple (state_, option_)
{
  open ();
}

SequencerOSS::~SequencerOSS ()
{
  close ();
}

int SequencerOSS::getCtrlRate () throw (Error)
{
  int rate = 0;
  if (ioctl (seqfd, SNDCTL_SEQ_CTRLRATE, &rate) == -1)
    {
      perror ("SequencerOSS::getCtrlRate SNDCTL_SEQ_CTRLRATE");
      throw CanNotGetCtrlRate ();
    }
#ifdef DEBUG
  //cout << "rate=" << rate << endl;
#endif// DEBUG

  return rate;
}
