/*
    SequcnerOSS.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SequencerOSS_H
#define SequencerOSS_H

#include <cstdio>
#include <cassert>

#if defined (HAVE_SYS_SOUNDCARD_H)
# include <sys/soundcard.h>
#elif defined (HAVE_MACHINE_SOUNDCARD_H)
# include <machine/soundcard.h>
#else
# error Open Sound System header file is not found?
#endif

#include "Sequencer/SequencerImple.h"

using namespace std;

SEQ_USE_EXTBUF();
extern int seqfd;

///dev/sequencer$B$r07$&0Y$NCj>]%/%i%9(B
//OSS$B$K0MB8$9$kItJ,$G(Btimer$BItJ,$r(Bimplement$B$9$k!#(B
class SequencerOSS:public SequencerImple
{
public:
  static void perrorSequencerDevice (const char* message)
    {
      string fullMessage = message;
      fullMessage += SEQUENCER_DEVICE;
      std::perror (fullMessage.c_str ());
    };
  
protected:
  static const char SEQUENCER_DEVICE[]="/dev/sequencer";
  //static const char SEQUENCER_DEVICE[]="/dev/sequencer2";
  static int open ();
  static int close ();
  static int nOfSynths;
  static int nOfMidis;
  static int getNRSynthsPrivate ();//$B<B:]$K(Bsynth device$B$N?t$r<hF@$9$k!#(B
  static int getNRMidisPrivate ();//$B<B:]$K(Bmidi device$B$N?t$r<hF@$9$k!#(B
  
public:
  SequencerOSS (State* state_, Option* option_);//constructor
  virtual ~SequencerOSS ();//destructor
  //synth device$B$N?t$r<hF@$9$k!#(B
  static int getNRSynths () {return nOfSynths;};
  //synthNumber$B$KBP1~$9$k(Bsynth device$B$N>pJs$rF@$k!#(B
  static int getSynthInfo (int synthNumber, synth_info* synthInfo);
  //midi device$B$N?t$r<hF@$9$k!#(B
  static int getNRMidis () {return nOfMidis;};
  //midiNumber$B$KBP1~$9$k(Bmidi device$B$N>pJs$rF@$k!#(B
  static int getMidiInfo (int midiNumber, midi_info* midiInfo);
  
  //
  //timer$B4XO"!#(B
  //
  virtual int getCtrlRate () throw (Error);
  virtual void waitTimer (ulong ticks)
    {traceWaitTimer (ticks);SEQ_WAIT_TIME (ticks)};
  virtual void deltaTimer (ulong ticks)
    {traceDeltaTimer (ticks);SEQ_DELTA_TIME(ticks)};
  virtual void startTimer ()
    {traceStartTimer ();SEQ_START_TIMER ();};//$B$3$l$O%?%$%^!<$r(B0$B$K$9$k!#(B
  //virtual void startTimer (){ioctl (seqfd,SNDCTL_TMR_START);};
  virtual void stopTimer (){traceStopTimer ();SEQ_STOP_TIMER();SEQ_DUMPBUF ();};
  //virtual void stopTimer (){ioctl (seqfd,SNDCTL_TMR_STOP);SEQ_DUMPBUF ();};
  //virtual void continueTimer (){SEQ_CONTINUE_TIMER();};
  //virtual void continueTimer (){ioctl (seqfd,SNDCTL_TMR_CONTINUE);};
  //virtual void timeBase (int timeBase){ioctl (seqfd,SNDCTL_TMR_TIMEBASE,timeBase);SEQ_DUMPBUF ();};

public:
  ///////exceptional classes
  class CanNotOpenDevSequencer:public CanNotOpenSequencerDevice {};
  class CanNotResetDevice:public SequencerError {};
  class CanNotCloseDevSequencer:public SequencerError {};
  class CanNotGetNRSynths:public SequencerError {};
  class CanNotGetSynthInfo:public SequencerError {};
  class CanNotGetNRMidis:public SequencerError {};
  class CanNotGetMidiInfo:public SequencerError {};
  class CanNotGetCtrlRate:public SequencerError {};
};
#endif// SequencerOSS_H
