/*
    Part.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */



#ifndef Part_H
#define Part_H
#include <pthread.h>

#include "Song/SongStatus.h"
#include "Sequencer/SequencerStatus.h"

class Part
{
public:
  static const int MAX_NOTES = 128;
  static const bool ON = true;
  static const bool OFF = false;
  
  
  Part ();//constructor
  //void reset (uchar receivedChannel, uchar defaltPort);//$BJQ?t$N(Breset
  void reset (uchar defaltPort, uchar receivedChannel);//$BJQ?t$N(Breset
  void reset ();//$BJQ?t$N(Breset
  uchar getNoteOnVelocity (const SongStatus& nowSongState) const;
  uchar getMaxVelocity (const SongStatus& nowSongStatus) const;
  //$BJL$N7W;;J}K!$G$d$C$F$_$k!#(B
  uchar getNoteOnVelocity2 (const SongStatus& nowSongState) const;
  uchar getMaxVelocity2 (const SongStatus& nowSongStatus) const;
  
  bool isMuted;
  const char* instrument;//$B3Z4oL>!#(B
  static pthread_once_t isInstrumentInited;

  static const uchar MAP_OFF = 0;
  static const uchar MAP_1   = 1;
  static const uchar MAP_2   = 2;
  uchar useForRhythmPart;
  bool isUseForRhythmPart (SequencerStatus::DeviceMode deviceMode)
    {
      if ((deviceMode & SequencerStatus::DEVICE_XG_ON) != 0)
	{//XG(Yamaha)
	  return ((bankMSB == 126) ||(bankMSB == 127));
	}
      else
	{//GM or 88 or 88Pro(Rolland)
	  return (useForRhythmPart == MAP_OFF);
	}
      return false;
    };
  //SequencerStatus::MIDI_PORT port;//CHANNEL MSG Rx PORT
  MIDI::PORT port;//CHANNEL MSG Rx PORT
  //MODEL ID=0x42 0x00 0x01 0x??(0x00-0x0F,0x10-0x1F) 0x0?(0,1=A,B port)

  //(port__, channel__)$B$r<u?.$7$F$$$k$+(B?
  bool isRecieve (MIDI::PORT port__, uchar channel__)
    {
      return ((port == port__) && (RxChannel == channel__));
    };

  bool lastNote[MAX_NOTES];
  bool note[MAX_NOTES];//$B$I$N(Bkey$B$,(Bnote on or off$B$+(B?
  double noteOnMachineTicks[MAX_NOTES];//note on$B$7$?;~$N(Bmachine ticks
  uchar velocity[MAX_NOTES];//note on$B$N(Bvelocity
  uchar maxVelocity;
  //note on$B$N(Bvelocity$B$N:GBg$NJ*$G0l2s$K(Bdump$B$9$k;~$N:GBg$r3P$($F$*$/!#(B
  //drum map$B$N;~$K(BLCD$B$rI=<($9$k;~$K;HMQ!#(B
  uchar lastMaxVelocity;
  //note on$B$N(Bvelocity$B$N:GBg$NJ*$G(Bdump,sync$B$7$?;~$NJ*$r3P$($F$*$/!#(B
  //drum map$B$N;~$K(BLCD$B$rI=<($9$k;~$K;HMQ!#(B
  uint32 lastTicks;//$B0lHV:G8e$KJQ2=$7$?;~4V!#(B(MIDI ticks)
  uint32 lastMachineTicks;//$B0lHV:G8e$KJQ2=$7$?;~4V!#(B(machine ticks)
  //$BC10L$O(Bmachine ticks$B$G$"$k$3$H$KCm0U!#(BSong$B$NJ}$N(Bticks$B$G$O$J$$!#(B
  //Song.rate$B$+$i7W;;=PMh$k!#(B
  int notesOnNumber;//note on$B$7$F$$$k?t!#(B
  
  uchar polyphonicKeyPressure[MAX_NOTES];//key pressure 0xAn note dd
  uchar channelPressure;//channel pressure 0xDn dd
  uchar program;//program number 0xCn pp
  uchar bankMSB;//bank MSB 0xBn 0x00 dM
  uchar bankLSB;//bank LSB 0xBn 0x20 dL
  
  static const int32 MAX_PITCH_BEND = 8192;
  static const int32 CENTER_PITCH_BEND = 0;
  int32 pitchBend;//piitch bend change 0xEn LSB MSB

  ///0xBn
  static const uint32 MAX_MODULATION = 16383;
  uint32 modulationWheel;//modulation wheel  0xBn 0x01 0xdM 7bit
                         //modulation wheel  0xBn 0x21 0xdL 7bit
  static const uint32 MAX_CHANNEL_VOLUME = 16383;
  uint32 channelVolume;//channel volume 0xBn 0x07 0xdM
                       //channel volume 0xBn 0x27 0xdL
  static const uint32 MAX_PAN = 8192;
  //static const uint32 CENTER_PAN=((0x40 << 7)+0x40);
  static const uint32 CENTER_PAN = 8192;
  uint32 pan;//pan 0xBn 0x0A 0xdM 7bit
             //pan 0xBn 0x2A 0xdM 7bit
  static const uint32 MAX_EXPRESSION = 16383;
  uint32 expression;//expression controller 0xBn 0x0B 0xdM 7bit
                    //expression controller 0xBn 0x2B 0xdM 7bit
  uchar damperPedal;//damper pedal(hold 1) 0xBn 0x40 dd 7bit
  uchar portamentSwitch;//portamento on/off 0xBn 0x41 dd
  uchar sostenute;//sostenute 0xBn 0x42 dd 7bit
  uchar softPedal;//soft pedal 0xBn 0x43 dd 7bit
  static const uchar MAX_REVERB_SEND_LEVEL = 127;
  uchar reverbSendLevel;//effect1 depth(reverb send level) 0xBn 0x5B dd
  uchar chorusSendLevel;//effect3 depth(chorus send level) 0xBn 0x5D dd
  uchar delaySendLevel;//effect4 depth (celeste (detune)depth) 0xBn 0x5E dd

  //////////////////////////////////////RPN,NRPN
  static const int PARAMETER_NULL = 0;
  static const int PARAMETER_RPN  = 1;
  static const int PARAMETER_NRPN = 2;
  int parameterKind;//$B8=:_(Brpn$B$+(Bnrpn$B$+(Bnull$B$+(B?
  uchar rpnMSB;
  uchar rpnLSB;
  uchar nrpnMSB;
  uchar nrpnLSB;

  ////////////////////////////////////////////////SysEx
  char vibrateRate;
  char vibrateDepth;
  char vibrateDelay;
  char tvfCutOff;
  char tvfResonance;
  char envelopeAttackTime;
  char envelopeDecayTime;
  char envelopeReleaseTime;
  
  uchar toneNumber;//CC#00 value 0-12 0x40 0x1? 0x00
  uchar RxChannel;//$B<u?.%A%c%s%M%k(B 0x40 0x1? 0x02//!!!!!!!!!!!!!!!!!!!!
  uchar RxPitchBend;//0x40 0x1? 0x03
  uchar RxChPressure;//0x40 0x1? 0x04
  uchar RxProgramChange;//0x40 0x1? 0x05
  uchar RxCongrolChange;//0x40 0x1? 0x06
  uchar RxPolyPressure;//0x40 0x1? 0x07
  uchar RxNoteMessage;//0x40 0x1? 0x08
  uchar RxRPN;//0x40 0x1? 0x09//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  uchar RxNRPN;//0x40 0x1? 0x0A//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  uchar RxModulation;//0x40 0x1? 0x0B
  uchar RxVolume;//0x40 0x1? 0x0C
  uchar RxPanpot;//0x40 0x1? 0x0D
  uchar RxExpression;//0x40 0x1? 0x0E
  uchar RxHold1;//0x40 0x1? 0x0F
  uchar RxPortamento;//0x40 0x1? 0x10
  uchar RxSostenute;//0x40 0x1? 0x11
  uchar RxSoft;//0x40 0x1? 0x12
  uchar MonoPolyMode;//0x40 0x1? 0x13
  uchar assignMode;//0x40 0x1? 0x14

  uchar RxBankSelect;//0x40 0x1? 0x23//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  uchar RxBandSelectLSB;//0x40 0x1? 0x24//!!!!!!!!!!!!!!!!!!!!!!!!!!!
};
#endif //Part_h
