/*
    Sequencer.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#include "SequencerImple.h"

SequencerImple::SequencerImple (State* globalState, Option* midiOption):
  Sequencer (globalState, midiOption)
{
  //do nothing
}

SequencerImple::~SequencerImple ()
{
  //do nothing
}

/////////////////////////////////////////////////////////event handling
void SequencerImple::allNoteOffSoundOff ()
{
  DEBUG_MESSAGE ("SequencerImple::allNoteOffSoundOff called");
  
  //////////////////////////////////////////////////////////all sound off
  //NOTE:0xBn 0x78 0x0 sound off
  //port a, port b$B$,;H$($k$+$I$&$+$OD4$Y$J$/$FNI$$!#(B
  //noteOff$BEy$G$d$C$F$$$k$+$i(B
  for (MIDI::PORT port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  controlChange (port, channel, 0x78, 0);
	}
    }
  
  //portChange (MIDI::PORT_A);
  //trace all note off
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  for (int j = 0; j < Part::MAX_NOTES; j++)
	    {
	      traceNoteOff (cacheState.part[port][channel], j);
	    }
	}
    }
  
  dumpBuf ();
  sync ();
}


void SequencerImple::noteOff (int port, uchar channel,
			      uchar note, uchar velocity)
{
  assert (note < 128);
  assert (velocity < 128);
  
  if (checkPortChannel (port, channel))
    {
      doNoteOff (port, channel, note, velocity);
      traceNoteOff (port, channel, note, velocity);
    }
}

void SequencerImple::noteOn (int port, uchar channel,
			     uchar note, uchar velocity)
{
  assert (note < 128);
  assert (velocity < 128);
  
  if (isPronounce (port, channel, velocity))
    {//$B$3$N<u?.%A%c%s%M%k$O(Bmute$B$5$l$F$$$J$$!#(B
      //velocity == 0$B$N;~$O(Bnote off$B!#(B
      //seek$BCf(B,mute$BCf$O(Bnote on$B$OL5;k$9$k!#(B
      if (checkPortChannel (port, channel))
	{
	  doNoteOn (port, channel, note, velocity);
	  traceNoteOn (port, channel, note, velocity);
	}
    }
}

void SequencerImple::polyphonicKeyPressure (int port, uchar channel,
					    uchar note,uchar keyPressure)
{
  assert (note<128);
  assert (keyPressure<128);
  
  if (checkPortChannel (port, channel))
    {
      doPolyphonicKeyPressure (port, channel, note, keyPressure);
      tracePolyphonicKeyPressure (port, channel, note, keyPressure);
    }
}


void SequencerImple::controlChange (int port, uchar channel,
				    uchar control, uchar value)
{
  assert (control < 128);
  //assert (value<128);
  if (value>=128)
    {
      DEBUG_MESSAGE ("SeqeuncerMIDI::controlChange value>=128!!!");
      value=127;
    }
  
  if (checkPortChannel (port, channel))
    {
      doControlChange (port, channel, control, value);
#if 0
      cerr << "controlChange " << hex
	   << (int)(MIDI::CONTROL_CHANGE|channel) << " "
	   << (int) control << " "
	   << (int) value << dec << endl;
#endif

      traceControlChange (port, channel,control,value);
    }
}

void SequencerImple::programChange (int port, uchar channel, uchar program)
{
  assert (program<128);
  
  if (checkPortChannel (port, channel))
    {
      doProgramChange (port, channel, program);
#if 0
      cerr << "programChange " << hex
	   << (int)(MIDI::PROGRAM_CHANGE|channel) << " "
	   << (int) program << dec << endl;
#endif

      traceProgramChange (port, channel, program);
    }
}

void SequencerImple::channelPressure (int port, uchar channel, uchar pressure)
{
  assert (pressure<128);
  
  if (checkPortChannel (port, channel))
    {
      doChannelPressure (port, channel, pressure);
      traceChannelPressure (port, channel, pressure);
    }
}

void SequencerImple::pitchBendChange (int port, uchar channel,
				     uchar lsb, uchar msb)
{
  assert (lsb < 128);
  assert (msb < 128);
  
  if (checkPortChannel (port, channel))
    {
      doPitchBendChange (port, channel, lsb, msb);
      tracePitchBendChange (port, channel, lsb,msb);
    }
}

void SequencerImple::exclusive (const uchar* udata, uint32 size,
			       int port = -1, int channel=-1)
{
  //channel=-1$B$OL5JQ99(B
  //$B$3$3$G%A%c%s%M%k$K$h$C$F(Bexclusive$B$r=PNO$9$k%]!<%H@h$rJQ99$9$k!#(B
  //$B8=:_$O<BAuITMW!#(B
  
  doExclusive (udata, size, port, channel);
  traceExclusive (udata, size, channel);
}
