/*
    SequcnerNull.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SequencerNull_H
#define SequencerNull_H

#include <cstdio>
#include <unistd.h>

#include "Sequencer.h"

using namespace std;

class SequencerNull:public Sequencer
{
protected:
  static const int ctrlRate = 100;//100$B$H$$$&?t;z<+BN$K$O0UL#$,L5$$!#(B
  bool isTimerWorking;
  timeval startTime;
  ulong waitTime;

  //thread$B$,(Bsleep$B$9$k0Y$NJ*!#(B
  Mutex dummyMutex;
  CondVar dummyCond;
  //abstime$B$^$G(Bsleep$B$9$k!#(B
  void threadSleep (timeval abstime)
    {
      timeval nowTime;
      gettimeofday (&nowTime,NULL);
      if (abstime.tv_sec-nowTime.tv_sec<0 ||
	  (abstime.tv_sec==nowTime.tv_sec &&
	   abstime.tv_usec-nowTime.tv_usec <= 0))
	return;//$B4{$K;~4V$,2a$.$F$$$?$+$iBT$?$:$KLa$k!#(B

#if 1
      timespec spec;
      spec.tv_sec = abstime.tv_sec;
      spec.tv_nsec = abstime.tv_usec*1000;
      dummyMutex.lock ();
      dummyCond.timedWait (dummyMutex,&spec);
      dummyMutex.unlock ();
#else
      //usleep$B$r;H$&$Y$-$+(B???
      const ulong usec=(abstime.tv_sec-nowTime.tv_sec)*1000+
	               (abstime.tv_usec-nowTime.tv_usec);
      usleep (usec);
#endif
    };
  
  virtual bool checkPortChannel (MIDI::PORT port, uchar channel)
    {return false;};
public:
  SequencerNull (State* globalState,Option* midiOption);//constructor
  virtual ~SequencerNull ();//destructor
  
  virtual void dumpBuf () {traceDumpBuf ();};
  virtual void allNoteOffSoundOff (){};

  //midi event$B$r<B9T$9$k!#(B
  virtual void noteOff (int port, uchar channel, uchar note, uchar velocity)
    {traceNoteOff (port, channel, note, velocity);};
  virtual void noteOn (int port, uchar channel, uchar note, uchar velocity)
    {traceNoteOn (port, channel, note, velocity);};
  virtual void polyphonicKeyPressure (int port, uchar channel,
				      uchar note, uchar keyPressure)
    {tracePolyphonicKeyPressure (port, channel, note, keyPressure);};
  virtual void controlChange (int port, uchar channel,
			      uchar control, uchar value)
    {traceControlChange (port, channel, control, value);};
  virtual void programChange (int port, uchar channel, uchar  program)
    {traceProgramChange (port, channel, program);};
  virtual void channelPressure (int port, uchar channel, uchar channelPressure)
    {traceChannelPressure (port, channel, channelPressure);};
  virtual void pitchBendChange (int port, uchar channel, uchar lsb, uchar msb)
    {tracePitchBendChange (port, channel, lsb, msb);};
  virtual void exclusive (const uchar* udata, uint32 size,
			  int port = -1, int channel=-1)
    {traceExclusive (udata, size, channel);};
  virtual void portChange (int port) {tracePortChange (port);};
  virtual int getCtrlRate () throw (Error){return ctrlRate;};
  virtual void waitTimer (ulong ticks)
    {
      traceWaitTimer (ticks);
      if (isTimerWorking && waitTime < ticks)
	waitTime = ticks;
    };
  virtual void deltaTimer (ulong ticks)
    {
      traceDeltaTimer (ticks);
      if (isTimerWorking)
	waitTime += ticks;
    };
  virtual void startTimer ()
    {
      traceStartTimer ();
      isTimerWorking = true;
      waitTime = 0;
      gettimeofday (&startTime, NULL);
    };
  virtual void stopTimer ()
    {
      traceStopTimer ();
      isTimerWorking = false;
    };
  virtual void sync ()
    {
      if (!isTimerWorking) return;//timer$B$,F0$$$F$$$J$1$l$P2?$b$7$J$$!#(B
      //$B<!$NBT$A;~4V$r7W;;$9$k!#(B
      timeval absTime = startTime;
      absTime.tv_sec += waitTime/ctrlRate;
      absTime.tv_usec += 1000000*(waitTime-(waitTime/ctrlRate)*ctrlRate)/ctrlRate;
      if (absTime.tv_usec >= 1000000)
	{//$B7+>e$,$j=hM}(B
	  absTime.tv_sec += absTime.tv_usec/1000000;
	  absTime.tv_usec %= 1000000;
	}
      //$BBT$D!#(B
      threadSleep (absTime);
    };
};
#endif// SequencerNull_H
