/*
    SequencerStatus.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SequencerStatus_H
#define SequencerStatus_H

#include "util/mytype.h"
#include "MIDI.h"

class SequencerStatus
{
public:
  SequencerStatus ();
  void reset ();

  //////////////////////////////////////////////////////midi mode
  typedef int DeviceMode;
  //followings are not sended.
  static const DeviceMode DEVICE_UNKNOWN = 0;
  //GS Reset
  // data for GS 0xF0 0x41 dev 0x42 0x12 0x40 0x00 0x7F 0x00 0x41 0xF7
  static const DeviceMode DEVICE_GS_ON = (1 << 0);
  //GM System ON
  //data for GM 0xF0 0x7E 0x7F 0x09 0x01 0xF7
  static const DeviceMode DEVICE_GM_ON = (1 << 1);
  // data for SC-88
  //system mode set 0xF0 0x41 dev  0x42 0x12 0x00 0x00 0x7f dd   sum  0xF7
  static const DeviceMode DEVICE_SC88_ON = (1 << 2);
  static const DeviceMode DEVICE_SC88PRO_ON = (1 << 3);// data for SC-88Pro
  //XG System ON
  //0xF0 0x43 0x1n 0x4C 0x00 0x00 0x7E 0x00 0xF7
  static const DeviceMode DEVICE_XG_ON = (1 << 4);// data for XG

  DeviceMode deviceMode;//current Mode
  
  //typedef int MIDI_PORT;
  //static const MIDI_PORT PORT_A = 0;
  //static const MIDI_PORT PORT_B = 1;
  //static const int MAX_MIDI_PORT = 2;//A,B$B$N(Bport
  //MIDI_PORT port;//$B8=:_$N=PNO@h$N%]!<%H(B
  MIDI::PORT port;//$B8=:_$N=PNO@h$N%]!<%H(B

  //$B$3$N(Bchannel$B$,(Bmute$B$5$l$F$$$k$+(B?
  //part$BHV9f$G$J$/(Bpart$B$N<u?.%A%c%s%M%k$G$"$k$3$H$KCm0U!#(B
  //bool isMuted[MAX_MIDI_PORT][MIDI::MAX_CHANNEL];
  bool isMuted[MIDI::MAX_PORT][MIDI::MAX_CHANNEL];
  
  ////////////////////////////////////////////////////display data
  static const int MAX_LETTERS = 32;
  char displayedLetter[MAX_LETTERS+1];
  uint32 letterTicks;
  double letterMachineTicks;
  static const int PAGE_NUM = 10;
  static const int DOT_SIZE = 64;
  uchar dotData[PAGE_NUM][DOT_SIZE];
  uint32 dotTicks;
  double dotMachineTicks;
  int changedPage;
  uchar displayPage;//0x00-0x0A 00:bar display 01:page1 ... 0a:page10
  uchar displayTime;//0x00-0x0F 0-7.2 (sec)
  uint32 pageTicks;
  double pageMachineTicks;
  
  static const uint32 MAX_MASTER_VOLUME = 16383;
  uint32 masterVolume;//0x40 0x00 0x04
  static const char MAX_MASTER_PAN = 64;
  static const char CENTER_MASTER_PAN = 64;
  char masterPan;//0x40 0x00 0x06
  
};
#endif// SequencerStatus_H
