#include "SequencerSerial.h"
#include "Sequencer/MIDI.h"

SequencerSerial::SequencerSerial (State* globalState,Option* option__):
  SequencerImple (globalState, option__),
  timedSerial (new TimedSerial (getenv (DEFAULT_DEVICE)))
{
}

SequencerSerial::~SequencerSerial ()
{
  delete timedSerial;
}



bool SequencerSerial::checkPortChannel (MIDI::PORT port, uchar channel)
{
  //note 0<= chennalϤĤ⿿
  if ((port == MIDI::PORT_A || port == MIDI::PORT_B) &&
      channel < MIDI::MAX_CHANNEL)
    {
      if(cacheState.sequencerStatus.port != port)
	portChange (port);
      return true;
    }

  return false;
}

void SequencerSerial::sendAllChannel (uchar data[3])
{
  //port b, port aνäport a᤹ɬפʤʤ롣
  //port a, port bνʤ餽θport aᤷƤɬפ롣
  portChange (MIDI::PORT_B);
  data[0] -= MIDI::MAX_CHANNEL;
  for (int channel = 0;
       channel < MIDI::MAX_CHANNEL;
       channel++)
    {
      timedSerial->write (data, 3);
      data[0]++;
    }
  portChange (MIDI::PORT_A);
  for (int channel = 0;
       channel < MIDI::MAX_CHANNEL;
       channel++)
    {
      timedSerial->write (data, 3);
      data[0]++;
    }
}


//////////////////////////////////////////////////////////////////////

void SequencerSerial::dumpBuf ()
{
  traceDumpBuf ();
}


void SequencerSerial::reset ()
{
  allNoteOffSoundOff ();
  
  ////////////////////////////reset all controllers
  uchar reset[3] = {0xB0,0x79,0x00};
  sendAllChannel (reset);
  sync ();
  
  traceReset ();
}


//midi event¹Ԥ롣
void SequencerSerial::portChange (int port)
{
  //0xF5 pp
  //pp<=1 port A,pp>=2 port B
  assert (port == MIDI::PORT_A || port == MIDI::PORT_B);
  
  tracePortChange (port);
  uchar buffer[2];
  buffer[0] = 0xF5;
  buffer[1] = static_cast<uchar> (port+1);
  timedSerial->write (buffer, 2);
}

int SequencerSerial::getCtrlRate () throw (Error)
{
  return TimedSerial::CTRL_RATE;
}

void SequencerSerial::waitTimer (ulong ticks)
{
  timedSerial->waitTimer (ticks);
}

void SequencerSerial::deltaTimer (ulong ticks)
{
  timedSerial->deltaTimer (ticks);
}

void SequencerSerial::startTimer ()
{
  timedSerial->startTimer ();
}

void SequencerSerial::stopTimer ()
{
  timedSerial->stopTimer ();
}

void SequencerSerial::sync ()
{
  timedSerial->sync ();
}


//midi event¹Ԥ롣
void SequencerSerial::doNoteOff (int port, uchar channel,
				 uchar note, uchar velocity)
{
      uchar buffer[3];
      buffer[0] = MIDI::NOTE_OFF | channel;
      buffer[1] = note;
      buffer[2] = velocity;
      timedSerial->write (buffer, 3);
}

void SequencerSerial::doNoteOn (int port, uchar channel,
				uchar note, uchar velocity)
{
  uchar buffer[3];
  buffer[0] = MIDI::NOTE_ON | channel;
  buffer[1] = note;
  buffer[2] = velocity;
  timedSerial->write (buffer, 3);
}

void SequencerSerial::doPolyphonicKeyPressure (int port, uchar channel,
					       uchar note, uchar keyPressure)
{
  uchar buffer[3];
  buffer[0] = MIDI::KEY_PRESSURE | channel;
  buffer[1] = note;
  buffer[2] = keyPressure;
  timedSerial->write (buffer, 3);
}

void SequencerSerial::doControlChange (int port, uchar channel,
				       uchar control, uchar value)
{
  uchar buffer[3];
  buffer[0] = MIDI::CONTROL_CHANGE | channel;
  buffer[1] = control;
  buffer[2] = value;
  timedSerial->write (buffer, 3);
}

void SequencerSerial::doProgramChange (int port, uchar channel, uchar program)
{
  uchar buffer[2];
  buffer[0] = MIDI::PROGRAM_CHANGE | channel;
  buffer[1] = program;
  timedSerial->write (buffer, 2);
}

void SequencerSerial::doChannelPressure (int port, uchar channel,
					 uchar channelPressure)
{
  uchar buffer[2];
  buffer[0] = MIDI::CHANNEL_PRESSURE | channel;
  buffer[1] = channelPressure;
  timedSerial->write (buffer, 2);
}

void SequencerSerial::doPitchBendChange (int port, uchar channel,
				       uchar lsb, uchar msb)
{
  uchar buffer[3];
  buffer[0] = MIDI::PITCH_BEND | channel;
  buffer[1] = lsb;
  buffer[2] = msb;
  timedSerial->write (buffer, 3);
}

void SequencerSerial::doExclusive (const uchar* udata, uint32 size,
				   int port = -1, int channel = -1)
{
  timedSerial->write (udata, size);
}



////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////for dynamic loading
///////////////////////////////////////////////////////////newSequencer<T>
//////////////////////////////////////////////////////////infoSequencer<T>
////////////////////////////////////////////////////////////////////////////
#include "Sequencer/InfoSequencer.h"

static Sequencer* newSequencerSerial (State* state_, Option* option_)
{
  return new SequencerSerial (state_, option_);
}

InfoSequencer infoSequencerSerial =
{
  newSequencerSerial
};

