/*
    SequcnerSerial.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SequencerSerial_H
#define SequencerSerial_H

#include "Sequencer/SequencerImple.h"
#include "TimedSerial.h"

using namespace std;

//adaptor TimedSerial to Sequencer
class SequencerSerial:public SequencerImple
{
private:
  //$B$3$N4D6-JQ?t$G;XDj$5$l$k(Bdevice$B$KBP$7$F1iAU$9$k!#(B
  static const char DEFAULT_DEVICE[] = "WIDI_SEQUENCER_SERIAL_DEVICE";
  
protected:
  static const int ctrlRate = 100;//100$B$H$$$&?t;z<+BN$K$O0UL#$,L5$$!#(B
  TimedSerial* timedSerial;
  
  virtual bool checkPortChannel (MIDI::PORT port, uchar channel);
  void sendAllChannel (uchar data[3]);
public:
  SequencerSerial (State* globalState, Option* option__);//constructor
  virtual ~SequencerSerial ();//destructor
  
  virtual void dumpBuf ();
  virtual void reset ();
  
  //midi event$B$r<B9T$9$k!#(B
  virtual void portChange (int port);
  virtual int getCtrlRate () throw (Error);
  virtual void waitTimer (ulong ticks);
  virtual void deltaTimer (ulong ticks);
  virtual void startTimer ();
  virtual void stopTimer ();
  virtual void sync ();

protected:
  //midi event$B$r<B9T$9$k!#(B
  virtual void doNoteOff (int port, uchar channel, uchar note, uchar velocity);
  virtual void doNoteOn (int port, uchar channel, uchar note, uchar velocity);
  virtual void doPolyphonicKeyPressure (int port, uchar channel,
				      uchar note, uchar keyPressure);
  virtual void doControlChange (int port, uchar channel,
			      uchar control, uchar value);
  virtual void doProgramChange (int port, uchar channel, uchar  program);
  virtual void doChannelPressure (int port, uchar channel,
				uchar channelPressure);
  virtual void doPitchBendChange (int port, uchar channel,
				  uchar lsb, uchar msb);
  virtual void doExclusive (const uchar* udata, uint32 size,
			    int port = -1, int channel = -1);  
};
#endif// SequencerSerial_H
