#include "IO/MainIO.h"
#include "Song/SongFactory.h"
#include "SequencerSerial.h"
#include "Widi/WidiController.h"
#include "Widi/ServerCommand.h"
#include "Widi/SongManager.h"

int main (int argc,const char** argv)
{
  if (argc<=1)
    {
      cerr << "Usage:" << argv[0] << " files..." << endl;
      exit (1);
    }

  Option option;
  option.isRealTime = true;
  State state;
  SongCommand songCommand;
  ServerCommand serverCommand;
  SongManager* songManager = new SongManager ();
  WidiController* widiController = new WidiController (&serverCommand,
						       songManager);
  MainIO* io = new MainIO (&state, widiController, &option);
  
  Sequencer* sequencer=new SequencerSerial (&state,&option);
  sequencer->reset ();
  sequencer->sync ();
  
  for (int i=1;i<argc;i++)
    {
      Song* song = SongFactory::newSong (argv[i],&state,&songCommand,*io,&option);
#if 1
      song->play (*sequencer);
#else
      pthread_t playing;
      song->createPlayThread (&playing,NULL,*sequencer);
      void* ret;
      pthread_join (playing,&ret);
#endif      
    }

  delete sequencer;
  delete widiController;
  delete songManager;
  
  delete io;
  cout << "all done" << endl;
  return 0;
}
