#include "Sequencer.h"
#include "InfoSequencer.h"

#if defined (HAVE_SYS_SOUNDCARD_H) || defined (HAVE_MACHINE_SOUNDCARD_H)
# include "Sequencer/OSS/MIDI/SequencerMIDI.h"
#else
# include "Sequencer/SequencerNull.h"
#endif

Sequencer* Sequencer::newSequencer (DlHandler** dlHandler_, const string& name,
				    State* state_, Option* option_)
{
  assert (state_ != NULL);
  assert (option_ != NULL);
  assert (dlHandler_ != NULL);
  
  *dlHandler_ = NULL;
  if (name.length () == 0)
    goto CREATE_DEFAULT_SEQUENCER;

  *dlHandler_ = DlHandler::newOpenDlHandler ("Sequencer",
					     name, option_);

  if (*dlHandler_ == NULL)
    goto CREATE_DEFAULT_SEQUENCER;

  Sequencer* ret;
  try
    {
      ret = InfoSequencer::createSequencer (*dlHandler_, name,
					    state_, option_);
    }
  catch (Sequencer::SequencerError&)
    {
      delete *dlHandler_;
      *dlHandler_ = NULL;
      goto CREATE_DEFAULT_SEQUENCER;
    }

  if (ret == NULL)
    {
      delete *dlHandler_;
      *dlHandler_ = NULL;
      goto CREATE_DEFAULT_SEQUENCER;
    }
  
  return ret;

 CREATE_DEFAULT_SEQUENCER:
#if defined (HAVE_SYS_SOUNDCARD_H) || defined (HAVE_MACHINE_SOUNDCARD_H)
  return new SequencerMIDI (state_, option_->midiDeviceNum, option_);
#else
  return new SequencerNull (state_, option_);
#endif
}
