/*
    SMFTrack.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "SMFTrack.h"

const char SMFTrack::OFF_RUNNING_STATUS;

SMFTrack::SMFTrack (const char* const bufferHead,int size):
  SMFBufferStream (), lastCommand (OFF_RUNNING_STATUS), port (MIDI::PORT_A)
{
  //$B$^$:(Bheader$B$,>pJsJ,M-$k$+%A%'%C%/(B
  if (size<=8)
    {
      Error error ("too small truck chunck length");
      throw (error);
    }

  SMFBufferStream tmpBufferStream (bufferHead,size);
  loadSMFTrack (tmpBufferStream);
}

SMFTrack::SMFTrack (SMFBufferStream& smfBufferStream):
  SMFBufferStream (), lastCommand (OFF_RUNNING_STATUS), port (MIDI::PORT_A)
{
  loadSMFTrack (smfBufferStream);
}

SMFTrack::~SMFTrack ()
{
  //do nothing
}


void SMFTrack::loadSMFTrack (SMFBufferStream& smfBufferStream) throw (Error)
{
#ifdef DEBUG
  //cerr.write (smfBufferStream.getCurrentBufferPtr (),4);
  //cerr << endl;
#endif
  char MTrk[5];
  MTrk[4]='\0';

  smfBufferStream.read (MTrk,4);
#ifdef DEBUG
  //cerr << "chunck magic=" << MTrk << endl;
#endif
  if (strcmp (MTrk,"MTrk")!=0)
    {
      Error error ("this chunk is not Track chunk.");
#ifdef DEBUG
      //cerr << "chunck magic=" << MTrk << endl;
#endif
      throw (error);
    }
  trackLength=smfBufferStream.get32B ();
#ifdef DEBUG
  //cerr << "trackLength=" << trackLength << endl;
#endif
  loadBuffer (smfBufferStream.getCurrentBufferPtr (),trackLength);
  smfBufferStream.seekgCur (trackLength);
}

int SMFTrack::peekVariableLength () throw (Error)
{
  int value=0;
  int tmp;
  int tmpIndex=index;
  do
    {
      if (tmpIndex+1<bufferSize)
	{
	  Error error ("unexpected end of track.");
	  throw (error);
	}
      tmp=buffer[tmpIndex++];
      value=(value << 7)+(tmp&0x7f);
    }while ((tmp&0x80)!=0);//
  return value;
}
