/*
    SMFTrack.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SMFTrack_H
#define SMFTrack_H

#include <cassert>

#include "Sequencer/MIDI.h"
#include "SMFBufferStream.h"

using namespace std;

class SMFTrack:public SMFBufferStream
{
private:
  int trackLength;//track$B$ND9$5(B
  uint32 ticks;//$B8=:_1iAU$7$F$$$k2U=j$N:G=i$+$i$N(Bticks$B?t!#(B
               //==~0$B$J$i$3$N%H%i%C%/$N1iAU$O=*N;$7$?;v$r0UL#$9$k!#(B
  
  static const char OFF_RUNNING_STATUS = 0;
  char lastCommand;//running status$B$KMQ$$$k!#A02s<B9T$7$?(Bcommand
  MIDI::PORT port;//$B8=:_(Bport a$B$+(Bport b$B$K$J$C$F$$$k$+(B
  int peekVariableLength () throw (Error);//$B$3$l$O(Bindex$B$r?J$a$J$$!#(B
  
public:
  SMFTrack (const char* const bufferHeader,int size);
  SMFTrack (SMFBufferStream& smfBufferStream);
  virtual ~SMFTrack ();//default destructor
private:
  void loadSMFTrack (SMFBufferStream& smfBufferStream) throw (Error);
  
public:
  void rewind ()
    {
      SMFBufferStream::rewind ();
      ticks=0;
      lastCommand=OFF_RUNNING_STATUS;
      port = MIDI::PORT_A;
    };//$BFI$_9~$_0LCV$r@hF,$KLa$9!#(B
  uint32 peekABSDeltaTime ()
    {return (isEnd ())?~0:ticks+peekVariableLength ();};
  //$B$3$l$O(Bindex$B$r?J$a$J$$!#Kt(Bticks$B$rJQ2=$5$;$J$$!#(B
  uint32 getABSDeltaTicks (){return (isEnd ())?~0:ticks+=getVariableLength ();};
  //$B$3$l$O(Bindex$B$r?J$a$k!#Kt(Bticks$B$K2C;;$9$k!#(B
  uint32 getABSTicks () const {return ticks;};//$B8=:_1iAUCf$N(Btick$B?t$rJV$9!#(B
  void setRunningStatus (char command){lastCommand=command;};
  //running status$B$r3P$($F$*$/!#(B
  char getRunningStatus ()
    {assert (isSetRunningStatus ());return lastCommand;};
  //running status$B$r<h$j=P$9!#(B
  void offRunningStatus () {lastCommand=OFF_RUNNING_STATUS;};
  //runnign status$B2r=|(B
  bool isSetRunningStatus () const {return (lastCommand!=OFF_RUNNING_STATUS);};
  //running status$B$,@_Dj$5$l$F$$$k$+(B?
  //lastCommand&0x80==0$B$J$iA02s$N(Bcommand$B$OL5$+$C$?$H$$$&;v!#(B
  //lastCommand&0x80!=0$B$J$iA02s$N(Bcommand$B$,$"$C$?!#(B
  int getTrackLength () const {return trackLength;};
  MIDI::PORT getPort () const {return port;};
  void setPort (MIDI::PORT port__)
    {
      assert (port__ == MIDI::PORT_A || port__ == MIDI::PORT_B);
      port = port__;
    };
  bool isEnd () const {return (ticks==~(uint32)0);};
  //track$B$,:G8e$^$G1iAU$7$?$+(B?
  void setEnd () {ticks=~0;};//track$B$,:G8e$^$G1iAU$7$?$H$9$k!#(B
};
#endif // SMFTrack_H
