/*
    Song.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef Song_H
#define Song_H

#include <pthread.h>

#include "Widi/State.h"
#include "Widi/Option.h"
#include "SongStatus.h"
#include "SongCommand.h"
#include "SongInputStream.h"
#include "IO/MainIO.h"
#include "Sequencer/Sequencer.h"


class Song
{
protected:
  Option* const option;
  State* const state;//threadǶͭդξ֤ؤΥ饹ؤΥݥ󥿡
  SongStatus* cacheSongStatus;//դξ֤򥭥å夷Ƥ٤ʪ
  SongCommand* const songCommand;//threadǶͭsongCommandؤΥݥ󥿡
  //֤򥢥åץǡȤѹäȤΤ롣
  //դФcommand򥭥å夷Ƥ٤ʪ
  //state롣
  SongCommand cacheSongCommand;
  inline void realUpdateStatus (Sequencer& sequencer)
    {
      sequencer.updateStatus ();
      state->updateSongStatus (cacheSongStatus);
    };
  inline void updateStatus (Sequencer& sequencer)
    {
      //cerr << "DEBUG:now SMF locking" << endl;
#ifdef USE_SPIN_LOCK
      while (state->trylock ()!=0)
	;//spin lock,do nothing
#else
      state->lock ();
#endif
      realUpdateStatus (sequencer);
      state->unlock ();
      state->broadcast ();
    };
  inline void tryUpdateStatus (Sequencer& sequencer)
    {
      if (state->trylock ()==0)
	//if (option->isRealTime && state->trylock ()==0)
	{
	  realUpdateStatus (sequencer);
	  state->unlock ();
	  state->broadcast ();
	}
    };
  MainIO& io;//å䥨顼Ϥ衣
  //̿Ūʥ顼exceptionбƽ³Ԥ褦warning
  //error messageǤƽ³Ԥ롣
  
public:
  //for playing
  Song (State* State_, SongCommand* songCommand_,
	MainIO& userIO, Option* option__);
#if 0
  //for converting
  Song (State* State_, SongCommand* songCommand_,
	MainIOConvert2Dumb& userIO, Option* midiOption,
	DumbEventList& dumbEventList_);
#endif
  virtual ~Song ();
  virtual void loadFile (const string& fileName);
  virtual void loadFile (SongInputStream& songInputStream);
  virtual void loadBuffer (const char* buffer, int bufferSize)
    {doLoadBuffer (buffer, bufferSize); preSearch ();};
  //bufferdataɤ߹ޤƤƤɤ롣
  //sub classǼ롣
  
  //ʤξupdate롣(㤨бջ֤ȤfilenameȤState롣)
  virtual void prePlay ()
    {
      cacheSongStatus->reset ();
      state->lock ();
      state->updateSongStatus (cacheSongStatus);
      state->unlock ();
      state->broadcast ();
    };
  
  //դ롣
  virtual void play (Sequencer& sequencer);//throw (Error)
  //threadƱդ򳫻Ϥ롣
  int createPlayThread (pthread_t* thread, pthread_attr_t* attr,
			Sequencer& sequncer);

  const string& getFileName () const {return cacheSongStatus->getFileName ();};
  const string& getTitle () const {return cacheSongStatus->getTitle ();};
  //ջ֤֤ñ̤SongStatus::SONG_CTRL_RATE HzǤ롣
  uint32 getPlayTime () const {return cacheSongStatus->totalLogicalTicks;};
  
protected:
  //loadBuffer,ºݤ˥ǡɤ߹ࡣ
  virtual void doLoadBuffer (const char* buffer, int bufferSize) = 0;//throw (Error)
  //֤侮Ȥ롣
  virtual void preSearch () = 0;
  virtual void resetDevice ();//???

  //֤ǽ˰֤᤹
  //â¾ѿѤʤ
  virtual void realRewind ()=0;
  //seqencerưݾڤƤ롣
  //㤨device is busyȤʤȤ
  //deviceν֤ݾڤʤΤǡȽɬפ롣
  virtual void realPause (Sequencer& sequencer, uint32 ticksOffset,
			  double machineTicksOffset,double logicalTicksOffset);
  virtual void realContinuePlay (Sequencer& sequencer);
  
  
  //ºݤˤǱդ롣
  //play method
  //sub classǼ롣
  virtual void doPlay (Sequencer& sequencer) = 0;
  //doPlayǤ
  double calcDMachineTicks (uint32 playTicks_, uint lastTicks_,
			    uint32 setTempoTicks_,
			    uint32 tempo_, uint32 oldTempo_,
			    int rate_,int16 division_,
			    double skew_ = 1.0)
    {
      //if (lastTicks<setTempoTicks && (uint32)dMachineTicks>0)
      //cerr << "setTempoTicks is effective" << endl;
      //      oldTempo        set tempo      cacheSongStatus->tempo
      //  lastTicks--------->setTempoTicks------>playTicks;
      //   |                                      |
      //Ǹwait timeticks    ܤƤսtick
      //lastTickssetTempTicksδ֤1machneTicks꾮λɬ
      //ˤʤ롣lastTicksplayTicksޤǤԤ֤Ф٤ɬס
      //嵭ʳǤlastTicks==setTempoTicksˤʤ롣
      //1000000.0tempousecñ̤ʤΤǡ
      return (static_cast<double>(playTicks_-setTempoTicks_)*tempo_+
	      static_cast<double>(setTempoTicks_-lastTicks_)*oldTempo_)*rate_
	/(1000000.0*skew_*division_);
    };
  double calcDLogicalTicks (uint32 playTicks_, uint lastTicks_,
			    uint32 setTempoTicks_,
			    uint32 tempo_, uint32 oldTempo_,
			    int16 division_)
    {return calcDMachineTicks (playTicks_, lastTicks_, setTempoTicks_,
			       tempo_, oldTempo_,
			       SongStatus::SONG_CTRL_RATE,division_,1.0);};
  
  //doPlay method
  //Ǽºݤsequcer˥ǡ񤭹Ǥıդξ֤ݻѿ
  //롣
  virtual void dumpBuffer (Sequencer& sequencer,
			   uint32 lastTicks, uint32 playTicks,
			   double& machineTicks, double dMachineTicks,
			   double& logicalTicks, double dLogicalTicks,
			   double& lastDumpTicks);  //throw (SongCommand songCommand_);

  //seek
  //seekingϱվ֤ιڤӡ
  //֤Ʊ(sequencer->waitTime, sequencer->sync)򤷤ʤȡ
private:
  bool isSeeking_;//seek椫?
  ulong positionToSeek;//seek֤ФƤ
protected:
  bool isSeeking () const {return isSeeking_;};
  void enterSeeking (Sequencer* sequencer)
    {
      assert (isSeeking_ == false);
      assert (sequencer != NULL);
      assert (sequencer->isSeeking () == false);
      sequencer->enterSeeking ();
      isSeeking_ = true;
    };
  void leaveSeeking (Sequencer* sequencer)
    {
      assert (isSeeking_ == true);
      assert (sequencer != NULL);
      assert (sequencer->isSeeking () == true);
      sequencer->leaveSeeking ();
      isSeeking_ = false;
    };
  void setSeekPosition (ulong position_)
    {
      assert (isSeeking () == false);
      positionToSeek = position_;
    };
  ulong getSeekPosition () const
    {
      assert (isSeeking () == true);
      return positionToSeek;
    };


protected:
  ////////////////////////////////////////////////for thread stuff
  struct PlayInfo
  {
    Song* song;
    Sequencer* seq;
  };
  static PlayInfo playInfo;
  static Mutex playInfoLock;
  static void* startPlay (PlayInfo* playInfo);  
};

#endif //Song_H
