/*
    SongBufferStream.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "SongBufferStream.h"

SongBufferStream::SongBufferStream (const char* const bufferHead,int size):
  buffer (NULL),bufferSize (-1),index (-1)
{
  loadBuffer (bufferHead,size);
}

SongBufferStream::SongBufferStream ():
  buffer (NULL),bufferSize (-1),index (-1)
{
  //do nogthing
}

void SongBufferStream::loadBuffer (const char* const bufferHead,int size) throw (Error)
{
  if (buffer!=NULL)
    {
      Error error ("this SongBufferStream is already initialized.");
      throw (error);
    }
  if (size<0)
    {
      Error error ("invalid buffer size.(size<0)");
      throw (error);
    }
  

  bufferSize=size;
  buffer=new char[bufferSize];
  memcpy (buffer,bufferHead,bufferSize);
  index=0;
}

SongBufferStream::~SongBufferStream ()
{
  delete[] buffer;
}


int SongBufferStream::get8 () throw (Error)
{
  if (buffer==NULL)
    {
      Error error ("Buffer is not loaded.");
      throw (error);
    }
  
  if (index+1>bufferSize)
    { // ⤦ɤǤʤǡʤ
      Error error ("Unexpected end of buffer.");
      throw (error);
    }

  int value=(unsigned char) buffer[index++];
  return value; // 8 bit ǡ
}


int SongBufferStream::get16B () throw (Error)
{
  uint32 value;
  value =(get8 () << 8);
  value+= get8 ();
  return value;
}

int SongBufferStream::get16L () throw (Error)
{
  uint32 value;
  value = get8 ();
  value+=(get8 () << 8);
  return value;
}

int SongBufferStream::get24B () throw (Error)
{
  uint32 value;
  value =(get8 () << 16);
  value+=(get8 () <<  8);
  value+= get8 ();
  return value;
}

int SongBufferStream::get32B () throw (Error)
{
  uint32 value;
  value =(get8 () << 24);
  value+=(get8 () << 16);
  value+=(get8 () <<  8);
  value+= get8 ();
  return value;
}

void SongBufferStream::read (char* ptr,int n) throw (Error)
{
  if (buffer==NULL)
    {
      Error error ("Buffer is not loaded.");
      throw (error);
    }
  
  if (index+n>bufferSize)
    {//ɤ߹٤dataĤäƤ뤫check
      Error error ("Unexpected end of buffer.");
      throw (error);
    }

  memcpy (ptr,buffer+index,n);
  index+=n;
}

void SongBufferStream::seekgCur (int n)
{
  index+=n;
  if (index<0)
    {
      Error error ("invalid seeking.buffer under.");
      throw (error);
    }
      
  if (index>bufferSize)
    {
      Error error ("invalid seeking.buffer over");
      throw (error);
    }
}
