/*
    SongStatus.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <cstring>

#include "SongStatus.h"

using namespace std;


const int SongStatus::SONG_CTRL_RATE;

const SongStatus::DeviceType SongStatus::FORMAT_UNKNOWN;
const SongStatus::DeviceType SongStatus::FORMAT_GS;
const SongStatus::DeviceType SongStatus::FORMAT_GM;
const SongStatus::DeviceType SongStatus::FORMAT_SC88;
const SongStatus::DeviceType SongStatus::FORMAT_SC88PRO;
const SongStatus::DeviceType SongStatus::FORMAT_XG;

SongStatus::SongStatus ()
{
  fullReset ();
}

SongStatus::~SongStatus ()
{
  //do nothing
};

void SongStatus::reset ()
{
  addTime.tv_sec = 0;
  addTime.tv_usec = 0;
  baseTime.tv_sec = 0;
  baseTime.tv_usec = 0;
  
  state = NOT_STARTED;
  ticks = 0;
  lastTicks = 0;
  machineTicks = 0;
  logicalTicks = 0;
  ticksOffset = 0;
  machineTicksOffset = 0;
  logicalTicksOffset = 0;
  events=0;

  skew=1.0;

  measures = 0;
  lastMeasureTicks = 0;
  lastMeasureMachineTicks = 0;
  lastMeasureLogicalTicks = 0;
  ticksFromLastMeasure = 0;
}

void SongStatus::fullReset ()
{
  reset ();
  totalEvents = 0;
  totalTicks = 0;
  totalMachineTicks = 0;
  totalLogicalTicks = 0;
  totalMeasures = 0;
  ticksPerMeasure = 96;//$B$^$!!":,5r$OL5$$$,!"<h$j4:$($:!"(Bwrd$B$HF1$8$K$7$F$*$/(B
  deviceType = FORMAT_UNKNOWN;
  format = FILE_FORMAT_UNKNOWN;
  nTracks = 0;
  division = 96;
  tempo = 500000;
  bpm = 0;
  //default$B$O(B4/4$BGo;R(B
  nn = 4;
  dd = 4;
  cc = 0;
  bb = 0;
  sf = 0;
  mi = 0;

  rate = -1;
  fileName = "";
  title = "";
}

void SongStatus::setFileName (const string& fileName_)
{
  fileName = fileName_;
}

void SongStatus::setTitle (const string& title_)
{
  title = title_;
}
