/*
    SongStatus.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef SongStatus_H
#define SongStatus_H

#include <ctime>
#include <cassert>
#include <string>
#include <sys/time.h>//why needs this???


#include "util/mytype.h"

using namespace std;

class SongStatus
{
public:
  //1000hz i.e 1msec$B$r(B1$B;~4VC10L$H$9$k!#(B
  static const int SONG_CTRL_RATE=1000;
  ///////////////////////////////////////////playing status
  enum PlayingStatus
  {
    NOT_STARTED,
    LOADING,//file$BFI$_9~$_Cf(B,file$B$rFbIt7A<0$KJQ49Cf!#(B
    PLAYING,
    PAUSING,//DOING_COMMAND$B$NFCJL$J>l9g!#(B
    SEEKING,//$B1iAU0LCV$rJQ99Cf!#(B
    STOPPED,//$B1iAU$rDd;_$5$;$?!#(B
    DOING_COMMAND,//SongCommand$B$r<B9TCf!#(B
    DONE,
  };
  PlayingStatus state;

  ///////////////////////////////////////////////////midi status
  timeval addTime;//addTime$B$@$11iAU$7$F!"(Bpause$B$7$?8e!"(B
  timeval baseTime;//baseTime$B$+$i1iAU$r:F3+!#(B
  //$BEvA37P2a;~4V$O(B(nowTime-baseTime)+addTime$B$H$J$k!#(B

  //$B;~4V$NC10L$O(B3$B<oN`M-$k!#(B
  //ticks:midi ticks
  //machineTicks:sequencer timer ticks (rate Hz)
  //logicalTicks:song ticks (SongStatus::SONG_CTRL_RATE Hz)
  //$B$3$3$G!"1iAUB.EY$rJQ2=$5$;$k$H(BmachineTicks$B$O$=$l$K1F6A$5$l$k!#(B($B<B1iAU;~4V(B)
  //logicalTicks$B$O(Bmidi ticks$B$+$iF@$i$l$kO@M}>e$N1iAU;~4V!#(B($BM}O@1iAU;~4V(B)
  uint32 ticks;
  uint32 lastTicks;
  double machineTicks;
  double logicalTicks;
  //pause$B$7$?;~$N(Bticks
  //$B$G%]!<%:2r=|$7$F$+$i$N%?%$%^!<$NCM$O(BmachineTicks-machineTicksOffset$B$K$J$k!#(B
  uint32 ticksOffset;
  double machineTicksOffset;
  double logicalTicksOffset;
  int rate; 

  uint32 totalTicks;//$B6JA4BN$N(Bticks$B?t!#(B
  uint32 totalMachineTicks;//$B6JA4BN$N(Bmachine ticks$B?t!#(B
  uint32 totalLogicalTicks;//$B6JA4BN$N(Blogical ticks$B?t!#(B
  uint32 totalMeasures;//$B6JA4BN$N>.@a?t!#(B
  uint32 measures;//$B8=:_1iAUCf$N>.@aHV9f!#(B0$B>.@aL\$+$i;O$^$k!#(B
  uint32 lastMeasureTicks;//$B:G8e$K(Bmeasure$B$,(Bincrement$B$7$?(Bticks
  double lastMeasureMachineTicks;//$B:G8e$K(Bmeasure$B$,(Bincrement$B$7$?(Bmachine ticks
  double lastMeasureLogicalTicks;//$B:G8e$K(Bmeasure$B$,(Bincrement$B$7$?(Bmachine ticks
  //$B>.@aFb$G$N(Bticks$B?t!#EvA3(B<ticksPerMeasure
  //ticksFromLastMeasure=ticks-lastMeasureTicks$B$K$J$kH&!#(B
  uint32 ticksFromLastMeasure;
  uint events;//$B8=:_$N(Bevents
  uint totalEvents;//$B6JA4BN$N(BDumb Song events

  ////////////////format
  typedef int DeviceType ;
  static const DeviceType FORMAT_UNKNOWN=0;
  static const DeviceType FORMAT_GS     =(1 << 0);// data for GS
  static const DeviceType FORMAT_GM     =(1 << 1);// data for GM
  static const DeviceType FORMAT_SC88   =(1 << 2);// data for SC-88
  static const DeviceType FORMAT_SC88PRO=(1 << 3);// data for SC-88Pro
  static const DeviceType FORMAT_XG     =(1 << 4);// data for XG
  
  enum Format
  {
    FILE_FORMAT_UNKNOWN,
    SMF_0,
    SMF_1,
    SMF_2,
    RCP_V2_5,  //???
    RCP_V2_5F,//???$B0lBN$I$&$$$&<oN`$,$"$k$N$+ITL@!#(B
    RCP_V2_5WIN,
    R36,
    G18,
    G36,
  };
protected:
  string fileName;
  string title;
  
  uint32 ticksPerMeasure;//1$B>.@aEv$j$N(Bticks$B?t!#(B
  DeviceType deviceType;//file$B$,$I$N2;8;MQ$+!#(B
  Format format;//format 
  int16 nTracks;//track$B$N?t(B

  /////tempo$B4XO"(B
  int16 division;//division 4$BJ,2;Id$NJ,2rG=(B(tick)Ticks Pre Quater Notes
  uint32 tempo;//tempo 4$BJ,2;Id$ND9$5(B(usec)
  uint32 bpm;//beat per minute
  //RCP$B$N;~$O(Bheader$B$N(Bbpm$B$r;X$7$F$=$l$+$iJQ2=$7$J$$!#(B

  //tempo$B$r$I$l$@$1$:$i$9$+(B?(skew$B$G3d$j;;$9$k!#$J$N$G!"(B1.0$B$N;~(B, $BDL>o:F@8!#(B)
  static const double MIN_SKEW = 0.1;//???
  static const double MAX_SKEW = 3.0;//???
  double skew;
  int16 key;//key$B$r$I$l$@$1$:$i$9$+(B?
  /////time signature
  int nn;//$BGo;R$NJ,;R(B
  int dd;//$BGo;R$NJ,Jl(B
  int cc;
  int bb;
  /////key signature
  int sf;//# of # or flat
         //if sf>=0 sf meas number of sharps
         //if sf<0  sf means number of flats
  int mi;//=0:major =1:minor
  //SongStatus& operator=(const SongStatus& src);

public:
  SongStatus ();
  virtual ~SongStatus ();
  //$B1iAUCf$KJQ2=$9$k(Bparemeter$B$r(Breset$B$9$k!#(B
  //song$B8GM-$J$b$N(B(e.g. filename.)$B$O(Breset$B$7$J$$!#(B
  void reset ();
  //reset+song$B8GM-$J$b$N(B(e.g. filename.)$B$N(Breset
  void fullReset ();
  void setFileName (const string& fileName_);
  void setTitle (const string& title_);

  virtual void setFormat (Format format_){format=format_;};
  virtual void setNTracks (int16 nTracks_){nTracks=nTracks_;};
  virtual void setDivision (int16 division_)
    {
      assert (division!=0);
      division=division_;
      ticksPerMeasure=division*4*nn/dd;
    };
  virtual void setTempo (uint32 newTempo){tempo=newTempo;};
  virtual void setBpm (uint32 bpm_)
    {
      assert (bpm_!=0);
      bpm=bpm_;
      tempo=(uint32) ((60.0)*1000000)/bpm;
    };
  virtual void setSkew (double skew_)
    {
      if (skew_ < MIN_SKEW) skew = MIN_SKEW;
      if (skew_ > MAX_SKEW) skew = MAX_SKEW;
      skew = skew_;
    };
  virtual void setTimeSignature (int nn_,int dd_,int cc_,int bb_)
    {
      assert (dd_!=0);
      nn=nn_;
      dd=dd_;
      cc=cc_;
      bb=bb_;
      ticksPerMeasure=division*4*nn/dd;
    };
  virtual void setTimeSignature (int nn_,int dd_)
    {
      //assert (dd_!=0);
      nn=nn_;
      //XXX
      if (dd_!=0)
	dd=dd_;
      ticksPerMeasure=division*4*nn/dd;
    };
  virtual void setKeySignature (int8 sf_,int8 mi_)
    {
      sf=sf_;
      mi=mi_;
    };
  //virtual void setTicksPerMeasure (){ticksPerMeasure=division*4*nn/dd;};

  /////////////////////////////////access method
  uint32 getTicksPerMeasure () const {return ticksPerMeasure;};
  DeviceType getDeviceType () const {return deviceType;};
  const string& getFileName () const {return fileName;};
  bool isTitleSet () const {return title.length () > 0;};
  const string& getTitle () const {return title;};
  Format getFormat () const {return format;};
  int16 getNTracks () const {return nTracks;};
  int16 getDivision () const{return division;};
  uint32 getTempo () const {return tempo;};
  uint32 getBpm () const {return bpm;};
  double getSkew () const {return skew;};
  int16 getKey () const {return key;};
  int getNn () const {return nn;};
  int getDd () const {return dd;};
  int getCc () const {return cc;};
  int getBb () const {return bb;};
  int getSf () const {return sf;};
  int getMi () const {return mi;};//=0:major =1:minor
};

#endif// SongStatus_H
