/*
    MidiEvent.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <iostream>

#include "MidiEvent.h"

using namespace std;

MidiEvent::MidiEvent ():
  next (NULL), event (NULL), ticks (0), logicalTicks (0.0)
{
  /*do nothing*/;
}

MidiEvent:: ~MidiEvent ()
{
  if (event != NULL)
    {
      if (event->eventType == MidiEventNode::MIDI_SYS_EX)
	delete[] (dynamic_cast<MidiSysEx*>(event))->udata;
    }
  delete event;
}

void MidiEvent::setNext (MidiEvent* next_)
{
  assert (event != NULL);
  assert (next == NULL);
  assert (next_ != NULL);
  next=next_;
}

void MidiEvent::setEventFormat (SongStatus::Format format,
				uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiFormat* eventFormat=new MidiFormat;
  eventFormat->eventType=MidiEventNode::FORMAT;
  eventFormat->format=format;
  event=eventFormat;
}

void MidiEvent::setEventNTracks (int16 nTracks,
				 uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiNTracks* eventNTracks=new MidiNTracks;
  eventNTracks->eventType=MidiEventNode::NTRACKS;
  eventNTracks->nTracks=nTracks;
  event=eventNTracks;
}

void MidiEvent::setEventDivision (int16 division,
				  uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiDivision* midiDivision=new MidiDivision;
  midiDivision->eventType=MidiEventNode::DIVISION;
  midiDivision->division=division;
  event=midiDivision;
}

void MidiEvent::setEventTempo (uint32 tempo,
			       uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiTempo* midiTempo=new MidiTempo;
  midiTempo->eventType=MidiEventNode::TEMPO;
  midiTempo->tempo=tempo;
  event=midiTempo;
}

void MidiEvent::setEventBpm (uint32 bpm, uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiBPM* midiBPM=new MidiBPM;
  midiBPM->eventType=MidiEventNode::BPM;
  midiBPM->bpm=bpm;
  event=midiBPM;
}

void MidiEvent::setEventTimeSignature (int nn, int dd, int cc, int bb,
				       uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiTimeSignature* midiTimeSignature=new MidiTimeSignature;
  midiTimeSignature->eventType=MidiEventNode::TIME_SIGNATURE;
  midiTimeSignature->nn=nn;
  midiTimeSignature->dd=dd;
  midiTimeSignature->cc=cc;
  midiTimeSignature->bb=bb;
  event=midiTimeSignature;
}

void MidiEvent::setEventTimeSignature2 (int nn, int dd,
					uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiTimeSignature2* midiTimeSignature2=new MidiTimeSignature2;
  midiTimeSignature2->eventType=MidiEventNode::TIME_SIGNATURE2;
  midiTimeSignature2->nn=nn;
  midiTimeSignature2->dd=dd;
  event=midiTimeSignature2;
}

void MidiEvent::setEventKeySignature (int sf, int mi,
				      uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiKeySignature* midiKeySignature=new MidiKeySignature;
  midiKeySignature->eventType=MidiEventNode::KEY_SIGNATURE;
  midiKeySignature->sf=sf;
  midiKeySignature->mi=mi;
  event=midiKeySignature;
}

void MidiEvent::setEventMidiNoteOff (int port, uchar channel, uchar note, uchar velocity,
				     uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiNoteOff* noteOff=new MidiNoteOff;
  noteOff->eventType=MidiEventNode::MIDI_NOTE_OFF;
  noteOff->channel=channel;
  noteOff->note=note;
  noteOff->velocity=velocity;
  event=noteOff;
}

void MidiEvent::setEventMidiNoteOn (int port, uchar channel, uchar note, uchar velocity,
				    uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiNoteOn* noteOn=new MidiNoteOn;
  noteOn->eventType=MidiEventNode::MIDI_NOTE_ON;
  noteOn->channel=channel;
  noteOn->note=note;
  noteOn->velocity=velocity;
  event=noteOn;
}

void MidiEvent::setEventMidiKeyPressure (int port, uchar channel, uchar note,
					 uchar  keyPressure,
					 uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiKeyPressure* midiKeyPressure=new MidiKeyPressure;
  midiKeyPressure->eventType=MidiEventNode::MIDI_KEY_PRESSURE;
  midiKeyPressure->channel=channel;
  midiKeyPressure->note=note;
  midiKeyPressure->keyPressure=keyPressure;
  event=midiKeyPressure;
}

void MidiEvent::setEventMidiControlChange (int port, uchar channel,
					   uchar control, uchar value,
					   uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiControlChange* controlChange=new MidiControlChange;
  controlChange->eventType=MidiEventNode::MIDI_CONTROL_CHANGE;
  controlChange->port=port;
  controlChange->channel=channel;
  controlChange->control=control;
  controlChange->value=value;
  event=controlChange;
}

void MidiEvent::setEventMidiProgramChange (int port, uchar channel,
					   uchar program,
					   uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiProgramChange* programChange=new MidiProgramChange;
  programChange->eventType=MidiEventNode::MIDI_PROGRAM_CHANGE;
  programChange->port=port;
  programChange->channel=channel;
  programChange->program=program;
  event=programChange;
}

void MidiEvent::setEventMidiChannelPressure (int port, uchar channel,
					     uchar pressure,
					     uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiChannelPressure* channelPressure=new MidiChannelPressure;
  channelPressure->eventType=MidiEventNode::MIDI_CHANNEL_PRESSURE;
  channelPressure->port=port;
  channelPressure->channel=channel;
  channelPressure->pressure=pressure;
  event=channelPressure;
}

void MidiEvent::setEventMidiPitchBend (int port, uchar channel,
				       uchar lsb, uchar msb,
				       uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiPitchBend* pitchBend=new MidiPitchBend;
  pitchBend->eventType=MidiEventNode::MIDI_PITCH_BEND;
  pitchBend->port=port;
  pitchBend->channel=channel;
  pitchBend->lsb=lsb;
  pitchBend->msb=msb;
  event=pitchBend;
}

void MidiEvent::setEventMidiSysEx (const uchar* data, uint32 size,
				   uint ticks_, double logicalTicks_,
				   int port = -1, int channel = -1)
{
  assert (event == NULL);
  assert (size>0);
  setTicks (ticks_, logicalTicks_);
  MidiSysEx* sysEx=new MidiSysEx;
  sysEx->eventType=MidiEventNode::MIDI_SYS_EX;
  sysEx->udata=new uchar[size];
  memcpy (sysEx->udata, data, size);
  sysEx->size=size;
  sysEx->port=port;
  sysEx->channel=channel;
  event=sysEx;
}

void MidiEvent::setEventMidiPortChange (int port,
					uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiPortChange* portChange=new MidiPortChange;
  portChange->eventType=MidiEventNode::MIDI_PORT_CHANGE;
  portChange->port=port;
  event=portChange;
}


void MidiEvent::setEventTextOut (const char* data, int size,
				 uint32 ticks_, double logicalTicks_)
{
  assert (event == NULL);
  assert (size>0);
  setTicks (ticks_, logicalTicks_);
  MidiTextOut* text=new MidiTextOut;
  text->eventType=MidiEventNode::TEXT_OUT;
  text->text=data;
  event=text;
}

void MidiEvent::setEventEnd (uint ticks_, double logicalTicks_)
{
  assert (event == NULL);
  setTicks (ticks_, logicalTicks_);
  MidiEnd* end=new MidiEnd;
  end->eventType=MidiEventNode::END;
  event=end;
}

//for Debug
void MidiEvent::printEvent () const
{
  cout << "ticks=" << ticks << " logicalTicks=" << logicalTicks << " ";
  switch (event->eventType)
    {
      //Song EVENT
    case MidiEventNode::FORMAT:
      cout << "eventType=MidiEventNode::FORMAT" << endl;
      break;
    case MidiEventNode::NTRACKS:
      cout << "eventType=MidiEventNode::NTRACKS="
	   << (dynamic_cast<MidiNTracks*>(event))->nTracks << endl;
      break;
    case MidiEventNode::DIVISION:
      cout << "eventType=MidiEventNode::DIVISION:division="
	   << (dynamic_cast<MidiDivision*>(event))->division
	   << endl;
      break;
    case MidiEventNode::TEMPO:
      cout << "eventType=MidiEventNode::TEMPO:tempo="
	   << (dynamic_cast<MidiTempo*>(event))->tempo << endl;
      break;
    case MidiEventNode::BPM:
      cout << "eventType=MidiEventNode::BPM:bpm="
	   << (dynamic_cast<MidiBPM*>(event))->bpm << endl;
      break;
    case MidiEventNode::TIME_SIGNATURE:
      {
	MidiTimeSignature* time=dynamic_cast<MidiTimeSignature*>(event);
	cout << "eventType=MidiEventNode::TIME_SIGNATURE"
	     << ":nn=" << time->nn
	     << ":dd=" << time->dd
	     << ":cc=" << time->cc
	     << ":bb=" << time->bb
	     << endl;
      }
      break;
    case MidiEventNode::TIME_SIGNATURE2:
      {
	MidiTimeSignature2* time=dynamic_cast<MidiTimeSignature2*>(event);
	cout << "eventType=MidiEventNode::TIME_SIGNATURE2"
	     << ":nn=" << time->nn
	     << ":dd=" << time->dd
	     << endl;
      }
      break;
    case MidiEventNode::KEY_SIGNATURE:
      {
	MidiKeySignature* key=dynamic_cast<MidiKeySignature*>(event);
	cout << "eventType=MidiEventNode::KEY_SIGNATURE"
	     << ":sf=" << key->sf
	     << ":mi=" << key->mi
	     << endl;
      }
      break;
      //MIDI EVENT
    case MidiEventNode::MIDI_NOTE_OFF:
      {
	MidiNoteOff* noteOff=dynamic_cast<MidiNoteOff*>(event);
	cout << "eventType=MidiEventNode::NoteOff,channel="
	     << (int) noteOff->channel
	     << " note=" << (int) noteOff->note
	     << " velocity=" << (int) noteOff->velocity
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_NOTE_ON:
      {
	MidiNoteOn* noteOn=dynamic_cast<MidiNoteOn*>(event);
	cout << "eventType=MidiEventNode::NoteOn,channel="
	     << (int) noteOn->channel
	     << " note=" << (int) noteOn->note
	     << " velocity=" << (int) noteOn->velocity
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_KEY_PRESSURE:
      {
	MidiKeyPressure* keyPressure=dynamic_cast<MidiKeyPressure*>(event);
	cout << "eventType=MidiEventNode::keyPressure,channel="
	     << (int) keyPressure->channel
	     << " note=" << (int) keyPressure->note
	     << " keyPressure=" << (int) keyPressure->keyPressure
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_CONTROL_CHANGE:
      {
	MidiControlChange* controlChange=dynamic_cast<MidiControlChange*>(event);
	cout << "eventType=MidiEventNode::controlChange,channel="
	     << (int) controlChange->channel
	     << " control=" << (int) controlChange->control
	     << " value=" << (int) controlChange->value
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_PROGRAM_CHANGE:
      {
	MidiProgramChange* programChange=dynamic_cast<MidiProgramChange*>(event);
	cout << "eventType=MidiEventNode::programChange,channel="
	     << (int) programChange->channel
	     << " program=" << (int) programChange->program
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_CHANNEL_PRESSURE:
      {
	MidiChannelPressure* channelPressure=
	  dynamic_cast<MidiChannelPressure*>(event);
	cout << "eventType=MidiEventNode::channelPressure,channel="
	     << (int) channelPressure->channel
	     << " pressure=" << (int) channelPressure->pressure
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_PITCH_BEND:
      {
	MidiPitchBend* pitchBend=dynamic_cast<MidiPitchBend*>(event);
	cout << "eventType=MidiEventNode::pitchBend,channel="
	     << (int) pitchBend->channel
	     << " lsb=" << (int) pitchBend->lsb
	     << " msb=" << (int) pitchBend->msb
	     << endl;
      }
      break;
    case MidiEventNode::MIDI_SYS_EX:
      {
	MidiSysEx* sysEx=dynamic_cast<MidiSysEx*>(event);
	cout << "eventType=MidiEventNode::sysEx,size=" << (int) sysEx->size
	     << " channel="<< sysEx->channel << endl;
      }
      break;
      
    case MidiEventNode::MIDI_PORT_CHANGE:
      {
	MidiPortChange* portChange=dynamic_cast<MidiPortChange*>(event);
	cout << "eventType=MidiEventNode::portChange,port=" << portChange->port
	     << endl;
      }
      break;
    case MidiEventNode::TEXT_OUT:
      cout << "eventType=MidiEventNode::TEXT_OUT"
	   << ":text=" << (dynamic_cast<MidiTextOut*>(event))->text
	   << endl;
      break;
    case MidiEventNode::END:
      cout << "eventType=MidiEventNode::END" << endl;
      break;
    case MidiEventNode::NOT_SET:
      cout << "eventType=MidiEventNode::NOT_SET" << endl;
      break;
    }
}
