/*
    MidiEventList.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "MidiEventList.h"

MidiEventList::MidiEventList ():
  length (0),first (NULL),last (NULL),
  totalTicks (0),totalLogicalTicks (0),totalMeasures (0)
{
  /*do nothing*/;
}

MidiEventList::~MidiEventList ()
{
  const MidiEvent* tmp1 = first;
  while (tmp1!=NULL)
    {
      const MidiEvent* tmp2 = tmp1->getNext ();
      delete tmp1;
      tmp1 = tmp2;
    }
}

void MidiEventList::appendFormatEvent (SongStatus::Format format,
				       uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventFormat (format, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendNTracksEvent (int16 nTracks,
					uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventNTracks (nTracks, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendDivisionEvent (int16 division,
					 uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventDivision (division, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendTempoEvent (uint32 tempo,
				      uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventTempo (tempo, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendBpmEvent (uint32 bpm,
				    uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventBpm (bpm, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendTimeSignatureEvent (int nn, int dd, int cc, int bb,
					      uint ticks_,
					      double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventTimeSignature (nn, dd, cc, bb, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendTimeSignature2Event (int nn, int dd,
					       uint ticks_,
					       double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventTimeSignature2 (nn, dd, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendKeySignatureEvent (int sf, int mi,
					     uint ticks_,
					     double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventKeySignature (sf, mi, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiNoteOffEvent (int port, uchar channel,
					    uchar note, uchar velocity,
					    uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiNoteOff (port, channel, note, velocity, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiNoteOnEvent (int port, uchar channel,
					   uchar note, uchar velocity,
					   uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiNoteOn (port, channel, note, velocity,
			     ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiKeyPressureEvent (int port, uchar channel,
						uchar note, uchar keyPressure,
						uint ticks_,
						double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiKeyPressure (port, channel, note, keyPressure,
				  ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiControlChangeEvent (int port, uchar channel,
						  uchar control, uchar value,
						  uint ticks_,
						  double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiControlChange (port, channel, control, value,
				    ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiProgramChangeEvent (int port, uchar channel,
						  uchar program,
						  uint ticks_,
						  double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiProgramChange (port, channel, program,
				    ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiChannelPressureEvent (int port, uchar channel,
						    uchar pressure,
						    uint ticks_,
						    double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiChannelPressure (port, channel, pressure,
				      ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiPitchBendEvent (int port, uchar channel,
					      uchar lsb, uchar msb,
					      uint ticks_,
					      double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiPitchBend (port, channel, lsb, msb,
				ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendMidiSysExEvent (const uchar* data, uint32 size,
					  uint ticks_, double logicalTicks_,
					  int port = -1, int channel = -1)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiSysEx (data, size, ticks_, logicalTicks_, port, channel);
  append (event);
}

void MidiEventList::appendMidiPortChangeEvent (int port, uint ticks_,
					       double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventMidiPortChange (port, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendTextOutEvent (const char* data, int size,
					uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventTextOut (data,size, ticks_, logicalTicks_);
  append (event);
}

void MidiEventList::appendEndEvent (uint ticks_, double logicalTicks_)
{
  MidiEvent* event = new MidiEvent ();
  event->setEventEnd (ticks_, logicalTicks_);
  append (event);
}

//for debug
void MidiEventList::printEvent () const
{
  const MidiEvent* tmp1 = first;
  while (tmp1 != NULL)
    {
      tmp1->printEvent ();
      tmp1 = tmp1->getNext ();
    }
}
