/*
    MidiEventList.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#ifndef MidiEventList_H
#define MidiEventList_H

#include "MidiEvent.h"

class MidiEventList
{
protected:
  uint length;
  MidiEvent* first;
  MidiEvent* last;

  uint32 totalTicks;
  double totalLogicalTicks;
  uint32 totalMeasures;

  //linked list$B$N?t$r?t$($F(Blength$B$H0lCW$9$k$+D4$Y$k!#(B(for debug)
  bool checkList ()
    {
      uint nowLength = 0;
      for (const MidiEvent* now = first; now != last; now = now->getNext ())
	nowLength++;
      return (nowLength == length);
    };
public:
  MidiEventList ();
  ~MidiEventList ();

public:
  inline void append (MidiEvent* event)
    {
      length++;
      if (first==NULL)
	{
	  first=event;
	  last=event;
	}
      else
	{
	  last->setNext (event);
	  last=event;
	}
      assert (checkList ());
    };


  void appendFormatEvent (SongStatus::Format format,
			  uint ticks_, double logicalTicks_);
  void appendNTracksEvent (int16 nTracks, uint ticks_, double logicalTicks_);
  void appendDivisionEvent (int16 division, uint ticks_, double logicalTicks_);
  void appendTempoEvent (uint32 tempo, uint ticks_, double logicalTicks_);
  void appendBpmEvent (uint32 bpm, uint ticks_, double logicalTicks_);
  void appendTimeSignatureEvent (int nn, int dd, int cc, int bb,
				 uint ticks_, double logicalTicks_);
  void appendTimeSignature2Event (int nn, int dd,
				  uint ticks_, double logicalTicks_);
  void appendKeySignatureEvent (int sf, int mi,
				uint ticks_, double logicalTicks_);

  void appendMidiNoteOffEvent (int port, uchar channel,
			       uchar note, uchar velocity,
			       uint ticks_, double logicalTicks_);
  void appendMidiNoteOnEvent (int port, uchar channel,
			      uchar note, uchar velocity,
			      uint ticks_, double logicalTicks_);
  void appendMidiKeyPressureEvent (int port, uchar channel,
				   uchar note, uchar  keyPressure,
				   uint ticks_, double logicalTicks_);
  void appendMidiControlChangeEvent (int port, uchar channel,
				     uchar control, uchar value,
				     uint ticks_, double logicalTicks_);
  void appendMidiProgramChangeEvent (int port, uchar channel, uchar program,
				     uint ticks_, double logicalTicks_);
  void appendMidiChannelPressureEvent (int port, uchar channel, uchar pressure,
				       uint ticks_, double logicalTicks_);
  void appendMidiPitchBendEvent (int port, uchar channel, uchar lsb, uchar msb,
				 uint ticks_, double logicalTicks_);
  //channel$B;XDj$OK\MhITMW$@$,%]!<%H;XDj$N0Y$K$"$k!#(B
  //channel==-1$B$N;~$O%]!<%H;XDjL5$7!#(B
  void appendMidiSysExEvent (const uchar* data, uint32 size,
			     uint ticks_, double logicalTicks_,
			     int port = -1, int channel=-1);
  void appendMidiPortChangeEvent (int port, uint ticks_, double logicalTicks_);
  void appendTextOutEvent (const char* data, int size,
			   uint ticks_, double logicalTicks_);
  void appendWaitTimeEvent  (ulong waitMachineTicks_,
			     uint ticks_, double logicalTicks_);
  void appendEndEvent (uint ticks_, double logicalTicks_);

  
  void setTotals (uint32 totalTicks_, double totalLogicalTicks_,
		  uint32 totalMeasures_)
    {
      totalTicks = totalTicks_;
      totalLogicalTicks = totalLogicalTicks_;
      totalMeasures = totalMeasures_;
    };
  uint getLength () const {return length;};
  const MidiEvent* getFirstEvent () const {return first;};
  uint32 getTotalTicks () const {return totalTicks;};
  double getTotalLogicalTicks () const {return totalLogicalTicks;};
  uint32 getTotalMeasures () const {return totalMeasures;};

  void printEvent () const;//for debug
};

#endif //MidiEventList_H
