/*
    SongEvent.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "SongEvent.h"
#include "Song/SongFactory.h"

SongEvent::SongEvent (const char* const fileName, State* state_,
		      SongCommand* songCommand_,
		      MainIO& userIO, Option* option_):
  Song (state_,songCommand_,userIO,option_),midiEventList (NULL)
{
  midiEventList=new MidiEventList ();
#if 0
  WidiController widiController;//dummy
  MainIOConvert2Midi mainIOConvert (state, &widiController,
				    option,midiEventList);
  Song* songConvert=newSong (fileName,state,songCommand,mainIOConvert,option,
			     *midiEventList);
  SequencerConvert2Midi sequencerConvert (state,option,midiEventList);
  songConvert->play (sequencerConvert);//convert!
  
  state->lock ();
  midiEventList->setTotals (state->songStatus.ticks,
			    state->songStatus.logicalTicks,
			    state->songStatus.measures);
  *cacheSongStatus=state->songStatus;
  cacheSongStatus->totalTicks = state->songStatus.ticks;
  cacheSongStatus->totalMachineTicks = state->songStatus.machineTicks;
  cacheSongStatus->totalLogicalTicks = state->songStatus.logicalTicks;
  cacheSongStatus->totalMeasures = state->songStatus.measures;
  cacheSongStatus->totalEvents = midiEventList->getLength ();
  state->unlock ();
  delete songConvert;
#endif
  
#ifdef DEBUG
  //midiEventList->printEvent ();
  cerr << "totalTicks=" << cacheSongStatus->totalTicks << endl;
  cerr << "totalLogicalTicks=" << cacheSongStatus->totalLogicalTicks << endl;
  cerr << "totalMesureas=" << cacheSongStatus->totalMeasures << endl;
#endif

}

SongEvent::~SongEvent ()
{
  delete midiEventList;
}

void SongEvent::doPlay (Sequencer& sequencer)
{
  DEBUG_MESSAGE ("SongEvent::doPlay");
  uint32 lastTicks = 0;//$BA02s(BwaitTime$B$7$?$H$-$N@dBP(Btick$B?t!#(B

  double machineTicks = 0;//$B8=:_1iAU$7$F$$$k@dBP(Bmachine ticks
  double dMachineTicks = 0;//$B7W;;MQ!#(B
  double lastDumpMachineTicks = 0;//$BA02s(BdumpBuf$B$7$F$+$i$N(Bmachine tick$B?t!#(B

  double logicalTicks = 0;//$B8=:_1iAU$7$F$$$k@dBP(Blogical ticks
  double dLogicalTicks = 0;//$B7W;;MQ!#(B

  cacheSongStatus->lastMeasureTicks = 0;//$BA02s>.@a$N=*$j$@$C$?;~$N(Bticks$B?t!#(B
  cacheSongStatus->lastMeasureMachineTicks = 0;//$BA02s>.@a$N=*$j$@$C$?;~$N(BmachineTicks$B?t!#(B
  cacheSongStatus->lastMeasureLogicalTicks = 0;//$BA02s>.@a$N=*$j$@$C$?;~$N(BlogicalTicks$B?t!#(B
  cacheSongStatus->ticksFromLastMeasure = 0;//$B>.@aFb$N(Bticks$B?t!#(B
  uint32 setTempoTicks = 0;//set tempo$B$7$?$H$-$N;~4V7W;;$NJd@5MQ!#(B
  uint32 oldTempo = cacheSongStatus->getTempo ();//set tempo$B$7$?$H$-$N;~4V7W;;$NJd@5MQ!#(B
  
  const MidiEvent* event = midiEventList->getFirstEvent ();
  uint length = 0;//$B8=:_CeL\$7$F$$$k%$%Y%s%H(B
  while (event != NULL)
    {
      //$B;~4VBT$A$,H/@8$7$?$i(Bbuffer$B$r(Bdump$B$9$k!#(B
      //$BBT$A;~4V7W;;!#(BMIDI ticks$B?t$+$i(Bmachine ticks$B?t$r7W;;!#(B
      dMachineTicks = 0;
      dLogicalTicks = 0;
      const uint32 playTicks = event->getTicks ();
      if (lastTicks<playTicks)
	{
	  dMachineTicks =
	    calcDMachineTicks (playTicks, lastTicks, setTempoTicks,
			       cacheSongStatus->getTempo (), oldTempo,
			       cacheSongStatus->rate,
			       cacheSongStatus->getDivision (),
			       cacheSongStatus->getSkew ());
	  dLogicalTicks =
	    calcDLogicalTicks (playTicks, lastTicks, setTempoTicks,
			       cacheSongStatus->getTempo (), oldTempo,
			       cacheSongStatus->getDivision ());
	  
	  //$B0l>.@aJU$j$N(Bticks$B?t(B=division*4*nn/dd$B$J$N$G!#(B
	  const int measureAdd=//$B2?>.@a?J$s$@$+(B
	    (playTicks-cacheSongStatus->lastMeasureTicks)*
	    cacheSongStatus->getDd ()/(uint)cacheSongStatus->getDivision()/
	    cacheSongStatus->getNn ()/4;
	  if (measureAdd>0)
	    {
	      cacheSongStatus->lastMeasureTicks += measureAdd*
		cacheSongStatus->getDivision ()*cacheSongStatus->getNn ()*4/
		cacheSongStatus->getDd ();
	      cacheSongStatus->lastMeasureMachineTicks = machineTicks;
	      cacheSongStatus->lastMeasureLogicalTicks = logicalTicks;
	      cacheSongStatus->measures += measureAdd;
	    }
	  cacheSongStatus->ticksFromLastMeasure =
	    playTicks-cacheSongStatus->lastMeasureTicks;
	}
      assert (dMachineTicks >= 0);
      assert (dLogicalTicks >= 0);
      
      if (static_cast<ulong> (machineTicks+dMachineTicks)-static_cast<ulong>(machineTicks)>0)
	{//$BBT$DI,MW$,$"$C$?!#(B
	  //buffer$B$r<B:]$K=q$-=P$7$F!"1iAU$N>uBV$r99?7!#(B
	  try
	    {
	      dumpBuffer (sequencer,lastTicks,playTicks,
			  machineTicks,dMachineTicks,
			  logicalTicks,dLogicalTicks,
			  lastDumpMachineTicks);
	    }
	  catch (SongCommand& command_)
	    {
	      if (command_.peekCommand () == SongCommand::REWIND)
		{//$B:G=i$KLa$C$F1iAU$7D>$9!#(B
		  cacheSongStatus->reset ();
		  realRewind ();
		  lastTicks = 0;
		  lastDumpMachineTicks = 0;
		  machineTicks = 0;
		  dMachineTicks = 0;
		  logicalTicks = 0;
		  dLogicalTicks = 0;
		  setTempoTicks = 0;
		  oldTempo = cacheSongStatus->getTempo ();
		  sequencer.startTimer ();
		  
		  event=midiEventList->getFirstEvent ();
		  length = 0;
		  continue;
		}
	      else if (command_.peekCommand () == SongCommand::ABORT)
		{
		  //XXX note off
		  cacheSongStatus->state = SongStatus::PLAYING;
		  goto END_OF_DATA;//$B6/@)=*N;!#(B
		  break;
		}
	      else
		{
		  //XXX$B8=:_$N=j(BSET$B$O(Bsupport$B$7$J$$!#(B
		  assert (command_.peekCommand () == SongCommand::SET);
		  cacheSongStatus->state = SongStatus::PLAYING;
		}
	    }
	  lastTicks = playTicks;
	  setTempoTicks = playTicks;
	  oldTempo = cacheSongStatus->getTempo ();
	}
      
      //event$B$r2r@O$9$k!#(B
      const MidiEventNode* eventNode=event->getEventNode ();
      while (event != NULL && playTicks == event->getTicks ())
	{
	  switch (eventNode->eventType)
	    {
	      //Song EVENT
	    case MidiEventNode::FORMAT:
	      cacheSongStatus->setFormat (dynamic_cast<const MidiFormat*>(eventNode)->format);
	      break;
	    case MidiEventNode::NTRACKS:
	      cacheSongStatus->setNTracks (dynamic_cast<const MidiNTracks*>(eventNode)->nTracks);
	      break;
	    case MidiEventNode::DIVISION:
	      cacheSongStatus->setDivision (dynamic_cast<const MidiDivision*>(eventNode)->division);
	      break;
	    case MidiEventNode::TEMPO:
	      oldTempo=cacheSongStatus->getTempo ();
	      setTempoTicks=playTicks;
	      cacheSongStatus->setTempo (dynamic_cast<const MidiTempo*>(eventNode)->tempo);
	      
	      break;
	    case MidiEventNode::BPM:
	      oldTempo = cacheSongStatus->getTempo ();
	      setTempoTicks = playTicks;
	      cacheSongStatus->setBpm (dynamic_cast<const MidiBPM*>(eventNode)->bpm);
	      break;
	    case MidiEventNode::TIME_SIGNATURE:
	      {
		const MidiTimeSignature* timeSignature=dynamic_cast<const MidiTimeSignature*>(eventNode);
		cacheSongStatus->setTimeSignature (timeSignature->nn,timeSignature->dd,timeSignature->cc,timeSignature->bb);
	      }
	    break;
	    case MidiEventNode::TIME_SIGNATURE2:
	      {
		const MidiTimeSignature2* timeSignature2=dynamic_cast<const MidiTimeSignature2*>(eventNode);
		cacheSongStatus->setTimeSignature (timeSignature2->nn,timeSignature2->dd);
	      }
	    break;
	    case MidiEventNode::KEY_SIGNATURE:
	      {
		const MidiKeySignature* keySignature=dynamic_cast<const MidiKeySignature*>(eventNode);
		cacheSongStatus->setKeySignature (keySignature->sf,keySignature->mi);
	      }
	    break;
	    case MidiEventNode::MIDI_NOTE_OFF:
	      {
		const MidiNoteOff* noteOff=dynamic_cast<const MidiNoteOff*>(eventNode);
		sequencer.noteOff (noteOff->port,noteOff->channel,noteOff->note,noteOff->velocity);
	      }
	    break;
	    case MidiEventNode::MIDI_NOTE_ON:
	      {
		const MidiNoteOn* noteOn=dynamic_cast<const MidiNoteOn*>(eventNode);
		sequencer.noteOn (noteOn->port,noteOn->channel,noteOn->note,noteOn->velocity);
	      }
	    break;
	    case MidiEventNode::MIDI_KEY_PRESSURE:
	      {
		const MidiKeyPressure* keyPressure=dynamic_cast<const MidiKeyPressure*>(eventNode);
		sequencer.polyphonicKeyPressure (keyPressure->port,keyPressure->channel,keyPressure->note,keyPressure->keyPressure);
	      }
	    break;
	    case MidiEventNode::MIDI_CONTROL_CHANGE:
	      {
		const MidiControlChange* controlChange=dynamic_cast<const MidiControlChange*>(eventNode);
		sequencer.controlChange (controlChange->port,controlChange->channel,controlChange->control,controlChange->value);
	      }
	    break;
	    case MidiEventNode::MIDI_PROGRAM_CHANGE:
	      {
		const MidiProgramChange* programChange=dynamic_cast<const MidiProgramChange*>(eventNode);
		sequencer.programChange (programChange->port,programChange->channel,programChange->program);
	      }
	    break;
	    case MidiEventNode::MIDI_CHANNEL_PRESSURE:
	      {
		const MidiChannelPressure* channelPressure=dynamic_cast<const MidiChannelPressure*>(eventNode);
		sequencer.channelPressure (channelPressure->port,channelPressure->channel,channelPressure->pressure);
	      }
	    break;
	    case MidiEventNode::MIDI_PITCH_BEND:
	      {
		const MidiPitchBend* pitchBend=dynamic_cast<const MidiPitchBend*>(eventNode);
		sequencer.pitchBendChange (pitchBend->port,pitchBend->channel,pitchBend->lsb,pitchBend->msb);
	      }
	    break;
	    case MidiEventNode::MIDI_SYS_EX:
	      {
		const MidiSysEx* sysEx=dynamic_cast<const MidiSysEx*>(eventNode);
		sequencer.exclusive (sysEx->udata,sysEx->size);
	      }
	    break;
	    case MidiEventNode::MIDI_PORT_CHANGE:
	      sequencer.portChange (dynamic_cast<const MidiPortChange*>(eventNode)->port);
	      break;
	    case MidiEventNode::TEXT_OUT:
	      io.normalMessage (dynamic_cast<const MidiTextOut*>(eventNode)->text.c_str ());
	      break;
	    case MidiEventNode::END:
	      goto END_OF_DATA;
	      break;
	    case MidiEventNode::NOT_SET:
	      //should not be reached
	      cerr << "ERROR:SongEvent:doPlay case NOT_SET is reached." << endl;
	      break;
	    };
	  event=event->getNext ();
	  length++;
	  cacheSongStatus->events=length;
	}
    }
  
 END_OF_DATA:
  ;
}


void SongEvent::realRewind ()
{
  /////////////////////////////////////////track$B$r:G=i$N0LCV$K$9$k!#(B
  cacheSongStatus->totalTicks=midiEventList->getTotalTicks ();
  cacheSongStatus->totalMachineTicks=static_cast<uint> (midiEventList->getTotalLogicalTicks ()*cacheSongStatus->rate/SongStatus::SONG_CTRL_RATE);
  cacheSongStatus->totalLogicalTicks=static_cast<uint> (midiEventList->getTotalLogicalTicks ());
  cacheSongStatus->totalMeasures=midiEventList->getTotalMeasures ();
  cacheSongStatus->totalEvents=midiEventList->getLength ();
}
