/*
    G36Track.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "G36Track.h"

const int G36Track::G36_TRACK_HEADER_SIZE;

G36Track::G36Track ():
  RCPTrack ()
{
  //do nothing
}

G36Track::~G36Track ()
{
  //do nothing
}


void G36Track::readTrackData (RCPBufferStream& rcpBufferStream) throw (Error)
{
  //read data size
  const uchar lsb0=rcpBufferStream.get8 ();
  const uchar msb0=rcpBufferStream.get8 ();
  const uchar lsb1=rcpBufferStream.get8 ();
  const uchar msb1=rcpBufferStream.get8 ();
  dataSize=(msb1 << 24)+(lsb1 << 16)+(msb0 << 8)+lsb0;
#ifdef DEBUG
  cerr << "DEBUG:G36Track:track size=" << dataSize
       << " to " << hex << rcpBufferStream.tellg ()+dataSize << endl;
#endif
  trackIndex=rcpBufferStream.get8 ();//read track number
  rythm=rcpBufferStream.get8 ();
  //read rythm OFF=0x00,RH1=0x80,RH2=0xA0,RH3=0xC0,RH4=0xC0,
  setRCPMidiChannel (rcpBufferStream.get8 ());//read midi channel
  uchar const tmp=rcpBufferStream.get8 ();//read key+ OFF=0x80//???
  if (tmp==0x80 || tmp==0)
    keyPlus=0;
  else
    keyPlus=tmp-128;
  stPlus=rcpBufferStream.get8 ();//read st+
  mode=rcpBufferStream.get8 ();//read mode
  rcpBufferStream.read (comment,MAX_COMMENT_LENGTH);//read comment

  //read track data
  loadBuffer
    (rcpBufferStream.getCurrentBufferPtr (),dataSize-G36_TRACK_HEADER_SIZE);
  rcpBufferStream.seekgCur (dataSize-G36_TRACK_HEADER_SIZE);

#ifdef DEBUG
  /*
  cerr << "trackIndex=" << (int) trackIndex << endl;
  cerr << "rythm=" << (int) rythm << endl;
  cerr << "midiChannel=" << (int) midiChannel << endl;
  cerr << "keyPlus=" << (int) keyPlus << endl;
  cerr << "stPlus=" << (int) stPlus << endl;
  cerr << "mode=" << (int) mode << endl;
  */
#endif
}

uint32 G36Track::peekStepTime () const
{
  return (uchar) buffer[index+2]+(buffer[index+3] << 8);
}
