/*
    RCP.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef RCP_H
#define RCP_H

#include "IO/MainIO.h"
#include "Song/Song.h"
#include "Song/SongInputStream.h"
#include "Song/recomposer/RCPBufferStream.h"
#include "Rythm.h"
#include "UserExclusive.h"
#include "RCPTrack.h"

class RCP:public Song
{
protected:
  //magic number
  static const int RCP_MAGIC_LENGTH = 32;
  static const char RCP_MAGIC[RCP_MAGIC_LENGTH+1] = 
  "RCM-PC98V2.0(C)COME ON MUSIC\r\n\0\0";
  //"COME ON MUSIC RECOMPOSER RCP3.0\0";
  //$B$3$ND9$50J2<$@$C$?$i(B.rcp file$B$H$7$F$OC;$9$.$k!#(B
  static const int RCP_LEAST_FILE_LENGTH = 0x5B1;
  
  char title[64+1];//title
  //char memo[336+1];//memo
  char memo[360+1];//memo 360$B$J$N$O(Bg18,g36$B$XBP1~$9$k0Y(B
  uchar key;//??? key
  void setKey (uchar setKey);
  //key$B$r@_Dj$7$F!"BP1~$9$k(BsongStatus.sf,songStatus.mi$B$r@_Dj$9$k!#(B
  char playBias;//??? play bias
  static const int RCP_MAX_FILENAME_LENGTH = 16;
  char controlFileNameCM6[RCP_MAX_FILENAME_LENGTH+1];//config file name .CM6
  char controlFileNameGSD[RCP_MAX_FILENAME_LENGTH+1];//config file name .GSD
  uchar toneNameTB[32];//??? tonename .TB?$B$NCM!#(B
  //uchar trackNumber;
  //songStatus.nTrack$B$rMQ$$$k!#(B
  static const int MAX_RYTHMS = 32;
  static const int RCP_RYTHMS_NUMBER = 32;
  Rythm** rythm;
  static const int RCP_MAX_USER_EXCLUSIVE = 8;
  UserExclusive userExclusive[RCP_MAX_USER_EXCLUSIVE];
  RCPTrack** rcpTrack;//track$B$,4v$DM-$k$+ITL@$J$N$GF0E*$K3NJ]!#(B
  virtual RCPTrack* newTrack ();
  
public:
  //constructors
  //for playing
  RCP (State* state_, SongCommand* songCommand_, MainIO& userIO, 
       Option* option_);
  RCP (const string& fileName, State* state_, SongCommand* songCommand_, 
       MainIO& userIO, Option* option_);
  RCP (SongInputStream& songInputStream, State* state_,
       SongCommand* songCommand_, MainIO& userIO, Option* option_);

#if 0
  //for converting
  RCP (const string& fileName, State* state_, SongCommand* songCommand_, 
       MainIOConvert2Dumb& userIO,
       Option* option_, DumbEventList& dumbEventList_);
#endif
  
  //destructor
  virtual ~RCP ();
  //$B$3$N%U%!%$%k$KBP1~$7$F$$$k$+(B?
  static bool isSupported (const string& fileName_);
  virtual void dumpTrack ();

protected:
    //loadBuffer$B$NJd=u(B,$B<B:]$K%G!<%?$rFI$_9~$`!#(B
  virtual void doLoadBuffer (const char* buffer, int bufferSize);//throw (Error)
  //$B1iAUAm;~4V$d>.@a?t$H$+$r5a$a$k!#(B
  virtual void preSearch ();
  
  //RCP$B$N(Bheader$B$rFI$_9~$`!#(B
  virtual void readHeader (RCPBufferStream& rcpBufferStream) throw (Error);
  void doPlay (Sequencer& sequencer);
  //play method$B$NJd=u!#(B
  //$B<B:]$K$3$3$G1iAU$9$k!#(B
  //$B0J2<(BdoPlay$B$NJd=u(B
  uint32 doNoteOff (Sequencer& sequncer, uint32 noteOffOffset);
  ////note off$B$N=hM}!#(B
  //noteOffOffset$B$h$j>.$5$$(Bgate time$B$r;}$DJ*$r(Bnote off$B$9$k!#(B
  //$BJV$jCM$O(BnoteOffOffset$B$h$jBg$-$$(Bgate time$B$NCf$G:GBg$NJ*!#(B
  virtual void getParameter (int playTrack, uchar& note, uint32& stepTime,
			     uchar& gateTime, uchar& velocity);
  void doEvent (Sequencer& sequncer, int playTrack, uint32& oldTempo,
		uint32& setTempoTicks, uint16& nEndOfTracks,
		double machineTicks, double logicalTicks);
  //$B<B:]$K(B(RCP$B$N(B)$B%$%Y%s%H$r2r<a<B9T$9$k!#(B
  void doRCPCommand (Sequencer& sequencer, int playTrack,
		     uint32& oldTemp, uint32& setTempoTicks, 
		     uint16& nEndOfTracks,
		     double machineTicks, double logicalTicks);
  ////////////////note on$B0J30$N(Bcommand$B$r<B9T$9$k!#(B
  void doUserExclusive(Sequencer& sequencer, int playTrack,
		       const uchar command, const uchar gateTime,
		       const uchar velocity);
  virtual void channelExclusive (Sequencer& sequencer, int playTrack, 
				 const uchar command, const uchar gateTime,
				 const uchar velocity);
  virtual void externalCommand (int playTrack, uchar gateTime, uchar velocity);//unsupport
  virtual void bankAndProgramLSB (Sequencer& sequencer, int playTrack,
				  uchar gateTime, uchar velocity);
  void bankAndProgramMSB (Sequencer& sequencer, int playTrack, uchar gateTime,
			  uchar velocity);
  void keyScan (Sequencer& sequencer, int playTrack, uchar gateTime);//XXX not supported yet.
  void midiChannelChange (int playTrack, uchar gateTime);
  virtual void tempoChange (int playTrack, uint32& oldTempo,
			    uint32& setTempoTicks,
			    uchar gateTime, uchar velocity);//XXX
  void afterTouch (Sequencer& sequencer, int playTrack, uchar gateTime);
  void controlChange (Sequencer& sequencer, int playTrack,
		      uchar gateTime, uchar velocity);
  void programChange (Sequencer& sequencer, int playTrack, uchar gateTime);
  void afterTouchPolyphonic (Sequencer& sequencer,
			     int playTrack, uchar gateTime, uchar velocity);
  void pitchBend (Sequencer& sequencer,
		  int playTrack, uchar gateTime, uchar velocity);
  ////////////////////////////////////////////////////////SysEx YAMAHA
  void yamahaBase (int playTrack, uchar gateTime, uchar velocity);//???
  void yamahaDeviceID (int playTrack, uchar gateTime, uchar velocity);//???
  void yamahaAddressAndParameter (Sequencer& sequencer,int playTrack,
				  uchar gateTime, uchar velocity);//???
  void yamahaXG (Sequencer& sequencer,
		 int playTrack, uchar gateTime, uchar velocity);//???
  
  ////////////////////////////////////////////////////////SysEx ROLAND
  void rolandBase (int playTrack, uchar gateTime, uchar velocity);
  void rolandParameter (Sequencer& sequencer,
			int playTrack, uchar gateTime, uchar velocity);
  void rolandDeviceAndModelID (int playTrack, uchar gateTime, uchar velocity);
  
  /////////////////////////////////////////$B@)8f7O(B
  void keyChange (int playTrack, uint32 stepTime);
  virtual void commentStart (int playTrack);
  void loopEnd (int playTrack, uint32 stepTime);
  void loopStart (int playTrack);
  virtual void sameMeasure (int playTrack,
			    double machineTicks, double logicalTicks);
  void measureEnd (int playTrack, double machineTicks, double logicalTicks);
  void endOfTrack (int playTrack, uint16& nEndOfTracks);

  //////////////////////////////////////////////////////////for play control
  //$B1iAU0LCV$r:G=i$K4,$-La$9!#(B($B0?$O1iAU0LCV$r;XDj0LCV$^$G0\F0$9$k!#(B)
  virtual void realRewind ();
};
#endif// RCP_H

