/*
    RCPTrack.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef RCPTrack_H
#define RCPTrack_H

#include <vector>

#include "Sequencer/Part.h"
#include "Song/recomposer/RCPBufferStream.h"

using namespace std;

class RCPTrack:public SongBufferStream
{
public:
  /////////////////////////////////////////header$B>pJs(B
  uint32 dataSize;
  static const int RCP_TRACK_HEADER_SIZE=44;
  uchar trackIndex;
  static const uchar RCP_RYTHM_ON=0x80;
  static const uchar RCP_RYTHM_OFF=0x00;
  uchar rythm;//ON=0x80,OFF=0x00
  static const char RCP_KEYPLUS_OFF = 0x80;
  char keyPlus;//read key+ OFF=0x80//???
  uchar stPlus;
  static const uchar RCP_MODE_PLAY=0;
  static const uchar RCP_MODE_MUTE=1;
  static const uchar RCP_MODE_MIX=2;
  static const uchar RCP_MODE_REC=4;
  uchar mode;//0:Play 1:Mute 2:Mix 4:Rec
  static const int MAX_COMMENT_LENGTH=36;
  char comment[MAX_COMMENT_LENGTH];

private:
  //headerMidiChannel$B$O(Btrack data$B$N(Bheader$B$K;XDj$5$l$F$$$k(BMIDI channel$B$r(B
  //$B3P$($F$*$/!#(Brewind$B$9$k;~$KI,MW!#(B
  uchar headerMidiChannel;//MPU=0..15,RS=16..31,OFF=0xFF
  uchar midiChannel;//MPU=0..15,RS=16..31,OFF=0xFF
  
  //////////////////////////////////////////$B1iAU$N$?$a$N>pJs(B
public:
  static const int32 RCP_NOTE_OFF=-1;
  int32 gateTimes[Part::MAX_NOTES];//gate time$B2;$NLD$C$F$$$k;D$j;~4V(B($B2;D9(B)
  //=RCP_NOTE_OFF$B$J$i(Bnote on$B$7$F$$$J$$!#(B
  uint32 ticks;//$B8=:_1iAU$7$F$$$k2U=j$N:G=i$+$i$N(Bticks$B?t!#(B
               //==~0$B$J$i$3$N%H%i%C%/$N1iAU$O=*N;$7$?;v$r0UL#$9$k!#(B
  uint32 measures;//$B8=:_1iAU$7$F$$$k2U=j$N:G=i$+$i$N>.@a?t!#(B
  
  ////////////////////////////////////////for loop
  struct Loop
  {
  public:
    uint32 loopStart;
    int loopCount;
  };
  vector<Loop> loopStack;
  void pushLoopStack ();
  void popLoopStack ();
  int getLeftLoopCount ();
  //$B;D$C$F$$$k(Bloop count$B$rJV$9!#(B
  //$B$^$@@_Dj$5$l$F$$$J$$$H$-$O(B-1$B$rJV$9!#(B
  void setLoopCount (uchar loopCount);
  void decrimentLoopCount ();
  void setLoopStart ();
  ///////////////////////////////////for same measure
  struct SameMeasure
  {
  public:
    uint32 fromIndex;//$B$3$3$G(Bsame measure event$B$,$"$C$?!#(B
    uchar sameMeasureNo;
    uint32 offsetAddress;
    bool isInSameMeasure;
    SameMeasure (){reset ();};
    void reset ()
      {
	fromIndex=0;
	sameMeasureNo=0;
	offsetAddress=0;
	isInSameMeasure=false;
      };
  };
  SameMeasure sameMeasure;
  void setSameMeasure (uint16 measureNumber,uint16 offsetAddress);
  void unsetSameMeasure ();
  //$B8=:_$N(Bsame measure$B$K$J$C$F$$$kItJ,$rGK4~$7$F!"(Bsame measure$B$r(B
  //$B8F$S=P$7$?=j$KLa$k!#(B
  bool isInSameMeasure (){return sameMeasure.isInSameMeasure;};//same measure$BFb$+(B?

  /////////////////////////////////////for SysEx
  bool isYamahaAddressInited;
  uchar yamahaAddressHigh;//$B>e0L%"%I%l%9(B
  uchar yamahaAddressMiddle;//$B>e0L%"%I%l%9(B
  bool isYamahaIDInited;
  uchar yamahaDeviceID;//device ID
  uchar yamahaModelID;//model ID

  bool isRolandAddressInited;
  uchar rolandAddressHigh;//$B>e0L%"%I%l%9(B
  uchar rolandAddressMiddle;//$BCf0L%"%I%l%9(B
  bool isRolandIDInited;
  uchar rolandDeviceID;//device ID
  uchar rolandModelID;//model ID

  ///////////////////////////////////////////////////methods
public:
  RCPTrack ();
  virtual void readTrackData (RCPBufferStream& rcpBufferStream) throw (Error);
  ~RCPTrack ();
  uint32 peekABSDeltaTime ()
    {return (isEnd ())?~0:ticks+peekStepTime ();};
  //$B$3$l$O(Bindex$B$r?J$a$J$$!#(Bticks$B$r2C;;$7$J$$!#(B
  uint32 getABSDeltaTicks (){return (isEnd ())?~0:ticks+=peekStepTime ();};
  //XXX$B$3$l$O(Bindex$B$r?J$a$J$$!#$,!"(Bticks$B$r2C;;$9$k!#(B//XXX
  uint32 getABSTicks () const {return ticks;};//$B8=:_1iAUCf$N(Btick$B?t$rJV$9!#(B
  uint32 getNoteOffABSTicks (int* note) const;
  void addTicks (uint32 stepTimes){ticks+=stepTimes;};
  //stepTimes$B$@$1(Bticks$B$r?J$a$k!#(B
  virtual uint32 peekStepTime () const;
  bool isEnd () const {return (ticks==~(uint32)0);};
  //track$B$,:G8e$^$G1iAU$7$?$+(B?
  void setEnd () {ticks=~0;};//track$B$,:G8e$^$G1iAU$7$?$H$9$k!#(B
  void rewind ();


  uchar getRCPMidiChannel () const {return midiChannel;};
  void setRCPMidiChannel (uchar midiChannel__) {midiChannel = midiChannel__;};
  uchar getMidiChannel () const {return midiChannel % MIDI::MAX_CHANNEL;}
  int getMidiPort () const {return midiChannel/MIDI::MAX_CHANNEL;};
};

#endif // RCPTrack_H
