/*
    UserExclusive.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "UserExclusive.h"

const int UserExclusive::RCP_MAX_MEMO_LENGTH;
const int UserExclusive::RCP_MAX_USER_EXCLUSIVE_LENGTH;

UserExclusive::UserExclusive ()
{
  memset (memo,0x20,RCP_MAX_MEMO_LENGTH);
  memset (exclusive,0xF7,RCP_MAX_USER_EXCLUSIVE_LENGTH);
}

  
UserExclusive::~UserExclusive ()
{
  //do nothing
}

void UserExclusive::load
(RCPBufferStream& rcpBufferStream,bool midiIsG36=false)
{
  isG36=midiIsG36;
  if (isG36)
    {
      rcpBufferStream.read (memo,RCP_MAX_MEMO_LENGTH-1);
      rcpBufferStream.read (exclusive,RCP_MAX_USER_EXCLUSIVE_LENGTH);
    }
  else
    {
      rcpBufferStream.read (memo,RCP_MAX_MEMO_LENGTH);
      rcpBufferStream.read (exclusive,RCP_MAX_USER_EXCLUSIVE_LENGTH-1);
    }
}

const uchar UserExclusive::GATE_DATA=0x80;
const uchar UserExclusive::VELOCITY=0x81;
const uchar UserExclusive::MIDI_CHANNEL=0x82;
const uchar UserExclusive::CLEAR_CHECK_SUM=0x83;
const uchar UserExclusive::CHECK_SUM=0x84;
const uchar UserExclusive::END_OF_EXCLUSIVE=0xF7;

uchar UserExclusive::storeData
(uchar storedData[RCP_MAX_USER_EXCLUSIVE_LENGTH+2],
 const uchar gateData,const uchar velocity,const uchar rcpMidiChannel)
{
  storedData[0]=0xF0;//beginning of exclusive event
  uchar length=1;
  uchar checkSum=0;
  for (int i=0;
       i<RCP_MAX_USER_EXCLUSIVE_LENGTH-((isG36)?0:1) &&
	 exclusive[i]!=END_OF_EXCLUSIVE;
       i++)
    {
      switch (exclusive[i])
	{
	case GATE_DATA: // gate data
	  checkSum+=storedData[length++]=gateData;
	  break;
	case VELOCITY: // velocity
	  checkSum+=storedData[length++]=velocity;
	  break;
	case MIDI_CHANNEL: // midi channel
	  checkSum+=storedData[length++]=rcpMidiChannel;
	  break;
	case CLEAR_CHECK_SUM: //clear check sum
	  checkSum=0;
	  break;
	case CHECK_SUM: //check sum
	  storedData[length++]=128-(checkSum & 0x7F);
	  break;
	default:
	  checkSum+=storedData[length++]=exclusive[i];
	  break;
	}
    }
  
  storedData[length++]=END_OF_EXCLUSIVE;//end of exclusive
  return (length>2)?length:0;//length==2$B$N;~$O2?$bAw$kJ*$,L5$+$C$?!#(B
}
