#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <iostream>
#include <dlfcn.h>
#include <unistd.h>

#include "IO/MainIO.h"
#include "IO/InfoIO.h"
#include "Sequencer/OSS/MIDI/SequencerMIDI.h"
#include "Song/SMF/SMF.h"
#include "Song/recomposer/RCP.h"
#include "Song/SongFactory.h"

int
main (int argc,const char** argv)
{
  if (argc<=1)
    {
      exit (1);
    }

  Option option;
  //option.isQuickStart=true;
  State state;
  SongCommand songCommand;
  ServerCommand serverCommand;
#define MAX_PATH_NAME 256
  const char pathPrefix[]="/home/yamahata/proj/mimpi/src/IO/NCurses/";
  char ioPath[MAX_PATH_NAME];
  //cerr << "DEBUG:dlopen" << endl;
  strcpy (ioPath,pathPrefix);
  strcat (ioPath,"IONCurses.so");
  assert (dlopen (ioPath,RTLD_LAZY|RTLD_GLOBAL)!=NULL);
  strcpy (ioPath,pathPrefix);
  strcat (ioPath,"IONCurses.so");
  //strcat (ioPath,"IONLevel.so");
  //strcat (ioPath,"IONTrace.so");
  //strcat (ioPath,"../X/Xlib/IOXlib.so");
  void* handle=dlopen (ioPath,RTLD_LAZY|RTLD_GLOBAL);
  if (!handle)
    {
      cerr << "dlopen:" << ioPath << ":"<< dlerror () << endl;
      exit (1);
    }
  //cerr << "DEBUG:dlsym" << endl;
  InfoIO* infoIO=(InfoIO*) dlsym (handle,"infoIONCurses");
  //InfoIO* infoIO=(InfoIO*) dlsym (handle,"infoIONLevel");
  //InfoIO* infoIO=(InfoIO*) dlsym (handle,"infoIONTrace");
  //InfoIO* infoIO=(InfoIO*) dlsym (handle,"infoIOXlib");
  //cerr << "DEBUG:dlsym done" << endl;
  assert (infoIO!=NULL);
  option.isRealTime=true;
  //option.isRealTime=false;
  MainIO* io=infoIO->newMainIO (&state,&serverCommand,&option);
  //MainIO* io=new MainIO (&state,&option);
  

  SequencerOSS::open ();
  Sequencer* sequencer=new SequencerMIDI (&state,0,&option);
  //Sequencer* sequencer=new SequencerMIDI (&state,1,&option);
  sequencer->reset ();
  sequencer->sync ();

  for (int i=1;i<argc;i++)
    {
      Song* song = SongFactory::newSong (argv[i],&state,&songCommand,*io,&option);
#if 1
      song->play (*sequencer);
#else
      pthread_t playing;
      song->createPlayThread (&playing,NULL,*sequencer);
      void* ret;
      pthread_join (playing,&ret);
#endif      
    }

  delete sequencer;
  SequencerOSS::close ();

  delete io;
  dlclose (handle);
  cout << "all done" << endl;
  exit (0);
}
