/*
    ArrayStream.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include "ArrayStream.h"

ArrayStream::ArrayStream (const char* const bufferHead,int size):
  DataStream ("",""),
  buffer (NULL),bufferSize (-1),index (-1)
{
  loadBuffer (bufferHead,size);
}

ArrayStream::ArrayStream ():
  DataStream ("",""),
  buffer (NULL),bufferSize (-1),index (-1)
{
  //do nogthing
}

void ArrayStream::loadBuffer (const char* const bufferHead,int size) throw (Error)
{
  if (buffer!=NULL)
    {
      Error error ("this ArrayStream is already initialized.");
      throw (error);
    }
  if (size<0)
    {
      Error error ("invalid buffer size.(size<0)");
      throw (error);
    }
  

  bufferSize=size;
  buffer=new char[bufferSize];
  memcpy (buffer,bufferHead,bufferSize);
  index=0;
}

ArrayStream::~ArrayStream ()
{
  delete[] buffer;
}


uint8 ArrayStream::get8 ()
{
  if (buffer==NULL)
    {
      Error error ("Buffer is not loaded.");
      throw (error);
    }
  
  if (index+1>bufferSize)
    { // ⤦ɤǤʤǡʤ
      Error error ("Unexpected end of buffer.");
      throw (error);
    }

  uint8 value=static_cast<unsigned char>(buffer[index++]);
  return value; // 8 bit ǡ
}

void ArrayStream::read (char* ptr,int n) throw (Error)
{
  if (buffer==NULL)
    {
      Error error ("Buffer is not loaded.");
      throw (error);
    }
  
  if (index+n>bufferSize)
    {//ɤ߹٤dataĤäƤ뤫check
      Error error ("Unexpected end of buffer.");
      throw (error);
    }

  memcpy (ptr,buffer+index,n);
  index+=n;
}

void ArrayStream::seekgCur (int n)
{
  index+=n;
  if (index<0)
    {
      Error error ("invalid seeking.buffer under.");
      throw (error);
    }
      
  if (index>bufferSize)
    {
      Error error ("invalid seeking.buffer over");
      throw (error);
    }
}


