/*
    ArrayStream.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef ArrayStream_H
#define ArrayStream_H

#include <iostream>
#include <fstream>

#include "util/mytype.h"
#include "util/Error.h"
#include "DataStream.h"

using namespace std;

class ArrayStream:public DataStream
{
protected:
  char* buffer;//buffer==NULL not initialized  //data$B$rJ];}$7$F$*$/J*!#(B
  int bufferSize;//buffer$B$ND9$5(B
  int index;//$B8=:_FI$_9~$s$G$$$k0LCV!#(B
public:
  ArrayStream (const char* const bufferHead,int size);
  ArrayStream ();
  virtual ~ArrayStream ();
  //$B<B:]$K(Bbuffer$B$K%G!<%?$rFI$_9~$`!#(B
  virtual void loadBuffer (const char* const BufferHead,int size) throw (Error);
  //$B$A$c$s$H%G!<%?$rFI$_9~$s$@$+(B?($B=i4|2=$7$?$+(B?)
  virtual bool isInitialized () const {return (buffer!=NULL);};
  //buffer$B$ND9$5$rF@$k!#(B
  int getBufferSize () const {return bufferSize;};
  //$B;D$j$N(Bbuffer$B$ND9$5$rF@$k!#(B
  int getLeftLength ()const{return bufferSize-index;};
  virtual uint8 get8 ();
  void read (char* ptr,int n) throw (Error);
  void read (signed char *ptr,int n) {read ((char*)ptr,n);};
  void read (unsigned char* ptr,int n){read ((char*)ptr,n);};
  //1data$B$@$1@hFI$_$9$k!#(B
  uchar peekUChar () const {return (unsigned char) buffer[index];};
  char peekChar () const {return buffer[index];};
  //$BFI$_9~$_0LCV$rJQ99$9$k!#(Bcurrent$B$N0LCV$+$i$NAjBPE*0LCVJQ99!#(B
  void seekgCur (int n);
  //$B8=:_0LCV$rJV$9!#(B
  virtual long tellg (){return index;};
  //1byte$BJ,$@$1La$k!#(B
  void unputchar () {seekgCur (-1);};
  //$BFI$_9~$_0LCV$r@hF,$KLa$9!#(B
  virtual long rewind (){index=0;return 0;};
  //$B:G=i$+$i(Bn$B$N0LCV$KFI$_9~$_0LCV$rJQ99$9$k!#(B
  void setg (int n) {rewind ();seekgCur (n);}
  //$B8=:_;X$7$F$$$k>l=j$rJV$9!#(B
  const char* getCurrentBufferPtr () {return buffer+index;};

  virtual bool isRandomAccessible () const {return true;};
};
#endif//  ArrayStream_H
