#include <cassert>
#include <algorithm>

#include "BufferStream.h"

using namespace std;

BufferStream::BufferStream (DataStream* dataStream_,
			    long maxBufferSize_=
			    MEMORY_BUFFER2_DEFUALT_MAX_BUFFER_SIZE,
			    bool doesCreateRetrieveThread=true):
  DataStream (dataStream_->getParentName (),dataStream_->getBaseName ()),
  Threaded<MemoryBuffer2>(dataStream_,new MemoryBuffer2 ()),
  maxBufferSize (maxBufferSize_)
{
  if (doesCreateRetrieveThread)
    createRetrieveThread ();
}


BufferStream::~BufferStream ()
{
}

void BufferStream::close ()
{
  assert (memoryBuffer!=NULL);
  delete memoryBuffer;
  memoryBuffer=NULL;
}

//!!! NOTE !!! doesBlock=false$B$N;~$O(Bthread$B$r;H$C$F$$$kI,MW$,$"$k!#(B
long BufferStream::read (char* buffer,long size_,bool doesBlock)
{
  DEBUG_MESSAGE ("BufferStream::read locking");

  lock.lock ();
  assert (memoryBuffer!=NULL);
  assert (!(doesBlock==false &&
	    (doesRetrieveThreadRunning || dataStream==NULL)));
  if (doesBlock==false)
    {//non-blocking mode
      assert ((dataStream==NULL)==(memoryBuffer->isAllStored ()));
      if (doesRetrieveThreadRunning || dataStream==NULL)
	{
	  lock.unlock ();
	  return memoryBuffer->read (buffer,size_,doesBlock);
	}
      else
	{
	  assert (false);
	  lock.unlock ();
	  throw UnavailableOperation ();
	}
    }
  lock.unlock ();
  DEBUG_MESSAGE ("BufferStream::read unlocked");
  

  //blocking mode
  long readSize=0;
  while (readSize<size_)
    {
      long readySize;

      DEBUG_MESSAGE ("BufferStram::read while locking");
      lock.lock ();
      assert ((dataStream==NULL)==(doesRetrieveThreadRunning==false));
      const bool doesNeedRetrieving=
	(!doesRetrieveThreadRunning && dataStream!=NULL);
      lock.unlock ();
      DEBUG_MESSAGE ("BufferStram::read while unlocked");
      if (doesNeedRetrieving &&
	  ((readySize=memoryBuffer->readyBufferSize ())==0))
	{
	  DEBUG_MESSAGE ("BufferStream::read need retreiving from dataStream");
	  char* buffer_=new char[readySize] ;
	  DEBUG_MESSAGE ("BufferStream::read retreiving locking");
	  lock.lock ();
	  dataStream->read (buffer_,readySize);
	  if (dataStream->eod ())
	    {
	      delete dataStream;
	      dataStream=NULL;
	      lock.unlock ();
	      memoryBuffer->doEOD ();
	    }
	  else
	    lock.unlock ();
	  DEBUG_MESSAGE ("BufferStream::read retreiving unlocked");
	  memoryBuffer->cat (buffer_,readySize);
	  delete buffer_;
	}
      
      DEBUG_MESSAGE ("BufferStream::read now memoryBuffer->read");
      assert (size_>readSize);
      const long currentSize=memoryBuffer->read (buffer,size_-readSize);
      DEBUG_MESSAGE ("BufferStream::read now memoryBuffer->read done");
      buffer+=currentSize;
      readSize+=currentSize;
      if (memoryBuffer->eod ())
	break;
#ifdef DEBUG
      if (currentSize==0)
	{
	  assert (!memoryBuffer->eod ());
	  assert (readSize<size_);
	  //assert (dataStream!=NULL);
	  //assert (doesRetrieveThreadRunning);
	  cerr << "BufferStream::read currentSize=0" << endl;
	}
#endif
    }

  return readSize;
}
