#ifndef DataStream_H
#define DataStream_H

#include <string>
#include <list>

#include "util/mytype.h"

using namespace std;

class DataStream
{
protected:
  //name=parentName+baseName
  //name=parentName/baseName normal file
  //name=parentName#baseName archive
  string name;
  string parentName;
  string baseName;
  
  class EntryNode
  {
  public:
    string filename;//filename
    class UserData{public:virtual ~UserData (){};};
    UserData* userData;//added Data !!!should not be array!!!
    //DataStream* cache;//should cache? ???
    EntryNode ():userData (NULL){/*do nothing*/};
    EntryNode (const string& filename_):filename (filename_),userData (NULL){};
    //memory leakʤۡ
    virtual ~EntryNode () {delete userData;};
  };
  list<EntryNode*> children;

  string dataTypeInfo;
  //static void setDataTypeInfo (DataTypeInfo* this);
public:
  typedef list<EntryNode*> ChildrenList;
  
  //////////////////////////////////////////////////////////㳰Ѥclass
  class ErrorOccured {};//base class of all exceptional class
  class UnexpectedEndOfData:public ErrorOccured {};
  class UnavailableOperation:public ErrorOccured {};
  class CanNotOpen:public ErrorOccured {};
  class CanNotClose:public ErrorOccured {};
  
  //static bool isSupported (const char* data,int size,int offset=0);
  //static int getVersion ();
  DataStream (const char* name_);//path=path name or URL
  DataStream (const char* parentName_,const char* baseName);
  DataStream (const string parentName_,const string baseName);
  virtual ~DataStream ();
  
  //for data 
  virtual void close ()// throw (UnavailableOperation)
    {assert (false);throw UnavailableOperation ();};
  virtual long read (char* buffer,long size)=0;
  virtual uint8 get8 ()
    {char ret;read (&ret,1);return static_cast<uint8>(ret);};
  virtual uint16 get16L ()//little endian
    {
      uint16 ret0=get8 ();
      uint16 ret1=get8 ();
      return ret0+(ret1 << 8);
    };
  virtual uint32 get24L ()//little endian
    {
      uint32 ret0=get8 ();
      uint32 ret1=get8 ();
      uint32 ret2=get8 ();
      return ret0+(ret1 << 8)+(ret2 << 16);
    };
  virtual uint32 get32L ()//little endian
    {
      uint32 ret0=get8 ();
      uint32 ret1=get8 ();
      uint32 ret2=get8 ();
      uint32 ret3=get8 ();
      return ret0+(ret1 << 8)+(ret2 << 16)+(ret3 << 24);
    };
  virtual uint16 get16B ()//big endian
    {
      uint16 ret0=get8 ();
      uint16 ret1=get8 ();
      return (ret0 << 8)+ret1;
    };
  virtual uint32 get24B ()//big endian
    {
      uint32 ret0=get8 ();
      uint32 ret1=get8 ();
      uint32 ret2=get8 ();
      return (ret0 << 16)+(ret1 << 8)+ret2;
    };
  virtual uint32 get32B ()//big endian
    {
      uint32 ret0=get8 ();
      uint32 ret1=get8 ();
      uint32 ret2=get8 ();
      uint32 ret3=get8 ();
      return (ret0 << 24)+(ret1 << 16)+(ret2 << 8)+ret3;
    };
  
  enum Whence
  {
    beg,
    cur,
    end
  };
  virtual long seekg (long pos,Whence whence=cur)//throw (UnavailableOperation)
    {assert (false);throw UnavailableOperation ();return 0;};
  virtual long rewind () {return seekg (0,beg);};
  virtual long tellg ()=0;
  virtual bool eod (){return true;};//???

  //for archive
  virtual DataStream* newChild (uint n);
  virtual DataStream* newChild (const string childname);
  //shuld overrided by Archive
  virtual DataStream* newChild (ChildrenList::iterator& i){return NULL;};
  virtual bool isArchive () const {return !children.empty ();};
  virtual int getNOfChildren () const {return children.size ();};
  virtual ChildrenList& getChildrenList () {return children;};
  
  ///////////////////////////////////////////////////////////access method
  virtual const string& getName () const {return name;};
  virtual const string& getBaseName () const {return baseName;};
  virtual const string& getParentName () const {return parentName;};
  virtual const char getDelimiter () const {return '/';};
  const string& getDataTypeInfo () const {return dataTypeInfo;};
  virtual bool isRandomAccessible () const =0;
  
};
#endif //DataStream_H
