#include <cstdio>
#include <sys/stat.h>
#include <unistd.h>
#include "dirent.h"

#include "Directory.h"
#include "FileStream.h"

using namespace std;

bool Directory::isSupport (const char* filename)
{
  if (filename==NULL)
    return false;

  struct stat attribute;
  if (stat (filename,&attribute)==-1)
    {
      perror ("FileStream::isSupport stat");
      return false;
    }

  if (S_ISDIR (attribute.st_mode))
    return true;//directory 

  return false;
}

Directory::Directory (const char* path):
  Archive (path)
{
  makeFileList ();
}

Directory::Directory (const char* parentDir,const char* dirName):
  Archive (parentDir,dirName)
{
  makeFileList ();
}

void Directory::makeFileList ()
{
  DIR* dir=opendir (name.c_str ());
  if (dir==NULL) throw CanNotOpenDirectory();

  dirent* dirEntry;
  while ((dirEntry=readdir (dir))!=NULL)
    {
      if (strcmp (dirEntry->d_name,".")==0 ||
	  strcmp (dirEntry->d_name,"..")==0)
	continue;

      //children.push_back (name+"/"+dirEntry->d_name);//??? waring ???
      children.push_back (new EntryNode (dirEntry->d_name));//??? waring ???
    }

  if (closedir (dir)==-1)
    throw CanNotCloseDirectory();
}

DataStream* Directory::newChild (ChildrenList::iterator& i)
{
  //validity check
  if ((*i)==NULL) return NULL;

  string filename=name+"/"+(*i)->filename;
  string childFileName=(*i)->filename;
  (*i)=NULL;
  
  struct stat fileStat;
  stat (filename.c_str (),&fileStat);
  if (S_ISDIR (fileStat.st_mode))
    return new Directory (name.c_str (),childFileName.c_str ());
  
  return new FileStream (name.c_str (),childFileName.c_str ());
}
