#include <cstdio>
#include <cassert>
#include <sys/stat.h>
#include <unistd.h>
#include <algorithm>

#include "FileStream.h"

using namespace std;

bool FileStream::isSupport (const char* filename)
{
  if (filename==NULL)
    return false;

  struct stat attribute;
  if (stat (filename,&attribute)==-1)
    {
      perror ("FileStream::isSupport stat");
      return false;
    }

  if (S_ISREG (attribute.st_mode))
    return true;//regular file

  return false;
}


FileStream::FileStream (const char* path):
  DataStream (path),file (NULL)
{
  setDataTypeInfo (this);
  openIfstream ();
};

FileStream::FileStream (const char* parentDirectory,const char* baseName):
  DataStream (parentDirectory,baseName),file (NULL)
{
  setDataTypeInfo (this);
  openIfstream ();
};

FileStream::~FileStream ()
{
  delete file;
}

void FileStream::openIfstream ()
{
  assert (file==NULL);
  file=new ifstream (name.c_str ());
  //file size$B<hF@(B
  file->seekg (0,ios::end);
  fileSize=file->tellg ();
  if (fileSize<0)
    throw InvalidFileSize ();
  //$B%U%!%$%k$N:G=i$KLa$7$F$*$/(B
  file->seekg (0,ios::beg);
}

void FileStream::close ()
{
  //file->close ();
  delete file;
  file=NULL;
};

long FileStream::read (char* buffer,long size_)
{
  assert (file!=NULL);
  const long before=file->tellg ();
  file->read (buffer,size_);
  //assert (!file->fail ());
  const long after=file->tellg ();
  return (after-before);
};


long FileStream::seekg (long pos,Whence whence=cur)
{
  assert (file!=NULL);
  
  switch (cur)
    {
    case beg:
      file->seekg (pos,ios::beg);
      break;
    case cur:
      file->seekg (pos,ios::cur);
      break;
    case end:
      file->seekg (pos,ios::end);
      break;
    }
  return file->tellg ();
}


bool FileStream::eod ()
{
  assert (file!=NULL);
  file->peek ();//to set eof flag
  //DEBUG_MESSAGE ("FileStream::eod");
  return file->eof ();
}
