#include "LZH.h"
#include "LZHMember.h"

LZH::LZH (const char* path):
  Archive (path)
{
  makeArchiveList ();
}

LZH::LZH (const char* parentDirectory,const char* baseName):
  Archive (parentDirectory,baseName)
{
  makeArchiveList ();
}

LZH::LZH (const char* name,const char* data,long size):
  Archive (name,data,size)
{
  makeArchiveList ();
}


LZH::LZH (DataStream* dataStream):
  Archive (dataStream)
{
  makeArchiveList ();
}

LZH::~LZH ()
{
  //do nothing
}


void LZH::makeArchiveList ()
{
  //append EntryNode to the children list
  assert (!dataStream->isArchive ());
  dataStream->rewind ();
  while (!dataStream->eod ())
    {
      EntryNode* entryNode=readHeaderAndSkipData ();
      if (entryNode!=NULL)
	{
	  children.push_back (entryNode);
#ifdef DEBUG
	  cerr << "LZH::makeArvhieList " << entryNode->filename << endl;
#endif
	}
    }
  delete dataStream;
  dataStream=NULL;
}

#define EXTEND_GENERIC		0
#define EXTEND_UNIX		'U'
#define EXTEND_MSDOS		'M'
#define EXTEND_MACOS		'm'
#define EXTEND_OS9		'9'
#define EXTEND_OS2		'2'
#define EXTEND_OS68K		'K'
#define EXTEND_OS386		'3'		/* OS-9000??? */
#define EXTEND_HUMAN		'H'
#define EXTEND_CPM		'C'
#define EXTEND_FLEX		'F'
#define EXTEND_RUNSER		'R'
/*	this OS type is not official */
#define EXTEND_TOWNSOS		'T'
#define EXTEND_XOSK		'X'


DataStream::EntryNode* LZH::readHeaderAndSkipData ()
{
  EntryNode* entryNode=new EntryNode ();
  ArchiveEntryNode* archiveEntryNode=new ArchiveEntryNode ();
  LZHEntryNode* lzhEntryNode=new LZHEntryNode ();
  entryNode->userData=archiveEntryNode;
  archiveEntryNode->userData=lzhEntryNode;
  
 RETRY_READ:
  //header size
  if (dataStream->eod () || (lzhEntryNode->headerSize=dataStream->get8 ())==0)
    {//error !! may be macbinary??? XXX
      delete entryNode;
      return NULL;
    }

  const long headerSize=lzhEntryNode->headerSize;
  char* const data=new char[headerSize-1];
#define LZH_MIN_HEADER_SIZE 22
  if (dataStream->read (data,headerSize-1)<headerSize-1 ||
      headerSize<LZH_MIN_HEADER_SIZE)
    {//too short header size
      delete[] data;
      delete entryNode;
      return NULL;
    }
#undef LZH_MIN_HEADER_SIZE
  MemoryStream header (name.c_str (),data,headerSize-1);
  //header.open (data,headerSize-1);
  delete[] data;
  
  lzhEntryNode->headerCheckSum=header.get8 ();//check sum
  header.read(lzhEntryNode->methodID,LZH_METHOD_ID_LENGTH);//method
  archiveEntryNode->compressedSize=header.get32L ();//packed size/skip size
  archiveEntryNode->uncompressedSize=header.get32L ();//original size
  header.seekg (4);//skip Original file date/time (Generic time stamp)
  header.seekg (1);//skip File attribute

  lzhEntryNode->headerLevel=header.get8 ();//header level
  const uchar headerLevel=lzhEntryNode->headerLevel;//for simplicity
  //assert (headerLevel==0||headerLevel==1||headerLevel==2);
  if (headerLevel!=0 && headerLevel!=1 && headerLevel!=2)
    {
      cerr << "LZH::readHeaderAndSkipData:Unkown header Level="
	   << static_cast<int>(headerLevel) << endl;
      delete entryNode;
      return NULL;
    }
#ifdef DEBUG
  cerr << "LZH::readHeaderAndSkipData:header level=" << static_cast<int>(headerLevel) << endl;
#endif
  
  int fileNameLength=0;
  if (headerLevel!=2)
    {
      //headerˤheaderSize-1ɤ߹Ǥʤä顣
      char data[2];

      if (dataStream->read (data,2)<2)
      {
	delete entryNode;
	return NULL;
      }
      header.cat (data,2);

      //header sizetotal header sizeˤ롣
      //(level=0,1λ+2,headersize,checksumʬ)
      lzhEntryNode->headerSize+=2;

      fileNameLength=header.get8 ();//file name length
      if (fileNameLength>0)
	{//file nameɤ߹
	  char* fileName=new char[fileNameLength+1];
	  if (header.read (fileName,fileNameLength,false)<fileNameLength)
	  {
	    delete[] fileName;
	    delete entryNode;
	    return NULL;
	  }
	  fileName[fileNameLength]='\0';
	  entryNode->filename=fileName;
	  delete[] fileName;
	}
    }

  //header level==2ʤheader size2byte(little endian)ä
  //headerʬɤ߹ߡ
  if (headerLevel==2 && lzhEntryNode->headerCheckSum!=0)
    {
      const long leftHeaderSize=(lzhEntryNode->headerCheckSum << 8);
      lzhEntryNode->headerSize+=leftHeaderSize;
      char* leftHeaderData=new char[leftHeaderSize];
      if (dataStream->read(leftHeaderData,leftHeaderSize)<leftHeaderSize)
	{
	  delete[] leftHeaderData;
	  delete entryNode;
	  return NULL;
	}
      delete[] leftHeaderData;
    }
  

  //file crc
  int extendType=EXTEND_GENERIC;
  if(lzhEntryNode->headerSize-fileNameLength>=24)
    {//EXTEND FORMAT
      lzhEntryNode->crc=header.get16L ();//file crc
      extendType=header.get8 ();//??? (level==0λΤǤ?)OS ID
    }
  else if(lzhEntryNode->headerSize-fileNameLength==22)
    {//Generic with CRC
      lzhEntryNode->crc=header.get16L ();//file crc
      extendType=EXTEND_GENERIC;
    }
  else if(lzhEntryNode->headerSize-fileNameLength==20)
    {// Generic no CRC
      extendType=EXTEND_GENERIC;
    }
  else
    {//error!
      delete entryNode;
      return NULL;
    }

  if(extendType==EXTEND_UNIX && lzhEntryNode->headerLevel==0)
    {
      header.get8 ();//skip minor_version
      header.get32L ();//skip unix_last_modified_stamp
      header.get16L ();//skip unix_mode
      header.get16L ();//skip unix_uid
      header.get16L ();//skip unix_gid
    }
  if (lzhEntryNode->headerLevel>0)
    {
      try
	{
	  readExtendedHeader (header,entryNode,archiveEntryNode,lzhEntryNode);
	}
      catch (UnexpectedEndOfData unexpectedEndOfData)
	{
	  delete entryNode;
	  return NULL;
	}
    }
  

  //headerɤ߼꽪λ
  header.close ();
  
  //filenameunixѴ롣
  //XXX

  //get offset to data
  lzhEntryNode->offset=dataStream->tellg ();
  //make DataStream to data

  const long compressedSize=archiveEntryNode->compressedSize;//for simplicity
  char* dataBuffer=new char[compressedSize];
  if (dataStream->read (dataBuffer,compressedSize)<compressedSize)
    {
      delete[] dataBuffer;
      delete entryNode;
      return NULL;
    }
  assert (lzhEntryNode->dataStream==NULL);
  MemoryStream* memoryStream=new MemoryStream
    (name.c_str (),entryNode->filename.c_str (),dataBuffer,compressedSize);
  delete[] dataBuffer;
  memoryStream->doEOD ();
  lzhEntryNode->dataStream=memoryStream;
  
  if (strncmp("-lhd-",lzhEntryNode->methodID,5)==0)
    {//skip directory entry
      delete entryNode;
      entryNode=new EntryNode ();
      archiveEntryNode=new ArchiveEntryNode ();
      lzhEntryNode=new LZHEntryNode ();
      entryNode->userData=archiveEntryNode;
      archiveEntryNode->userData=lzhEntryNode;
      goto RETRY_READ;
    }

  return entryNode;
}

void LZH::readExtendedHeader (MemoryStream& header,
			      EntryNode* entryNode,
			      ArchiveEntryNode* archiveEntryNode,
			      LZHEntryNode* lzhEntryNode)
{
  assert (!header.eod ());
  assert (entryNode->userData==archiveEntryNode);
  assert (archiveEntryNode->userData==lzhEntryNode);
  //assert (lzhEntryNode->headerLevel==1||lzhEntryNode->headerLevel==2);
  
#define LEVEL_ONE_HEADER_EXTENTDED_HEADER_SIZE_OFFSET 25
  //skip Extend Header
  if (lzhEntryNode->headerLevel==1)
    {
      const long extendHeaderSizeOffset=lzhEntryNode->headerSize
	-LEVEL_ONE_HEADER_EXTENTDED_HEADER_SIZE_OFFSET
	-entryNode->filename.length ()-2;
      if (header.seekg (extendHeaderSizeOffset)<extendHeaderSizeOffset)
	throw UnexpectedEndOfData ();
    }
#define LEVEL_ONE_HEADER_EXTENTDED_HEADER_SIZE_OFFSET 25

  int extendHeaderSize;
  while((extendHeaderSize=header.get16L ())!= 0)
    {
      if(lzhEntryNode->headerLevel==1)
	{
	  char* const data=new char[extendHeaderSize];
	  if (dataStream->read (data,extendHeaderSize)<extendHeaderSize)
	    {
	      delete[] data;
	      throw UnexpectedEndOfData();
	    }
	  header.cat (data,extendHeaderSize);
	  delete[] data;
	  //header sizetotal header size롣
	  lzhEntryNode->headerSize+=extendHeaderSize;
	  //compressedSizeskip sizeäƤΤ롣
	  archiveEntryNode->compressedSize-=extendHeaderSize;
	}

      int extendType=header.get8 ();
      switch(extendType)
	{
	case 0://header crc
	  header.seekg (extendHeaderSize-3);
	  break;
	case 1://filename
	  {
	    assert (entryNode->filename.length ()==0);
	    int nameLength=extendHeaderSize-3;
	    char* const fileName=new char[nameLength+1];
	    header.read (fileName,nameLength);
	    fileName[nameLength] = '\0';
	    entryNode->filename=fileName;
	    delete[] fileName;
	  }
	  break;
	case 2://directory
	  {
	    int dirLength=extendHeaderSize-3;
	    char* const dirName=new char[dirLength];
	    header.read (dirName,dirLength);
	    //convert delimiter
	    for (int i=0;i<dirLength;i++)
	      if (static_cast<uchar>(dirName[i])==0xFF) dirName[i]='/';
	    entryNode->filename=dirName+entryNode->filename;
	    delete[] dirName;
	  }
	  break;
	case 0x40://MS-DOS attribute
	  if(extendType==EXTEND_MSDOS ||
	     extendType==EXTEND_HUMAN ||
	     extendType==EXTEND_GENERIC)
	    header.get16L ();// attribute
	  break;
	case 0x50://UNIX permission
	  if(extendType==EXTEND_UNIX)
	    header.get16L ();//unix_mode
	  break;
	case 0x51://UNIX gid and uid
	  if(extendType==EXTEND_UNIX)
	    {
	      header.get16L ();//unix gid
	      header.get16L ();//unix uid
	    }
	  break;
	case 0x52://UNIX group name
	  header.seekg (extendHeaderSize-3);
	  break;
	case 0x53://UNIX user name
	  header.seekg (extendHeaderSize-3);
	  break;
	case 0x54://UNIX last modified time
	  if(extendType==EXTEND_UNIX)
	    header.get32L ();//unix_last_modified_stamp
	  break;
	default://other headers
	  header.seekg (extendHeaderSize-3);
	  break;
	}
    }

#if 0
  switch(extend_type)
    {
    case EXTEND_MSDOS:
      msdos_to_unix_filename(filename, name_length);
    case EXTEND_HUMAN:
      ; /* ignored */
    break;

#ifdef OSK
    case EXTEND_OS68K:
    case EXTEND_XOSK:
#endif
    case EXTEND_UNIX:
      break;

    case EXTEND_MACOS:
      macos_to_unix_filename(filename, name_length);
      break;

    default:
      generic_to_unix_filename(filename, name_length);
    }
#endif

}

#undef EXTEND_GENERIC  
#undef EXTEND_UNIX     
#undef EXTEND_MSDOS    
#undef EXTEND_MACOS    
#undef EXTEND_OS9      
#undef EXTEND_OS2      
#undef EXTEND_OS68K    
#undef EXTEND_OS386    
#undef EXTEND_HUMAN    
#undef EXTEND_CPM      
#undef EXTEND_FLEX     
#undef EXTEND_RUNSER   
#undef EXTEND_TOWNSOS  
#undef EXTEND_XOSK     


DataStream* LZH::newChild (ChildrenList::iterator& i)
{
  //need more validity check
  dynamic_cast<LZHEntryNode*>
    (dynamic_cast<Archive::ArchiveEntryNode*>((*i)->userData)->userData);
  if ((*i)==NULL) return NULL;
  
  EntryNode* entryNodeOfChild=(*i);
  *i=NULL;//get rid of this node

  if (entryNodeOfChild!=NULL)
    return new LZHMember (entryNodeOfChild);
  return NULL;
}
