#include <cassert>
#include <algorithm>

#include "MemoryStream.h"

using namespace std;

MemoryStream::MemoryStream (const char* path):
  DataStream (path),memoryBuffer (NULL)
{
  //do nothing
}

MemoryStream::MemoryStream (const char* parentDirectory,const char* baseName):
  DataStream (parentDirectory,baseName),memoryBuffer (NULL)
{
  //do nothing
}

MemoryStream::MemoryStream (const char* path,const char* data_,long size_):
  DataStream (path),memoryBuffer (new MemoryBuffer ())
{
  cat (data_,size_);
}

MemoryStream::MemoryStream (const char* parentDirectory,const char* baseName,
			    const char* data_,long size_):
  DataStream (parentDirectory,baseName),memoryBuffer (new MemoryBuffer ())
{
  cat (data_,size_);
}


MemoryStream::~MemoryStream ()
{
  delete memoryBuffer;
}

void MemoryStream::close ()
{
  assert (memoryBuffer!=NULL);
  delete memoryBuffer;
  memoryBuffer=NULL;
}

long MemoryStream::read (char* buffer,long size_,bool doesBlock)
{
  assert (memoryBuffer!=NULL);
  return memoryBuffer->read (buffer,size_,doesBlock);
};


long MemoryStream::seekg (long pos,Whence whence=cur)
{
  assert (memoryBuffer!=NULL);
  MemoryBuffer::Whence whence_;
  switch (cur)
    {
    case beg:
      whence_=MemoryBuffer::beg;
      break;
    case cur:
      whence_=MemoryBuffer::cur;
      break;
    case end:
      whence_=MemoryBuffer::end;
      break;
    }

  return memoryBuffer->seekg (pos,whence_);
}
