#include <cassert>
#include <unistd.h>
#include <netdb.h>

#include "Socket.h"

using namespace std;

Socket::Socket (int domain, int type_, int protocol=0) throw (Error):
  file (NULL)
{
  socket=::socket (domain, type_, protocol);
  if (socket == -1)
    {
      //perror (Socket::Socket:);
      throw (Error ("Socket::Socket", strerror (errno)));
    }
}

Socket::Socket (const char* host, int port) throw (Error):
  socket (SOCKET_IS_CLOSED), file (NULL)
{
  //create socket
  socket=::socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (socket == -1)
    throw (Error ("Socket::Socket:socket==-1", strerror (errno)));
  
  //connect
  sockaddr_in in;
  in.sin_port=htons (port);
  in.sin_family=AF_INET;
  //in.sin_zero//???

  //#if defined(_POSIX_REENTRANT_FUNCTIONS) || defined(_POSIX_THREAD_SAFE_FUNCTIONS)//???
#ifdef HAVE_GETHOSTBYNAME_R
  hostent resultBuffer;
#define BUFFER_LENGTH 1024 //??? is this enough?
  char buffer[BUFFER_LENGTH];
  int hErrNo;//dummy
  hostent* hostEnt;
  const int ret =gethostbyname_r (host, &resultBuffer,
				  buffer, BUFFER_LENGTH, &hostEnt, &hErrNo);
#undef BUFFER_LENGTH 
  if (ret != 0)
    throw (Error ("Socket::Socket:hostEnt==NULL"));//XXX
  memcpy (&in.sin_addr, hostEnt->h_addr, hostEnt->h_length);
  connect (&in);
#else
  hostentLock.lock ();
  hostent* hostEnt=gethostbyname (host);
  if (hostEnt == NULL)
    {
      hostentLock.unlock ();
      throw (Error ("Socket::Socket:hostEnt==NULL"));//XXX
    }
  memcpy (&in.sin_addr, hostEnt->h_addr, hostEnt->h_length);
  hostentLock.unlock ();
  connect (&in);
#endif
}


Socket::Socket (SOCKET socket_):
  socket (socket_), file (NULL)
{
  assert (socket>=0);
}


Socket::~Socket () throw (Error)
{
  if (file != NULL)
    fclose ();
  if (socket != SOCKET_IS_CLOSED)
    close ();
}


void Socket::bind (sockaddr_un* myAddress) throw (Error)
{
  assert (socket != SOCKET_IS_CLOSED);
  const int length=sizeof (myAddress->sun_family)+strlen (myAddress->sun_path);
  if (::bind (socket, (sockaddr*) myAddress, length) == -1)
    throw (Error ("Socket::bind", strerror (errno)));
}

void Socket::bind (sockaddr_in* myAddress) throw (Error)
{
  assert (socket != SOCKET_IS_CLOSED);
  assert (myAddress != NULL);
  if (::bind (socket, (sockaddr*) myAddress, sizeof (*myAddress)) == -1)
    throw (Error ("Socket::bind", strerror (errno)));
}

void Socket::listen (int backlog=5) throw (Error)
{
  assert (socket != SOCKET_IS_CLOSED);
  if (::listen (socket, backlog) == -1)
    throw (Error ("Socket::listen", strerror (errno)));
}

void Socket::shutdown (int how) throw (Error)
{
  assert (how == 0||how == 1||how == 2);
  assert (socket != SOCKET_IS_CLOSED);
  if (::shutdown (socket, how) == -1)
    throw (Error ("Socket::shutdown", strerror (errno)));
  socket=SOCKET_IS_CLOSED;
}

void Socket::close () throw (Error)
{
  assert (socket != SOCKET_IS_CLOSED);
  if (::close (socket) == -1)
    throw (Error ("Socket::close", strerror (errno)));
  socket=SOCKET_IS_CLOSED;
}

Socket::SOCKET Socket::accept (sockaddr* address,
			       int* addressLength) throw (Error)
{
  const SOCKET returnSocket=::accept (socket, address, addressLength);
  if (returnSocket == -1)
    throw (Error ("Socket::acceptn", strerror (errno)));
  return returnSocket;
}
    
void Socket::connect (sockaddr_un* servAddr) throw (Error)
{
  assert (servAddr != NULL);
  const int length=sizeof (servAddr->sun_family)+strlen (servAddr->sun_path);
  if (::connect (socket, (sockaddr*) servAddr, length) == -1)
    throw (Error ("Socket::connect", strerror (errno)));
}

void Socket::connect (sockaddr_in* servAddr) throw (Error)
{
  assert (servAddr != NULL);
  if (::connect (socket, (sockaddr*) servAddr, sizeof (*servAddr)) == -1)
    throw (Error ("Socket::connect", strerror (errno)));
}

int Socket::send (const void* message, int length, unsigned int flags=0)
{
  assert (socket != SOCKET_IS_CLOSED);
  return ::send (socket, message, length, flags);
}

int Socket::recv (void* buffer, int length, unsigned int flags=0)
{
  assert (socket != SOCKET_IS_CLOSED);
  return ::recv (socket, buffer, length, flags);
}

#if 0
int Socket::sendto
(const void* msg, int len, unsigned int flags, const sockaddr* to, int tolen)
{
}

int Socket::recvfrom
(void* buf, int len, unsigned int flags, sockaddr* from, int* formlen)
{
}
#endif

void Socket::fdopen (char* mode)
{
  assert (socket != SOCKET_IS_CLOSED);
  assert (file == NULL);
  file=::fdopen (socket, mode);
}

char* Socket:: fgets (char* buffer, int n)
{
  assert (file != NULL);
  return ::fgets (buffer, n, file);
}

int Socket::fgetc ()
{
  assert (file != NULL);
  //return ::getc (file);
  return ::fgetc (file);
}

long Socket::fread (void* buffer, long bufferSize)
{
  assert (file != NULL);
  return static_cast<long>(::fread (buffer, 1, bufferSize, file));
}

long Socket::fwrite (void* buffer, long bufferSize)
{
  assert (file != NULL);
  return static_cast<long>(::fwrite (buffer, 1, bufferSize, file));
}

int Socket::fflush ()
{
  assert (file != NULL);
  return ::fflush (file);
}

int Socket::fclose ()
{
  assert (file != NULL);
  const int ret=::fclose (file);
  file=NULL;
  return ret;
}
