#ifndef Socket_H
#define Socket_H
#include <cstdio>
#include <cerrno>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>

#include "util/Error.h"

#ifndef HAVE_GETHOSTBYNAME_R
# include "util/Mutex.h"
#endif

class Socket
{
private:
  typedef int SOCKET;
  enum {SOCKET_IS_CLOSED=-1};
  SOCKET socket;//file dedcriptor for socket
  FILE* file;

//#if !defined(_POSIX_REENTRANT_FUNCTIONS) && !defined(_POSIX_THREAD_SAFE_FUNCTIONS)//???
#ifndef HAVE_GETHOSTBYNAME_R
  Mutex hostentLock;
#endif
public:
  Socket (int domain,int type_,int protocol=0) throw (Error);
  Socket (const char* host,int port) throw (Error);//XXX
  //for use accept
  //e.g Socket acceptedSocket(aSocket.accpet (&address,&addressLength));
  Socket (SOCKET socket_);
  ~Socket () throw (Error);
  void bind (sockaddr_un* myAddress) throw (Error);
  void bind (sockaddr_in* myAddress) throw (Error);
  void listen (int backlog=5) throw (Error);
  void shutdown (int how) throw (Error);
  void close () throw (Error);

  SOCKET accept (sockaddr* addr, int* addrlen) throw (Error);
  void connect (sockaddr_un* servAddr) throw (Error);
  void connect (sockaddr_in* servAddr) throw (Error);

  int send (const void* message,int length,unsigned int flags=0);
  int recv (void* buffer,int length,unsigned int flags=0);

#if 0
  int sendto
  (const void* msg,int len,unsigned int flags,const sockaddr* to,int tolen);
  int recvfrom
  (void* buffer,int length,unsigned int flags,sockaddr* from,int* formLength);
#endif
  ////////////////////////////////////////for File
  void fdopen (char* mode);
  char* fgets (char* buffer,int n);
  int fgetc ();
  long fread (void* buffer,long bufferSize);
  long fwrite (void* buffer,long bufferSize);
  int fflush ();
  int fclose ();
};
#endif // Socket_H
