#ifndef Threaded_H
#define Threaded_H

#include "DataStream.h"
#include "MemoryBuffer.h"

//mixin
template <class T>//T=MemoryBuffer or MemoryBuffer2
class Threaded
{
protected:
  Threaded (DataStream* dataStream_,T* memoryBuffer_):
    dataStream (dataStream_),memoryBuffer (memoryBuffer_),
    doesRetrieveThreadRunning (false),doCancelRetrieveThread (false)
    {assert (dataStream!=NULL);assert (memoryBuffer!=NULL);};

  virtual ~Threaded () {delete dataStream;delete memoryBuffer;};
  //read the data from
  DataStream* dataStream;
  //store the data to
  T* memoryBuffer;

  //for retrieving thread
  //memoryBufferlocklockƤʤ֤access롣
  //dataStreamˤϡretrieve threadäƤʤƱaccessʤΤ
  //doesRetrieveThreadRunningǧƤaccess롣
  //doesRetrieveThreadRunning,doCancelRetrieveThread
  //accessˤlocklockɬפ롣
  pthread_t thread;
  Mutex lock;
  CondVar cond;//dataɲääretrive threadλȤѡ
  bool doesRetrieveThreadRunning;
  bool doCancelRetrieveThread;

  virtual void createRetrieveThread ();
  static void* startRetrieve (Threaded* me);
  virtual void retrieve ();
  virtual void cancelRetrieveThread ();//need this???
};

#define UNIT_BUFFER_SIZE 1024L //ĤŬ?

template <class T>
void Threaded<T>::createRetrieveThread ()
{
  DEBUG_MESSAGE ("Threaded::createRetrieveStream calling");
  pthread_attr_t attr;
  pthread_attr_init (&attr);
  int ret=pthread_attr_setdetachstate (&attr,PTHREAD_CREATE_DETACHED);
  if (ret!=0)
    cerr << "Threaded::createRetriveRhread:"
	 << "pthread_attr_setdetachstate:" << strerror (ret)
	 << endl;
  ret=pthread_create (&thread,&attr,&startRetrieve,this);
  if (ret!=0)
    cerr << "Threaded::createRetriveThread:"
	 << "pthread_create:" << strerror (ret)
	 << endl;
  ret=pthread_attr_destroy (&attr);
  if (ret!=0)
    cerr << "Threaded::createRetriveThread:"
	 << "pthread_attr_destroy:" << strerror (ret)
	 << endl;
  DEBUG_MESSAGE ("Threaded::createRetrieveStream call done");

  //wait
  lock.lock ();
  //retrieve threadΤˤϤäΤΤ
  //dataStream!=NULLΤ,ǡretrieveб
  //Ǥ롣
  while (doesRetrieveThreadRunning==false && dataStream!=NULL)
    cond.wait (lock);
  lock.unlock ();

  DEBUG_MESSAGE ("Threaded::createRetrieveStream all done.");
}

template <class T>
void* Threaded<T>::startRetrieve (Threaded* me)
{
  me->retrieve ();
  return NULL;
}

template <class T>
void Threaded<T>::retrieve ()
{
  DEBUG_MESSAGE ("Threaded::retrieve calling");
  lock.lock ();
  assert (!doesRetrieveThreadRunning);
  doesRetrieveThreadRunning=true;
  lock.unlock ();
  cond.broadcast ();//signal retrieve thread is starting
  
  DEBUG_MESSAGE ("Threaded::retrieve now retrieving");
  //retrieve data and store to memoryBuffer
  for (;;)
    {
      char buffer[UNIT_BUFFER_SIZE];
      DEBUG_MESSAGE ("Threaded::retrieve for locking");
      lock.lock ();
      const long currentSize=dataStream->read (buffer,UNIT_BUFFER_SIZE);
      const bool isDataStreamEod=dataStream->eod ();
      const bool isDoCancelRetrieveThread=doCancelRetrieveThread;
      lock.unlock ();
      DEBUG_MESSAGE ("Threaded::retrieve for unlocked");
      memoryBuffer->cat (buffer,currentSize);
#ifdef DEBUG
      cerr << "Threaded::retrieve currentSize=" << currentSize << endl;
#endif
      if (isDataStreamEod)
	{
	  DEBUG_MESSAGE ("Threaded::retrieve dataStream->eod ()=true");
	  break;//dataκǸޤɤ߽Ф
	}
      DEBUG_MESSAGE ("Threaded::retrieve now checking doCancelRetrieveThread");
      if (isDoCancelRetrieveThread)
	{//retrieve threadλȤΤä
	  DEBUG_MESSAGE ("Threaded::retrieve doCancelRetrieveThread=true");
	  break;
	}
    }
  
  DEBUG_MESSAGE ("Threaded::retrieve now retrieved.now cleaning up");
  //clean up
  lock.lock ();
  doesRetrieveThreadRunning=false;
  delete dataStream;
  dataStream=NULL;
  lock.unlock ();
  //thread safeˤ뤿˺ݤɤäƤΤա
  DEBUG_MESSAGE ("Threaded::retrieve memoryBuffer->doEOD ()");
  memoryBuffer->doEOD ();
  assert (memoryBuffer->isAllStored ());
#ifdef DEBUG
  cerr << "Threaded::retrieve totalSize=" << memoryBuffer->getTotalBufferSize () << endl;
#endif

  ///signal that retrieving data is over.  
  cond.broadcast ();

  DEBUG_MESSAGE ("Threaded::retrieve all done");
}

template <class T>
void Threaded<T>::cancelRetrieveThread ()
{
  DEBUG_MESSAGE ("Threaded::cancelRetrieveThread calling");
  lock.lock ();
  assert (doCancelRetrieveThread==false);
  if (!doesRetrieveThreadRunning)
    {
      assert (dataStream==NULL);
      lock.unlock ();
      return;
    }

  DEBUG_MESSAGE ("Threaded::cancelRetrieveThread cancelling.");
  doCancelRetrieveThread=true;
  //joining retrieving thread
  while (doesRetrieveThreadRunning)
    cond.wait (lock);
  //joined
  doCancelRetrieveThread=false;
  lock.unlock ();
  
  DEBUG_MESSAGE ("Threaded::cancelRetrieveThread all done.");
}

#undef UNIT_BUFFER_SIZE 

#endif // Threaded_H
