#include <cassert>
#include <algorithm>

#include "util/mytype.h"
#include "ThreadedMemoryStream.h"

  
#define UNIT_BUFFER_SIZE 1024L //ĤŬ?

ThreadedMemoryStream::ThreadedMemoryStream
(const char* parentName,const char* baseName):
  DataStream (parentName,baseName),
  Threaded<MemoryBuffer>(NULL,NULL)
{
  //do nothing
}

ThreadedMemoryStream::ThreadedMemoryStream
(DataStream* dataStream_,bool doesCreateRetrieveThread=true):
  DataStream (dataStream_->getParentName ().c_str (),
	      dataStream_->getBaseName ().c_str ()),
  Threaded<MemoryBuffer>(dataStream_,new MemoryBuffer ())
{
  if (doesCreateRetrieveThread==true)
    createRetrieveThread ();
}

ThreadedMemoryStream::~ThreadedMemoryStream ()
{
  close ();
}

void ThreadedMemoryStream::close ()
{
  //dataStreamretrieve methodνλ˲롣
  cancelRetrieveThread ();
  assert (dataStream==NULL);
  
  //multi threadȴ
  //retrieveThreadλ뤳ȤǧƤdeleteɬפ롣
  delete memoryBuffer;
  memoryBuffer=NULL;
}

long ThreadedMemoryStream::read (char* buffer,long size_,bool doesBlock)
{
  DEBUG_MESSAGE ("ThreadedMemoryStream::read calling");
  assert (memoryBuffer!=NULL);
  if (doesBlock==false)//blockˤ롣
    return memoryBuffer->read (buffer,size_,false);
  
  ////////////////////////////////ɬפʤɤ߹फeodˤʤޤblock롣
  lock.lock ();
  if (doesRetrieveThreadRunning || dataStream==NULL)
    {//retrieve threadäƤΤǤˤޤɤ
      //ƤΥǡretrieve
      lock.unlock ();
      return memoryBuffer->read (buffer,size_,true);
    }
  lock.unlock ();
  
  //retrieve threadäƤʤΤǡdataäƤɬפ롣
  //ޤbufferɤ߽Ф
  long readSize=memoryBuffer->read (buffer,size_,false);
  if (memoryBuffer->eod ()) return readSize;//dataκǸޤɤ߽Ф
  if (readSize<size_)
    {//bufferͭΤǤ­ʤäΤdataStreamɤ߽Ф
      assert (dataStream!=NULL);
      while (readSize<size_)
	{
	  lock.lock ();
	  long currentSize=dataStream->read (buffer+readSize,size_-readSize);
	  const bool isDataStreamEod=dataStream->eod ();
	  lock.unlock ();
	  //bufferˤФƤ
	  memoryBuffer->cat (buffer+readSize,currentSize);
	  memoryBuffer->seekg (currentSize);//ɤ߹֤߰Ĵ
	  readSize+=currentSize;
	  
	  if (isDataStreamEod)
	    {//dataκǸޤɤ߽Ф
	      lock.lock ();
	      delete dataStream;
	      dataStream=NULL;
	      lock.unlock ();
	      assert (!memoryBuffer->eod ());
	      memoryBuffer->doEOD ();
	      break;
	    }
	}
    }

  return readSize;
}


//!!!NOTE!!!
//whence=end,doesBlock=trueλdataޤblock롣
//whence=end,doesBlock=falseλϺbufferκǸ˰ưǡ
//θǡ夹ǽ롣(when dataStream->eod()==false)
long ThreadedMemoryStream::seekg (long pos,Whence whence=cur,bool doesBlock)
{
  assert (memoryBuffer!=NULL);
  long offsetFromBegin=0;
  MemoryBuffer::Whence whence_;
  switch (cur)
    {
    case beg:
      offsetFromBegin=pos;
      whence_=MemoryBuffer::beg;
      break;
    case cur:
      offsetFromBegin=memoryBuffer->tellg ()+pos;
      whence_=MemoryBuffer::cur;
      break;
    case end:
      //assert (memoryBuffer->eod ());
      assert (((pos<=0) && (doesBlock==true)) ||
	      ((pos=0)  && (doesBlock==false)));
      whence_=MemoryBuffer::end;
      break;
    }

  if (!doesBlock)
    return memoryBuffer->seekg (pos,whence_);//block˰ư롣

  ////////////////////////////////////////ɬפʰ֤˰ưǤޤblock롣
  lock.lock ();
  assert ((dataStream==NULL)==(memoryBuffer->isAllStored ()));
  if (doesRetrieveThreadRunning || dataStream==NULL)
    {
      if (whence==end)
	{
	  while (doesRetrieveThreadRunning)
	    cond.wait (lock);
	  lock.unlock ();
	  return memoryBuffer->seekg (pos,MemoryBuffer::end);
	}
      else
	{
	  while (memoryBuffer->getTotalBufferSize ()<offsetFromBegin)
	    cond.wait (lock);
	  lock.unlock ();
	  return memoryBuffer->seekg (pos,whence_);
	}
    }

  //retrieve threadäƤʤ
  //NOTE retrieve threadäƤdataStreamaccessΤ
  //thread safeǤʤʤ롣Ǥretrieve threadäƤʤҤȤ
  //ݾںѤʤΤdataStreamaccessɤ
  //doesBlock=trueݾںѡ
  assert ((dataStream==NULL)==(memoryBuffer->isAllStored ()));
  if (memoryBuffer->isAllStored () ||
      (whence!=end && offsetFromBegin<memoryBuffer->getTotalBufferSize ()))
    {//block˰ưǤ롣
      lock.unlock ();
      return memoryBuffer->seekg (pos,whence_);
    }

  //retrieve threadäƤʤΤǼdataäƤɬפ롣
  //ư֤ޤǥǡɤ߹ߡ
  assert (dataStream!=NULL);
  while (memoryBuffer->getTotalBufferSize ()<offsetFromBegin)
    {
      if (whence==end)
	{//whence==end,doesBlock==trueʤΤǡ
	  //dataStreamκǸޤɤ߽ФɬפΤǡ
	  offsetFromBegin=memoryBuffer->getTotalBufferSize ()+UNIT_BUFFER_SIZE;
	  assert (offsetFromBegin>0);
	}
      char buffer[UNIT_BUFFER_SIZE];
      const long currentSize=dataStream->read
	(buffer,min(offsetFromBegin-memoryBuffer->getTotalBufferSize ()+1,
		    UNIT_BUFFER_SIZE));
      //bufferˤФƤ
      memoryBuffer->cat (buffer,currentSize);
      if (dataStream->eod ())
	{//dataκǸޤɤ߽Ф
	  assert (!memoryBuffer->isAllStored ());
	  assert (!memoryBuffer->eod ());
	  delete dataStream;
	  dataStream=NULL;
	  memoryBuffer->doEOD ();
	  break;
	}
    }

  lock.unlock ();
  return memoryBuffer->seekg (0,MemoryBuffer::end);
}
