/*
    ServerCommand.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef ServerCommand_H
#define ServerCommand_H

#include "util/mytype.h"
#include "util/CondVar.h"
#include "Sequencer/Sequencer.h"
#include "PlayingStyle.h"
#include "DeviceResetType.h"

class ServerCommand
{
public:
  enum Command
  {
    NONE,           //̿᤬ʤ
    
    //class Songž̿
    PLAY,           //ճ
    PAUSE,          //հ
    CONTINUE_PLAY,  //߲
    REWIND,         //ʤλϤ롣
    BACK,           //ᤷ
    STOP,           //,ߤαդ
    FORWARD,        //
    SET,            //հ֤֤ޤǤ˰ư

    SET_SKEW,//skewѹ롣(skewϥƥݤ򤺤餹Τ˻Ȥ1.0̾)
    
    //midi port˥ǡϤ롣
    MIDI_PORT_NOTE_ON,
    MIDI_PORT_NOTE_OFF,
    MIDI_PORT_POLYPHONIC_KEY_PRESSURE,
    //MSB,LSBƱ˻ꤹɬפ롣
    //RPN,NRPNξ֤᤹Ȥ򤷤ʤΤǡ
    //RPN,NRPNMIDI_PORT_PRN,MIDI_PORT_NPRN,
    //Ȥȡ
    MIDI_PORT_CONTROL_CHANGE,
    MIDI_PORT_RPN,
    MIDI_PORT_NRPN,
    MIDI_PORT_PROGRAM_CHANGE,//MSB,LSBƱ˻ꤹɬפ???
    MIDI_PORT_CHANNEL_PRESSURE,
    MIDI_PORT_PITCH_BEND_CHANGE,
    MIDI_PORT_SYS_EX,

    SET_MUTE,//port,channelmute򤹤롢ʤꤹ롣
    TOGGLE_MUTE,//port,channelmute򤹤롢ʤtoggle롣
    
    //server¹Ԥ̿
    PREV,           //ʤ롣
    NEXT,           //ʤ
    DONE,           //դƤʤαդλ
    OPEN,           //
    QUIT,           //λ

    ADD_FILE,       //եꥹȤɲ

    //filenameǻꤵ줿ʪդ롣եꥹȤˤʤɲá
    //ΤΤ϶λ롣
    PLAY_WITH_FILENAME,

    //ե⡼ɤѹ롣
    SET_PLAYING_MODE,

    //deviceνˡѹ롣
    SET_DEVICE_RESET_TYPE,
  };
protected:
  Mutex commandLock;
  CondVar commandCond;
  //commandNONEˤʤޤԤäơå򤷤Ƥ֤롣
  void waitLockCommand ();
  void setCommand (Command command_);

  Command command;

  //հֻñ̤logical ticks
  uint32  position;
  //skew
  double skew;
  //MIDI_PORT 
  static const int MAX_MIDI_PARAMETER = 8;
  uchar midiParameter[MAX_MIDI_PARAMETER];
  
  //MIDI_PORT_PORT_OUT,MIDI_PORT_SYS_EX
  int sysExSize;
  uchar* sysExData;
  int channel;
  uchar* newCopySysEx (const uchar* src_, int sysExSize_)
    {
      uchar* ret = new uchar[sysExSize_];
      memcpy (ret, src_, sysExSize_);
      return ret;
    };
  
  //SET_MUTE
  int muteSize;
  Sequencer::Muting* muting;

  //addFile
  string filename;

  //playing mode
  PlayingStyle::PlayingMode playingMode;

  //deviceˡѡ
  DeviceResetType::DeviceResetType deviceResetType;

public:
  ServerCommand ();
  ~ServerCommand ();

  void lock () {commandLock.lock ();};
  void unlock () {commandLock.unlock ();};
  void broadcast () {commandCond.broadcast ();};
  void waitCond () {commandCond.wait (commandLock);};
  //ꤵƤcommandreset롣
  //lockϸƤӽФ¦ǹԤȡ
  void reset ()
    {
      command=NONE;
      filename.erase ();
      position=0;
      skew = 1.0;
      memset (midiParameter, 0, sizeof (midiParameter));
      sysExSize = -1;
      delete[] sysExData;
      sysExData = NULL;
      channel = -1;
      muteSize = -1;
      delete[] muting;
      muting = NULL;
      playingMode = PlayingStyle::CONTINUOUS_PLAY;
      deviceResetType = DeviceResetType::NONE;
    };
  //ꤵƤcommandФ
  //ꤵƤcommandNONE롣
  //lockϸƤӽФ¦ǹԤȡ
  //ޤcommandФ顢broadcast ()
  void getCommand (ServerCommand* serverCommand_);

  Command peekCommand () const {return command;};
  const string& getFileName () const
    {
      assert (command == ADD_FILE || command == PLAY_WITH_FILENAME);
      return filename;
    };
  uint32 getPosition () const {assert (command==SET);return position;};
  double getSkew () const {assert (command == SET_SKEW);return skew;};
  const uchar* getMidiParameter () const
    {
      assert (command == MIDI_PORT_NOTE_ON ||
	      command == MIDI_PORT_NOTE_OFF ||
	      command == MIDI_PORT_POLYPHONIC_KEY_PRESSURE ||
	      command == MIDI_PORT_CONTROL_CHANGE ||
	      command == MIDI_PORT_RPN ||
	      command == MIDI_PORT_NRPN ||
	      command == MIDI_PORT_PROGRAM_CHANGE ||
	      command == MIDI_PORT_CHANNEL_PRESSURE ||
	      command == MIDI_PORT_PITCH_BEND_CHANGE);
      return midiParameter;
    };
  int getSysExDataSize () const
    {
      assert (command == MIDI_PORT_SYS_EX);
      return sysExSize;
    };
  const uchar* getSysExData () const 
    {
      assert (command == MIDI_PORT_SYS_EX);
      return sysExData;
    };
  int getMuteSize () const
    {
      assert (command == SET_MUTE || command == TOGGLE_MUTE);
      assert (muting != NULL);
      return muteSize;
    }
  const Sequencer::Muting* getMuting () const
    {
      assert (command == SET_MUTE || command == TOGGLE_MUTE);
      assert (muting != NULL);
      return muting;
    };

  PlayingStyle::PlayingMode getPlayingMode () const
    {
      assert (command == SET_PLAYING_MODE);
      return playingMode;
    };

  DeviceResetType::DeviceResetType getDeviceResetType () const
    {
      assert (command == SET_DEVICE_RESET_TYPE);
      return deviceResetType;
    };
  
  ////////////////////////////////////////////method for playing
  void play ();
  void pause ();
  void continuePlay ();
  void rewind ();
  void back ();
  void stop ();
  void forward ();
  //հposition_logicalTicksǻꤹ롣
  void setPosition (uint32 position_);
  void setSkew (double skew_);

  void setCommandNoteOn (int port, uchar channel, uchar note, uchar velocity);
  void setCommandNoteOff (int port, uchar channel, uchar note, uchar velocity);
  void setCommandPolyphonicKeyPressure (int port, uchar channel,
					uchar note, uchar keyPressure);
  //control chage
  //0xB0|channel msbControl msbValue
  //0xB0|channel lsbControl lsbValue
  //롣
  void setCommandControlChange (int port, uchar channel,
				uchar msbControl, uchar msbValue,
				uchar lsbControl, uchar lsbValue);
  //channelФ
  //RPN rpnMSB,rpnLSBꤷơ
  //0xB0|channel 0x65 rpnMSB    (RPN MSB)
  //0xB0|channel 0x64 rpnLSB    (RPN LSB)
  //0xB0|channel cc1  cc1Value
  //(cc1 = 0x06(dataEntry) or 0x60(incriment) or 0x61(decriment)Ǥ롣)
  //0xB0|channel cc2  cc2Value
  //(dataEntryλΤ(cc1 == 0x06λΤ),cc2 = 0x26, increment,decriment
  //Ϥͤϰ̣ʤ)
  //롣
  void setCommandRPN (int port, uchar channel, uchar rpnMSB, uchar rpnLSB,
		      uchar cc1, uchar cc1Value, uchar cc2, uchar cc2Value);
  //parameterΰ̣rpnλƱ
  void setCommandNRPN (int port, uchar channel, uchar rpnMSB, uchar rpnLSB,
		       uchar cc1, uchar cc1Value, uchar cc2, uchar cc2Value);
  void setCommandProgramChange (int port, uchar channel, uchar program);
  void setCommandChannelPressure (int port, uchar channel,
				  uchar channelPressure);
  void setCommandPicthBendChange (int port, uchar channel,
				  uchar lsb_, uchar msb_);
  void setCommandSysEx  (const uchar* data_, int sysExSize_,
			 int port = -1, int channel_ = -1);

  void setMute (const Sequencer::Muting* muting_, int muteSize_,
		bool isToggle = false);
  void toggleMute (const Sequencer::Muting* muting_, int muteSize_)
    {setMute (muting_, muteSize_, true);};
  
  void prev ();
  void next ();
  void done ();
  void open ();
  void quit ();
  void addFile (const string& filename_);
  void playWithFileName (const string& filename_);

  void setPlayingMode (PlayingStyle::PlayingMode playingMode__);
  void setDeviceResetType (DeviceResetType::DeviceResetType deviceResetType__);
  
  void getFileList ();//XXX  
};
#endif // ServerCommand_H
