#include "SongManager.h"

using namespace std;

const uint SongManager::DEFAULT_MAX_SONG_HISTORY;

SongManager::SongManager ():
  isFreezed (false),
  playingStyle (PlayingStyle::DEFAULT_PLAYING_STYLE),
  loopStyle (PlayingStyle::DEFAULT_LOOP_STYLE),
  isStarted (false),historySize (DEFAULT_MAX_SONG_HISTORY), isInHistory (false)
{
}

SongManager::~SongManager ()
{
  Monitor monitor (this);//??? needs this?
}

bool SongManager::moveToPrevSong ()
{
  Monitor monitor (this);
  if (!isInHistory && songFileList.empty ())
    return false;
  
  switch (loopStyle)
    {
    case PlayingStyle::REPEAT:
      return true;
      break;
      
    default:
      if (isInHistory)
	{
	  if (currentHistorySong != playedSongList.begin ())
	    {
	      currentHistorySong--;
	      return true;
	    }
	  else
	    return false;
	}
      else
	{
	  if (playedSongList.size () == 0)
	    {
	      currentSong = songFileList.begin ();
	      return false;
	    }
	  
	  isInHistory = true;
	  currentHistorySong = playedSongList.end ();
	  currentHistorySong--;
	  return true;
	}
      break;
    }

  return false;
}

const string SongManager::getSongName () const
{
  Monitor monitor (this);

  if (isInHistory)
    return (*currentHistorySong).getFileName ();

  if (!isStarted)
    return "";
  
  return (*currentSong).getFileName ();
}

void SongManager::arrangeSongList ()
{
  assert (!songFileList.empty ());
  workingFileList = songFileList;
  switch (playingStyle)
    {
    case PlayingStyle::NORMAL:
      //do nothing
      break;
    case PlayingStyle::NORMAL_REVERSE:
      reverse (workingFileList.begin (), workingFileList.end ());
      break;
    case PlayingStyle::SHUFFLE:
      workingFileList.randomShuffle ();
      break;
    case PlayingStyle::RANDOM:
      //do nothing
      break;
    case PlayingStyle::FILENAME_ORDER:
      workingFileList.fileNameSortLess ();
      break;
    case PlayingStyle::FILENAME_REVERSE_ORDER:
      workingFileList.fileNameSortGreater ();
      break;
    case PlayingStyle::TITLE_ORDER:
      workingFileList.titleSortLess ();
      break;
    case PlayingStyle::TITLE_REVERSE_ORDER:
      workingFileList.titleSortGreater ();
      break;
    case PlayingStyle::FILESIZE_ORDER:
      //XXX
      //not implemented
      break;
    case PlayingStyle::FILESIZE_REVERSE_ORDER:
      //XXX
      //not implemented
      break;
    case PlayingStyle::TIME_ORDER:
      workingFileList.playTimeSortLess ();
      break;
    case PlayingStyle::TIME_REVERSE_ORDER:
      workingFileList.playTimeSortGreater ();
      break;
    case PlayingStyle::ALBUM:
      //ŪϿǱդɤȦ
      //do nothing
      break;
    case PlayingStyle::ALBUM_REVERSE:
      //ŪϿεսɤȦ
      reverse (workingFileList.begin (), workingFileList.end ());
      break;
    }
}

bool SongManager::moveToNextSong ()
{
  Monitor monitor (this);
  if (!isInHistory && songFileList.empty ())
    return "";//̤ϿƤʤ

  if (!isStarted)
    {
      arrangeSongList ();
      isStarted = true;
      currentSong = workingFileList.begin ();
      addHistory (*currentSong);
      return true;
    }

  if (isInHistory)
    {
      if (loopStyle == PlayingStyle::REPEAT)
	return true;
      
      currentHistorySong++;
      if (currentHistorySong == playedSongList.end ())
	{
	  if (songFileList.empty ())
	    {//̤ϿƤʤ
	      currentHistorySong--;
	      return false;
	    }
	  
	  isInHistory = false;
	}
      else
	return true;
    }
  
  if (loopStyle == PlayingStyle::REPEAT)
    return true;
  
  switch (playingStyle)
    {
    case PlayingStyle::RANDOM:
      currentSong = workingFileList.get (random (workingFileList.size ()));
      break;
      
    case PlayingStyle::NORMAL://falling through
    case PlayingStyle::NORMAL_REVERSE://falling through
    case PlayingStyle::SHUFFLE://falling through
    case PlayingStyle::FILENAME_ORDER://falling through
    case PlayingStyle::FILENAME_REVERSE_ORDER://falling through
    case PlayingStyle::TITLE_ORDER://falling through
    case PlayingStyle::TITLE_REVERSE_ORDER://falling through
    case PlayingStyle::FILESIZE_ORDER://falling through
    case PlayingStyle::FILESIZE_REVERSE_ORDER://falling through
    case PlayingStyle::TIME_ORDER://falling through
    case PlayingStyle::TIME_REVERSE_ORDER://falling through
    case PlayingStyle::ALBUM://falling through
    case PlayingStyle::ALBUM_REVERSE:
      currentSong++;
      break;
    }
  
  if (currentSong == workingFileList.end ())
    {
      switch (loopStyle)
	{
	case PlayingStyle::LOOP:
	  currentSong = workingFileList.begin ();
	  break;
	case PlayingStyle::STOP_AT_END:
	  currentSong--;
	  break;
	case PlayingStyle::REPEAT:
	  //should not be reached
	  DEBUG_MESSAGE ("SongManager::moveToNextSong error!!");
	  break;
	}
    }

  addHistory (*currentSong);
  return true;
}

void SongManager::setPlayingStyle (PlayingStyle::PlayingStyle playingStyle__)
{
  Monitor monitor (this);
  playingStyle = playingStyle__;
  arrangeSongList ();
}

void SongManager::setLoopStyle (PlayingStyle::LoopStyle loopStyle__)
{
  Monitor monitor (this);
  loopStyle = loopStyle__;
}

void SongManager::appendSongListItem (const string& songName)
{
  Monitor monitor(this);
  songFileList.push_back (SongFileItem (songName));
  workingFileList.push_back (SongFileItem (songName));
}

void SongManager::appendSongListItemAndMoveTo (const string& songName)
{
  DEBUG_MESSAGE ("SongManager::appendSongListItemAndMoveTo called");
  Monitor monitor(this);
  songFileList.push_back (SongFileItem (songName));
  workingFileList.push_back (SongFileItem (songName));
  currentSong = workingFileList.end ();
  if (currentSong != workingFileList.begin ())
    currentSong--;
}

void SongManager::moveTo (const string& songName)
{
  DEBUG_MESSAGE ("SongManager::moveTo called");
  currentSong = workingFileList.find (songName);
  if (currentSong == workingFileList.end ())
    {
      if (currentSong != workingFileList.begin ())
	currentSong--;
    }
}

bool SongManager::removeSongListItem (const string& songName)
{
  Monitor monitor (this);
  SongFileList::iterator target = workingFileList.find (songName);
  if (target == workingFileList.end ())
    return false;//songNameʪϿƤʤä

  songFileList.erase (songFileList.find (*target));
  SongFileList::SongItem tmp = target;
  if (!isInHistory && tmp == currentSong)
    currentSong = workingFileList.erase (target);
  else
    workingFileList.erase (target);

  if (workingFileList.empty ())
    {
      assert (songFileList.empty ());
      currentSong = workingFileList.begin ();
      isStarted = false;
    }
  
  return true;
}

void SongManager::clearSongList ()
{
  Monitor monitor(this);
  songFileList.erase (songFileList.begin (), songFileList.end ());
  workingFileList.erase (workingFileList.begin (), workingFileList.end ());
  currentSong = workingFileList.begin ();
  isStarted = false;
}

void SongManager::readSongListFile (const string& listFileName)
{
  Monitor monitor (this);
  //XXX not implemented yet

  ifstream ifs (listFileName.c_str ());
  if (!ifs)
    throw CanNotOpenFile();

  
}

void SongManager::searchDirectory (const string direcotry, bool isRecuresive)
{
  Monitor monitor (this);
  
  //XXX not implemented yet
  
}


void SongManager::writeSongListFile (const string& listFileName,
				     int mode = ios::out | ios::noreplace)
{
  Monitor monitor (this);

  //XXX not implemeneted yet
  ofstream ofs (listFileName.c_str (), mode);
  if (!ofs)
    throw CanNotOpenFile ();
  
  for (SongFileList::SongItem i = songFileList.begin ();
       i != songFileList.end ();
       i++)
    {
      //XXX full pathˤɬפ롣
      ofs << (*i).getFileName () << "\n";
      ofs << (*i).getPlayTime () << "\n";
      //XXX title"\n"ȤޤΤŬ֤ɬפ롣
      ofs << (*i).getTitle () << "\n";
      ofs << "---\n";
    }
}

void SongManager::setHistorySize (uint historySize__)
{
  Monitor monitor (this);
  if (historySize__ > playedSongList.max_size ())
    historySize__ = playedSongList.max_size ();

  //;ʬ˳ФƤҥȥ꡼ڤͤᡣ
  while (historySize__ < playedSongList.size ())
    playedSongList.pop_front ();
  
  historySize = historySize__;
}
