#ifndef SongManager_H
#define SongManager_H

#include <list>

#include "PlayingStyle.h"
#include "SongFileList.h"
#include "util/Mutex.h"
#include "util/Random.h"


using namespace std;

class SongManager
{
public:
  //դʤФƤ
  //128ʤФƤнʬ?
  static const uint DEFAULT_MAX_SONG_HISTORY = 128;
private:
  //songmanagerѤ¾ѡ
  Mutex globalLock;
  Random random;

  //֤lock˻ѡ
  mutable Mutex localLock;
  class Monitor
  {
  private:
    const SongManager* const songManager;
  public:
    Monitor (const SongManager* songManager__):
      songManager (songManager__)
      {songManager->localLock.lock ();};
    ~Monitor ()
      {songManager->localLock.unlock ();};
  };
  friend Monitor;
  
  //TimerSignal timerSignal;
  //bool isInTimerd;
  //TimerSignal::Tag timerTag;
  bool isFreezed;

  //ζʤ֤Ȥ
  PlayingStyle::PlayingStyle playingStyle;
  PlayingStyle::LoopStyle loopStyle;

  //٤ǤmoveToNextSong () ƤФ줿?
  bool isStarted;
  
  SongFileList songFileList;
  SongFileList workingFileList;//(sortꤹ١)
  
  //դΤФƤ
  uint historySize;//ФƤҥȥ꡼ο
  SongFileList playedSongList;
  void addHistory (const SongFileItem& lastSong)
    {
      playedSongList.push_back (lastSong);
      if (playedSongList.size () > historySize)
	playedSongList.pop_front ();
    };
  
  
  
  //߻ؤƤ֡
  //workingFileList򤵤iterator
  SongFileList::SongItem currentSong;
  
  //ºݤ˱դƤΤcurrentSong뤤currentHistory?
  bool isInHistory;
  //playedSongListؤiterator
  SongFileList::SongItem currentHistorySong;
  
  //ɤ߹song listΥꥹ
  list<string> songListList;

  //addSongListItem,addSongListFile˻Ȥpath
  list<string> path;

  //songFileList.size ()!= 0ǤȤ
  //moveToNextSongƤФ줿ȤƤФơ
  //workingFileListplayingStyle˱ꤵ롣
  void arrangeSongList ();
  
public:
  SongManager ();
  virtual ~SongManager ();

  ///////////////////////////////////////////////////interface to WidiServer
  PlayingStyle::PlayingStyle getPlayingSytle () const
    {
      Monitor monitor (this);
      return playingStyle;
    };
  PlayingStyle::LoopStyle getLoopStyle () const
    {
      Monitor monitor (this);
      return loopStyle;
    }
  
  //const methodǤ̵Ȥա
  //֤ͤ˶ʤäɤ
  //äʤtrueʤäfalse֤
  bool moveToPrevSong ();
  
  //֤Ǥա
  const string getSongName () const;
  
  //const methodǤ̵Ȥա
  //֤ͤ˶ʤäɤ
  //äʤtrueʤäfalse֤
  bool moveToNextSong ();

  bool isEnd ()
    {
      Monitor monitor (this);
      return (!isInHistory && (currentSong == --workingFileList.end ()));
    };

  //֤Ǥա
  const SongFileList getSongList () const
    {
      Monitor monitor (this);
      return songFileList;
    };

  //֤Ǥա
  const list<string> getSongListList () const
    {
      Monitor monitor (this);
      return songListList;
    };
  
  
  //֤Ǥա
  const list<string> getPath () const
    {
      Monitor monitor (this);
      return path;
    };

  int getSontListSize () const
    {
      Monitor monitor (this);
      return songFileList.size ();
    };
  
  bool doesHave (const string& fileName)// const//SongFileList::find is not const method :-<
    {
      Monitor monitor (this);
      return (songFileList.find (fileName) != songFileList.end ());
    };
  
  
  /////////////////////////////////////////////////interface to subclass of IO
  void setPlayingStyle (PlayingStyle::PlayingStyle playingStyle__);
  void setLoopStyle (PlayingStyle::LoopStyle loopStyle__);
  
  void appendSongListItem (const string& songName);
  void appendSongListItemAndMoveTo (const string& songName);
  void moveTo (const string& songName);
  
  //֤ϤϤƤ줿Τºݤ˺줿?
  //ꤵ줿ΤϿƤˤкơtrue֤롣
  //ꤵ줿ΤϿƤ̵äfalse֤롣
  //Ϻǽ˸ĤäΰĤΤߺ
  //⤷ʣƱΤϿƤ顢ܰʹߤϻĤΤա
  bool removeSongListItem (const string& songName);

  //֤ϤϤƤ줿Τºݤ˺줿?
  //ƤƤֹˤΤ롣
  bool removeSongListItem (const uint32 i);

  void clearSongList ();

  //ꤷSongListɲä롣
  //ɤ߹ΤΤߤˤclearSongList () ˸ƤǤȡ
  void readSongListFile (const string& listFileName);
  void searchDirectory (const string direcotry, bool isRecuresive);
  void writeSongListFile (const string& listFileName,
			  int mode = ios::out | ios::noreplace);
  void setHistorySize (uint historySize__);
  ///exceptional classes
  class SError {};
  class FileIsNotFound:public SError {};
  class CanNotOpenFile:public SError {};
  class DirecotryIsNotFound:public SError {};
  class FileExists:public SError {};

  
  uint getHistorySize () const
    {
      Monitor monitor (this);
      return historySize;
    };

  //SongManagerξ֤򤹤Τ롣
  void broadcast ()
    {
      Monitor (this);
      //XXX
      //timerSignal ();
    };
  //SongManagerξΤ롣
  void freeze ()
    {
      Monitor monitor (this);
      isFreezed = true;
    };
  //SongManagerξΤ
  void thaw ()
    {
      Monitor monitor (this);
      isFreezed = false;
    };
  
  ///////////////////////////////////////////////////////////////////locking
  void lock ()
    {
      Monitor monitor (this);
      globalLock.lock ();
    };
  int tryLock ()
    {
      Monitor monitor (this);
      return globalLock.trylock ();
    };
  void unlock ()
    {
      Monitor monitor (this);
      globalLock.unlock ();
    };
};
#endif// SongManager_H
