#include "Timer.h"
#include "util/PThreadAttribute.h"
#include "util/PThreadStartRoutine.h"

Timer::Timer ():
  isFinish (false), tagIndex (0)
{
  PThreadAttribute attribute;
  pthread_create (&loopThread, &attribute.getPThreadAttribute (),
		  &PThreadStartRoutine<Timer, &Timer::timerLoop>, this);
}

Timer::~Timer ()
{
  mutex.lock ();
  isFinish = true;
  mutex.unlock ();
  condVar.signal ();
  pthread_join (loopThread, NULL);

  for (list<ListNode*>::iterator i = timeOuts.begin ();
       i != timeOuts.end ();
       i++)
    delete (*i);
}

void Timer::timerLoop ()
{
  DEBUG_MESSAGE ("Timer::timerLoop called");
  mutex.lock ();

  while (1)
    {
      int ret;
      if (timeOuts.size () > 0)
	{
	  //DEBUG_MESSAGE ("Timer::timerLoop found waiting event");
	  //cerr << "timeOuts.size () = " << timeOuts.size () << endl;
	  //DEBUG_MESSAGE ("Timer::timerLoop waiting");
	  //Ԥ
	  timespec nextTime = (*timeOuts.begin ())->timeSpec;
#if 0
	  cerr << "nextTime.tv_sec=" << nextTime.tv_sec
	       << ":nextTime.tv_nsec=" << nextTime.tv_nsec
	       << endl;
#endif
	  ret = condVar.timedWait (mutex, &nextTime);
	}
      else
	{//timeOuts.size () == 0
	  //⤦ԤäƤʪʤ
	  //Ԥ󤬶ʤΤǾ֤ѲޤԤġ
	  ret = 0;
	  condVar.wait (mutex);
	}
      assert (ret == 0 || ret == ETIMEDOUT);

#if 0
      if (ret == 0)
	DEBUG_MESSAGE ("Timer::timerLoop signaled");
      else if (ret == ETIMEDOUT)
	DEBUG_MESSAGE ("Timer::timerLoop timedout");
      else
	DEBUG_MESSAGE ("Timer::timerLoop error!");
#endif
      
      //֤褿Τ,timer handerƤӽФ
      timeval timeVal;
      gettimeofday (&timeVal, NULL);
      while (timeOuts.size () > 0 &&
	     compareTime ((*timeOuts.begin ())->timeSpec, timeVal) <= 0)
	{
	  ListNode* listNode = *timeOuts.begin ();
	  timeOuts.pop_front ();
	  //줫¹Ԥtag
	  for (TimerSignal::Connections::iterator i = listNode->signal.getBegin ();
	       i != listNode->signal.getEnd ();
	       i++)
	    tags.erase ((*i).first);
	  
	  //signal()ϻ֤뤫ΤʤΤunlockƼ¹Ԥ롣
	  mutex.unlock ();
	  listNode->signal ();
	  delete listNode;
	  mutex.lock ();
	  
	  gettimeofday (&timeVal, NULL);
	}
      
      if (isFinish)
	break;//λ롣ĤäƤ륿ޡ¹ԤȤʤƤ?
    }
  mutex.unlock ();

  DEBUG_MESSAGE ("Timer::timerLoop finished");
}



bool Timer::disconnect (Timer::Tag tag)
{
  bool ret = false;
  mutex.lock ();
  map<Tag, ListNode*>::iterator node = tags.find (tag);
  if (node != tags.end ())
    {
      ret = true;

      const bool tmp = (*node).second->signal.disconnect (tag);
      assert (tmp == true);
      map<Tag, ListNode*>::size_type erasedNumber = tags.erase (tag);
      assert (erasedNumber == 1);
      
      
      if ((*node).second->signal.size () == 0)
	{//nodeϤ⤦ɬפʤΤǼ롣
	  //DEBUG_MESSAGE ("Timer::disconnect: deleting node");
	  for (list<ListNode*>::iterator i = timeOuts.begin ();
	       i != timeOuts.end ();
	       i++)
	    {
	      if ((*node).second == *i)
		{
		  assert ((*i)->signal.size () == 0);
		  delete (*i);
		  timeOuts.erase (i);
		  break;
		}
	    }
	}
    }
  mutex.unlock ();

  return ret;
}
