#ifndef Timer_H
#define Timer_H

#include <map>
#include <sys/time.h>

#include "TimerSignal.h"
#include "util/Mutex.h"
#include "util/CondVar.h"
#include "util/compareTime.h"

using namespace std;

class Timer
{
public:
  typedef TimerSignal::Tag Tag;
private:
  Mutex mutex;
  CondVar condVar;
  bool isFinish;//main loopȴ뤫ݤ
  
  class ListNode
  {
  public:
    timespec timeSpec;
    TimerSignal signal;
    ListNode (timespec timeSpec__):timeSpec (timeSpec__) {};
  };
  list<ListNode*> timeOuts;
  map<Tag, ListNode*> tags;
  Tag tagIndex;

  pthread_t loopThread;
  void timerLoop ();
public:
  Timer ();
  ~Timer ();

  //interval milli secondtimerHandlerinvoke롣
  template <typename T>
  Tag connect (uint32 interval,
	       T* obj,void (T::*timerHandler) (void* arg), void* data);
  //tagǼΤϿä롣
  //ºݤä줿true֤
  //⤦Ǥ˼¹ԤƤϿƤʤäfalse֤
  bool disconnect (Tag tag);
};

template <typename T>
Timer::Tag Timer::connect (uint32 interval,
			   T* obj, void (T::*timerHandler) (void* arg),
			   void* data)
{
  //DEBUG_MESSAGE ("Timer::connect called");
  //invokeл֤롣
  timeval timeVal;
  gettimeofday (&timeVal, NULL);
  timespec timeSpec;
  timeSpec.tv_sec = timeVal.tv_sec + interval/1000;
  timeSpec.tv_nsec = timeVal.tv_usec*1000 + (interval%1000)*1000000;
  if (timeSpec.tv_nsec > 999999999)
    {
      timeSpec.tv_sec += timeSpec.tv_nsec/1000000000;
      timeSpec.tv_nsec %= 1000000000;
    }

  mutex.lock ();
      
  if (tags.size () == tags.max_size ())
    {//ϿƤtagƤޤä
      DEBUG_MESSAGE ("Timer::connect:can not connect. Error!");
      exit (1);
    }

  //ȤƤʤtag򸫤Ĥ롣
  while (tags.find (tagIndex) != tags.end ())
    {
      tagIndex++;
      if (tagIndex >= tags.max_size ())
	tagIndex = 0;
    }
  

  //򸫤Ĥ롣
  list<ListNode*>::iterator i = timeOuts.begin ();
  while (i != timeOuts.end () &&
	 compareTime (timeSpec, (*i)->timeSpec) > 0)
    i++;

  //ºݤconnect롣
  ListNode* node;
  if (i != timeOuts.end () &&
      compareTime ((*i)->timeSpec, timeSpec) == 0)
    {//λ֤nodeˤä
      (*i)->signal.connect (tagIndex, obj, timerHandler, data);
      node = *i;
    }
  else
    {//nodeɬפ롣
      node = new ListNode (timeSpec);
      node->signal.connect (tagIndex, obj, timerHandler, data);
      timeOuts.insert (i, node);
    }

  tags[tagIndex] = node;
  const int retTag = tagIndex++;
  if (tagIndex >= tags.max_size ())
    tagIndex = 0;
  mutex.unlock ();
  condVar.signal ();
  
  //DEBUG_MESSAGE ("Timer::connect done");
  return retTag;
}

#endif //Timer_H
