/*
    WidiServer.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef WidiServer_H
#define WidiServer_H

#include <list>

#include "State.h"
#include "ServerCommand.h"
#include "SongManager.h"
#include "WidiController.h"
#include "IO/InfoIO.h"
#include "IO/IO.h"
#include "IO/MainIO.h"
#include "Sequencer/Sequencer.h"
#include "Song/Song.h"
#include "Song/SongCommand.h"
#include "util/DlHandler.h"

using namespace std;

class WidiServer
{
private:
  State* const state;
  SongCommand songCommand;
  SongManager* const songManager;
  ServerCommand serverCommand;
  //SongManager$B$H(BserverCommand$B$r0l$D$K$^$H$a$k!#(B
  WidiController* const widiController;

  Option* option;
  MainIO* mainIO;
  DlHandler* mainDlHandler;
  list<IO*> io;
  int nIO;//io$B$N?t!#(B= io.size ()
  list<DlHandler*> ioDlHandler;
  list<DlHandler*> preloadDlHandler;
  
  PlayingStyle::PlayingMode playingMode;
  DeviceResetType::DeviceResetType deviceResetType;
  
  //$BA06J1iAU$7=*$($?;~!"=*N;$9$k$+(Bevent$BBT$A%k!<%W$K$O$$$k$+(B
  //default$B$G$O(BisQuitWhenFinished = true
  //$B$3$l$O!"(BIO,MainIO$B$,$=$l$>$l%f!<%6!<$+$i$N%$%Y%s%HBT$A%k!<%W$KF~$k$+$K(B
  //$B0MB8$9$k!#0l$D$G$b%$%Y%s%HBT$A%k!<%W$KF~$k$J$i!"(B
  //isQuitWhenFinished=false$B$H$J$k!#(B
  bool isQuitWhenFinished;

public:
  WidiServer (State* state__, Option* option__);
  virtual ~WidiServer ();

  ////////////////////////////////////////////////////////main loop
  void mainLoop (list<string> files, Sequencer* serquencer_);
  
  ServerCommand* getServerCommand (){return &serverCommand;};
private:

  //main loop$B$NJd=u(B
  //$B$3$l$r8F$S=P$9;~$O(BsongCommand,state$B$r6&$K$r(Block$B$7$F$$$k;v(B
  //serverCommand$B$O(Block$B$7$F$$$J$$$3$H!#(B
  void joinAndIgnoreServerCommandDone (pthread_t* thread, pthread_t* ioThread);
  //$B8F$S=P$7;~$K(Bstate$B$r(Block$B$7$F$$$F$O$$$1$J$$(B
  void createPlayThread (Song* song, Sequencer* serquencer,
			 pthread_t* thread, pthread_t* ioThread);

  //createPlayThread$B$G:n$C$?(Bthread$B$r(Bjoin$B$9$k!#(B
  //serverCommand,songCommand,state$B$r(Block$B$7$F$$$J$$>uBV$G8F$S=P$9;v!#(B
  //$B2?8N$J$i(Bclass Song$B$,(Bstate->songStatus.state=DONE$B$K=q$-49$($i$l$J$$$+$i!#(B
  //songState.state==DONE ---> mainIO$B$,(BserverCommand::DONE$B$H$J$i$J$$$+$i!#(B
  void joinPlayThread (pthread_t* thread, pthread_t* ioThread);


  
  //lock$B$9$k=gHV$O(B(1)songCommand,(2)state$B$N=g$G$9$k;v!#(B
  //$B$3$l$O(Bsong$B$,1iAUCf$G$"$k$3$H$rA0Ds$7$F$$$F!"$=$l0J30$N;~$K8F$P$l$k$H(B
  //dead lock$B$9$k62$l$,$"$k$N$GCm0U!#(B
  //$B$D$^$j(Bsong$B$,1iAUCf$GL5$$;~$O(Bsong command$B$KL$=hM}$N(Bcommand$B$,;D$C$F$$$J$$(B
  //$B;v$rJ]>Z$9$kI,MW$,$"$k!#(B
  void waitLockSongCommand ();
  //$B$3$l$O(BsongCommand$B$r(Block$B$7$?>uBV$G8F$S=P$9I,MW$,$"$k!#(B
  //$B2?$+(Bcommand$B$r<B9TCf$J$i$=$l$,=*N;$9$k$^$GBT$D!#(B
  //$BJV$C$F$/$k$H$-$b(Bstate->$B%m%C%/$5$l$?>uBV$GJV$C$F$/$k!#(B
  void waitLockState (pthread_t* thread, pthread_t* ioThread);

  void lockSongCommandAndStatus (pthread_t* thread, pthread_t* ioThread)
    {waitLockSongCommand ();waitLockState (thread, ioThread);};
  void unlockSongCommandAndStatus ()
    {state->unlock ();songCommand.unlock ();songCommand.broadcast ();};
  
  
  ////////////////////////////////////////////method for playing
  void play (Song*& song, Sequencer* sequencer,
	     //list<string>& files, list<string>::iterator& i,
	     SongManager& songManager__, 
	     pthread_t* thread, pthread_t* ioThread);
  //filename$B$G;XDj$5$l$?J*$r1iAU$9$k!#(B
  //$B$b$7(Bfilename$B$,(Bfiles$B$NCf$K$J$1$l$PDI2C$9$k!#(B
  //$B:F@8Cf$N$b$N$,$"$l$P6/@)=*N;!#(B
  void playWithFileName (Song*& song, Sequencer* sequencer,
			 //list<string>& files,
			 SongManager& songManager__,
			 const string& filename,
			 pthread_t* thread, pthread_t* ioThread);
  typedef void (WidiServer::*unlockDoCommand) ();
  
  void lockDoCommand (unlockDoCommand doCommand,
		      pthread_t* thread, pthread_t* ioThread);
  void unlockPause ();
  void unlockContinuePlay ();
  void unlockRewind ();
  void unlockForward ();
  void unlockBack ();
  void stop (Song*& song, pthread_t* thread, pthread_t* ioThread);
  void set (ulong position, pthread_t* thread, pthread_t* ioThread);
  

  typedef void (WidiServer::*unlockDoMidiCommand) (ServerCommand& tmpCommand);
  void lockDoMidiCommand (unlockDoMidiCommand doCommand,
			  Song* song,  ServerCommand& tmpCommand,
			  pthread_t* thread, pthread_t* ioThread);
  void unlockSetSkew (ServerCommand& tmpCommand);
  void unlockMidiNoteOn (ServerCommand& tmpCommand);
  void unlockMidiNoteOff (ServerCommand& tmpCommand);
  void unlockMidiPolyphonicKeyPressure (ServerCommand& tmpCommand);
  void unlockMidiControlChange (ServerCommand& tmpCommand);
  void unlockMidiRPN (ServerCommand& tmpCommand);
  void unlockMidiNRPN (ServerCommand& tmpCommand);
  void unlockMidiProgramChange (ServerCommand& tmpCommand);
  void unlockMidiChannelPressure (ServerCommand& tmpCommand);
  void unlockMidiPitchBendChange (ServerCommand& tmpCommand);
  void unlockMidiSysEx (ServerCommand& tmpCommand);
  void unlockSetMute (ServerCommand& tmpCommand);
  void unlockToggleMute (ServerCommand& tmpCommand);
  
  void prev (Song*& song, Sequencer* sequencer,
	     //list<string>& files, list<string>::iterator& i,
	     SongManager& songManger__,
	     pthread_t* thread, pthread_t* ioThread);
  void next (Song*& song, Sequencer* sequencer,
	     //list<string>& files, list<string>::iterator& i,
	     SongManager& songManager__,
	     pthread_t* thread, pthread_t* ioThread);
  void open (pthread_t* thread, pthread_t* ioThread);

  //$B6J$N=*N;=hM}$r$9$k!#(B
  //$B$^$@!"1iAU$9$k6J$,;D$C$F$$$k$J$i(Btrue
  //$BJV$jCM$OA06J1iAU=*N;$J$i(B false
  //$B$rJV$9!#(B
  bool done (Song*& song, Sequencer* sequencer,
	     //list<string>& files, list<string>::iterator& i,
	     SongManager& songManager__,
	     pthread_t* thread, pthread_t* ioThread);

  void getFileList () const {};//XXX

  //seeking$BCf$N;~$O8F$s$G$O$$$1$J$$!#(B
  //$B$=$l0J30$N>uBV$K$D$$$F$O8F$s$G$bBg>fIW!#(B
  //songCommand,state$B$NN>J}$r(Block$B$7$F$+$i8F$S=P$9;v!#(B
  void abort (pthread_t* thread, pthread_t* ioThread);
};
#endif WidiServer_H
