/*
    tesetWidiServer.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <cstdio>
#include <sys/ioctl.h>
#include <iostream>
#include <unistd.h>
#include <getopt.h>

#include <list>

#include "WidiServer.h"
#include "IO/MainIO.h"
#include "IO/InfoIO.h"
#include "Sequencer/SequencerNull.h"
#include "Sequencer/OSS/MIDI/SequencerMIDI.h"
#include "util/DlHandler.h"

void printUsage (const char* progName)
{
  cerr << "Usage:" << progName  << "[options] midi-file..." << endl
       << "options:" << endl
       << "--convert,-c\n"
       << "--io <module name> ,-i <module name>\n"
       << "--norealtime,-n\n"
       << endl;
}

int
main (int argc,char** argv)
{
  char* progName=argv[0];
  if (argc<=1)
    {
      printUsage (progName);
      exit (-1);
    }
  State state;
  Option option;
  option.isQuickStart=true;
  option.isRealTime=true;
  option.modulePath=new char*[2];
  *(option.modulePath)=new char[strlen (WIDI_LIB_DIR)+1];
  strcpy (*(option.modulePath),WIDI_LIB_DIR);
  *(option.modulePath+1)=NULL;
  option.mainIO="Xlib";
  option.ioNames=NULL;
  WidiServer server (&state,&option);

  SequencerOSS::open ();
  Sequencer* sequencer=new SequencerMIDI (&state,0,&option);
  //Sequencer* sequencer=new SequencerNull (&state,0,&option);
    
  list<const char*> files;
  for (int i=optind;i<argc;i++)
    files.push_back (argv[i]);
  server.mainLoop (files,sequencer);
  
  delete sequencer;
  SequencerOSS::close ();

  delete[] option.modulePath[1];
  delete[] option.modulePath;
  exit (0);
}

