#ifndef nanosleep_H
#define nanosleep_H

#ifndef HAVE_NANOSLEEP

#include <unistd.h>
#include <ctime>
#include <sys/time.h>

#include "util/CondVar.h"
#include "util/getABSTimeSpec.h"

using namespace std;

ifdef __cplusplus
extern "C" {
#endif
//for compatibility
int nanosleep (const timespec* req, timespec* rem)
{
  if (req == NULL ||
      req->tv_sec < 0 || req->tv_nsec < 0 || req->tv_nsec > 999999999)
    {
      errno = EINVAL;
      return -1;
    }
  
  
  unsigned long usec = req->tv_sec * 1000 + req->tv_nsec/1000;
  
  usleep (usec);
  
  return 0;
};


#if 0
int nanosleep(const timespec* req, timespec* rem)
{
  if (req == NULL ||
      req->tv_sec < 0 || req->tv_nsec < 0 || req->tv_nsec > 999999999)
    return EINVAL;

  Mutex mutex;
  CondVar condVar;

  timespec timeSpec;
  timeval timeVal;
  gettimeofday (timeVal, NULL);
  timeSpec.tv_sec = timeVal.tv_sec + req->tv_sec;
  timeSpec.tv_nsec = timeVal.tv_usec*1000 + req->tv_nsec;
  if (timeSpec.tv_nsec >= 1000000000)
    {//$B7+>e$,$j=hM}!#(B
      timeSpec.tv_sec += timeSpec.tv_nsec/1000000000;
      timeSpec.tv_nsec %= 1000000000;
    }
  
  mutex.lock ();
  const int ret = condVar.timeWait (mutex, timeSpec);
  mutex.unlock ();

  if (ret == ETIMEDOUT)
    return 0;

  if (ret == EINTR)
    {
      if (rem !=NULL)
	{
	  timeval afterTimeVal;
	  gettimeofday (&afterTimeVal, NULL);
	  rem->tv_sec = req->tv_sec-(afterTimeVal.tv_sec-timeVal.tv_sec);
	  rem->tv_nsec = req->rv_nsec-(afterTimeVal.tv_usec-timeVal.tv_usec)*1000;
	  if (rem->tv_nsec < 0)
	    {//$B7+2<$,$j=hM}(B
	      rem->tv_nsec += 1000000000;
	      rem->tv_sec -= 1;
	    }
	  assert (rem->tv_sec >= 0);
	  assert (rem->tv_nsec >= 0);
	  assert (rem->tv_nsec < 1000000000);
#if 0
	  if (rem->tv_sec < 0)
	    errno = ETIMEDOUT;
	    retirn -1;
#endif
	}
      errno = EINTR;
      return -1;
    }

  assert (ret == 0);
  return 0;
};
#endif

#ifdef __cplusplus
}
#endif


#endif // HAVE_NANOSLEEP

#endif //nanosleep_H

