/*
    CondVar.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef CondVar_H
#define CondVar_H

#include <cassert>

#include "Mutex.h"

using namespace std;

class CondVar
{
private:
  pthread_cond_t cond;
public:
  CondVar () throw (Error)
    {
      const int ret = pthread_cond_init (&cond,NULL);
      if (ret != 0)
	error (ret);
    };
  CondVar (const pthread_condattr_t* attr) throw (Error)
    {
      const int ret=pthread_cond_init (&cond,attr);
      if (ret != 0)
	error (ret);
    };
  ~CondVar ()
    {
      const int ret = pthread_cond_destroy (&cond);
      if (ret != 0)
	error (ret);
    };
  inline void broadcast () throw (Error)
    {
      const int ret = pthread_cond_broadcast (&cond);
      if (ret != 0)
	error (ret);
    };
  inline void signal () throw (Error)
    {
      const int ret = pthread_cond_signal (&cond);
      if (ret != 0)
	error (ret);
    };
  inline void wait (Mutex& mutex) throw (Error)
    {
      const int ret = pthread_cond_wait (&cond,&mutex.mutex);
      if (ret != 0)
	error (ret);
    };
  inline int timedWait (Mutex& mutex,const timespec* abstime) throw (Error)
    {
      const int ret=pthread_cond_timedwait (&cond,&mutex.mutex,abstime);
      assert (ret == 0 || ret == ETIMEDOUT || ret == EINTR);
      if (ret != 0 && ret != ETIMEDOUT && ret != EINTR)
	error (ret);
      return ret;
    }
};
#endif// CondVar_H
