#include <iostream>

#include "DlHandler.h"

using namespace std;

DlHandler::DlHandler (Option* option_):
  option (option_), handle (NULL)
{
  /*do nothing*/
}

DlHandler::~DlHandler ()
{
  close ();
}

#ifdef RTLD_GLOBAL
const void* DlHandler::open (const string& filename,
			     int flag = RTLD_GLOBAL | RTLD_LAZY)
#else
const void* DlHandler::open (const string& filename, int flag = RTLD_LAZY)//Does this work well?
#endif
{
  assert (handle == NULL);
  handle = dlopen (filename.c_str (), flag);

  if (handle != NULL)
    return handle;//library found

  for (list<string>::const_iterator i = option->modulePath.begin ();
       i != option->modulePath.end ();
       i++)
    {//library not found.search direcotries
      string absFilename;
      absFilename = *i;
      absFilename += "/";
      absFilename += filename;

      handle = dlopen (absFilename.c_str (), flag);
      if (handle != NULL)
	break;
    }
#if 0
  if (handle == NULL)
    {//library not found in the directoryies.assuming statically linked.
      cerr << "DlHandler:can not load plug in=" << filename
	   << " in following dirctories.";
      for (list<string>::const_iterator i = option->modulePath.begint ();
	   i != option->modulePath.end ();
	   i++)
	cerr << ":" << *i;
      cerr << "\nassuming statically linked" << endl;
      handle = dlopen (NULL, flag);
    }
#endif
  if (handle == NULL)
    {//library not found at all.
      cerr << "DlHandler:can not load plug in=" << filename
	   << " in following dirctories.";
      for (list<string>::const_iterator i = option->modulePath.begin ();
	   i != option->modulePath.end ();
	   i++)
	cerr << ":" << *i;
      cerr << endl << dlerror () << endl;
    }
  return handle;
}

const void* DlHandler::openLibSo (const string& prefix, const string& name)
{//"lib"+prefix+name+".so"$B$r(Bopen$B$9$k!#(B
  string libPrefixNameSo = "lib";
  libPrefixNameSo += prefix;
  libPrefixNameSo += name;
  libPrefixNameSo += ".so";

  const void* ret = open (libPrefixNameSo);
  return ret;
}

void* DlHandler::sym (string& symbol)
{
  assert (handle != NULL);
  void* ret;
  ret = dlsym (handle, symbol.c_str ());
  if (ret == NULL)
    {
      cerr << "DlHandler:can not resolve symbol=" << symbol << endl;
      cerr << dlerror () << endl;
    }
  return ret;
}

void DlHandler::close ()
{
  if (handle != NULL)
    {
      dlclose (handle);
      handle = NULL;
    }
}

DlHandler* DlHandler::newOpenDlHandler (const string& prefix,
					const string& name,
					Option* option)
{
  assert (option != NULL);
  DlHandler* retHandler = new DlHandler (option);
  if (retHandler->openLibSo (prefix, name) == NULL)
    {
      delete retHandler;
      retHandler = NULL;
    }
  return retHandler;
}
