/*
    Mutex.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#ifndef Mutex_H
#define Mutex_H

#include <cerrno>
#include <pthread.h>

#include "Error.h"
#include "misc.h"

using namespace std;

class CondVar;

class Mutex
{
private:
  friend CondVar;
  pthread_mutex_t mutex;
public:
  Mutex () throw (Error)
    {
      const int ret=pthread_mutex_init (&mutex,NULL);
      if (ret!=0)
	error (ret);
    };
  Mutex (const pthread_mutexattr_t* attr) throw (Error)
    {
      const int ret=pthread_mutex_init (&mutex,attr);
      if (ret!=0)
	error (ret);
    }
  ~Mutex () throw (Error)
    {
      const int ret=pthread_mutex_destroy (&mutex);
      if (ret!=0)
	error (ret);
    };
  inline void lock () throw (Error)
    {
      const int ret=pthread_mutex_lock (&mutex);
      if (ret!=0)
	error (ret);
    };
  inline int trylock () throw (Error)
    {
      const int ret = pthread_mutex_trylock (&mutex);
      if (ret != 0 && ret != EBUSY)
	error (ret);
      return ret;
    };
  inline void unlock () throw (Error)
    {
      const int ret=pthread_mutex_unlock (&mutex);
      if (ret!=0)
	error (ret);
    };
};
#endif //Mutex_H
